#define TOKEN_DONE_KW                         1
#define TOKEN_SEMICOLON                       2
#define TOKEN_DEBUG_KW                        3
#define TOKEN_OPEN_BRACE                      4
#define TOKEN_CLOSE_BRACE                     5
#define TOKEN_FILENAME_KW                     6
#define TOKEN_QUOTED                          7
#define TOKEN_NAME                            8
#define TOKEN_LEVEL_KW                        9
#define TOKEN_INTEGER                        10
#define TOKEN_PDU_KW                         11
#define TOKEN_GOP_KW                         12
#define TOKEN_GOG_KW                         13
#define TOKEN_DEFAULT_KW                     14
#define TOKEN_LAST_EXTRACTED_KW              15
#define TOKEN_DROP_UNASSIGNED_KW             16
#define TOKEN_DISCARD_PDU_DATA_KW            17
#define TOKEN_EXPIRATION_KW                  18
#define TOKEN_IDLE_TIMEOUT_KW                19
#define TOKEN_LIFETIME_KW                    20
#define TOKEN_SHOW_TREE_KW                   21
#define TOKEN_SHOW_TIMES_KW                  22
#define TOKEN_GOP_TREE_KW                    23
#define TOKEN_TRANSFORM_KW                   24
#define TOKEN_MATCH_KW                       25
#define TOKEN_STRICT_KW                      26
#define TOKEN_EVERY_KW                       27
#define TOKEN_LOOSE_KW                       28
#define TOKEN_REPLACE_KW                     29
#define TOKEN_INSERT_KW                      30
#define TOKEN_PROTO_KW                       31
#define TOKEN_TRANSPORT_KW                   32
#define TOKEN_PAYLOAD_KW                     33
#define TOKEN_CRITERIA_KW                    34
#define TOKEN_ACCEPT_KW                      35
#define TOKEN_REJECT_KW                      36
#define TOKEN_EXTRACT_KW                     37
#define TOKEN_FROM_KW                        38
#define TOKEN_LAST_PDU_KW                    39
#define TOKEN_SLASH                          40
#define TOKEN_ON_KW                          41
#define TOKEN_START_KW                       42
#define TOKEN_STOP_KW                        43
#define TOKEN_NO_TREE_KW                     44
#define TOKEN_PDU_TREE_KW                    45
#define TOKEN_FRAME_TREE_KW                  46
#define TOKEN_BASIC_TREE_KW                  47
#define TOKEN_TRUE_KW                        48
#define TOKEN_FALSE_KW                       49
#define TOKEN_FLOATING                       50
#define TOKEN_NULL_TREE_KW                   51
#define TOKEN_FULL_TREE_KW                   52
#define TOKEN_MEMBER_KW                      53
#define TOKEN_EXTRA_KW                       54
#define TOKEN_COMMA                          55
#define TOKEN_OPEN_PARENS                    56
#define TOKEN_CLOSE_PARENS                   57
#define TOKEN_AVP_OPERATOR                   58
#define TOKEN_PIPE                           59
#define TOKEN_DOTED_IP                       60
#define TOKEN_COLONIZED                      61
