/* iana-snap-pid.h
 * SNAP PIDs for the IANA's OUI 00:00:5e
 * See
 *
 *	http://www.iana.org/assignments/ethernet-numbers
 *
 * $Id: iana_snap_pid.h 28287 2009-05-06 00:59:59Z guy $
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define IANA_PID_MARS_DATA_SHORT	0x0001	/* RFC 2022 */
#define IANA_PID_NHRP_RESERVED		0x0002	/* RFC 2332 */
#define IANA_PID_MARS_NHRP_CONTROL	0x0003	/* RFC 2022, RFC 2332 */
#define IANA_PID_MARS_DATA_LONG		0x0004	/* RFC 2022 */
#define IANA_PID_SCSP			0x0005	/* RFC 2334 */
#define IANA_PID_VRID			0x0006
#define IANA_PID_L2TP			0x0007	/* RFC 3070 */
#define IANA_PID_VPN_ID			0x0008	/* RFC 2684 */
#define IANA_PID_MSDP_GRE_PROTO_TYPE	0x0009
