; wsutil.def
; $Id: libwsutil.def 41523 2012-03-13 08:57:04Z etxrab $

; Add symbols that should be exported out of libwsutil.dll
;
; DATA symbols must be declared WS_VAR_IMPORT in the header file!
;
; NOTE: TABS DON'T WORK IN THIS FILE, USE SPACES INSTEAD!!!
;
EXPORTS
; airpdcap.c
AirPDcapWepDecrypt

; crc6.c
update_crc6_by_bytes

; crc10.c
update_crc10_by_bytes

; crc8.c
crc8_0x2F

; crc16.c
crc16_0x5935
crc16_ccitt
crc16_ccitt_seed
crc16_x25_ccitt

; crc16-plain.c
crc16_plain_update

; crc32.c
crc32_ccitt
crc32_ccitt_seed
crc32_ccitt_table_lookup
crc32c_calculate
crc32c_calculate_no_swap
crc32c_table_lookup
crc32_mpeg2_seed

; crcdrm.c
crc_drm

; file_util.c
getenv_utf8
ws_stdio_fopen
ws_stdio_freopen
ws_stdio_mkdir
ws_stdio_open
ws_stdio_remove
ws_stdio_rename
ws_stdio_stat64
ws_stdio_unlink
ws_init_dll_search_path
ws_load_library
ws_module_open

; inet_aton.c
inet_aton

; inet_ntop.c
; Note: inet_v6defs.h renames this function for Windows (only)
ws_inet_ntop

; inet_pton.c
; Note: inet_v6defs.h renames this function for Windows (only)
ws_inet_pton

; mpeg-audio.c
mpa_bitrate
mpa_frequency
mpa_layer
mpa_padding
mpa_samples
mpa_version

; privileges.c
init_process_policies
get_cur_groupname
get_cur_username
npf_sys_is_running
relinquish_special_privs_perm
running_with_special_privs
started_with_special_privs

; strptime.c
strptime

; str_util.c
ascii_strdown_inplace
ascii_strup_inplace
isprint_string
isdigit_string

; type_util.c
type_util_gdouble_to_guint64
type_util_guint64_to_gdouble

; unicode-utils.c
utf_16to8
utf_8to16
utf_8to16_snprintf
arg_list_utf_16to8

; wsgetopt.c
getopt
optarg DATA
optind DATA
opterr DATA
optopt DATA
