#include <windows.h>
#include "richedit.h"
#include "../ui/win32/file_dlg_win32.h"

WIRESHARK_OPENFILENAME_TEMPLATE DIALOG 0, 0, 368, 94
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    // Filter button/entry
    // PUSHBUTTON "Filter:", EWFD_FILTER_BTN, 7, 4, 35, 14
    LTEXT "Display filter:", EWFD_FILTER_LBL, 7, 7, 49, 14
    CONTROL    "",        EWFD_FILTER_EDIT, RICHEDIT_CLASS, ES_AUTOHSCROLL, 52, 5, 88, 12, WS_EX_CLIENTEDGE

    CHECKBOX "MAC name resolution", EWFD_MAC_NR_CB, 7, 32, 100, 8,  BS_AUTOCHECKBOX | WS_GROUP | WS_TABSTOP
    CHECKBOX "Network name resolution", EWFD_NET_NR_CB, 7, 47, 100, 8,  BS_AUTOCHECKBOX | WS_GROUP | WS_TABSTOP
    CHECKBOX "Transport name resolution", EWFD_TRANS_NR_CB, 7, 62, 100, 8,  BS_AUTOCHECKBOX | WS_GROUP | WS_TABSTOP

    LTEXT "Filename:", EWFD_PT_FILENAME, 164, 4, 40, 8
    LTEXT "-", EWFD_PTX_FILENAME, 211, 4, 150, 8
    LTEXT "Format:", EWFD_PT_FORMAT, 164, 19, 40, 8
    LTEXT "-", EWFD_PTX_FORMAT, 211, 19, 150, 8
    LTEXT "Size:", EWFD_PT_SIZE, 164, 34, 40, 8
    LTEXT "-", EWFD_PTX_SIZE, 211, 34, 150, 8
    LTEXT "Packets:", EWFD_PT_PACKETS, 164, 49, 40, 8
    LTEXT "-", EWFD_PTX_PACKETS, 211, 49, 150, 8
    LTEXT "First Packet:", EWFD_PT_FIRST_PKT, 164, 64, 40, 8
    LTEXT "-", EWFD_PTX_FIRST_PKT, 211, 64, 150, 8
    LTEXT "Elapsed:", EWFD_PT_ELAPSED, 164, 79, 40, 8
    LTEXT "-", EWFD_PTX_ELAPSED, 211, 79, 150, 8
}

WIRESHARK_SAVEFILENAME_TEMPLATE DIALOG 0, 0, 368, 40
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
}

WIRESHARK_EXPORT_SPECIFIED_PACKETS_FILENAME_TEMPLATE DIALOG 0, 0, 368, 128
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    GROUPBOX "Packet Range", -1, 5, 21, 220, 102

    CONTROL  "Captured", EWFD_CAPTURED_BTN, "Button", BS_AUTORADIOBUTTON | WS_GROUP, 120, 32, 45, 10
    CONTROL  "Displayed", EWFD_DISPLAYED_BTN, "Button", BS_AUTORADIOBUTTON, 170,  32, 47, 10

    CONTROL  "All packets", EWFD_ALL_PKTS_BTN, "Button", BS_AUTORADIOBUTTON | WS_GROUP, 16, 44, 51, 10
    CONTROL  "Selected packet", EWFD_SEL_PKT_BTN, "Button", BS_AUTORADIOBUTTON,  16, 56, 68, 10
    CONTROL  "Marked packets", EWFD_MARKED_BTN, "Button", BS_AUTORADIOBUTTON,  16, 68, 67, 10
    CONTROL  "First to last marked", EWFD_FIRST_LAST_BTN, "Button",  BS_AUTORADIOBUTTON, 16, 80, 75, 10
    CONTROL  "Range:", EWFD_RANGE_BTN, "Button",  BS_AUTORADIOBUTTON, 16, 92, 35, 10
    EDITTEXT EWFD_RANGE_EDIT, 53, 91, 50, 12, ES_AUTOHSCROLL
    CONTROL  "Remove Ignored packets", EWFD_REMOVE_IGN_CB, "Button",  BS_AUTOCHECKBOX | WS_TABSTOP, 16, 104, 100, 10

    LTEXT "0", EWFD_ALL_PKTS_CAP, 120, 45, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_SEL_PKT_CAP, 120, 57, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_MARKED_CAP, 120, 69, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_FIRST_LAST_CAP, 120, 81, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_RANGE_CAP, 120, 93, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_IGNORED_CAP, 120, 105, 39, 8, SS_RIGHT

    LTEXT "0", EWFD_ALL_PKTS_DISP, 170, 45, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_SEL_PKT_DISP, 170, 57, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_MARKED_DISP, 170, 69, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_FIRST_LAST_DISP, 170, 81, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_RANGE_DISP, 170, 93, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_IGNORED_DISP, 170, 105, 41, 8, SS_RIGHT
}

WIRESHARK_MERGEFILENAME_TEMPLATE DIALOG 0, 0, 368, 94
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    // Filter button/entry
    // PUSHBUTTON "Filter:", EWFD_FILTER_BTN, 7, 4, 35, 14
    LTEXT "Display filter:", EWFD_FILTER_LBL, 7, 7, 49, 14
    CONTROL    "",        EWFD_FILTER_EDIT, RICHEDIT_CLASS, ES_AUTOHSCROLL, 52, 5, 88, 12, WS_EX_CLIENTEDGE

    CONTROL  "Prepend packets to existing file", EWFD_MERGE_PREPEND_BTN, "Button", BS_AUTORADIOBUTTON | WS_GROUP, 7, 32, 120, 8
    CONTROL  "Merge packets chronologically", EWFD_MERGE_CHRONO_BTN, "Button", BS_AUTORADIOBUTTON, 7, 47, 120, 8
    CONTROL  "Append packets to existing file", EWFD_MERGE_APPEND_BTN, "Button", BS_AUTORADIOBUTTON, 7, 62, 120, 8

    LTEXT "Filename:", EWFD_PT_FILENAME, 164, 4, 40, 8
    LTEXT "-", EWFD_PTX_FILENAME, 211, 4, 150, 8
    LTEXT "Format:", EWFD_PT_FORMAT, 164, 19, 40, 8
    LTEXT "-", EWFD_PTX_FORMAT, 211, 19, 150, 8
    LTEXT "Size:", EWFD_PT_SIZE, 164, 34, 40, 8
    LTEXT "-", EWFD_PTX_SIZE, 211, 34, 150, 8
    LTEXT "Packets:", EWFD_PT_PACKETS, 164, 49, 40, 8
    LTEXT "-", EWFD_PTX_PACKETS, 211, 49, 150, 8
    LTEXT "First Packet:", EWFD_PT_FIRST_PKT, 164, 64, 40, 8
    LTEXT "-", EWFD_PTX_FIRST_PKT, 211, 64, 150, 8
    LTEXT "Elapsed:", EWFD_PT_ELAPSED, 164, 79, 40, 8
    LTEXT "-", EWFD_PTX_ELAPSED, 211, 79, 150, 8
}

WIRESHARK_EXPORTFILENAME_TEMPLATE DIALOG 0, 0, 319, 109
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    // Provide hints for dialog layout and size
    LTEXT    "", stc32, 1, 1, 260, 0

    GROUPBOX "Packet Range", -1, 5, 2, 220, 102

    CONTROL  "Captured", EWFD_CAPTURED_BTN, "Button", BS_AUTORADIOBUTTON | WS_GROUP, 120, 13, 45, 10
    CONTROL  "Displayed", EWFD_DISPLAYED_BTN, "Button", BS_AUTORADIOBUTTON, 170,  13, 47, 10

    CONTROL  "All packets", EWFD_ALL_PKTS_BTN, "Button", BS_AUTORADIOBUTTON | WS_GROUP, 16, 25, 51, 10
    CONTROL  "Selected packet", EWFD_SEL_PKT_BTN, "Button", BS_AUTORADIOBUTTON,  16, 37, 68, 10
    CONTROL  "Marked packets", EWFD_MARKED_BTN, "Button", BS_AUTORADIOBUTTON,  16, 49, 67, 10
    CONTROL  "First to last marked", EWFD_FIRST_LAST_BTN, "Button",  BS_AUTORADIOBUTTON, 16, 61, 75, 10
    CONTROL  "Range:", EWFD_RANGE_BTN, "Button",  BS_AUTORADIOBUTTON, 16, 73, 35, 10
    EDITTEXT EWFD_RANGE_EDIT, 53, 72, 50, 12, ES_AUTOHSCROLL
    CONTROL  "Remove Ignored packets", EWFD_REMOVE_IGN_CB, "Button",  BS_AUTOCHECKBOX | WS_TABSTOP, 16, 85, 100, 10

    LTEXT "0", EWFD_ALL_PKTS_CAP,   120, 26, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_SEL_PKT_CAP,    120, 38, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_MARKED_CAP,     120, 50, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_FIRST_LAST_CAP, 120, 62, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_RANGE_CAP,      120, 74, 39, 8, SS_RIGHT
    LTEXT "0", EWFD_IGNORED_CAP,    120, 86, 39, 8, SS_RIGHT

    LTEXT "0", EWFD_ALL_PKTS_DISP,   170, 26, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_SEL_PKT_DISP,    170, 38, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_MARKED_DISP,     170, 50, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_FIRST_LAST_DISP, 170, 62, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_RANGE_DISP,      170, 74, 41, 8, SS_RIGHT
    LTEXT "0", EWFD_IGNORED_DISP,    170, 86, 41, 8, SS_RIGHT

    GROUPBOX "Packet Format", EWFD_PKT_FORMAT_GB, 234, 2, 119, 102
    CONTROL  "Packet summary line", EWFD_PKT_SUMMARY_CB, "Button",  BS_AUTOCHECKBOX | WS_TABSTOP, 245, 15, 84, 10
    CONTROL  "Packet details:", EWFD_PKT_DETAIL_CB, "Button", BS_AUTOCHECKBOX | WS_TABSTOP, 245, 27, 95, 10
    COMBOBOX EWFD_PKT_DETAIL_COMBO, 257, 39, 74, 45, CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL  "Packet Bytes", EWFD_PKT_BYTES_CB, "Button", BS_AUTOCHECKBOX | WS_TABSTOP, 245, 56, 80, 10
    CONTROL  "Each packet on a new page", EWFD_PKT_NEW_PAGE_CB, "Button",  BS_AUTOCHECKBOX | WS_TABSTOP, 245, 70, 106, 10

}


WIRESHARK_EXPORTRAWFILENAME_TEMPLATE DIALOG 0, 0, 200, 18
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    LTEXT    "-", EWFD_EXPORTRAW_ST, 5, 5, 180, 8
}

WIRESHARK_EXPORTSSLKEYSFILENAME_TEMPLATE DIALOG 0, 0, 200, 18
STYLE WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | DS_3DLOOK | DS_CONTROL
FONT 8, "MS Shell Dlg"
{
    LTEXT    "-", EWFD_EXPORTSSLKEYS_ST, 5, 5, 180, 8
}
