/* DO NOT EDIT
	This filter was automatically generated
	from nspi.idl and nspi.cnf.
	
	Pidl is a perl based IDL compiler for DCE/RPC idl files. 
	It is maintained by the Samba team, not the Wireshark team.
	Instructions on how to download and install Pidl can be 
	found at http://wiki.wireshark.org/Pidl

	$Id: packet-dcerpc-nspi.c 34713 2010-10-30 14:35:05Z morriss $
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <string.h>
#include <epan/packet.h>

#include "packet-dcerpc.h"
#include "packet-dcerpc-nt.h"
#include "packet-windows-common.h"
#include "packet-dcerpc-nspi.h"

/* Ett declarations */
static gint ett_dcerpc_nspi = -1;
static gint ett_nspi_MAPIUID = -1;
static gint ett_nspi_input_locale = -1;
static gint ett_nspi_MAPI_SETTINGS = -1;
static gint ett_nspi_SPropTagArray = -1;
static gint ett_nspi_instance_key = -1;
static gint ett_nspi_MAPINAMEID = -1;
static gint ett_nspi_SPropertyRestriction = -1;
static gint ett_nspi_SAndRestriction = -1;
static gint ett_nspi_SRestriction_CTR = -1;
static gint ett_nspi_SRestriction = -1;
static gint ett_nspi_SSortOrder = -1;
static gint ett_nspi_SSortOrderSet = -1;
static gint ett_nspi_NAME_STRING = -1;
static gint ett_nspi_SBinary = -1;
static gint ett_nspi_FILETIME = -1;
static gint ett_nspi_SShortArray = -1;
static gint ett_nspi_MV_LONG_STRUCT = -1;
static gint ett_nspi_LPSTR = -1;
static gint ett_nspi_SLPSTRArray = -1;
static gint ett_nspi_SBinaryArray = -1;
static gint ett_nspi_SGuidArray = -1;
static gint ett_nspi_MV_UNICODE_STRUCT = -1;
static gint ett_nspi_SDateTimeArray = -1;
static gint ett_nspi_SPropValue_CTR = -1;
static gint ett_nspi_SPropValue = -1;
static gint ett_nspi_SRow = -1;
static gint ett_nspi_SRowSet = -1;


/* Header field declarations */
static gint hf_nspi_NspiGetHierarchyInfo_unknown1 = -1;
static gint hf_nspi_SPropValue_CTR_ft = -1;
static gint hf_nspi_SPropValue_CTR_object = -1;
static gint hf_nspi_input_locale_language = -1;
static gint hf_nspi_SPropValue_CTR_MVft = -1;
static gint hf_nspi_SDateTimeArray_lpft = -1;
static gint hf_nspi_SSortOrderSet_aSort = -1;
static gint hf_nspi_SGuidArray_lpguid = -1;
static gint hf_nspi_SPropValue_dwAlignPad = -1;
static gint hf_nspi_NspiQueryRows_instance_key = -1;
static gint hf_nspi_SSortOrderSet_cCategories = -1;
static gint hf_nspi_MAPINAMEID_ulKind = -1;
static gint hf_nspi_NspiDNToEph_server_dn = -1;
static gint hf_nspi_MAPIUID_ab = -1;
static gint hf_nspi_SPropValue_CTR_b = -1;
static gint hf_nspi_SRow_ulAdrEntryPad = -1;
static gint hf_nspi_SPropValue_CTR_null = -1;
static gint hf_nspi_SBinary_cb = -1;
static gint hf_nspi_SBinaryArray_lpbin = -1;
static gint hf_nspi_SShortArray_cValues = -1;
static gint hf_nspi_SRowSet_aRow = -1;
static gint hf_nspi_NspiBind_mapiuid = -1;
static gint hf_nspi_SGuidArray_cValues = -1;
static gint hf_nspi_instance_key_cValues = -1;
static gint hf_nspi_SRestriction_CTR_resProperty = -1;
static gint hf_nspi_NspiQueryRows_lRows = -1;
static gint hf_nspi_MV_UNICODE_STRUCT_cValues = -1;
static gint hf_nspi_SSortOrder_ulOrder = -1;
static gint hf_nspi_NspiGetHierarchyInfo_RowSet = -1;
static gint hf_nspi_MAPINAMEID_lID = -1;
static gint hf_nspi_NspiQueryRows_settings = -1;
static gint hf_nspi_SBinaryArray_cValues = -1;
static gint hf_nspi_NspiGetMatches_unknown2 = -1;
static gint hf_nspi_SLPSTRArray_strings = -1;
static gint hf_nspi_FILETIME_dwHighDateTime = -1;
static gint hf_nspi_NspiGetProps_REPL_values = -1;
static gint hf_nspi_SRow_cValues = -1;
static gint hf_nspi_SPropValue_ulPropTag = -1;
static gint hf_nspi_NspiGetProps_settings = -1;
static gint hf_nspi_NspiDNToEph_instance_key = -1;
static gint hf_nspi_SRestriction_CTR_resAnd = -1;
static gint hf_nspi_NspiGetMatches_RowSet = -1;
static gint hf_nspi_MV_UNICODE_STRUCT_lpi = -1;
static gint hf_nspi_SSortOrderSet_cSorts = -1;
static gint hf_nspi_SSortOrder_ulPropTag = -1;
static gint hf_nspi_SPropValue_CTR_MVbin = -1;
static gint hf_nspi_opnum = -1;
static gint hf_nspi_NspiUnbind_status = -1;
static gint hf_nspi_SAndRestriction_lpRes = -1;
static gint hf_nspi_NspiDNToEph_size = -1;
static gint hf_nspi_SPropValue_CTR_MVi = -1;
static gint hf_nspi_SPropertyRestriction_lpProp = -1;
static gint hf_nspi_NspiGetMatches_unknown3 = -1;
static gint hf_nspi_MAPI_SETTINGS_flag = -1;
static gint hf_nspi_instance_key_value = -1;
static gint hf_nspi_NspiGetMatches_settings = -1;
static gint hf_nspi_MV_LONG_STRUCT_cValues = -1;
static gint hf_nspi_SRowSet_cRows = -1;
static gint hf_nspi_MAPI_SETTINGS_service_provider = -1;
static gint hf_nspi_SPropValue_CTR_l = -1;
static gint hf_nspi_SAndRestriction_cRes = -1;
static gint hf_nspi_NspiQueryRows_flag = -1;
static gint hf_nspi_NspiQueryRows_REQ_properties = -1;
static gint hf_nspi_SPropValue_CTR_MVl = -1;
static gint hf_nspi_NspiGetProps_flag = -1;
static gint hf_nspi_MAPISTATUS_status = -1;
static gint hf_nspi_SPropValue_CTR_MVszW = -1;
static gint hf_nspi_LPSTR_lppszA = -1;
static gint hf_nspi_SPropValue_CTR_MVszA = -1;
static gint hf_nspi_property_type = -1;
static gint hf_nspi_handle = -1;
static gint hf_nspi_NspiGetMatches_restrictions = -1;
static gint hf_nspi_NspiQueryRows_RowSet = -1;
static gint hf_nspi_SRow_lpProps = -1;
static gint hf_nspi_MAPI_SETTINGS_input_locale = -1;
static gint hf_nspi_SPropertyRestriction_ulPropTag = -1;
static gint hf_nspi_SPropTagArray_aulPropTag = -1;
static gint hf_nspi_NspiGetHierarchyInfo_unknown2 = -1;
static gint hf_nspi_SBinary_lpb = -1;
static gint hf_nspi_NspiGetHierarchyInfo_settings = -1;
static gint hf_nspi_SSortOrderSet_cExpanded = -1;
static gint hf_nspi_NspiQueryRows_unknown = -1;
static gint hf_nspi_input_locale_method = -1;
static gint hf_nspi_NspiGetMatches_instance_key = -1;
static gint hf_nspi_SPropValue_CTR_lpszA = -1;
static gint hf_nspi_SPropValue_CTR_bin = -1;
static gint hf_nspi_SShortArray_lpi = -1;
static gint hf_nspi_SPropValue_CTR_lpszW = -1;
static gint hf_nspi_NspiGetMatches_PropTagArray = -1;
static gint hf_nspi_SLPSTRArray_cValues = -1;
static gint hf_nspi_SPropValue_CTR_err = -1;
static gint hf_nspi_SRestriction_PTTYPE = -1;
static gint hf_nspi_NAME_STRING_str = -1;
static gint hf_nspi_SPropertyRestriction_relop = -1;
static gint hf_nspi_SPropValue_CTR_MVguid = -1;
static gint hf_nspi_MAPINAMEID_lpguid = -1;
static gint hf_nspi_FILETIME_dwLowDateTime = -1;
static gint hf_nspi_NspiGetMatches_REQ_properties = -1;
static gint hf_nspi_SPropTagArray_cValues = -1;
static gint hf_nspi_SPropValue_CTR_lpguid = -1;
static gint hf_nspi_SPropValue_CTR_i = -1;
static gint hf_nspi_MAPI_SETTINGS_codepage = -1;
static gint hf_nspi_NspiDNToEph_flag = -1;
static gint hf_nspi_NspiBind_unknown = -1;
static gint hf_nspi_MAPI_SETTINGS_handle = -1;
static gint hf_nspi_NspiGetProps_REQ_properties = -1;
static gint hf_nspi_MV_LONG_STRUCT_lpl = -1;
static gint hf_nspi_NspiGetMatches_unknown1 = -1;
static gint hf_nspi_SDateTimeArray_cValues = -1;
static gint hf_nspi_NspiBind_settings = -1;

static gint proto_dcerpc_nspi = -1;
/* Version information */


static e_uuid_t uuid_dcerpc_nspi = {
	0xf5cc5a18, 0x4264, 0x101a,
	{ 0x8c, 0x59, 0x08, 0x00, 0x2b, 0x2f, 0x84, 0x26 }
};
static guint16 ver_dcerpc_nspi = 56;

const value_string nspi_MAPITAGS_vals[] = {
	{ PR_ACKNOWLEDGEMENT_MODE, "PR_ACKNOWLEDGEMENT_MODE" },
	{ PR_ACKNOWLEDGEMENT_MODE_ERROR, "PR_ACKNOWLEDGEMENT_MODE_ERROR" },
	{ PR_ALTERNATE_RECIPIENT_ALLOWED, "PR_ALTERNATE_RECIPIENT_ALLOWED" },
	{ PR_ALTERNATE_RECIPIENT_ALLOWED_ERROR, "PR_ALTERNATE_RECIPIENT_ALLOWED_ERROR" },
	{ PR_AUTHORIZING_USERS, "PR_AUTHORIZING_USERS" },
	{ PR_AUTHORIZING_USERS_ERROR, "PR_AUTHORIZING_USERS_ERROR" },
	{ PR_AUTO_FORWARD_COMMENT, "PR_AUTO_FORWARD_COMMENT" },
	{ PR_AUTO_FORWARD_COMMENT_UNICODE, "PR_AUTO_FORWARD_COMMENT_UNICODE" },
	{ PR_AUTO_FORWARD_COMMENT_ERROR, "PR_AUTO_FORWARD_COMMENT_ERROR" },
	{ PR_AUTO_FORWARDED, "PR_AUTO_FORWARDED" },
	{ PR_AUTO_FORWARDED_ERROR, "PR_AUTO_FORWARDED_ERROR" },
	{ PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID, "PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID" },
	{ PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID_ERROR, "PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID_ERROR" },
	{ PR_CONTENT_CORRELATOR, "PR_CONTENT_CORRELATOR" },
	{ PR_CONTENT_CORRELATOR_ERROR, "PR_CONTENT_CORRELATOR_ERROR" },
	{ PR_CONTENT_IDENTIFIER, "PR_CONTENT_IDENTIFIER" },
	{ PR_CONTENT_IDENTIFIER_UNICODE, "PR_CONTENT_IDENTIFIER_UNICODE" },
	{ PR_CONTENT_IDENTIFIER_ERROR, "PR_CONTENT_IDENTIFIER_ERROR" },
	{ PR_CONTENT_LENGTH, "PR_CONTENT_LENGTH" },
	{ PR_CONTENT_LENGTH_ERROR, "PR_CONTENT_LENGTH_ERROR" },
	{ PR_CONTENT_RETURN_REQUESTED, "PR_CONTENT_RETURN_REQUESTED" },
	{ PR_CONTENT_RETURN_REQUESTED_ERROR, "PR_CONTENT_RETURN_REQUESTED_ERROR" },
	{ PR_CONVERSATION_KEY, "PR_CONVERSATION_KEY" },
	{ PR_CONVERSATION_KEY_ERROR, "PR_CONVERSATION_KEY_ERROR" },
	{ PR_CONVERSION_EITS, "PR_CONVERSION_EITS" },
	{ PR_CONVERSION_EITS_ERROR, "PR_CONVERSION_EITS_ERROR" },
	{ PR_CONVERSION_WITH_LOSS_PROHIBITED, "PR_CONVERSION_WITH_LOSS_PROHIBITED" },
	{ PR_CONVERSION_WITH_LOSS_PROHIBITED_ERROR, "PR_CONVERSION_WITH_LOSS_PROHIBITED_ERROR" },
	{ PR_CONVERTED_EITS, "PR_CONVERTED_EITS" },
	{ PR_CONVERTED_EITS_ERROR, "PR_CONVERTED_EITS_ERROR" },
	{ PR_DEFERRED_DELIVERY_TIME, "PR_DEFERRED_DELIVERY_TIME" },
	{ PR_DEFERRED_DELIVERY_TIME_ERROR, "PR_DEFERRED_DELIVERY_TIME_ERROR" },
	{ PR_DELIVER_TIME, "PR_DELIVER_TIME" },
	{ PR_DELIVER_TIME_ERROR, "PR_DELIVER_TIME_ERROR" },
	{ PR_DISCARD_REASON, "PR_DISCARD_REASON" },
	{ PR_DISCARD_REASON_ERROR, "PR_DISCARD_REASON_ERROR" },
	{ PR_DISCLOSURE_OF_RECIPIENTS, "PR_DISCLOSURE_OF_RECIPIENTS" },
	{ PR_DISCLOSURE_OF_RECIPIENTS_ERROR, "PR_DISCLOSURE_OF_RECIPIENTS_ERROR" },
	{ PR_DL_EXPANSION_HISTORY, "PR_DL_EXPANSION_HISTORY" },
	{ PR_DL_EXPANSION_HISTORY_ERROR, "PR_DL_EXPANSION_HISTORY_ERROR" },
	{ PR_DL_EXPANSION_PROHIBITED, "PR_DL_EXPANSION_PROHIBITED" },
	{ PR_DL_EXPANSION_PROHIBITED_ERROR, "PR_DL_EXPANSION_PROHIBITED_ERROR" },
	{ PR_EXPIRY_TIME, "PR_EXPIRY_TIME" },
	{ PR_EXPIRY_TIME_ERROR, "PR_EXPIRY_TIME_ERROR" },
	{ PR_IMPLICIT_CONVERSION_PROHIBITED, "PR_IMPLICIT_CONVERSION_PROHIBITED" },
	{ PR_IMPLICIT_CONVERSION_PROHIBITED_ERROR, "PR_IMPLICIT_CONVERSION_PROHIBITED_ERROR" },
	{ PR_IMPORTANCE, "PR_IMPORTANCE" },
	{ PR_IMPORTANCE_ERROR, "PR_IMPORTANCE_ERROR" },
	{ PR_IPM_ID, "PR_IPM_ID" },
	{ PR_IPM_ID_ERROR, "PR_IPM_ID_ERROR" },
	{ PR_LATEST_DELIVERY_TIME, "PR_LATEST_DELIVERY_TIME" },
	{ PR_LATEST_DELIVERY_TIME_ERROR, "PR_LATEST_DELIVERY_TIME_ERROR" },
	{ PR_MESSAGE_CLASS, "PR_MESSAGE_CLASS" },
	{ PR_MESSAGE_CLASS_UNICODE, "PR_MESSAGE_CLASS_UNICODE" },
	{ PR_MESSAGE_CLASS_ERROR, "PR_MESSAGE_CLASS_ERROR" },
	{ PR_MESSAGE_DELIVERY_ID, "PR_MESSAGE_DELIVERY_ID" },
	{ PR_MESSAGE_DELIVERY_ID_ERROR, "PR_MESSAGE_DELIVERY_ID_ERROR" },
	{ PR_MESSAGE_SECURITY_LABEL, "PR_MESSAGE_SECURITY_LABEL" },
	{ PR_MESSAGE_SECURITY_LABEL_ERROR, "PR_MESSAGE_SECURITY_LABEL_ERROR" },
	{ PR_OBSOLETED_IPMS, "PR_OBSOLETED_IPMS" },
	{ PR_OBSOLETED_IPMS_ERROR, "PR_OBSOLETED_IPMS_ERROR" },
	{ PR_ORIGINALLY_INTENDED_RECIPIENT_NAME, "PR_ORIGINALLY_INTENDED_RECIPIENT_NAME" },
	{ PR_ORIGINALLY_INTENDED_RECIPIENT_NAME_ERROR, "PR_ORIGINALLY_INTENDED_RECIPIENT_NAME_ERROR" },
	{ PR_ORIGINAL_EITS, "PR_ORIGINAL_EITS" },
	{ PR_ORIGINAL_EITS_ERROR, "PR_ORIGINAL_EITS_ERROR" },
	{ PR_ORIGINATOR_CERTIFICATE, "PR_ORIGINATOR_CERTIFICATE" },
	{ PR_ORIGINATOR_CERTIFICATE_ERROR, "PR_ORIGINATOR_CERTIFICATE_ERROR" },
	{ PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED, "PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED" },
	{ PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED_ERROR, "PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED_ERROR" },
	{ PR_ORIGINATOR_RETURN_ADDRESS, "PR_ORIGINATOR_RETURN_ADDRESS" },
	{ PR_ORIGINATOR_RETURN_ADDRESS_ERROR, "PR_ORIGINATOR_RETURN_ADDRESS_ERROR" },
	{ PR_PARENT_KEY, "PR_PARENT_KEY" },
	{ PR_PARENT_KEY_ERROR, "PR_PARENT_KEY_ERROR" },
	{ PR_PRIORITY, "PR_PRIORITY" },
	{ PR_PRIORITY_ERROR, "PR_PRIORITY_ERROR" },
	{ PR_ORIGIN_CHECK, "PR_ORIGIN_CHECK" },
	{ PR_ORIGIN_CHECK_ERROR, "PR_ORIGIN_CHECK_ERROR" },
	{ PR_PROOF_OF_SUBMISSION_REQUESTED, "PR_PROOF_OF_SUBMISSION_REQUESTED" },
	{ PR_PROOF_OF_SUBMISSION_REQUESTED_ERROR, "PR_PROOF_OF_SUBMISSION_REQUESTED_ERROR" },
	{ PR_READ_RECEIPT_REQUESTED, "PR_READ_RECEIPT_REQUESTED" },
	{ PR_READ_RECEIPT_REQUESTED_ERROR, "PR_READ_RECEIPT_REQUESTED_ERROR" },
	{ PR_RECEIPT_TIME, "PR_RECEIPT_TIME" },
	{ PR_RECEIPT_TIME_ERROR, "PR_RECEIPT_TIME_ERROR" },
	{ PR_RECIPIENT_REASSIGNMENT_PROHIBITED, "PR_RECIPIENT_REASSIGNMENT_PROHIBITED" },
	{ PR_RECIPIENT_REASSIGNMENT_PROHIBITED_ERROR, "PR_RECIPIENT_REASSIGNMENT_PROHIBITED_ERROR" },
	{ PR_REDIRECTION_HISTORY, "PR_REDIRECTION_HISTORY" },
	{ PR_REDIRECTION_HISTORY_ERROR, "PR_REDIRECTION_HISTORY_ERROR" },
	{ PR_RELATED_IPMS, "PR_RELATED_IPMS" },
	{ PR_RELATED_IPMS_ERROR, "PR_RELATED_IPMS_ERROR" },
	{ PR_ORIGINAL_SENSITIVITY, "PR_ORIGINAL_SENSITIVITY" },
	{ PR_ORIGINAL_SENSITIVITY_ERROR, "PR_ORIGINAL_SENSITIVITY_ERROR" },
	{ PR_LANGUAGES, "PR_LANGUAGES" },
	{ PR_LANGUAGES_UNICODE, "PR_LANGUAGES_UNICODE" },
	{ PR_LANGUAGES_ERROR, "PR_LANGUAGES_ERROR" },
	{ PR_REPLY_TIME, "PR_REPLY_TIME" },
	{ PR_REPLY_TIME_ERROR, "PR_REPLY_TIME_ERROR" },
	{ PR_REPORT_TAG, "PR_REPORT_TAG" },
	{ PR_REPORT_TAG_ERROR, "PR_REPORT_TAG_ERROR" },
	{ PR_REPORT_TIME, "PR_REPORT_TIME" },
	{ PR_REPORT_TIME_ERROR, "PR_REPORT_TIME_ERROR" },
	{ PR_RETURNED_IPM, "PR_RETURNED_IPM" },
	{ PR_RETURNED_IPM_ERROR, "PR_RETURNED_IPM_ERROR" },
	{ PR_SECURITY, "PR_SECURITY" },
	{ PR_SECURITY_ERROR, "PR_SECURITY_ERROR" },
	{ PR_INCOMPLETE_COPY, "PR_INCOMPLETE_COPY" },
	{ PR_INCOMPLETE_COPY_ERROR, "PR_INCOMPLETE_COPY_ERROR" },
	{ PR_SENSITIVITY, "PR_SENSITIVITY" },
	{ PR_SENSITIVITY_ERROR, "PR_SENSITIVITY_ERROR" },
	{ PR_SUBJECT, "PR_SUBJECT" },
	{ PR_SUBJECT_UNICODE, "PR_SUBJECT_UNICODE" },
	{ PR_SUBJECT_ERROR, "PR_SUBJECT_ERROR" },
	{ PR_SUBJECT_IPM, "PR_SUBJECT_IPM" },
	{ PR_SUBJECT_IPM_ERROR, "PR_SUBJECT_IPM_ERROR" },
	{ PR_CLIENT_SUBMIT_TIME, "PR_CLIENT_SUBMIT_TIME" },
	{ PR_CLIENT_SUBMIT_TIME_ERROR, "PR_CLIENT_SUBMIT_TIME_ERROR" },
	{ PR_REPORT_NAME, "PR_REPORT_NAME" },
	{ PR_REPORT_NAME_UNICODE, "PR_REPORT_NAME_UNICODE" },
	{ PR_REPORT_NAME_ERROR, "PR_REPORT_NAME_ERROR" },
	{ PR_SENT_REPRESENTING_SEARCH_KEY, "PR_SENT_REPRESENTING_SEARCH_KEY" },
	{ PR_SENT_REPRESENTING_SEARCH_KEY_ERROR, "PR_SENT_REPRESENTING_SEARCH_KEY_ERROR" },
	{ PR_X400_CONTENT_TYPE, "PR_X400_CONTENT_TYPE" },
	{ PR_X400_CONTENT_TYPE_ERROR, "PR_X400_CONTENT_TYPE_ERROR" },
	{ PR_SUBJECT_PREFIX, "PR_SUBJECT_PREFIX" },
	{ PR_SUBJECT_PREFIX_UNICODE, "PR_SUBJECT_PREFIX_UNICODE" },
	{ PR_SUBJECT_PREFIX_ERROR, "PR_SUBJECT_PREFIX_ERROR" },
	{ PR_NON_RECEIPT_REASON, "PR_NON_RECEIPT_REASON" },
	{ PR_NON_RECEIPT_REASON_ERROR, "PR_NON_RECEIPT_REASON_ERROR" },
	{ PR_RECEIVED_BY_ENTRYID, "PR_RECEIVED_BY_ENTRYID" },
	{ PR_RECEIVED_BY_ENTRYID_ERROR, "PR_RECEIVED_BY_ENTRYID_ERROR" },
	{ PR_RECEIVED_BY_NAME, "PR_RECEIVED_BY_NAME" },
	{ PR_RECEIVED_BY_NAME_UNICODE, "PR_RECEIVED_BY_NAME_UNICODE" },
	{ PR_RECEIVED_BY_NAME_ERROR, "PR_RECEIVED_BY_NAME_ERROR" },
	{ PR_SENT_REPRESENTING_ENTRYID, "PR_SENT_REPRESENTING_ENTRYID" },
	{ PR_SENT_REPRESENTING_ENTRYID_ERROR, "PR_SENT_REPRESENTING_ENTRYID_ERROR" },
	{ PR_SENT_REPRESENTING_NAME, "PR_SENT_REPRESENTING_NAME" },
	{ PR_SENT_REPRESENTING_NAME_UNICODE, "PR_SENT_REPRESENTING_NAME_UNICODE" },
	{ PR_SENT_REPRESENTING_NAME_ERROR, "PR_SENT_REPRESENTING_NAME_ERROR" },
	{ PR_RCVD_REPRESENTING_ENTRYID, "PR_RCVD_REPRESENTING_ENTRYID" },
	{ PR_RCVD_REPRESENTING_ENTRYID_ERROR, "PR_RCVD_REPRESENTING_ENTRYID_ERROR" },
	{ PR_RCVD_REPRESENTING_NAME, "PR_RCVD_REPRESENTING_NAME" },
	{ PR_RCVD_REPRESENTING_NAME_UNICODE, "PR_RCVD_REPRESENTING_NAME_UNICODE" },
	{ PR_RCVD_REPRESENTING_NAME_ERROR, "PR_RCVD_REPRESENTING_NAME_ERROR" },
	{ PR_REPORT_ENTRYID, "PR_REPORT_ENTRYID" },
	{ PR_REPORT_ENTRYID_ERROR, "PR_REPORT_ENTRYID_ERROR" },
	{ PR_READ_RECEIPT_ENTRYID, "PR_READ_RECEIPT_ENTRYID" },
	{ PR_READ_RECEIPT_ENTRYID_ERROR, "PR_READ_RECEIPT_ENTRYID_ERROR" },
	{ PR_MESSAGE_SUBMISSION_ID, "PR_MESSAGE_SUBMISSION_ID" },
	{ PR_MESSAGE_SUBMISSION_ID_ERROR, "PR_MESSAGE_SUBMISSION_ID_ERROR" },
	{ PR_PROVIDER_SUBMIT_TIME, "PR_PROVIDER_SUBMIT_TIME" },
	{ PR_PROVIDER_SUBMIT_TIME_ERROR, "PR_PROVIDER_SUBMIT_TIME_ERROR" },
	{ PR_ORIGINAL_SUBJECT, "PR_ORIGINAL_SUBJECT" },
	{ PR_ORIGINAL_SUBJECT_UNICODE, "PR_ORIGINAL_SUBJECT_UNICODE" },
	{ PR_ORIGINAL_SUBJECT_ERROR, "PR_ORIGINAL_SUBJECT_ERROR" },
	{ PR_DISC_VAL, "PR_DISC_VAL" },
	{ PR_DISC_VAL_ERROR, "PR_DISC_VAL_ERROR" },
	{ PR_ORIG_MESSAGE_CLASS, "PR_ORIG_MESSAGE_CLASS" },
	{ PR_ORIG_MESSAGE_CLASS_UNICODE, "PR_ORIG_MESSAGE_CLASS_UNICODE" },
	{ PR_ORIG_MESSAGE_CLASS_ERROR, "PR_ORIG_MESSAGE_CLASS_ERROR" },
	{ PR_ORIGINAL_AUTHOR_ENTRYID, "PR_ORIGINAL_AUTHOR_ENTRYID" },
	{ PR_ORIGINAL_AUTHOR_ENTRYID_ERROR, "PR_ORIGINAL_AUTHOR_ENTRYID_ERROR" },
	{ PR_ORIGINAL_AUTHOR_NAME, "PR_ORIGINAL_AUTHOR_NAME" },
	{ PR_ORIGINAL_AUTHOR_NAME_UNICODE, "PR_ORIGINAL_AUTHOR_NAME_UNICODE" },
	{ PR_ORIGINAL_AUTHOR_NAME_ERROR, "PR_ORIGINAL_AUTHOR_NAME_ERROR" },
	{ PR_ORIGINAL_SUBMIT_TIME, "PR_ORIGINAL_SUBMIT_TIME" },
	{ PR_ORIGINAL_SUBMIT_TIME_ERROR, "PR_ORIGINAL_SUBMIT_TIME_ERROR" },
	{ PR_REPLY_RECIPIENT_ENTRIES, "PR_REPLY_RECIPIENT_ENTRIES" },
	{ PR_REPLY_RECIPIENT_ENTRIES_ERROR, "PR_REPLY_RECIPIENT_ENTRIES_ERROR" },
	{ PR_REPLY_RECIPIENT_NAMES, "PR_REPLY_RECIPIENT_NAMES" },
	{ PR_REPLY_RECIPIENT_NAMES_UNICODE, "PR_REPLY_RECIPIENT_NAMES_UNICODE" },
	{ PR_REPLY_RECIPIENT_NAMES_ERROR, "PR_REPLY_RECIPIENT_NAMES_ERROR" },
	{ PR_RECEIVED_BY_SEARCH_KEY, "PR_RECEIVED_BY_SEARCH_KEY" },
	{ PR_RECEIVED_BY_SEARCH_KEY_ERROR, "PR_RECEIVED_BY_SEARCH_KEY_ERROR" },
	{ PR_RCVD_REPRESENTING_SEARCH_KEY, "PR_RCVD_REPRESENTING_SEARCH_KEY" },
	{ PR_RCVD_REPRESENTING_SEARCH_KEY_ERROR, "PR_RCVD_REPRESENTING_SEARCH_KEY_ERROR" },
	{ PR_READ_RECEIPT_SEARCH_KEY, "PR_READ_RECEIPT_SEARCH_KEY" },
	{ PR_READ_RECEIPT_SEARCH_KEY_ERROR, "PR_READ_RECEIPT_SEARCH_KEY_ERROR" },
	{ PR_REPORT_SEARCH_KEY, "PR_REPORT_SEARCH_KEY" },
	{ PR_REPORT_SEARCH_KEY_ERROR, "PR_REPORT_SEARCH_KEY_ERROR" },
	{ PR_ORIGINAL_DELIVERY_TIME, "PR_ORIGINAL_DELIVERY_TIME" },
	{ PR_ORIGINAL_DELIVERY_TIME_ERROR, "PR_ORIGINAL_DELIVERY_TIME_ERROR" },
	{ PR_ORIGINAL_AUTHOR_SEARCH_KEY, "PR_ORIGINAL_AUTHOR_SEARCH_KEY" },
	{ PR_ORIGINAL_AUTHOR_SEARCH_KEY_ERROR, "PR_ORIGINAL_AUTHOR_SEARCH_KEY_ERROR" },
	{ PR_MESSAGE_TO_ME, "PR_MESSAGE_TO_ME" },
	{ PR_MESSAGE_TO_ME_ERROR, "PR_MESSAGE_TO_ME_ERROR" },
	{ PR_MESSAGE_CC_ME, "PR_MESSAGE_CC_ME" },
	{ PR_MESSAGE_CC_ME_ERROR, "PR_MESSAGE_CC_ME_ERROR" },
	{ PR_MESSAGE_RECIP_ME, "PR_MESSAGE_RECIP_ME" },
	{ PR_MESSAGE_RECIP_ME_ERROR, "PR_MESSAGE_RECIP_ME_ERROR" },
	{ PR_ORIGINAL_SENDER_NAME, "PR_ORIGINAL_SENDER_NAME" },
	{ PR_ORIGINAL_SENDER_NAME_UNICODE, "PR_ORIGINAL_SENDER_NAME_UNICODE" },
	{ PR_ORIGINAL_SENDER_NAME_ERROR, "PR_ORIGINAL_SENDER_NAME_ERROR" },
	{ PR_ORIGINAL_SENDER_ENTRYID, "PR_ORIGINAL_SENDER_ENTRYID" },
	{ PR_ORIGINAL_SENDER_ENTRYID_ERROR, "PR_ORIGINAL_SENDER_ENTRYID_ERROR" },
	{ PR_ORIGINAL_SENDER_SEARCH_KEY, "PR_ORIGINAL_SENDER_SEARCH_KEY" },
	{ PR_ORIGINAL_SENDER_SEARCH_KEY_ERROR, "PR_ORIGINAL_SENDER_SEARCH_KEY_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_NAME, "PR_ORIGINAL_SENT_REPRESENTING_NAME" },
	{ PR_ORIGINAL_SENT_REPRESENTING_NAME_UNICODE, "PR_ORIGINAL_SENT_REPRESENTING_NAME_UNICODE" },
	{ PR_ORIGINAL_SENT_REPRESENTING_NAME_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_NAME_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_ENTRYID, "PR_ORIGINAL_SENT_REPRESENTING_ENTRYID" },
	{ PR_ORIGINAL_SENT_REPRESENTING_ENTRYID_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_ENTRYID_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY, "PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY" },
	{ PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY_ERROR" },
	{ PR_START_DATE, "PR_START_DATE" },
	{ PR_START_DATE_ERROR, "PR_START_DATE_ERROR" },
	{ PR_END_DATE, "PR_END_DATE" },
	{ PR_END_DATE_ERROR, "PR_END_DATE_ERROR" },
	{ PR_OWNER_APPT_ID, "PR_OWNER_APPT_ID" },
	{ PR_OWNER_APPT_ID_ERROR, "PR_OWNER_APPT_ID_ERROR" },
	{ PR_RESPONSE_REQUESTED, "PR_RESPONSE_REQUESTED" },
	{ PR_RESPONSE_REQUESTED_ERROR, "PR_RESPONSE_REQUESTED_ERROR" },
	{ PR_SENT_REPRESENTING_ADDRTYPE, "PR_SENT_REPRESENTING_ADDRTYPE" },
	{ PR_SENT_REPRESENTING_ADDRTYPE_UNICODE, "PR_SENT_REPRESENTING_ADDRTYPE_UNICODE" },
	{ PR_SENT_REPRESENTING_ADDRTYPE_ERROR, "PR_SENT_REPRESENTING_ADDRTYPE_ERROR" },
	{ PR_SENT_REPRESENTING_EMAIL_ADDRESS, "PR_SENT_REPRESENTING_EMAIL_ADDRESS" },
	{ PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE, "PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE" },
	{ PR_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR, "PR_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR" },
	{ PR_ORIGINAL_SENDER_ADDRTYPE, "PR_ORIGINAL_SENDER_ADDRTYPE" },
	{ PR_ORIGINAL_SENDER_ADDRTYPE_UNICODE, "PR_ORIGINAL_SENDER_ADDRTYPE_UNICODE" },
	{ PR_ORIGINAL_SENDER_ADDRTYPE_ERROR, "PR_ORIGINAL_SENDER_ADDRTYPE_ERROR" },
	{ PR_ORIGINAL_SENDER_EMAIL_ADDRESS, "PR_ORIGINAL_SENDER_EMAIL_ADDRESS" },
	{ PR_ORIGINAL_SENDER_EMAIL_ADDRESS_UNICODE, "PR_ORIGINAL_SENDER_EMAIL_ADDRESS_UNICODE" },
	{ PR_ORIGINAL_SENDER_EMAIL_ADDRESS_ERROR, "PR_ORIGINAL_SENDER_EMAIL_ADDRESS_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE, "PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE" },
	{ PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_UNICODE, "PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_UNICODE" },
	{ PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS, "PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS" },
	{ PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE, "PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE" },
	{ PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR" },
	{ PR_CONVERSATION_TOPIC, "PR_CONVERSATION_TOPIC" },
	{ PR_CONVERSATION_TOPIC_UNICODE, "PR_CONVERSATION_TOPIC_UNICODE" },
	{ PR_CONVERSATION_TOPIC_ERROR, "PR_CONVERSATION_TOPIC_ERROR" },
	{ PR_CONVERSATION_INDEX, "PR_CONVERSATION_INDEX" },
	{ PR_CONVERSATION_INDEX_ERROR, "PR_CONVERSATION_INDEX_ERROR" },
	{ PR_ORIGINAL_DISPLAY_BCC, "PR_ORIGINAL_DISPLAY_BCC" },
	{ PR_ORIGINAL_DISPLAY_BCC_UNICODE, "PR_ORIGINAL_DISPLAY_BCC_UNICODE" },
	{ PR_ORIGINAL_DISPLAY_BCC_ERROR, "PR_ORIGINAL_DISPLAY_BCC_ERROR" },
	{ PR_ORIGINAL_DISPLAY_CC, "PR_ORIGINAL_DISPLAY_CC" },
	{ PR_ORIGINAL_DISPLAY_CC_UNICODE, "PR_ORIGINAL_DISPLAY_CC_UNICODE" },
	{ PR_ORIGINAL_DISPLAY_CC_ERROR, "PR_ORIGINAL_DISPLAY_CC_ERROR" },
	{ PR_ORIGINAL_DISPLAY_TO, "PR_ORIGINAL_DISPLAY_TO" },
	{ PR_ORIGINAL_DISPLAY_TO_UNICODE, "PR_ORIGINAL_DISPLAY_TO_UNICODE" },
	{ PR_ORIGINAL_DISPLAY_TO_ERROR, "PR_ORIGINAL_DISPLAY_TO_ERROR" },
	{ PR_RECEIVED_BY_ADDRTYPE, "PR_RECEIVED_BY_ADDRTYPE" },
	{ PR_RECEIVED_BY_ADDRTYPE_UNICODE, "PR_RECEIVED_BY_ADDRTYPE_UNICODE" },
	{ PR_RECEIVED_BY_ADDRTYPE_ERROR, "PR_RECEIVED_BY_ADDRTYPE_ERROR" },
	{ PR_RECEIVED_BY_EMAIL_ADDRESS, "PR_RECEIVED_BY_EMAIL_ADDRESS" },
	{ PR_RECEIVED_BY_EMAIL_ADDRESS_UNICODE, "PR_RECEIVED_BY_EMAIL_ADDRESS_UNICODE" },
	{ PR_RECEIVED_BY_EMAIL_ADDRESS_ERROR, "PR_RECEIVED_BY_EMAIL_ADDRESS_ERROR" },
	{ PR_RCVD_REPRESENTING_ADDRTYPE, "PR_RCVD_REPRESENTING_ADDRTYPE" },
	{ PR_RCVD_REPRESENTING_ADDRTYPE_UNICODE, "PR_RCVD_REPRESENTING_ADDRTYPE_UNICODE" },
	{ PR_RCVD_REPRESENTING_ADDRTYPE_ERROR, "PR_RCVD_REPRESENTING_ADDRTYPE_ERROR" },
	{ PR_RCVD_REPRESENTING_EMAIL_ADDRESS, "PR_RCVD_REPRESENTING_EMAIL_ADDRESS" },
	{ PR_RCVD_REPRESENTING_EMAIL_ADDRESS_UNICODE, "PR_RCVD_REPRESENTING_EMAIL_ADDRESS_UNICODE" },
	{ PR_RCVD_REPRESENTING_EMAIL_ADDRESS_ERROR, "PR_RCVD_REPRESENTING_EMAIL_ADDRESS_ERROR" },
	{ PR_ORIGINAL_AUTHOR_ADDRTYPE, "PR_ORIGINAL_AUTHOR_ADDRTYPE" },
	{ PR_ORIGINAL_AUTHOR_ADDRTYPE_UNICODE, "PR_ORIGINAL_AUTHOR_ADDRTYPE_UNICODE" },
	{ PR_ORIGINAL_AUTHOR_ADDRTYPE_ERROR, "PR_ORIGINAL_AUTHOR_ADDRTYPE_ERROR" },
	{ PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS, "PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS" },
	{ PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_UNICODE, "PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_UNICODE" },
	{ PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_ERROR, "PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_ERROR" },
	{ PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE, "PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE" },
	{ PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_UNICODE, "PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_UNICODE" },
	{ PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_ERROR, "PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_ERROR" },
	{ PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS, "PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS" },
	{ PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_UNICODE, "PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_UNICODE" },
	{ PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_ERROR, "PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_ERROR" },
	{ PR_TRANSPORT_MESSAGE_HEADERS, "PR_TRANSPORT_MESSAGE_HEADERS" },
	{ PR_TRANSPORT_MESSAGE_HEADERS_UNICODE, "PR_TRANSPORT_MESSAGE_HEADERS_UNICODE" },
	{ PR_TRANSPORT_MESSAGE_HEADERS_ERROR, "PR_TRANSPORT_MESSAGE_HEADERS_ERROR" },
	{ PR_DELEGATION, "PR_DELEGATION" },
	{ PR_DELEGATION_ERROR, "PR_DELEGATION_ERROR" },
	{ PR_TNEF_CORRELATION_KEY, "PR_TNEF_CORRELATION_KEY" },
	{ PR_TNEF_CORRELATION_KEY_ERROR, "PR_TNEF_CORRELATION_KEY_ERROR" },
	{ PR_CONTENT_INTEGRITY_CHECK, "PR_CONTENT_INTEGRITY_CHECK" },
	{ PR_CONTENT_INTEGRITY_CHECK_ERROR, "PR_CONTENT_INTEGRITY_CHECK_ERROR" },
	{ PR_EXPLICIT_CONVERSION, "PR_EXPLICIT_CONVERSION" },
	{ PR_EXPLICIT_CONVERSION_ERROR, "PR_EXPLICIT_CONVERSION_ERROR" },
	{ PR_IPM_RETURN_REQUESTED, "PR_IPM_RETURN_REQUESTED" },
	{ PR_IPM_RETURN_REQUESTED_ERROR, "PR_IPM_RETURN_REQUESTED_ERROR" },
	{ PR_MESSAGE_TOKEN, "PR_MESSAGE_TOKEN" },
	{ PR_MESSAGE_TOKEN_ERROR, "PR_MESSAGE_TOKEN_ERROR" },
	{ PR_NDR_REASON_CODE, "PR_NDR_REASON_CODE" },
	{ PR_NDR_REASON_CODE_ERROR, "PR_NDR_REASON_CODE_ERROR" },
	{ PR_NDR_DIAG_CODE, "PR_NDR_DIAG_CODE" },
	{ PR_NDR_DIAG_CODE_ERROR, "PR_NDR_DIAG_CODE_ERROR" },
	{ PR_NON_RECEIPT_NOTIFICATION_REQUESTED, "PR_NON_RECEIPT_NOTIFICATION_REQUESTED" },
	{ PR_NON_RECEIPT_NOTIFICATION_REQUESTED_ERROR, "PR_NON_RECEIPT_NOTIFICATION_REQUESTED_ERROR" },
	{ PR_DELIVERY_POINT, "PR_DELIVERY_POINT" },
	{ PR_DELIVERY_POINT_ERROR, "PR_DELIVERY_POINT_ERROR" },
	{ PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED, "PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED" },
	{ PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED_ERROR, "PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED_ERROR" },
	{ PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT, "PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT" },
	{ PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT_ERROR, "PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT_ERROR" },
	{ PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY, "PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY" },
	{ PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY_ERROR, "PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY_ERROR" },
	{ PR_PHYSICAL_DELIVERY_MODE, "PR_PHYSICAL_DELIVERY_MODE" },
	{ PR_PHYSICAL_DELIVERY_MODE_ERROR, "PR_PHYSICAL_DELIVERY_MODE_ERROR" },
	{ PR_PHYSICAL_DELIVERY_REPORT_REQUEST, "PR_PHYSICAL_DELIVERY_REPORT_REQUEST" },
	{ PR_PHYSICAL_DELIVERY_REPORT_REQUEST_ERROR, "PR_PHYSICAL_DELIVERY_REPORT_REQUEST_ERROR" },
	{ PR_PHYSICAL_FORWARDING_ADDRESS, "PR_PHYSICAL_FORWARDING_ADDRESS" },
	{ PR_PHYSICAL_FORWARDING_ADDRESS_ERROR, "PR_PHYSICAL_FORWARDING_ADDRESS_ERROR" },
	{ PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED, "PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED" },
	{ PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED_ERROR, "PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED_ERROR" },
	{ PR_PHYSICAL_FORWARDING_PROHIBITED, "PR_PHYSICAL_FORWARDING_PROHIBITED" },
	{ PR_PHYSICAL_FORWARDING_PROHIBITED_ERROR, "PR_PHYSICAL_FORWARDING_PROHIBITED_ERROR" },
	{ PR_PHYSICAL_RENDITION_ATTRIBUTES, "PR_PHYSICAL_RENDITION_ATTRIBUTES" },
	{ PR_PHYSICAL_RENDITION_ATTRIBUTES_ERROR, "PR_PHYSICAL_RENDITION_ATTRIBUTES_ERROR" },
	{ PR_PROOF_OF_DELIVERY, "PR_PROOF_OF_DELIVERY" },
	{ PR_PROOF_OF_DELIVERY_ERROR, "PR_PROOF_OF_DELIVERY_ERROR" },
	{ PR_PROOF_OF_DELIVERY_REQUESTED, "PR_PROOF_OF_DELIVERY_REQUESTED" },
	{ PR_PROOF_OF_DELIVERY_REQUESTED_ERROR, "PR_PROOF_OF_DELIVERY_REQUESTED_ERROR" },
	{ PR_RECIPIENT_CERTIFICATE, "PR_RECIPIENT_CERTIFICATE" },
	{ PR_RECIPIENT_CERTIFICATE_ERROR, "PR_RECIPIENT_CERTIFICATE_ERROR" },
	{ PR_RECIPIENT_NUMBER_FOR_ADVICE, "PR_RECIPIENT_NUMBER_FOR_ADVICE" },
	{ PR_RECIPIENT_NUMBER_FOR_ADVICE_UNICODE, "PR_RECIPIENT_NUMBER_FOR_ADVICE_UNICODE" },
	{ PR_RECIPIENT_NUMBER_FOR_ADVICE_ERROR, "PR_RECIPIENT_NUMBER_FOR_ADVICE_ERROR" },
	{ PR_RECIPIENT_TYPE, "PR_RECIPIENT_TYPE" },
	{ PR_RECIPIENT_TYPE_ERROR, "PR_RECIPIENT_TYPE_ERROR" },
	{ PR_REGISTERED_MAIL_TYPE, "PR_REGISTERED_MAIL_TYPE" },
	{ PR_REGISTERED_MAIL_TYPE_ERROR, "PR_REGISTERED_MAIL_TYPE_ERROR" },
	{ PR_REPLY_REQUESTED, "PR_REPLY_REQUESTED" },
	{ PR_REPLY_REQUESTED_ERROR, "PR_REPLY_REQUESTED_ERROR" },
	{ PR_REQUESTED_DELIVERY_METHOD, "PR_REQUESTED_DELIVERY_METHOD" },
	{ PR_REQUESTED_DELIVERY_METHOD_ERROR, "PR_REQUESTED_DELIVERY_METHOD_ERROR" },
	{ PR_SENDER_ENTRYID, "PR_SENDER_ENTRYID" },
	{ PR_SENDER_ENTRYID_ERROR, "PR_SENDER_ENTRYID_ERROR" },
	{ PR_SENDER_NAME, "PR_SENDER_NAME" },
	{ PR_SENDER_NAME_UNICODE, "PR_SENDER_NAME_UNICODE" },
	{ PR_SENDER_NAME_ERROR, "PR_SENDER_NAME_ERROR" },
	{ PR_SUPPLEMENTARY_INFO, "PR_SUPPLEMENTARY_INFO" },
	{ PR_SUPPLEMENTARY_INFO_UNICODE, "PR_SUPPLEMENTARY_INFO_UNICODE" },
	{ PR_SUPPLEMENTARY_INFO_ERROR, "PR_SUPPLEMENTARY_INFO_ERROR" },
	{ PR_TYPE_OF_MTS_USER, "PR_TYPE_OF_MTS_USER" },
	{ PR_TYPE_OF_MTS_USER_ERROR, "PR_TYPE_OF_MTS_USER_ERROR" },
	{ PR_SENDER_SEARCH_KEY, "PR_SENDER_SEARCH_KEY" },
	{ PR_SENDER_SEARCH_KEY_ERROR, "PR_SENDER_SEARCH_KEY_ERROR" },
	{ PR_SENDER_ADDRTYPE, "PR_SENDER_ADDRTYPE" },
	{ PR_SENDER_ADDRTYPE_UNICODE, "PR_SENDER_ADDRTYPE_UNICODE" },
	{ PR_SENDER_ADDRTYPE_ERROR, "PR_SENDER_ADDRTYPE_ERROR" },
	{ PR_SENDER_EMAIL_ADDRESS, "PR_SENDER_EMAIL_ADDRESS" },
	{ PR_SENDER_EMAIL_ADDRESS_UNICODE, "PR_SENDER_EMAIL_ADDRESS_UNICODE" },
	{ PR_SENDER_EMAIL_ADDRESS_ERROR, "PR_SENDER_EMAIL_ADDRESS_ERROR" },
	{ PR_CURRENT_VERSION, "PR_CURRENT_VERSION" },
	{ PR_CURRENT_VERSION_ERROR, "PR_CURRENT_VERSION_ERROR" },
	{ PR_DELETE_AFTER_SUBMIT, "PR_DELETE_AFTER_SUBMIT" },
	{ PR_DELETE_AFTER_SUBMIT_ERROR, "PR_DELETE_AFTER_SUBMIT_ERROR" },
	{ PR_DISPLAY_BCC, "PR_DISPLAY_BCC" },
	{ PR_DISPLAY_BCC_UNICODE, "PR_DISPLAY_BCC_UNICODE" },
	{ PR_DISPLAY_BCC_ERROR, "PR_DISPLAY_BCC_ERROR" },
	{ PR_DISPLAY_CC, "PR_DISPLAY_CC" },
	{ PR_DISPLAY_CC_UNICODE, "PR_DISPLAY_CC_UNICODE" },
	{ PR_DISPLAY_CC_ERROR, "PR_DISPLAY_CC_ERROR" },
	{ PR_DISPLAY_TO, "PR_DISPLAY_TO" },
	{ PR_DISPLAY_TO_UNICODE, "PR_DISPLAY_TO_UNICODE" },
	{ PR_DISPLAY_TO_ERROR, "PR_DISPLAY_TO_ERROR" },
	{ PR_PARENT_DISPLAY, "PR_PARENT_DISPLAY" },
	{ PR_PARENT_DISPLAY_UNICODE, "PR_PARENT_DISPLAY_UNICODE" },
	{ PR_PARENT_DISPLAY_ERROR, "PR_PARENT_DISPLAY_ERROR" },
	{ PR_MESSAGE_DELIVERY_TIME, "PR_MESSAGE_DELIVERY_TIME" },
	{ PR_MESSAGE_DELIVERY_TIME_ERROR, "PR_MESSAGE_DELIVERY_TIME_ERROR" },
	{ PR_MESSAGE_FLAGS, "PR_MESSAGE_FLAGS" },
	{ PR_MESSAGE_FLAGS_ERROR, "PR_MESSAGE_FLAGS_ERROR" },
	{ PR_MESSAGE_SIZE, "PR_MESSAGE_SIZE" },
	{ PR_MESSAGE_SIZE_ERROR, "PR_MESSAGE_SIZE_ERROR" },
	{ PR_MESSAGE_SIZE_EXTENDED, "PR_MESSAGE_SIZE_EXTENDED" },
	{ PR_PARENT_ENTRYID, "PR_PARENT_ENTRYID" },
	{ PR_PARENT_ENTRYID_ERROR, "PR_PARENT_ENTRYID_ERROR" },
	{ PR_SENTMAIL_ENTRYID, "PR_SENTMAIL_ENTRYID" },
	{ PR_SENTMAIL_ENTRYID_ERROR, "PR_SENTMAIL_ENTRYID_ERROR" },
	{ PR_CORRELATE, "PR_CORRELATE" },
	{ PR_CORRELATE_ERROR, "PR_CORRELATE_ERROR" },
	{ PR_CORRELATE_MTSID, "PR_CORRELATE_MTSID" },
	{ PR_CORRELATE_MTSID_ERROR, "PR_CORRELATE_MTSID_ERROR" },
	{ PR_DISCRETE_VALUES, "PR_DISCRETE_VALUES" },
	{ PR_DISCRETE_VALUES_ERROR, "PR_DISCRETE_VALUES_ERROR" },
	{ PR_RESPONSIBILITY, "PR_RESPONSIBILITY" },
	{ PR_RESPONSIBILITY_ERROR, "PR_RESPONSIBILITY_ERROR" },
	{ PR_SPOOLER_STATUS, "PR_SPOOLER_STATUS" },
	{ PR_SPOOLER_STATUS_ERROR, "PR_SPOOLER_STATUS_ERROR" },
	{ PR_TRANSPORT_STATUS, "PR_TRANSPORT_STATUS" },
	{ PR_TRANSPORT_STATUS_ERROR, "PR_TRANSPORT_STATUS_ERROR" },
	{ PR_MESSAGE_RECIPIENTS, "PR_MESSAGE_RECIPIENTS" },
	{ PR_MESSAGE_RECIPIENTS_ERROR, "PR_MESSAGE_RECIPIENTS_ERROR" },
	{ PR_MESSAGE_ATTACHMENTS, "PR_MESSAGE_ATTACHMENTS" },
	{ PR_MESSAGE_ATTACHMENTS_ERROR, "PR_MESSAGE_ATTACHMENTS_ERROR" },
	{ PR_SUBMIT_FLAGS, "PR_SUBMIT_FLAGS" },
	{ PR_SUBMIT_FLAGS_ERROR, "PR_SUBMIT_FLAGS_ERROR" },
	{ PR_RECIPIENT_STATUS, "PR_RECIPIENT_STATUS" },
	{ PR_RECIPIENT_STATUS_ERROR, "PR_RECIPIENT_STATUS_ERROR" },
	{ PR_TRANSPORT_KEY, "PR_TRANSPORT_KEY" },
	{ PR_TRANSPORT_KEY_ERROR, "PR_TRANSPORT_KEY_ERROR" },
	{ PR_MSG_STATUS, "PR_MSG_STATUS" },
	{ PR_MSG_STATUS_ERROR, "PR_MSG_STATUS_ERROR" },
	{ PR_MESSAGE_DOWNLOAD_TIME, "PR_MESSAGE_DOWNLOAD_TIME" },
	{ PR_MESSAGE_DOWNLOAD_TIME_ERROR, "PR_MESSAGE_DOWNLOAD_TIME_ERROR" },
	{ PR_CREATION_VERSION, "PR_CREATION_VERSION" },
	{ PR_CREATION_VERSION_ERROR, "PR_CREATION_VERSION_ERROR" },
	{ PR_MODIFY_VERSION, "PR_MODIFY_VERSION" },
	{ PR_MODIFY_VERSION_ERROR, "PR_MODIFY_VERSION_ERROR" },
	{ PR_HASATTACH, "PR_HASATTACH" },
	{ PR_HASATTACH_ERROR, "PR_HASATTACH_ERROR" },
	{ PR_BODY_CRC, "PR_BODY_CRC" },
	{ PR_BODY_CRC_ERROR, "PR_BODY_CRC_ERROR" },
	{ PR_NORMALIZED_SUBJECT, "PR_NORMALIZED_SUBJECT" },
	{ PR_NORMALIZED_SUBJECT_UNICODE, "PR_NORMALIZED_SUBJECT_UNICODE" },
	{ PR_NORMALIZED_SUBJECT_ERROR, "PR_NORMALIZED_SUBJECT_ERROR" },
	{ PR_RTF_IN_SYNC, "PR_RTF_IN_SYNC" },
	{ PR_RTF_IN_SYNC_ERROR, "PR_RTF_IN_SYNC_ERROR" },
	{ PR_ATTACH_SIZE, "PR_ATTACH_SIZE" },
	{ PR_ATTACH_SIZE_ERROR, "PR_ATTACH_SIZE_ERROR" },
	{ PR_ATTACH_NUM, "PR_ATTACH_NUM" },
	{ PR_ATTACH_NUM_ERROR, "PR_ATTACH_NUM_ERROR" },
	{ PR_PREPROCESS, "PR_PREPROCESS" },
	{ PR_PREPROCESS_ERROR, "PR_PREPROCESS_ERROR" },
	{ PR_INTERNET_ARTICLE_NUMBER, "PR_INTERNET_ARTICLE_NUMBER" },
	{ PR_INTERNET_ARTICLE_NUMBER_ERROR, "PR_INTERNET_ARTICLE_NUMBER_ERROR" },
	{ PR_NEWSGROUP_NAME, "PR_NEWSGROUP_NAME" },
	{ PR_NEWSGROUP_NAME_UNICODE, "PR_NEWSGROUP_NAME_UNICODE" },
	{ PR_NEWSGROUP_NAME_ERROR, "PR_NEWSGROUP_NAME_ERROR" },
	{ PR_ORIGINATING_MTA_CERTIFICATE, "PR_ORIGINATING_MTA_CERTIFICATE" },
	{ PR_ORIGINATING_MTA_CERTIFICATE_ERROR, "PR_ORIGINATING_MTA_CERTIFICATE_ERROR" },
	{ PR_PROOF_OF_SUBMISSION, "PR_PROOF_OF_SUBMISSION" },
	{ PR_PROOF_OF_SUBMISSION_ERROR, "PR_PROOF_OF_SUBMISSION_ERROR" },
	{ PR_NT_SECURITY_DESCRIPTOR, "PR_NT_SECURITY_DESCRIPTOR" },
	{ PR_NT_SECURITY_DESCRIPTOR_ERROR, "PR_NT_SECURITY_DESCRIPTOR_ERROR" },
	{ PR_CREATOR_SID, "PR_CREATOR_SID" },
	{ PR_CREATOR_SID_ERROR, "PR_CREATOR_SID_ERROR" },
	{ PR_LAST_MODIFIER_SID, "PR_LAST_MODIFIER_SID" },
	{ PR_LAST_MODIFIER_SID_ERROR, "PR_LAST_MODIFIER_SID_ERROR" },
	{ PR_MIME_HANDLER_CLASSIDS, "PR_MIME_HANDLER_CLASSIDS" },
	{ PR_MIME_HANDLER_CLASSIDS_ERROR, "PR_MIME_HANDLER_CLASSIDS_ERROR" },
	{ PR_URL_COMP_NAME_POSTFIX, "PR_URL_COMP_NAME_POSTFIX" },
	{ PR_URL_COMP_NAME_POSTFIX_ERROR, "PR_URL_COMP_NAME_POSTFIX_ERROR" },
	{ PR_URL_COMP_NAME_SET, "PR_URL_COMP_NAME_SET" },
	{ PR_URL_COMP_NAME_SET_ERROR, "PR_URL_COMP_NAME_SET_ERROR" },
	{ PR_SUBFOLDER_CT, "PR_SUBFOLDER_CT" },
	{ PR_SUBFOLDER_CT_ERROR, "PR_SUBFOLDER_CT_ERROR" },
	{ PR_DELETED_SUBFOLDER_CT, "PR_DELETED_SUBFOLDER_CT" },
	{ PR_DELETED_SUBFOLDER_CT_ERROR, "PR_DELETED_SUBFOLDER_CT_ERROR" },
	{ PR_DELETE_TIME, "PR_DELETE_TIME" },
	{ PR_DELETE_TIME_ERROR, "PR_DELETE_TIME_ERROR" },
	{ PR_AGE_LIMIT, "PR_AGE_LIMIT" },
	{ PR_AGE_LIMIT_ERROR, "PR_AGE_LIMIT_ERROR" },
	{ PR_TRUST_SENDER, "PR_TRUST_SENDER" },
	{ PR_TRUST_SENDER_ERROR, "PR_TRUST_SENDER_ERROR" },
	{ PR_ATTACH_VIRUS_SCAN_INFO, "PR_ATTACH_VIRUS_SCAN_INFO" },
	{ PR_ATTACH_VIRUS_SCAN_INFO_ERROR, "PR_ATTACH_VIRUS_SCAN_INFO_ERROR" },
	{ PR_ACCESS, "PR_ACCESS" },
	{ PR_ACCESS_ERROR, "PR_ACCESS_ERROR" },
	{ PR_ROW_TYPE, "PR_ROW_TYPE" },
	{ PR_ROW_TYPE_ERROR, "PR_ROW_TYPE_ERROR" },
	{ PR_INSTANCE_KEY, "PR_INSTANCE_KEY" },
	{ PR_INSTANCE_KEY_ERROR, "PR_INSTANCE_KEY_ERROR" },
	{ PR_ACCESS_LEVEL, "PR_ACCESS_LEVEL" },
	{ PR_ACCESS_LEVEL_ERROR, "PR_ACCESS_LEVEL_ERROR" },
	{ PR_MAPPING_SIGNATURE, "PR_MAPPING_SIGNATURE" },
	{ PR_MAPPING_SIGNATURE_ERROR, "PR_MAPPING_SIGNATURE_ERROR" },
	{ PR_RECORD_KEY, "PR_RECORD_KEY" },
	{ PR_RECORD_KEY_ERROR, "PR_RECORD_KEY_ERROR" },
	{ PR_STORE_RECORD_KEY, "PR_STORE_RECORD_KEY" },
	{ PR_STORE_RECORD_KEY_ERROR, "PR_STORE_RECORD_KEY_ERROR" },
	{ PR_STORE_ENTRYID, "PR_STORE_ENTRYID" },
	{ PR_STORE_ENTRYID_ERROR, "PR_STORE_ENTRYID_ERROR" },
	{ PR_MINI_ICON, "PR_MINI_ICON" },
	{ PR_MINI_ICON_ERROR, "PR_MINI_ICON_ERROR" },
	{ PR_ICON, "PR_ICON" },
	{ PR_ICON_ERROR, "PR_ICON_ERROR" },
	{ PR_OBJECT_TYPE, "PR_OBJECT_TYPE" },
	{ PR_OBJECT_TYPE_ERROR, "PR_OBJECT_TYPE_ERROR" },
	{ PR_ENTRYID, "PR_ENTRYID" },
	{ PR_ENTRYID_ERROR, "PR_ENTRYID_ERROR" },
	{ PR_BODY, "PR_BODY" },
	{ PR_BODY_UNICODE, "PR_BODY_UNICODE" },
	{ PR_BODY_ERROR, "PR_BODY_ERROR" },
	{ PR_REPORT_TEXT, "PR_REPORT_TEXT" },
	{ PR_REPORT_TEXT_UNICODE, "PR_REPORT_TEXT_UNICODE" },
	{ PR_REPORT_TEXT_ERROR, "PR_REPORT_TEXT_ERROR" },
	{ PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY, "PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY" },
	{ PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY_ERROR, "PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY_ERROR" },
	{ PR_REPORTING_DL_NAME, "PR_REPORTING_DL_NAME" },
	{ PR_REPORTING_DL_NAME_ERROR, "PR_REPORTING_DL_NAME_ERROR" },
	{ PR_REPORTING_MTA_CERTIFICATE, "PR_REPORTING_MTA_CERTIFICATE" },
	{ PR_REPORTING_MTA_CERTIFICATE_ERROR, "PR_REPORTING_MTA_CERTIFICATE_ERROR" },
	{ PR_RTF_SYNC_BODY_CRC, "PR_RTF_SYNC_BODY_CRC" },
	{ PR_RTF_SYNC_BODY_CRC_ERROR, "PR_RTF_SYNC_BODY_CRC_ERROR" },
	{ PR_RTF_SYNC_BODY_COUNT, "PR_RTF_SYNC_BODY_COUNT" },
	{ PR_RTF_SYNC_BODY_COUNT_ERROR, "PR_RTF_SYNC_BODY_COUNT_ERROR" },
	{ PR_RTF_SYNC_BODY_TAG, "PR_RTF_SYNC_BODY_TAG" },
	{ PR_RTF_SYNC_BODY_TAG_UNICODE, "PR_RTF_SYNC_BODY_TAG_UNICODE" },
	{ PR_RTF_SYNC_BODY_TAG_ERROR, "PR_RTF_SYNC_BODY_TAG_ERROR" },
	{ PR_RTF_COMPRESSED, "PR_RTF_COMPRESSED" },
	{ PR_RTF_COMPRESSED_ERROR, "PR_RTF_COMPRESSED_ERROR" },
	{ PR_RTF_SYNC_PREFIX_COUNT, "PR_RTF_SYNC_PREFIX_COUNT" },
	{ PR_RTF_SYNC_PREFIX_COUNT_ERROR, "PR_RTF_SYNC_PREFIX_COUNT_ERROR" },
	{ PR_RTF_SYNC_TRAILING_COUNT, "PR_RTF_SYNC_TRAILING_COUNT" },
	{ PR_RTF_SYNC_TRAILING_COUNT_ERROR, "PR_RTF_SYNC_TRAILING_COUNT_ERROR" },
	{ PR_ORIGINALLY_INTENDED_RECIP_ENTRYID, "PR_ORIGINALLY_INTENDED_RECIP_ENTRYID" },
	{ PR_ORIGINALLY_INTENDED_RECIP_ENTRYID_ERROR, "PR_ORIGINALLY_INTENDED_RECIP_ENTRYID_ERROR" },
	{ PR_HTML, "PR_HTML" },
	{ PR_HTML_ERROR, "PR_HTML_ERROR" },
	{ PR_INTERNET_APPROVED, "PR_INTERNET_APPROVED" },
	{ PR_INTERNET_APPROVED_UNICODE, "PR_INTERNET_APPROVED_UNICODE" },
	{ PR_INTERNET_APPROVED_ERROR, "PR_INTERNET_APPROVED_ERROR" },
	{ PR_INTERNET_CONTROL, "PR_INTERNET_CONTROL" },
	{ PR_INTERNET_CONTROL_UNICODE, "PR_INTERNET_CONTROL_UNICODE" },
	{ PR_INTERNET_CONTROL_ERROR, "PR_INTERNET_CONTROL_ERROR" },
	{ PR_INTERNET_DISTRIBUTION, "PR_INTERNET_DISTRIBUTION" },
	{ PR_INTERNET_DISTRIBUTION_UNICODE, "PR_INTERNET_DISTRIBUTION_UNICODE" },
	{ PR_INTERNET_DISTRIBUTION_ERROR, "PR_INTERNET_DISTRIBUTION_ERROR" },
	{ PR_INTERNET_FOLLOWUP_TO, "PR_INTERNET_FOLLOWUP_TO" },
	{ PR_INTERNET_FOLLOWUP_TO_UNICODE, "PR_INTERNET_FOLLOWUP_TO_UNICODE" },
	{ PR_INTERNET_FOLLOWUP_TO_ERROR, "PR_INTERNET_FOLLOWUP_TO_ERROR" },
	{ PR_INTERNET_LINES, "PR_INTERNET_LINES" },
	{ PR_INTERNET_LINES_ERROR, "PR_INTERNET_LINES_ERROR" },
	{ PR_INTERNET_MESSAGE_ID, "PR_INTERNET_MESSAGE_ID" },
	{ PR_INTERNET_MESSAGE_ID_UNICODE, "PR_INTERNET_MESSAGE_ID_UNICODE" },
	{ PR_INTERNET_MESSAGE_ID_ERROR, "PR_INTERNET_MESSAGE_ID_ERROR" },
	{ PR_INTERNET_NEWSGROUPS, "PR_INTERNET_NEWSGROUPS" },
	{ PR_INTERNET_NEWSGROUPS_UNICODE, "PR_INTERNET_NEWSGROUPS_UNICODE" },
	{ PR_INTERNET_NEWSGROUPS_ERROR, "PR_INTERNET_NEWSGROUPS_ERROR" },
	{ PR_INTERNET_ORGANIZATION, "PR_INTERNET_ORGANIZATION" },
	{ PR_INTERNET_ORGANIZATION_UNICODE, "PR_INTERNET_ORGANIZATION_UNICODE" },
	{ PR_INTERNET_ORGANIZATION_ERROR, "PR_INTERNET_ORGANIZATION_ERROR" },
	{ PR_INTERNET_NNTP_PATH, "PR_INTERNET_NNTP_PATH" },
	{ PR_INTERNET_NNTP_PATH_UNICODE, "PR_INTERNET_NNTP_PATH_UNICODE" },
	{ PR_INTERNET_NNTP_PATH_ERROR, "PR_INTERNET_NNTP_PATH_ERROR" },
	{ PR_INTERNET_REFERENCES, "PR_INTERNET_REFERENCES" },
	{ PR_INTERNET_REFERENCES_UNICODE, "PR_INTERNET_REFERENCES_UNICODE" },
	{ PR_INTERNET_REFERENCES_ERROR, "PR_INTERNET_REFERENCES_ERROR" },
	{ PR_SUPERSEDES, "PR_SUPERSEDES" },
	{ PR_SUPERSEDES_UNICODE, "PR_SUPERSEDES_UNICODE" },
	{ PR_SUPERSEDES_ERROR, "PR_SUPERSEDES_ERROR" },
	{ PR_POST_FOLDER_ENTRIES, "PR_POST_FOLDER_ENTRIES" },
	{ PR_POST_FOLDER_ENTRIES_ERROR, "PR_POST_FOLDER_ENTRIES_ERROR" },
	{ PR_POST_FOLDER_NAMES, "PR_POST_FOLDER_NAMES" },
	{ PR_POST_FOLDER_NAMES_UNICODE, "PR_POST_FOLDER_NAMES_UNICODE" },
	{ PR_POST_FOLDER_NAMES_ERROR, "PR_POST_FOLDER_NAMES_ERROR" },
	{ PR_POST_REPLY_FOLDER_ENTRIES, "PR_POST_REPLY_FOLDER_ENTRIES" },
	{ PR_POST_REPLY_FOLDER_ENTRIES_ERROR, "PR_POST_REPLY_FOLDER_ENTRIES_ERROR" },
	{ PR_POST_REPLY_FOLDER_NAMES, "PR_POST_REPLY_FOLDER_NAMES" },
	{ PR_POST_REPLY_FOLDER_NAMES_UNICODE, "PR_POST_REPLY_FOLDER_NAMES_UNICODE" },
	{ PR_POST_REPLY_FOLDER_NAMES_ERROR, "PR_POST_REPLY_FOLDER_NAMES_ERROR" },
	{ PR_POST_REPLY_DENIED, "PR_POST_REPLY_DENIED" },
	{ PR_POST_REPLY_DENIED_ERROR, "PR_POST_REPLY_DENIED_ERROR" },
	{ PR_NNTP_XREF, "PR_NNTP_XREF" },
	{ PR_NNTP_XREF_UNICODE, "PR_NNTP_XREF_UNICODE" },
	{ PR_NNTP_XREF_ERROR, "PR_NNTP_XREF_ERROR" },
	{ PR_INTERNET_PRECEDENCE, "PR_INTERNET_PRECEDENCE" },
	{ PR_INTERNET_PRECEDENCE_UNICODE, "PR_INTERNET_PRECEDENCE_UNICODE" },
	{ PR_INTERNET_PRECEDENCE_ERROR, "PR_INTERNET_PRECEDENCE_ERROR" },
	{ PR_IN_REPLY_TO_ID, "PR_IN_REPLY_TO_ID" },
	{ PR_IN_REPLY_TO_ID_UNICODE, "PR_IN_REPLY_TO_ID_UNICODE" },
	{ PR_IN_REPLY_TO_ID_ERROR, "PR_IN_REPLY_TO_ID_ERROR" },
	{ PR_LIST_HELP, "PR_LIST_HELP" },
	{ PR_LIST_HELP_UNICODE, "PR_LIST_HELP_UNICODE" },
	{ PR_LIST_HELP_ERROR, "PR_LIST_HELP_ERROR" },
	{ PR_LIST_SUBSCRIBE, "PR_LIST_SUBSCRIBE" },
	{ PR_LIST_SUBSCRIBE_UNICODE, "PR_LIST_SUBSCRIBE_UNICODE" },
	{ PR_LIST_SUBSCRIBE_ERROR, "PR_LIST_SUBSCRIBE_ERROR" },
	{ PR_LIST_UNSUBSCRIBE, "PR_LIST_UNSUBSCRIBE" },
	{ PR_LIST_UNSUBSCRIBE_UNICODE, "PR_LIST_UNSUBSCRIBE_UNICODE" },
	{ PR_LIST_UNSUBSCRIBE_ERROR, "PR_LIST_UNSUBSCRIBE_ERROR" },
	{ PR_ACTION, "PR_ACTION" },
	{ PR_ACTION_ERROR, "PR_ACTION_ERROR" },
	{ PR_ACTION_FLAG, "PR_ACTION_FLAG" },
	{ PR_ACTION_FLAG_ERROR, "PR_ACTION_FLAG_ERROR" },
	{ PR_ACTION_DATE, "PR_ACTION_DATE" },
	{ PR_ACTION_DATE_ERROR, "PR_ACTION_DATE_ERROR" },
	{ PR_FLAG_STATUS, "PR_FLAG_STATUS" },
	{ PR_FLAG_STATUS_ERROR, "PR_FLAG_STATUS_ERROR" },
	{ PR_FLAG_COMPLETE, "PR_FLAG_COMPLETE" },
	{ PR_FLAG_COMPLETE_ERROR, "PR_FLAG_COMPLETE_ERROR" },
	{ PR_SMTP_TEMP_TBL_DATA, "PR_SMTP_TEMP_TBL_DATA" },
	{ PR_SMTP_TEMP_TBL_DATA_ERROR, "PR_SMTP_TEMP_TBL_DATA_ERROR" },
	{ PR_SMTP_TEMP_TBL_DATA_2, "PR_SMTP_TEMP_TBL_DATA_2" },
	{ PR_SMTP_TEMP_TBL_DATA_2_ERROR, "PR_SMTP_TEMP_TBL_DATA_2_ERROR" },
	{ PR_SMTP_TEMP_TBL_DATA_3, "PR_SMTP_TEMP_TBL_DATA_3" },
	{ PR_SMTP_TEMP_TBL_DATA_3_ERROR, "PR_SMTP_TEMP_TBL_DATA_3_ERROR" },
	{ PR_CAL_START_TIME, "PR_CAL_START_TIME" },
	{ PR_CAL_START_TIME_ERROR, "PR_CAL_START_TIME_ERROR" },
	{ PR_CAL_END_TIME, "PR_CAL_END_TIME" },
	{ PR_CAL_END_TIME_ERROR, "PR_CAL_END_TIME_ERROR" },
	{ PR_CAL_RECURRING_ID, "PR_CAL_RECURRING_ID" },
	{ PR_CAL_RECURRING_ID_ERROR, "PR_CAL_RECURRING_ID_ERROR" },
	{ PR_DAV_SUBMIT_DATA, "PR_DAV_SUBMIT_DATA" },
	{ PR_DAV_SUBMIT_DATA_UNICODE, "PR_DAV_SUBMIT_DATA_UNICODE" },
	{ PR_DAV_SUBMIT_DATA_ERROR, "PR_DAV_SUBMIT_DATA_ERROR" },
	{ PR_CDO_EXPANSION_INDEX, "PR_CDO_EXPANSION_INDEX" },
	{ PR_CDO_EXPANSION_INDEX_ERROR, "PR_CDO_EXPANSION_INDEX_ERROR" },
	{ PR_IFS_INTERNAL_DATA, "PR_IFS_INTERNAL_DATA" },
	{ PR_IFS_INTERNAL_DATA_ERROR, "PR_IFS_INTERNAL_DATA_ERROR" },
	{ PR_CAL_REMINDER_NEXT_TIME, "PR_CAL_REMINDER_NEXT_TIME" },
	{ PR_CAL_REMINDER_NEXT_TIME_ERROR, "PR_CAL_REMINDER_NEXT_TIME_ERROR" },
	{ PR_OWA_URL, "PR_OWA_URL" },
	{ PR_OWA_URL_UNICODE, "PR_OWA_URL_UNICODE" },
	{ PR_OWA_URL_ERROR, "PR_OWA_URL_ERROR" },
	{ PR_DISABLE_FULL_FIDELITY, "PR_DISABLE_FULL_FIDELITY" },
	{ PR_DISABLE_FULL_FIDELITY_ERROR, "PR_DISABLE_FULL_FIDELITY_ERROR" },
	{ PR_URL_COMP_NAME, "PR_URL_COMP_NAME" },
	{ PR_URL_COMP_NAME_UNICODE, "PR_URL_COMP_NAME_UNICODE" },
	{ PR_URL_COMP_NAME_ERROR, "PR_URL_COMP_NAME_ERROR" },
	{ PR_ATTR_HIDDEN, "PR_ATTR_HIDDEN" },
	{ PR_ATTR_HIDDEN_ERROR, "PR_ATTR_HIDDEN_ERROR" },
	{ PR_ATTR_SYSTEM, "PR_ATTR_SYSTEM" },
	{ PR_ATTR_SYSTEM_ERROR, "PR_ATTR_SYSTEM_ERROR" },
	{ PR_ATTR_READONLY, "PR_ATTR_READONLY" },
	{ PR_ATTR_READONLY_ERROR, "PR_ATTR_READONLY_ERROR" },
	{ PR_P1_CONTENT, "PR_P1_CONTENT" },
	{ PR_P1_CONTENT_ERROR, "PR_P1_CONTENT_ERROR" },
	{ PR_P1_CONTENT_TYPE, "PR_P1_CONTENT_TYPE" },
	{ PR_P1_CONTENT_TYPE_ERROR, "PR_P1_CONTENT_TYPE_ERROR" },
	{ PR_ROWID, "PR_ROWID" },
	{ PR_ROWID_ERROR, "PR_ROWID_ERROR" },
	{ PR_DISPLAY_NAME, "PR_DISPLAY_NAME" },
	{ PR_DISPLAY_NAME_UNICODE, "PR_DISPLAY_NAME_UNICODE" },
	{ PR_DISPLAY_NAME_ERROR, "PR_DISPLAY_NAME_ERROR" },
	{ PR_ADDRTYPE, "PR_ADDRTYPE" },
	{ PR_ADDRTYPE_UNICODE, "PR_ADDRTYPE_UNICODE" },
	{ PR_ADDRTYPE_ERROR, "PR_ADDRTYPE_ERROR" },
	{ PR_EMAIL_ADDRESS, "PR_EMAIL_ADDRESS" },
	{ PR_EMAIL_ADDRESS_UNICODE, "PR_EMAIL_ADDRESS_UNICODE" },
	{ PR_EMAIL_ADDRESS_ERROR, "PR_EMAIL_ADDRESS_ERROR" },
	{ PR_COMMENT, "PR_COMMENT" },
	{ PR_COMMENT_UNICODE, "PR_COMMENT_UNICODE" },
	{ PR_COMMENT_ERROR, "PR_COMMENT_ERROR" },
	{ PR_DEPTH, "PR_DEPTH" },
	{ PR_DEPTH_ERROR, "PR_DEPTH_ERROR" },
	{ PR_PROVIDER_DISPLAY, "PR_PROVIDER_DISPLAY" },
	{ PR_PROVIDER_DISPLAY_UNICODE, "PR_PROVIDER_DISPLAY_UNICODE" },
	{ PR_PROVIDER_DISPLAY_ERROR, "PR_PROVIDER_DISPLAY_ERROR" },
	{ PR_CREATION_TIME, "PR_CREATION_TIME" },
	{ PR_CREATION_TIME_ERROR, "PR_CREATION_TIME_ERROR" },
	{ PR_LAST_MODIFICATION_TIME, "PR_LAST_MODIFICATION_TIME" },
	{ PR_LAST_MODIFICATION_TIME_ERROR, "PR_LAST_MODIFICATION_TIME_ERROR" },
	{ PR_RESOURCE_FLAGS, "PR_RESOURCE_FLAGS" },
	{ PR_RESOURCE_FLAGS_ERROR, "PR_RESOURCE_FLAGS_ERROR" },
	{ PR_PROVIDER_DLL_NAME, "PR_PROVIDER_DLL_NAME" },
	{ PR_PROVIDER_DLL_NAME_UNICODE, "PR_PROVIDER_DLL_NAME_UNICODE" },
	{ PR_PROVIDER_DLL_NAME_ERROR, "PR_PROVIDER_DLL_NAME_ERROR" },
	{ PR_SEARCH_KEY, "PR_SEARCH_KEY" },
	{ PR_SEARCH_KEY_ERROR, "PR_SEARCH_KEY_ERROR" },
	{ PR_PROVIDER_UID, "PR_PROVIDER_UID" },
	{ PR_PROVIDER_UID_ERROR, "PR_PROVIDER_UID_ERROR" },
	{ PR_PROVIDER_ORDINAL, "PR_PROVIDER_ORDINAL" },
	{ PR_PROVIDER_ORDINAL_ERROR, "PR_PROVIDER_ORDINAL_ERROR" },
	{ PR_FORM_VERSION, "PR_FORM_VERSION" },
	{ PR_FORM_VERSION_UNICODE, "PR_FORM_VERSION_UNICODE" },
	{ PR_FORM_VERSION_ERROR, "PR_FORM_VERSION_ERROR" },
	{ PR_FORM_CLSID, "PR_FORM_CLSID" },
	{ PR_FORM_CLSID_ERROR, "PR_FORM_CLSID_ERROR" },
	{ PR_FORM_CONTACT_NAME, "PR_FORM_CONTACT_NAME" },
	{ PR_FORM_CONTACT_NAME_UNICODE, "PR_FORM_CONTACT_NAME_UNICODE" },
	{ PR_FORM_CONTACT_NAME_ERROR, "PR_FORM_CONTACT_NAME_ERROR" },
	{ PR_FORM_CATEGORY, "PR_FORM_CATEGORY" },
	{ PR_FORM_CATEGORY_UNICODE, "PR_FORM_CATEGORY_UNICODE" },
	{ PR_FORM_CATEGORY_ERROR, "PR_FORM_CATEGORY_ERROR" },
	{ PR_FORM_CATEGORY_SUB, "PR_FORM_CATEGORY_SUB" },
	{ PR_FORM_CATEGORY_SUB_UNICODE, "PR_FORM_CATEGORY_SUB_UNICODE" },
	{ PR_FORM_CATEGORY_SUB_ERROR, "PR_FORM_CATEGORY_SUB_ERROR" },
	{ PR_FORM_HOST_MAP, "PR_FORM_HOST_MAP" },
	{ PR_FORM_HOST_MAP_ERROR, "PR_FORM_HOST_MAP_ERROR" },
	{ PR_FORM_HIDDEN, "PR_FORM_HIDDEN" },
	{ PR_FORM_HIDDEN_ERROR, "PR_FORM_HIDDEN_ERROR" },
	{ PR_FORM_DESIGNER_NAME, "PR_FORM_DESIGNER_NAME" },
	{ PR_FORM_DESIGNER_NAME_UNICODE, "PR_FORM_DESIGNER_NAME_UNICODE" },
	{ PR_FORM_DESIGNER_NAME_ERROR, "PR_FORM_DESIGNER_NAME_ERROR" },
	{ PR_FORM_DESIGNER_GUID, "PR_FORM_DESIGNER_GUID" },
	{ PR_FORM_DESIGNER_GUID_ERROR, "PR_FORM_DESIGNER_GUID_ERROR" },
	{ PR_FORM_MESSAGE_BEHAVIOR, "PR_FORM_MESSAGE_BEHAVIOR" },
	{ PR_FORM_MESSAGE_BEHAVIOR_ERROR, "PR_FORM_MESSAGE_BEHAVIOR_ERROR" },
	{ PR_DEFAULT_STORE, "PR_DEFAULT_STORE" },
	{ PR_DEFAULT_STORE_ERROR, "PR_DEFAULT_STORE_ERROR" },
	{ PR_STORE_SUPPORT_MASK, "PR_STORE_SUPPORT_MASK" },
	{ PR_STORE_SUPPORT_MASK_ERROR, "PR_STORE_SUPPORT_MASK_ERROR" },
	{ PR_STORE_STATE, "PR_STORE_STATE" },
	{ PR_STORE_STATE_ERROR, "PR_STORE_STATE_ERROR" },
	{ PR_IPM_SUBTREE_SEARCH_KEY, "PR_IPM_SUBTREE_SEARCH_KEY" },
	{ PR_IPM_SUBTREE_SEARCH_KEY_ERROR, "PR_IPM_SUBTREE_SEARCH_KEY_ERROR" },
	{ PR_IPM_OUTBOX_SEARCH_KEY, "PR_IPM_OUTBOX_SEARCH_KEY" },
	{ PR_IPM_OUTBOX_SEARCH_KEY_ERROR, "PR_IPM_OUTBOX_SEARCH_KEY_ERROR" },
	{ PR_IPM_WASTEBASKET_SEARCH_KEY, "PR_IPM_WASTEBASKET_SEARCH_KEY" },
	{ PR_IPM_WASTEBASKET_SEARCH_KEY_ERROR, "PR_IPM_WASTEBASKET_SEARCH_KEY_ERROR" },
	{ PR_IPM_SENTMAIL_SEARCH_KEY, "PR_IPM_SENTMAIL_SEARCH_KEY" },
	{ PR_IPM_SENTMAIL_SEARCH_KEY_ERROR, "PR_IPM_SENTMAIL_SEARCH_KEY_ERROR" },
	{ PR_MDB_PROVIDER, "PR_MDB_PROVIDER" },
	{ PR_MDB_PROVIDER_ERROR, "PR_MDB_PROVIDER_ERROR" },
	{ PR_RECEIVE_FOLDER_SETTINGS, "PR_RECEIVE_FOLDER_SETTINGS" },
	{ PR_RECEIVE_FOLDER_SETTINGS_ERROR, "PR_RECEIVE_FOLDER_SETTINGS_ERROR" },
	{ PR_VALID_FOLDER_MASK, "PR_VALID_FOLDER_MASK" },
	{ PR_VALID_FOLDER_MASK_ERROR, "PR_VALID_FOLDER_MASK_ERROR" },
	{ PR_IPM_SUBTREE_ENTRYID, "PR_IPM_SUBTREE_ENTRYID" },
	{ PR_IPM_SUBTREE_ENTRYID_ERROR, "PR_IPM_SUBTREE_ENTRYID_ERROR" },
	{ PR_IPM_OUTBOX_ENTRYID, "PR_IPM_OUTBOX_ENTRYID" },
	{ PR_IPM_OUTBOX_ENTRYID_ERROR, "PR_IPM_OUTBOX_ENTRYID_ERROR" },
	{ PR_IPM_WASTEBASKET_ENTRYID, "PR_IPM_WASTEBASKET_ENTRYID" },
	{ PR_IPM_WASTEBASKET_ENTRYID_ERROR, "PR_IPM_WASTEBASKET_ENTRYID_ERROR" },
	{ PR_IPM_SENTMAIL_ENTRYID, "PR_IPM_SENTMAIL_ENTRYID" },
	{ PR_IPM_SENTMAIL_ENTRYID_ERROR, "PR_IPM_SENTMAIL_ENTRYID_ERROR" },
	{ PR_VIEWS_ENTRYID, "PR_VIEWS_ENTRYID" },
	{ PR_VIEWS_ENTRYID_ERROR, "PR_VIEWS_ENTRYID_ERROR" },
	{ PR_COMMON_VIEWS_ENTRYID, "PR_COMMON_VIEWS_ENTRYID" },
	{ PR_COMMON_VIEWS_ENTRYID_ERROR, "PR_COMMON_VIEWS_ENTRYID_ERROR" },
	{ PR_FINDER_ENTRYID, "PR_FINDER_ENTRYID" },
	{ PR_FINDER_ENTRYID_ERROR, "PR_FINDER_ENTRYID_ERROR" },
	{ PR_CONTAINER_FLAGS, "PR_CONTAINER_FLAGS" },
	{ PR_CONTAINER_FLAGS_ERROR, "PR_CONTAINER_FLAGS_ERROR" },
	{ PR_FOLDER_TYPE, "PR_FOLDER_TYPE" },
	{ PR_FOLDER_TYPE_ERROR, "PR_FOLDER_TYPE_ERROR" },
	{ PR_CONTENT_COUNT, "PR_CONTENT_COUNT" },
	{ PR_CONTENT_COUNT_ERROR, "PR_CONTENT_COUNT_ERROR" },
	{ PR_CONTENT_UNREAD, "PR_CONTENT_UNREAD" },
	{ PR_CONTENT_UNREAD_ERROR, "PR_CONTENT_UNREAD_ERROR" },
	{ PR_CREATE_TEMPLATES, "PR_CREATE_TEMPLATES" },
	{ PR_CREATE_TEMPLATES_ERROR, "PR_CREATE_TEMPLATES_ERROR" },
	{ PR_DETAILS_TABLE, "PR_DETAILS_TABLE" },
	{ PR_DETAILS_TABLE_ERROR, "PR_DETAILS_TABLE_ERROR" },
	{ PR_SEARCH, "PR_SEARCH" },
	{ PR_SEARCH_ERROR, "PR_SEARCH_ERROR" },
	{ PR_SELECTABLE, "PR_SELECTABLE" },
	{ PR_SELECTABLE_ERROR, "PR_SELECTABLE_ERROR" },
	{ PR_SUBFOLDERS, "PR_SUBFOLDERS" },
	{ PR_SUBFOLDERS_ERROR, "PR_SUBFOLDERS_ERROR" },
	{ PR_STATUS, "PR_STATUS" },
	{ PR_STATUS_ERROR, "PR_STATUS_ERROR" },
	{ PR_ANR, "PR_ANR" },
	{ PR_ANR_UNICODE, "PR_ANR_UNICODE" },
	{ PR_ANR_ERROR, "PR_ANR_ERROR" },
	{ PR_CONTENTS_SORT_ORDER, "PR_CONTENTS_SORT_ORDER" },
	{ PR_CONTENTS_SORT_ORDER_ERROR, "PR_CONTENTS_SORT_ORDER_ERROR" },
	{ PR_CONTAINER_HIERARCHY, "PR_CONTAINER_HIERARCHY" },
	{ PR_CONTAINER_HIERARCHY_ERROR, "PR_CONTAINER_HIERARCHY_ERROR" },
	{ PR_CONTAINER_CONTENTS, "PR_CONTAINER_CONTENTS" },
	{ PR_CONTAINER_CONTENTS_ERROR, "PR_CONTAINER_CONTENTS_ERROR" },
	{ PR_FOLDER_ASSOCIATED_CONTENTS, "PR_FOLDER_ASSOCIATED_CONTENTS" },
	{ PR_FOLDER_ASSOCIATED_CONTENTS_ERROR, "PR_FOLDER_ASSOCIATED_CONTENTS_ERROR" },
	{ PR_DEF_CREATE_DL, "PR_DEF_CREATE_DL" },
	{ PR_DEF_CREATE_DL_ERROR, "PR_DEF_CREATE_DL_ERROR" },
	{ PR_DEF_CREATE_MAILUSER, "PR_DEF_CREATE_MAILUSER" },
	{ PR_DEF_CREATE_MAILUSER_ERROR, "PR_DEF_CREATE_MAILUSER_ERROR" },
	{ PR_CONTAINER_CLASS, "PR_CONTAINER_CLASS" },
	{ PR_CONTAINER_CLASS_UNICODE, "PR_CONTAINER_CLASS_UNICODE" },
	{ PR_CONTAINER_CLASS_ERROR, "PR_CONTAINER_CLASS_ERROR" },
	{ PR_CONTAINER_MODIFY_VERSION, "PR_CONTAINER_MODIFY_VERSION" },
	{ PR_CONTAINER_MODIFY_VERSION_ERROR, "PR_CONTAINER_MODIFY_VERSION_ERROR" },
	{ PR_AB_PROVIDER_ID, "PR_AB_PROVIDER_ID" },
	{ PR_AB_PROVIDER_ID_ERROR, "PR_AB_PROVIDER_ID_ERROR" },
	{ PR_DEFAULT_VIEW_ENTRYID, "PR_DEFAULT_VIEW_ENTRYID" },
	{ PR_DEFAULT_VIEW_ENTRYID_ERROR, "PR_DEFAULT_VIEW_ENTRYID_ERROR" },
	{ PR_ASSOC_CONTENT_COUNT, "PR_ASSOC_CONTENT_COUNT" },
	{ PR_ASSOC_CONTENT_COUNT_ERROR, "PR_ASSOC_CONTENT_COUNT_ERROR" },
	{ PR_PACKED_NAME_PROPS, "PR_PACKED_NAME_PROPS" },
	{ PR_PACKED_NAME_PROPS_ERROR, "PR_PACKED_NAME_PROPS_ERROR" },
	{ PR_IPM_APPOINTMENT_ENTRYID, "PR_IPM_APPOINTMENT_ENTRYID" },
	{ PR_IPM_APPOINTMENT_ENTRYID_ERROR, "PR_IPM_APPOINTMENT_ENTRYID_ERROR" },
	{ PR_IPM_CONTACT_ENTRYID, "PR_IPM_CONTACT_ENTRYID" },
	{ PR_IPM_CONTACT_ENTRYID_ERROR, "PR_IPM_CONTACT_ENTRYID_ERROR" },
	{ PR_IPM_JOURNAL_ENTRYID, "PR_IPM_JOURNAL_ENTRYID" },
	{ PR_IPM_JOURNAL_ENTRYID_ERROR, "PR_IPM_JOURNAL_ENTRYID_ERROR" },
	{ PR_IPM_NOTE_ENTRYID, "PR_IPM_NOTE_ENTRYID" },
	{ PR_IPM_NOTE_ENTRYID_ERROR, "PR_IPM_NOTE_ENTRYID_ERROR" },
	{ PR_IPM_TASK_ENTRYID, "PR_IPM_TASK_ENTRYID" },
	{ PR_IPM_TASK_ENTRYID_ERROR, "PR_IPM_TASK_ENTRYID_ERROR" },
	{ PR_REMINDERS_ONLINE_ENTRYID, "PR_REMINDERS_ONLINE_ENTRYID" },
	{ PR_REMINDERS_ONLINE_ENTRYID_ERROR, "PR_REMINDERS_ONLINE_ENTRYID_ERROR" },
	{ PR_REMINDERS_OFFLINE_ENTRYID, "PR_REMINDERS_OFFLINE_ENTRYID" },
	{ PR_REMINDERS_OFFLINE_ENTRYID_ERROR, "PR_REMINDERS_OFFLINE_ENTRYID_ERROR" },
	{ PR_IPM_DRAFTS_ENTRYID, "PR_IPM_DRAFTS_ENTRYID" },
	{ PR_IPM_DRAFTS_ENTRYID_ERROR, "PR_IPM_DRAFTS_ENTRYID_ERROR" },
	{ PR_OUTLOOK_2003_ENTRYIDS, "PR_OUTLOOK_2003_ENTRYIDS" },
	{ PR_OUTLOOK_2003_ENTRYIDS_ERROR, "PR_OUTLOOK_2003_ENTRYIDS_ERROR" },
	{ PR_FOLDER_WEBVIEWINFO, "PR_FOLDER_WEBVIEWINFO" },
	{ PR_FOLDER_WEBVIEWINFO_ERROR, "PR_FOLDER_WEBVIEWINFO_ERROR" },
	{ PR_FOLDER_XVIEWINFO_E, "PR_FOLDER_XVIEWINFO_E" },
	{ PR_FOLDER_XVIEWINFO_E_ERROR, "PR_FOLDER_XVIEWINFO_E_ERROR" },
	{ PR_FOLDER_VIEWS_ONLY, "PR_FOLDER_VIEWS_ONLY" },
	{ PR_FOLDER_VIEWS_ONLY_ERROR, "PR_FOLDER_VIEWS_ONLY_ERROR" },
	{ PR_FREEBUSY_ENTRYIDS, "PR_FREEBUSY_ENTRYIDS" },
	{ PR_FREEBUSY_ENTRYIDS_ERROR, "PR_FREEBUSY_ENTRYIDS_ERROR" },
	{ PR_DEF_MSG_CLASS, "PR_DEF_MSG_CLASS" },
	{ PR_DEF_MSG_CLASS_UNICODE, "PR_DEF_MSG_CLASS_UNICODE" },
	{ PR_DEF_MSG_CLASS_ERROR, "PR_DEF_MSG_CLASS_ERROR" },
	{ PR_DEF_FORM_NAME, "PR_DEF_FORM_NAME" },
	{ PR_DEF_FORM_NAME_UNICODE, "PR_DEF_FORM_NAME_UNICODE" },
	{ PR_DEF_FORM_NAME_ERROR, "PR_DEF_FORM_NAME_ERROR" },
	{ PR_GENERATE_EXCHANGE_VIEWS, "PR_GENERATE_EXCHANGE_VIEWS" },
	{ PR_GENERATE_EXCHANGE_VIEWS_ERROR, "PR_GENERATE_EXCHANGE_VIEWS_ERROR" },
	{ PR_AGING_PERIOD, "PR_AGING_PERIOD" },
	{ PR_AGING_PERIOD_ERROR, "PR_AGING_PERIOD_ERROR" },
	{ PR_AGING_GRANULARITY, "PR_AGING_GRANULARITY" },
	{ PR_AGING_GRANULARITY_ERROR, "PR_AGING_GRANULARITY_ERROR" },
	{ PR_ATTACHMENT_X400_PARAMETERS, "PR_ATTACHMENT_X400_PARAMETERS" },
	{ PR_ATTACHMENT_X400_PARAMETERS_ERROR, "PR_ATTACHMENT_X400_PARAMETERS_ERROR" },
	{ PR_ATTACH_DATA_OBJ, "PR_ATTACH_DATA_OBJ" },
	{ PR_ATTACH_DATA_OBJ_ERROR, "PR_ATTACH_DATA_OBJ_ERROR" },
	{ PR_ATTACH_DATA_BIN, "PR_ATTACH_DATA_BIN" },
	{ PR_ATTACH_ENCODING, "PR_ATTACH_ENCODING" },
	{ PR_ATTACH_ENCODING_ERROR, "PR_ATTACH_ENCODING_ERROR" },
	{ PR_ATTACH_EXTENSION, "PR_ATTACH_EXTENSION" },
	{ PR_ATTACH_EXTENSION_UNICODE, "PR_ATTACH_EXTENSION_UNICODE" },
	{ PR_ATTACH_EXTENSION_ERROR, "PR_ATTACH_EXTENSION_ERROR" },
	{ PR_ATTACH_FILENAME, "PR_ATTACH_FILENAME" },
	{ PR_ATTACH_FILENAME_UNICODE, "PR_ATTACH_FILENAME_UNICODE" },
	{ PR_ATTACH_FILENAME_ERROR, "PR_ATTACH_FILENAME_ERROR" },
	{ PR_ATTACH_METHOD, "PR_ATTACH_METHOD" },
	{ PR_ATTACH_METHOD_ERROR, "PR_ATTACH_METHOD_ERROR" },
	{ PR_ATTACH_LONG_FILENAME, "PR_ATTACH_LONG_FILENAME" },
	{ PR_ATTACH_LONG_FILENAME_UNICODE, "PR_ATTACH_LONG_FILENAME_UNICODE" },
	{ PR_ATTACH_LONG_FILENAME_ERROR, "PR_ATTACH_LONG_FILENAME_ERROR" },
	{ PR_ATTACH_PATHNAME, "PR_ATTACH_PATHNAME" },
	{ PR_ATTACH_PATHNAME_UNICODE, "PR_ATTACH_PATHNAME_UNICODE" },
	{ PR_ATTACH_PATHNAME_ERROR, "PR_ATTACH_PATHNAME_ERROR" },
	{ PR_ATTACH_RENDERING, "PR_ATTACH_RENDERING" },
	{ PR_ATTACH_RENDERING_ERROR, "PR_ATTACH_RENDERING_ERROR" },
	{ PR_ATTACH_TAG, "PR_ATTACH_TAG" },
	{ PR_ATTACH_TAG_ERROR, "PR_ATTACH_TAG_ERROR" },
	{ PR_RENDERING_POSITION, "PR_RENDERING_POSITION" },
	{ PR_RENDERING_POSITION_ERROR, "PR_RENDERING_POSITION_ERROR" },
	{ PR_ATTACH_TRANSPORT_NAME, "PR_ATTACH_TRANSPORT_NAME" },
	{ PR_ATTACH_TRANSPORT_NAME_UNICODE, "PR_ATTACH_TRANSPORT_NAME_UNICODE" },
	{ PR_ATTACH_TRANSPORT_NAME_ERROR, "PR_ATTACH_TRANSPORT_NAME_ERROR" },
	{ PR_ATTACH_LONG_PATHNAME, "PR_ATTACH_LONG_PATHNAME" },
	{ PR_ATTACH_LONG_PATHNAME_UNICODE, "PR_ATTACH_LONG_PATHNAME_UNICODE" },
	{ PR_ATTACH_LONG_PATHNAME_ERROR, "PR_ATTACH_LONG_PATHNAME_ERROR" },
	{ PR_ATTACH_MIME_TAG, "PR_ATTACH_MIME_TAG" },
	{ PR_ATTACH_MIME_TAG_UNICODE, "PR_ATTACH_MIME_TAG_UNICODE" },
	{ PR_ATTACH_MIME_TAG_ERROR, "PR_ATTACH_MIME_TAG_ERROR" },
	{ PR_ATTACH_ADDITIONAL_INFO, "PR_ATTACH_ADDITIONAL_INFO" },
	{ PR_ATTACH_ADDITIONAL_INFO_ERROR, "PR_ATTACH_ADDITIONAL_INFO_ERROR" },
	{ PR_ATTACH_CONTENT_ID, "PR_ATTACH_CONTENT_ID" },
	{ PR_ATTACH_CONTENT_ID_UNICODE, "PR_ATTACH_CONTENT_ID_UNICODE" },
	{ PR_ATTACH_CONTENT_ID_ERROR, "PR_ATTACH_CONTENT_ID_ERROR" },
	{ PR_ATTACH_CONTENT_LOCATION, "PR_ATTACH_CONTENT_LOCATION" },
	{ PR_ATTACH_CONTENT_LOCATION_UNICODE, "PR_ATTACH_CONTENT_LOCATION_UNICODE" },
	{ PR_ATTACH_CONTENT_LOCATION_ERROR, "PR_ATTACH_CONTENT_LOCATION_ERROR" },
	{ PR_ATTACH_FLAGS, "PR_ATTACH_FLAGS" },
	{ PR_ATTACH_FLAGS_ERROR, "PR_ATTACH_FLAGS_ERROR" },
	{ PR_ATTACH_CONTENT_DISPOSITION, "PR_ATTACH_CONTENT_DISPOSITION" },
	{ PR_ATTACH_CONTENT_DISPOSITION_UNICODE, "PR_ATTACH_CONTENT_DISPOSITION_UNICODE" },
	{ PR_ATTACH_CONTENT_DISPOSITION_ERROR, "PR_ATTACH_CONTENT_DISPOSITION_ERROR" },
	{ PR_SYNCEVENT_SUPPRESS_GUID, "PR_SYNCEVENT_SUPPRESS_GUID" },
	{ PR_SYNCEVENT_SUPPRESS_GUID_ERROR, "PR_SYNCEVENT_SUPPRESS_GUID_ERROR" },
	{ PR_DISPLAY_TYPE, "PR_DISPLAY_TYPE" },
	{ PR_DISPLAY_TYPE_ERROR, "PR_DISPLAY_TYPE_ERROR" },
	{ PR_TEMPLATEID, "PR_TEMPLATEID" },
	{ PR_TEMPLATEID_ERROR, "PR_TEMPLATEID_ERROR" },
	{ PR_PRIMARY_CAPABILITY, "PR_PRIMARY_CAPABILITY" },
	{ PR_PRIMARY_CAPABILITY_ERROR, "PR_PRIMARY_CAPABILITY_ERROR" },
	{ PR_SMTP_ADDRESS, "PR_SMTP_ADDRESS" },
	{ PR_SMTP_ADDRESS_UNICODE, "PR_SMTP_ADDRESS_UNICODE" },
	{ PR_SMTP_ADDRESS_ERROR, "PR_SMTP_ADDRESS_ERROR" },
	{ PR_7BIT_DISPLAY_NAME, "PR_7BIT_DISPLAY_NAME" },
	{ PR_7BIT_DISPLAY_NAME_UNICODE, "PR_7BIT_DISPLAY_NAME_UNICODE" },
	{ PR_7BIT_DISPLAY_NAME_ERROR, "PR_7BIT_DISPLAY_NAME_ERROR" },
	{ PR_ACCOUNT, "PR_ACCOUNT" },
	{ PR_ACCOUNT_UNICODE, "PR_ACCOUNT_UNICODE" },
	{ PR_ACCOUNT_ERROR, "PR_ACCOUNT_ERROR" },
	{ PR_ALTERNATE_RECIPIENT, "PR_ALTERNATE_RECIPIENT" },
	{ PR_ALTERNATE_RECIPIENT_ERROR, "PR_ALTERNATE_RECIPIENT_ERROR" },
	{ PR_CALLBACK_TELEPHONE_NUMBER, "PR_CALLBACK_TELEPHONE_NUMBER" },
	{ PR_CALLBACK_TELEPHONE_NUMBER_UNICODE, "PR_CALLBACK_TELEPHONE_NUMBER_UNICODE" },
	{ PR_CALLBACK_TELEPHONE_NUMBER_ERROR, "PR_CALLBACK_TELEPHONE_NUMBER_ERROR" },
	{ PR_CONVERSION_PROHIBITED, "PR_CONVERSION_PROHIBITED" },
	{ PR_CONVERSION_PROHIBITED_ERROR, "PR_CONVERSION_PROHIBITED_ERROR" },
	{ PR_DISCLOSE_RECIPIENTS, "PR_DISCLOSE_RECIPIENTS" },
	{ PR_DISCLOSE_RECIPIENTS_ERROR, "PR_DISCLOSE_RECIPIENTS_ERROR" },
	{ PR_GENERATION, "PR_GENERATION" },
	{ PR_GENERATION_UNICODE, "PR_GENERATION_UNICODE" },
	{ PR_GENERATION_ERROR, "PR_GENERATION_ERROR" },
	{ PR_GIVEN_NAME, "PR_GIVEN_NAME" },
	{ PR_GIVEN_NAME_UNICODE, "PR_GIVEN_NAME_UNICODE" },
	{ PR_GIVEN_NAME_ERROR, "PR_GIVEN_NAME_ERROR" },
	{ PR_GOVERNMENT_ID_NUMBER, "PR_GOVERNMENT_ID_NUMBER" },
	{ PR_GOVERNMENT_ID_NUMBER_UNICODE, "PR_GOVERNMENT_ID_NUMBER_UNICODE" },
	{ PR_GOVERNMENT_ID_NUMBER_ERROR, "PR_GOVERNMENT_ID_NUMBER_ERROR" },
	{ PR_OFFICE_TELEPHONE_NUMBER, "PR_OFFICE_TELEPHONE_NUMBER" },
	{ PR_OFFICE_TELEPHONE_NUMBER_UNICODE, "PR_OFFICE_TELEPHONE_NUMBER_UNICODE" },
	{ PR_OFFICE_TELEPHONE_NUMBER_ERROR, "PR_OFFICE_TELEPHONE_NUMBER_ERROR" },
	{ PR_HOME_TELEPHONE_NUMBER, "PR_HOME_TELEPHONE_NUMBER" },
	{ PR_HOME_TELEPHONE_NUMBER_UNICODE, "PR_HOME_TELEPHONE_NUMBER_UNICODE" },
	{ PR_HOME_TELEPHONE_NUMBER_ERROR, "PR_HOME_TELEPHONE_NUMBER_ERROR" },
	{ PR_INITIALS, "PR_INITIALS" },
	{ PR_INITIALS_UNICODE, "PR_INITIALS_UNICODE" },
	{ PR_INITIALS_ERROR, "PR_INITIALS_ERROR" },
	{ PR_KEYWORD, "PR_KEYWORD" },
	{ PR_KEYWORD_UNICODE, "PR_KEYWORD_UNICODE" },
	{ PR_KEYWORD_ERROR, "PR_KEYWORD_ERROR" },
	{ PR_LANGUAGE, "PR_LANGUAGE" },
	{ PR_LANGUAGE_UNICODE, "PR_LANGUAGE_UNICODE" },
	{ PR_LANGUAGE_ERROR, "PR_LANGUAGE_ERROR" },
	{ PR_LOCATION, "PR_LOCATION" },
	{ PR_LOCATION_UNICODE, "PR_LOCATION_UNICODE" },
	{ PR_LOCATION_ERROR, "PR_LOCATION_ERROR" },
	{ PR_MAIL_PERMISSION, "PR_MAIL_PERMISSION" },
	{ PR_MAIL_PERMISSION_ERROR, "PR_MAIL_PERMISSION_ERROR" },
	{ PR_MHS_COMMON_NAME, "PR_MHS_COMMON_NAME" },
	{ PR_MHS_COMMON_NAME_UNICODE, "PR_MHS_COMMON_NAME_UNICODE" },
	{ PR_MHS_COMMON_NAME_ERROR, "PR_MHS_COMMON_NAME_ERROR" },
	{ PR_ORGANIZATIONAL_ID_NUMBER, "PR_ORGANIZATIONAL_ID_NUMBER" },
	{ PR_ORGANIZATIONAL_ID_NUMBER_UNICODE, "PR_ORGANIZATIONAL_ID_NUMBER_UNICODE" },
	{ PR_ORGANIZATIONAL_ID_NUMBER_ERROR, "PR_ORGANIZATIONAL_ID_NUMBER_ERROR" },
	{ PR_SURNAME, "PR_SURNAME" },
	{ PR_SURNAME_UNICODE, "PR_SURNAME_UNICODE" },
	{ PR_SURNAME_ERROR, "PR_SURNAME_ERROR" },
	{ PR_ORIGINAL_ENTRYID, "PR_ORIGINAL_ENTRYID" },
	{ PR_ORIGINAL_ENTRYID_ERROR, "PR_ORIGINAL_ENTRYID_ERROR" },
	{ PR_ORIGINAL_DISPLAY_NAME, "PR_ORIGINAL_DISPLAY_NAME" },
	{ PR_ORIGINAL_DISPLAY_NAME_UNICODE, "PR_ORIGINAL_DISPLAY_NAME_UNICODE" },
	{ PR_ORIGINAL_DISPLAY_NAME_ERROR, "PR_ORIGINAL_DISPLAY_NAME_ERROR" },
	{ PR_ORIGINAL_SEARCH_KEY, "PR_ORIGINAL_SEARCH_KEY" },
	{ PR_ORIGINAL_SEARCH_KEY_ERROR, "PR_ORIGINAL_SEARCH_KEY_ERROR" },
	{ PR_POSTAL_ADDRESS, "PR_POSTAL_ADDRESS" },
	{ PR_POSTAL_ADDRESS_UNICODE, "PR_POSTAL_ADDRESS_UNICODE" },
	{ PR_POSTAL_ADDRESS_ERROR, "PR_POSTAL_ADDRESS_ERROR" },
	{ PR_COMPANY_NAME, "PR_COMPANY_NAME" },
	{ PR_COMPANY_NAME_UNICODE, "PR_COMPANY_NAME_UNICODE" },
	{ PR_COMPANY_NAME_ERROR, "PR_COMPANY_NAME_ERROR" },
	{ PR_TITLE, "PR_TITLE" },
	{ PR_TITLE_UNICODE, "PR_TITLE_UNICODE" },
	{ PR_TITLE_ERROR, "PR_TITLE_ERROR" },
	{ PR_DEPARTMENT_NAME, "PR_DEPARTMENT_NAME" },
	{ PR_DEPARTMENT_NAME_UNICODE, "PR_DEPARTMENT_NAME_UNICODE" },
	{ PR_DEPARTMENT_NAME_ERROR, "PR_DEPARTMENT_NAME_ERROR" },
	{ PR_OFFICE_LOCATION, "PR_OFFICE_LOCATION" },
	{ PR_OFFICE_LOCATION_UNICODE, "PR_OFFICE_LOCATION_UNICODE" },
	{ PR_OFFICE_LOCATION_ERROR, "PR_OFFICE_LOCATION_ERROR" },
	{ PR_PRIMARY_TELEPHONE_NUMBER, "PR_PRIMARY_TELEPHONE_NUMBER" },
	{ PR_PRIMARY_TELEPHONE_NUMBER_UNICODE, "PR_PRIMARY_TELEPHONE_NUMBER_UNICODE" },
	{ PR_PRIMARY_TELEPHONE_NUMBER_ERROR, "PR_PRIMARY_TELEPHONE_NUMBER_ERROR" },
	{ PR_OFFICE2_TELEPHONE_NUMBER, "PR_OFFICE2_TELEPHONE_NUMBER" },
	{ PR_OFFICE2_TELEPHONE_NUMBER_UNICODE, "PR_OFFICE2_TELEPHONE_NUMBER_UNICODE" },
	{ PR_OFFICE2_TELEPHONE_NUMBER_ERROR, "PR_OFFICE2_TELEPHONE_NUMBER_ERROR" },
	{ PR_MOBILE_TELEPHONE_NUMBER, "PR_MOBILE_TELEPHONE_NUMBER" },
	{ PR_MOBILE_TELEPHONE_NUMBER_UNICODE, "PR_MOBILE_TELEPHONE_NUMBER_UNICODE" },
	{ PR_MOBILE_TELEPHONE_NUMBER_ERROR, "PR_MOBILE_TELEPHONE_NUMBER_ERROR" },
	{ PR_RADIO_TELEPHONE_NUMBER, "PR_RADIO_TELEPHONE_NUMBER" },
	{ PR_RADIO_TELEPHONE_NUMBER_UNICODE, "PR_RADIO_TELEPHONE_NUMBER_UNICODE" },
	{ PR_RADIO_TELEPHONE_NUMBER_ERROR, "PR_RADIO_TELEPHONE_NUMBER_ERROR" },
	{ PR_CAR_TELEPHONE_NUMBER, "PR_CAR_TELEPHONE_NUMBER" },
	{ PR_CAR_TELEPHONE_NUMBER_UNICODE, "PR_CAR_TELEPHONE_NUMBER_UNICODE" },
	{ PR_CAR_TELEPHONE_NUMBER_ERROR, "PR_CAR_TELEPHONE_NUMBER_ERROR" },
	{ PR_OTHER_TELEPHONE_NUMBER, "PR_OTHER_TELEPHONE_NUMBER" },
	{ PR_OTHER_TELEPHONE_NUMBER_UNICODE, "PR_OTHER_TELEPHONE_NUMBER_UNICODE" },
	{ PR_OTHER_TELEPHONE_NUMBER_ERROR, "PR_OTHER_TELEPHONE_NUMBER_ERROR" },
	{ PR_TRANSMITTABLE_DISPLAY_NAME, "PR_TRANSMITTABLE_DISPLAY_NAME" },
	{ PR_TRANSMITTABLE_DISPLAY_NAME_UNICODE, "PR_TRANSMITTABLE_DISPLAY_NAME_UNICODE" },
	{ PR_TRANSMITTABLE_DISPLAY_NAME_ERROR, "PR_TRANSMITTABLE_DISPLAY_NAME_ERROR" },
	{ PR_PAGER_TELEPHONE_NUMBER, "PR_PAGER_TELEPHONE_NUMBER" },
	{ PR_PAGER_TELEPHONE_NUMBER_UNICODE, "PR_PAGER_TELEPHONE_NUMBER_UNICODE" },
	{ PR_PAGER_TELEPHONE_NUMBER_ERROR, "PR_PAGER_TELEPHONE_NUMBER_ERROR" },
	{ PR_USER_CERTIFICATE, "PR_USER_CERTIFICATE" },
	{ PR_USER_CERTIFICATE_ERROR, "PR_USER_CERTIFICATE_ERROR" },
	{ PR_PRIMARY_FAX_NUMBER, "PR_PRIMARY_FAX_NUMBER" },
	{ PR_PRIMARY_FAX_NUMBER_UNICODE, "PR_PRIMARY_FAX_NUMBER_UNICODE" },
	{ PR_PRIMARY_FAX_NUMBER_ERROR, "PR_PRIMARY_FAX_NUMBER_ERROR" },
	{ PR_HOME_FAX_NUMBER, "PR_HOME_FAX_NUMBER" },
	{ PR_HOME_FAX_NUMBER_UNICODE, "PR_HOME_FAX_NUMBER_UNICODE" },
	{ PR_HOME_FAX_NUMBER_ERROR, "PR_HOME_FAX_NUMBER_ERROR" },
	{ PR_COUNTRY, "PR_COUNTRY" },
	{ PR_COUNTRY_UNICODE, "PR_COUNTRY_UNICODE" },
	{ PR_COUNTRY_ERROR, "PR_COUNTRY_ERROR" },
	{ PR_LOCALITY, "PR_LOCALITY" },
	{ PR_LOCALITY_UNICODE, "PR_LOCALITY_UNICODE" },
	{ PR_LOCALITY_ERROR, "PR_LOCALITY_ERROR" },
	{ PR_STATE_OR_PROVINCE, "PR_STATE_OR_PROVINCE" },
	{ PR_STATE_OR_PROVINCE_UNICODE, "PR_STATE_OR_PROVINCE_UNICODE" },
	{ PR_STATE_OR_PROVINCE_ERROR, "PR_STATE_OR_PROVINCE_ERROR" },
	{ PR_STREET_ADDRESS, "PR_STREET_ADDRESS" },
	{ PR_STREET_ADDRESS_UNICODE, "PR_STREET_ADDRESS_UNICODE" },
	{ PR_STREET_ADDRESS_ERROR, "PR_STREET_ADDRESS_ERROR" },
	{ PR_POSTAL_CODE, "PR_POSTAL_CODE" },
	{ PR_POSTAL_CODE_UNICODE, "PR_POSTAL_CODE_UNICODE" },
	{ PR_POSTAL_CODE_ERROR, "PR_POSTAL_CODE_ERROR" },
	{ PR_POST_OFFICE_BOX, "PR_POST_OFFICE_BOX" },
	{ PR_POST_OFFICE_BOX_UNICODE, "PR_POST_OFFICE_BOX_UNICODE" },
	{ PR_POST_OFFICE_BOX_ERROR, "PR_POST_OFFICE_BOX_ERROR" },
	{ PR_TELEX_NUMBER, "PR_TELEX_NUMBER" },
	{ PR_TELEX_NUMBER_UNICODE, "PR_TELEX_NUMBER_UNICODE" },
	{ PR_TELEX_NUMBER_ERROR, "PR_TELEX_NUMBER_ERROR" },
	{ PR_ISDN_NUMBER, "PR_ISDN_NUMBER" },
	{ PR_ISDN_NUMBER_UNICODE, "PR_ISDN_NUMBER_UNICODE" },
	{ PR_ISDN_NUMBER_ERROR, "PR_ISDN_NUMBER_ERROR" },
	{ PR_ASSISTANT_TELEPHONE_NUMBER, "PR_ASSISTANT_TELEPHONE_NUMBER" },
	{ PR_ASSISTANT_TELEPHONE_NUMBER_UNICODE, "PR_ASSISTANT_TELEPHONE_NUMBER_UNICODE" },
	{ PR_ASSISTANT_TELEPHONE_NUMBER_ERROR, "PR_ASSISTANT_TELEPHONE_NUMBER_ERROR" },
	{ PR_HOME2_TELEPHONE_NUMBER, "PR_HOME2_TELEPHONE_NUMBER" },
	{ PR_HOME2_TELEPHONE_NUMBER_UNICODE, "PR_HOME2_TELEPHONE_NUMBER_UNICODE" },
	{ PR_HOME2_TELEPHONE_NUMBER_ERROR, "PR_HOME2_TELEPHONE_NUMBER_ERROR" },
	{ PR_ASSISTANT, "PR_ASSISTANT" },
	{ PR_ASSISTANT_UNICODE, "PR_ASSISTANT_UNICODE" },
	{ PR_ASSISTANT_ERROR, "PR_ASSISTANT_ERROR" },
	{ PR_SEND_RICH_INFO, "PR_SEND_RICH_INFO" },
	{ PR_SEND_RICH_INFO_ERROR, "PR_SEND_RICH_INFO_ERROR" },
	{ PR_WEDDING_ANNIVERSARY, "PR_WEDDING_ANNIVERSARY" },
	{ PR_WEDDING_ANNIVERSARY_ERROR, "PR_WEDDING_ANNIVERSARY_ERROR" },
	{ PR_BIRTHDAY, "PR_BIRTHDAY" },
	{ PR_BIRTHDAY_ERROR, "PR_BIRTHDAY_ERROR" },
	{ PR_HOBBIES, "PR_HOBBIES" },
	{ PR_HOBBIES_UNICODE, "PR_HOBBIES_UNICODE" },
	{ PR_HOBBIES_ERROR, "PR_HOBBIES_ERROR" },
	{ PR_MIDDLE_NAME, "PR_MIDDLE_NAME" },
	{ PR_MIDDLE_NAME_UNICODE, "PR_MIDDLE_NAME_UNICODE" },
	{ PR_MIDDLE_NAME_ERROR, "PR_MIDDLE_NAME_ERROR" },
	{ PR_DISPLAY_NAME_PREFIX, "PR_DISPLAY_NAME_PREFIX" },
	{ PR_DISPLAY_NAME_PREFIX_UNICODE, "PR_DISPLAY_NAME_PREFIX_UNICODE" },
	{ PR_DISPLAY_NAME_PREFIX_ERROR, "PR_DISPLAY_NAME_PREFIX_ERROR" },
	{ PR_PROFESSION, "PR_PROFESSION" },
	{ PR_PROFESSION_UNICODE, "PR_PROFESSION_UNICODE" },
	{ PR_PROFESSION_ERROR, "PR_PROFESSION_ERROR" },
	{ PR_PREFERRED_BY_NAME, "PR_PREFERRED_BY_NAME" },
	{ PR_PREFERRED_BY_NAME_UNICODE, "PR_PREFERRED_BY_NAME_UNICODE" },
	{ PR_PREFERRED_BY_NAME_ERROR, "PR_PREFERRED_BY_NAME_ERROR" },
	{ PR_SPOUSE_NAME, "PR_SPOUSE_NAME" },
	{ PR_SPOUSE_NAME_UNICODE, "PR_SPOUSE_NAME_UNICODE" },
	{ PR_SPOUSE_NAME_ERROR, "PR_SPOUSE_NAME_ERROR" },
	{ PR_COMPUTER_NETWORK_NAME, "PR_COMPUTER_NETWORK_NAME" },
	{ PR_COMPUTER_NETWORK_NAME_UNICODE, "PR_COMPUTER_NETWORK_NAME_UNICODE" },
	{ PR_COMPUTER_NETWORK_NAME_ERROR, "PR_COMPUTER_NETWORK_NAME_ERROR" },
	{ PR_CUSTOMER_ID, "PR_CUSTOMER_ID" },
	{ PR_CUSTOMER_ID_UNICODE, "PR_CUSTOMER_ID_UNICODE" },
	{ PR_CUSTOMER_ID_ERROR, "PR_CUSTOMER_ID_ERROR" },
	{ PR_TTYTDD_PHONE_NUMBER, "PR_TTYTDD_PHONE_NUMBER" },
	{ PR_TTYTDD_PHONE_NUMBER_UNICODE, "PR_TTYTDD_PHONE_NUMBER_UNICODE" },
	{ PR_TTYTDD_PHONE_NUMBER_ERROR, "PR_TTYTDD_PHONE_NUMBER_ERROR" },
	{ PR_FTP_SITE, "PR_FTP_SITE" },
	{ PR_FTP_SITE_UNICODE, "PR_FTP_SITE_UNICODE" },
	{ PR_FTP_SITE_ERROR, "PR_FTP_SITE_ERROR" },
	{ PR_GENDER, "PR_GENDER" },
	{ PR_GENDER_ERROR, "PR_GENDER_ERROR" },
	{ PR_MANAGER_NAME, "PR_MANAGER_NAME" },
	{ PR_MANAGER_NAME_UNICODE, "PR_MANAGER_NAME_UNICODE" },
	{ PR_MANAGER_NAME_ERROR, "PR_MANAGER_NAME_ERROR" },
	{ PR_NICKNAME, "PR_NICKNAME" },
	{ PR_NICKNAME_UNICODE, "PR_NICKNAME_UNICODE" },
	{ PR_NICKNAME_ERROR, "PR_NICKNAME_ERROR" },
	{ PR_PERSONAL_HOME_PAGE, "PR_PERSONAL_HOME_PAGE" },
	{ PR_PERSONAL_HOME_PAGE_UNICODE, "PR_PERSONAL_HOME_PAGE_UNICODE" },
	{ PR_PERSONAL_HOME_PAGE_ERROR, "PR_PERSONAL_HOME_PAGE_ERROR" },
	{ PR_BUSINESS_HOME_PAGE, "PR_BUSINESS_HOME_PAGE" },
	{ PR_BUSINESS_HOME_PAGE_UNICODE, "PR_BUSINESS_HOME_PAGE_UNICODE" },
	{ PR_BUSINESS_HOME_PAGE_ERROR, "PR_BUSINESS_HOME_PAGE_ERROR" },
	{ PR_CONTACT_VERSION, "PR_CONTACT_VERSION" },
	{ PR_CONTACT_VERSION_ERROR, "PR_CONTACT_VERSION_ERROR" },
	{ PR_CONTACT_ENTRYIDS, "PR_CONTACT_ENTRYIDS" },
	{ PR_CONTACT_ENTRYIDS_ERROR, "PR_CONTACT_ENTRYIDS_ERROR" },
	{ PR_CONTACT_ADDRTYPES, "PR_CONTACT_ADDRTYPES" },
	{ PR_CONTACT_ADDRTYPES_UNICODE, "PR_CONTACT_ADDRTYPES_UNICODE" },
	{ PR_CONTACT_ADDRTYPES_ERROR, "PR_CONTACT_ADDRTYPES_ERROR" },
	{ PR_CONTACT_DEFAULT_ADDRESS_INDEX, "PR_CONTACT_DEFAULT_ADDRESS_INDEX" },
	{ PR_CONTACT_DEFAULT_ADDRESS_INDEX_ERROR, "PR_CONTACT_DEFAULT_ADDRESS_INDEX_ERROR" },
	{ PR_CONTACT_EMAIL_ADDRESSES, "PR_CONTACT_EMAIL_ADDRESSES" },
	{ PR_CONTACT_EMAIL_ADDRESSES_UNICODE, "PR_CONTACT_EMAIL_ADDRESSES_UNICODE" },
	{ PR_CONTACT_EMAIL_ADDRESSES_ERROR, "PR_CONTACT_EMAIL_ADDRESSES_ERROR" },
	{ PR_COMPANY_MAIN_PHONE_NUMBER, "PR_COMPANY_MAIN_PHONE_NUMBER" },
	{ PR_COMPANY_MAIN_PHONE_NUMBER_UNICODE, "PR_COMPANY_MAIN_PHONE_NUMBER_UNICODE" },
	{ PR_COMPANY_MAIN_PHONE_NUMBER_ERROR, "PR_COMPANY_MAIN_PHONE_NUMBER_ERROR" },
	{ PR_CHILDRENS_NAMES, "PR_CHILDRENS_NAMES" },
	{ PR_CHILDRENS_NAMES_UNICODE, "PR_CHILDRENS_NAMES_UNICODE" },
	{ PR_CHILDRENS_NAMES_ERROR, "PR_CHILDRENS_NAMES_ERROR" },
	{ PR_HOME_ADDRESS_CITY, "PR_HOME_ADDRESS_CITY" },
	{ PR_HOME_ADDRESS_CITY_UNICODE, "PR_HOME_ADDRESS_CITY_UNICODE" },
	{ PR_HOME_ADDRESS_CITY_ERROR, "PR_HOME_ADDRESS_CITY_ERROR" },
	{ PR_HOME_ADDRESS_COUNTRY, "PR_HOME_ADDRESS_COUNTRY" },
	{ PR_HOME_ADDRESS_COUNTRY_UNICODE, "PR_HOME_ADDRESS_COUNTRY_UNICODE" },
	{ PR_HOME_ADDRESS_COUNTRY_ERROR, "PR_HOME_ADDRESS_COUNTRY_ERROR" },
	{ PR_HOME_ADDRESS_POSTAL_CODE, "PR_HOME_ADDRESS_POSTAL_CODE" },
	{ PR_HOME_ADDRESS_POSTAL_CODE_UNICODE, "PR_HOME_ADDRESS_POSTAL_CODE_UNICODE" },
	{ PR_HOME_ADDRESS_POSTAL_CODE_ERROR, "PR_HOME_ADDRESS_POSTAL_CODE_ERROR" },
	{ PR_HOME_ADDRESS_STATE_OR_PROVINCE, "PR_HOME_ADDRESS_STATE_OR_PROVINCE" },
	{ PR_HOME_ADDRESS_STATE_OR_PROVINCE_UNICODE, "PR_HOME_ADDRESS_STATE_OR_PROVINCE_UNICODE" },
	{ PR_HOME_ADDRESS_STATE_OR_PROVINCE_ERROR, "PR_HOME_ADDRESS_STATE_OR_PROVINCE_ERROR" },
	{ PR_HOME_ADDRESS_STREET, "PR_HOME_ADDRESS_STREET" },
	{ PR_HOME_ADDRESS_STREET_UNICODE, "PR_HOME_ADDRESS_STREET_UNICODE" },
	{ PR_HOME_ADDRESS_STREET_ERROR, "PR_HOME_ADDRESS_STREET_ERROR" },
	{ PR_HOME_ADDRESS_POST_OFFICE_BOX, "PR_HOME_ADDRESS_POST_OFFICE_BOX" },
	{ PR_HOME_ADDRESS_POST_OFFICE_BOX_UNICODE, "PR_HOME_ADDRESS_POST_OFFICE_BOX_UNICODE" },
	{ PR_HOME_ADDRESS_POST_OFFICE_BOX_ERROR, "PR_HOME_ADDRESS_POST_OFFICE_BOX_ERROR" },
	{ PR_OTHER_ADDRESS_CITY, "PR_OTHER_ADDRESS_CITY" },
	{ PR_OTHER_ADDRESS_CITY_UNICODE, "PR_OTHER_ADDRESS_CITY_UNICODE" },
	{ PR_OTHER_ADDRESS_CITY_ERROR, "PR_OTHER_ADDRESS_CITY_ERROR" },
	{ PR_OTHER_ADDRESS_COUNTRY, "PR_OTHER_ADDRESS_COUNTRY" },
	{ PR_OTHER_ADDRESS_COUNTRY_UNICODE, "PR_OTHER_ADDRESS_COUNTRY_UNICODE" },
	{ PR_OTHER_ADDRESS_COUNTRY_ERROR, "PR_OTHER_ADDRESS_COUNTRY_ERROR" },
	{ PR_OTHER_ADDRESS_POSTAL_CODE, "PR_OTHER_ADDRESS_POSTAL_CODE" },
	{ PR_OTHER_ADDRESS_POSTAL_CODE_UNICODE, "PR_OTHER_ADDRESS_POSTAL_CODE_UNICODE" },
	{ PR_OTHER_ADDRESS_POSTAL_CODE_ERROR, "PR_OTHER_ADDRESS_POSTAL_CODE_ERROR" },
	{ PR_OTHER_ADDRESS_STATE_OR_PROVINCE, "PR_OTHER_ADDRESS_STATE_OR_PROVINCE" },
	{ PR_OTHER_ADDRESS_STATE_OR_PROVINCE_UNICODE, "PR_OTHER_ADDRESS_STATE_OR_PROVINCE_UNICODE" },
	{ PR_OTHER_ADDRESS_STATE_OR_PROVINCE_ERROR, "PR_OTHER_ADDRESS_STATE_OR_PROVINCE_ERROR" },
	{ PR_OTHER_ADDRESS_STREET, "PR_OTHER_ADDRESS_STREET" },
	{ PR_OTHER_ADDRESS_STREET_UNICODE, "PR_OTHER_ADDRESS_STREET_UNICODE" },
	{ PR_OTHER_ADDRESS_STREET_ERROR, "PR_OTHER_ADDRESS_STREET_ERROR" },
	{ PR_OTHER_ADDRESS_POST_OFFICE_BOX, "PR_OTHER_ADDRESS_POST_OFFICE_BOX" },
	{ PR_OTHER_ADDRESS_POST_OFFICE_BOX_UNICODE, "PR_OTHER_ADDRESS_POST_OFFICE_BOX_UNICODE" },
	{ PR_OTHER_ADDRESS_POST_OFFICE_BOX_ERROR, "PR_OTHER_ADDRESS_POST_OFFICE_BOX_ERROR" },
	{ PR_USER_X509_CERTIFICATE, "PR_USER_X509_CERTIFICATE" },
	{ PR_USER_X509_CERTIFICATE_ERROR, "PR_USER_X509_CERTIFICATE_ERROR" },
	{ PR_SEND_INTERNET_ENCODING, "PR_SEND_INTERNET_ENCODING" },
	{ PR_SEND_INTERNET_ENCODING_ERROR, "PR_SEND_INTERNET_ENCODING_ERROR" },
	{ PR_STORE_PROVIDERS, "PR_STORE_PROVIDERS" },
	{ PR_STORE_PROVIDERS_ERROR, "PR_STORE_PROVIDERS_ERROR" },
	{ PR_AB_PROVIDERS, "PR_AB_PROVIDERS" },
	{ PR_AB_PROVIDERS_ERROR, "PR_AB_PROVIDERS_ERROR" },
	{ PR_TRANSPORT_PROVIDERS, "PR_TRANSPORT_PROVIDERS" },
	{ PR_TRANSPORT_PROVIDERS_ERROR, "PR_TRANSPORT_PROVIDERS_ERROR" },
	{ PR_DEFAULT_PROFILE, "PR_DEFAULT_PROFILE" },
	{ PR_DEFAULT_PROFILE_ERROR, "PR_DEFAULT_PROFILE_ERROR" },
	{ PR_AB_SEARCH_PATH, "PR_AB_SEARCH_PATH" },
	{ PR_AB_SEARCH_PATH_ERROR, "PR_AB_SEARCH_PATH_ERROR" },
	{ PR_AB_DEFAULT_DIR, "PR_AB_DEFAULT_DIR" },
	{ PR_AB_DEFAULT_DIR_ERROR, "PR_AB_DEFAULT_DIR_ERROR" },
	{ PR_AB_DEFAULT_PAB, "PR_AB_DEFAULT_PAB" },
	{ PR_AB_DEFAULT_PAB_ERROR, "PR_AB_DEFAULT_PAB_ERROR" },
	{ PR_FILTERING_HOOKS, "PR_FILTERING_HOOKS" },
	{ PR_FILTERING_HOOKS_ERROR, "PR_FILTERING_HOOKS_ERROR" },
	{ PR_SERVICE_NAME, "PR_SERVICE_NAME" },
	{ PR_SERVICE_NAME_UNICODE, "PR_SERVICE_NAME_UNICODE" },
	{ PR_SERVICE_NAME_ERROR, "PR_SERVICE_NAME_ERROR" },
	{ PR_SERVICE_DLL_NAME, "PR_SERVICE_DLL_NAME" },
	{ PR_SERVICE_DLL_NAME_UNICODE, "PR_SERVICE_DLL_NAME_UNICODE" },
	{ PR_SERVICE_DLL_NAME_ERROR, "PR_SERVICE_DLL_NAME_ERROR" },
	{ PR_SERVICE_ENTRY_NAME, "PR_SERVICE_ENTRY_NAME" },
	{ PR_SERVICE_ENTRY_NAME_UNICODE, "PR_SERVICE_ENTRY_NAME_UNICODE" },
	{ PR_SERVICE_ENTRY_NAME_ERROR, "PR_SERVICE_ENTRY_NAME_ERROR" },
	{ PR_SERVICE_UID, "PR_SERVICE_UID" },
	{ PR_SERVICE_UID_ERROR, "PR_SERVICE_UID_ERROR" },
	{ PR_SERVICE_EXTRA_UIDS, "PR_SERVICE_EXTRA_UIDS" },
	{ PR_SERVICE_EXTRA_UIDS_ERROR, "PR_SERVICE_EXTRA_UIDS_ERROR" },
	{ PR_SERVICES, "PR_SERVICES" },
	{ PR_SERVICES_ERROR, "PR_SERVICES_ERROR" },
	{ PR_SERVICE_SUPPORT_FILES, "PR_SERVICE_SUPPORT_FILES" },
	{ PR_SERVICE_SUPPORT_FILES_UNICODE, "PR_SERVICE_SUPPORT_FILES_UNICODE" },
	{ PR_SERVICE_SUPPORT_FILES_ERROR, "PR_SERVICE_SUPPORT_FILES_ERROR" },
	{ PR_SERVICE_DELETE_FILES, "PR_SERVICE_DELETE_FILES" },
	{ PR_SERVICE_DELETE_FILES_UNICODE, "PR_SERVICE_DELETE_FILES_UNICODE" },
	{ PR_SERVICE_DELETE_FILES_ERROR, "PR_SERVICE_DELETE_FILES_ERROR" },
	{ PR_AB_SEARCH_PATH_UPDATE, "PR_AB_SEARCH_PATH_UPDATE" },
	{ PR_AB_SEARCH_PATH_UPDATE_ERROR, "PR_AB_SEARCH_PATH_UPDATE_ERROR" },
	{ PR_PROFILE_NAME, "PR_PROFILE_NAME" },
	{ PR_PROFILE_NAME_UNICODE, "PR_PROFILE_NAME_UNICODE" },
	{ PR_PROFILE_NAME_ERROR, "PR_PROFILE_NAME_ERROR" },
	{ PR_SERVICE_INSTALL_ID, "PR_SERVICE_INSTALL_ID" },
	{ PR_SERVICE_INSTALL_ID_UNICODE, "PR_SERVICE_INSTALL_ID_UNICODE" },
	{ PR_SERVICE_INSTALL_ID_ERROR, "PR_SERVICE_INSTALL_ID_ERROR" },
	{ PR_ADMIN_SECURITY_DESCRIPTOR, "PR_ADMIN_SECURITY_DESCRIPTOR" },
	{ PR_ADMIN_SECURITY_DESCRIPTOR_ERROR, "PR_ADMIN_SECURITY_DESCRIPTOR_ERROR" },
	{ PR_IDENTITY_DISPLAY, "PR_IDENTITY_DISPLAY" },
	{ PR_IDENTITY_DISPLAY_UNICODE, "PR_IDENTITY_DISPLAY_UNICODE" },
	{ PR_IDENTITY_DISPLAY_ERROR, "PR_IDENTITY_DISPLAY_ERROR" },
	{ PR_IDENTITY_ENTRYID, "PR_IDENTITY_ENTRYID" },
	{ PR_IDENTITY_ENTRYID_ERROR, "PR_IDENTITY_ENTRYID_ERROR" },
	{ PR_RESOURCE_METHODS, "PR_RESOURCE_METHODS" },
	{ PR_RESOURCE_METHODS_ERROR, "PR_RESOURCE_METHODS_ERROR" },
	{ PR_RESOURCE_TYPE, "PR_RESOURCE_TYPE" },
	{ PR_RESOURCE_TYPE_ERROR, "PR_RESOURCE_TYPE_ERROR" },
	{ PR_STATUS_CODE, "PR_STATUS_CODE" },
	{ PR_STATUS_CODE_ERROR, "PR_STATUS_CODE_ERROR" },
	{ PR_IDENTITY_SEARCH_KEY, "PR_IDENTITY_SEARCH_KEY" },
	{ PR_IDENTITY_SEARCH_KEY_ERROR, "PR_IDENTITY_SEARCH_KEY_ERROR" },
	{ PR_OWN_STORE_ENTRYID, "PR_OWN_STORE_ENTRYID" },
	{ PR_OWN_STORE_ENTRYID_ERROR, "PR_OWN_STORE_ENTRYID_ERROR" },
	{ PR_RESOURCE_PATH, "PR_RESOURCE_PATH" },
	{ PR_RESOURCE_PATH_UNICODE, "PR_RESOURCE_PATH_UNICODE" },
	{ PR_RESOURCE_PATH_ERROR, "PR_RESOURCE_PATH_ERROR" },
	{ PR_STATUS_STRING, "PR_STATUS_STRING" },
	{ PR_STATUS_STRING_UNICODE, "PR_STATUS_STRING_UNICODE" },
	{ PR_STATUS_STRING_ERROR, "PR_STATUS_STRING_ERROR" },
	{ PR_X400_DEFERRED_DELIVERY_CANCEL, "PR_X400_DEFERRED_DELIVERY_CANCEL" },
	{ PR_X400_DEFERRED_DELIVERY_CANCEL_ERROR, "PR_X400_DEFERRED_DELIVERY_CANCEL_ERROR" },
	{ PR_HEADER_FOLDER_ENTRYID, "PR_HEADER_FOLDER_ENTRYID" },
	{ PR_HEADER_FOLDER_ENTRYID_ERROR, "PR_HEADER_FOLDER_ENTRYID_ERROR" },
	{ PR_REMOTE_PROGRESS, "PR_REMOTE_PROGRESS" },
	{ PR_REMOTE_PROGRESS_ERROR, "PR_REMOTE_PROGRESS_ERROR" },
	{ PR_REMOTE_PROGRESS_TEXT, "PR_REMOTE_PROGRESS_TEXT" },
	{ PR_REMOTE_PROGRESS_TEXT_UNICODE, "PR_REMOTE_PROGRESS_TEXT_UNICODE" },
	{ PR_REMOTE_PROGRESS_TEXT_ERROR, "PR_REMOTE_PROGRESS_TEXT_ERROR" },
	{ PR_REMOTE_VALIDATE_OK, "PR_REMOTE_VALIDATE_OK" },
	{ PR_REMOTE_VALIDATE_OK_ERROR, "PR_REMOTE_VALIDATE_OK_ERROR" },
	{ PR_CONTROL_FLAGS, "PR_CONTROL_FLAGS" },
	{ PR_CONTROL_FLAGS_ERROR, "PR_CONTROL_FLAGS_ERROR" },
	{ PR_CONTROL_STRUCTURE, "PR_CONTROL_STRUCTURE" },
	{ PR_CONTROL_STRUCTURE_ERROR, "PR_CONTROL_STRUCTURE_ERROR" },
	{ PR_CONTROL_TYPE, "PR_CONTROL_TYPE" },
	{ PR_CONTROL_TYPE_ERROR, "PR_CONTROL_TYPE_ERROR" },
	{ PR_DELTAX, "PR_DELTAX" },
	{ PR_DELTAX_ERROR, "PR_DELTAX_ERROR" },
	{ PR_DELTAY, "PR_DELTAY" },
	{ PR_DELTAY_ERROR, "PR_DELTAY_ERROR" },
	{ PR_XPOS, "PR_XPOS" },
	{ PR_XPOS_ERROR, "PR_XPOS_ERROR" },
	{ PR_YPOS, "PR_YPOS" },
	{ PR_YPOS_ERROR, "PR_YPOS_ERROR" },
	{ PR_CONTROL_ID, "PR_CONTROL_ID" },
	{ PR_CONTROL_ID_ERROR, "PR_CONTROL_ID_ERROR" },
	{ PR_INITIAL_DETAILS_PANE, "PR_INITIAL_DETAILS_PANE" },
	{ PR_INITIAL_DETAILS_PANE_ERROR, "PR_INITIAL_DETAILS_PANE_ERROR" },
	{ PR_DID, "PR_DID" },
	{ PR_DID_ERROR, "PR_DID_ERROR" },
	{ PR_SEQID, "PR_SEQID" },
	{ PR_SEQID_ERROR, "PR_SEQID_ERROR" },
	{ PR_DRAFTID, "PR_DRAFTID" },
	{ PR_DRAFTID_ERROR, "PR_DRAFTID_ERROR" },
	{ PR_CHECK_IN_TIME, "PR_CHECK_IN_TIME" },
	{ PR_CHECK_IN_TIME_ERROR, "PR_CHECK_IN_TIME_ERROR" },
	{ PR_CHECK_IN_COMMENT, "PR_CHECK_IN_COMMENT" },
	{ PR_CHECK_IN_COMMENT_UNICODE, "PR_CHECK_IN_COMMENT_UNICODE" },
	{ PR_CHECK_IN_COMMENT_ERROR, "PR_CHECK_IN_COMMENT_ERROR" },
	{ PR_VERSION_OP_CODE, "PR_VERSION_OP_CODE" },
	{ PR_VERSION_OP_CODE_ERROR, "PR_VERSION_OP_CODE_ERROR" },
	{ PR_VERSION_OP_DATA, "PR_VERSION_OP_DATA" },
	{ PR_VERSION_OP_DATA_ERROR, "PR_VERSION_OP_DATA_ERROR" },
	{ PR_VERSION_SEQUENCE_NUMBER, "PR_VERSION_SEQUENCE_NUMBER" },
	{ PR_VERSION_SEQUENCE_NUMBER_ERROR, "PR_VERSION_SEQUENCE_NUMBER_ERROR" },
	{ PR_ATTACH_ID, "PR_ATTACH_ID" },
	{ PR_ATTACH_ID_ERROR, "PR_ATTACH_ID_ERROR" },
	{ PR_PKM_DOC_STATUS, "PR_PKM_DOC_STATUS" },
	{ PR_PKM_DOC_STATUS_UNICODE, "PR_PKM_DOC_STATUS_UNICODE" },
	{ PR_PKM_DOC_STATUS_ERROR, "PR_PKM_DOC_STATUS_ERROR" },
	{ PR_MV_PKM_OPERATION_REQ, "PR_MV_PKM_OPERATION_REQ" },
	{ PR_MV_PKM_OPERATION_REQ_UNICODE, "PR_MV_PKM_OPERATION_REQ_UNICODE" },
	{ PR_MV_PKM_OPERATION_REQ_ERROR, "PR_MV_PKM_OPERATION_REQ_ERROR" },
	{ PR_PKM_DOC_INTERNAL_STATE, "PR_PKM_DOC_INTERNAL_STATE" },
	{ PR_PKM_DOC_INTERNAL_STATE_UNICODE, "PR_PKM_DOC_INTERNAL_STATE_UNICODE" },
	{ PR_PKM_DOC_INTERNAL_STATE_ERROR, "PR_PKM_DOC_INTERNAL_STATE_ERROR" },
	{ PR_VERSIONING_FLAGS, "PR_VERSIONING_FLAGS" },
	{ PR_VERSIONING_FLAGS_ERROR, "PR_VERSIONING_FLAGS_ERROR" },
	{ PR_PKM_LAST_UNAPPROVED_VID, "PR_PKM_LAST_UNAPPROVED_VID" },
	{ PR_PKM_LAST_UNAPPROVED_VID_ERROR, "PR_PKM_LAST_UNAPPROVED_VID_ERROR" },
	{ PR_MV_PKM_VERSION_LABELS, "PR_MV_PKM_VERSION_LABELS" },
	{ PR_MV_PKM_VERSION_LABELS_UNICODE, "PR_MV_PKM_VERSION_LABELS_UNICODE" },
	{ PR_MV_PKM_VERSION_LABELS_ERROR, "PR_MV_PKM_VERSION_LABELS_ERROR" },
	{ PR_MV_PKM_VERSION_STATUS, "PR_MV_PKM_VERSION_STATUS" },
	{ PR_MV_PKM_VERSION_STATUS_UNICODE, "PR_MV_PKM_VERSION_STATUS_UNICODE" },
	{ PR_MV_PKM_VERSION_STATUS_ERROR, "PR_MV_PKM_VERSION_STATUS_ERROR" },
	{ PR_PKM_INTERNAL_DATA, "PR_PKM_INTERNAL_DATA" },
	{ PR_PKM_INTERNAL_DATA_ERROR, "PR_PKM_INTERNAL_DATA_ERROR" },
	{ PR_LAST_CONFLICT, "PR_LAST_CONFLICT" },
	{ PR_LAST_CONFLICT_ERROR, "PR_LAST_CONFLICT_ERROR" },
	{ PR_CONFLICT_MSG_KEY, "PR_CONFLICT_MSG_KEY" },
	{ PR_CONFLICT_MSG_KEY_ERROR, "PR_CONFLICT_MSG_KEY_ERROR" },
	{ PR_REPL_HEADER, "PR_REPL_HEADER" },
	{ PR_REPL_HEADER_ERROR, "PR_REPL_HEADER_ERROR" },
	{ PR_REPL_STATUS, "PR_REPL_STATUS" },
	{ PR_REPL_STATUS_ERROR, "PR_REPL_STATUS_ERROR" },
	{ PR_REPL_CHANGES, "PR_REPL_CHANGES" },
	{ PR_REPL_CHANGES_ERROR, "PR_REPL_CHANGES_ERROR" },
	{ PR_REPL_RGM, "PR_REPL_RGM" },
	{ PR_REPL_RGM_ERROR, "PR_REPL_RGM_ERROR" },
	{ PR_RMI, "PR_RMI" },
	{ PR_RMI_ERROR, "PR_RMI_ERROR" },
	{ PR_INTERNAL_POST_REPLY, "PR_INTERNAL_POST_REPLY" },
	{ PR_INTERNAL_POST_REPLY_ERROR, "PR_INTERNAL_POST_REPLY_ERROR" },
	{ PR_NTSD_MODIFICATION_TIME, "PR_NTSD_MODIFICATION_TIME" },
	{ PR_NTSD_MODIFICATION_TIME_ERROR, "PR_NTSD_MODIFICATION_TIME_ERROR" },
	{ PR_PREVIEW_UNREAD, "PR_PREVIEW_UNREAD" },
	{ PR_PREVIEW_UNREAD_UNICODE, "PR_PREVIEW_UNREAD_UNICODE" },
	{ PR_PREVIEW_UNREAD_ERROR, "PR_PREVIEW_UNREAD_ERROR" },
	{ PR_PREVIEW, "PR_PREVIEW" },
	{ PR_PREVIEW_UNICODE, "PR_PREVIEW_UNICODE" },
	{ PR_PREVIEW_ERROR, "PR_PREVIEW_ERROR" },
	{ PR_ABSTRACT, "PR_ABSTRACT" },
	{ PR_ABSTRACT_UNICODE, "PR_ABSTRACT_UNICODE" },
	{ PR_ABSTRACT_ERROR, "PR_ABSTRACT_ERROR" },
	{ PR_DL_REPORT_FLAGS, "PR_DL_REPORT_FLAGS" },
	{ PR_DL_REPORT_FLAGS_ERROR, "PR_DL_REPORT_FLAGS_ERROR" },
	{ PR_BILATERAL_INFO, "PR_BILATERAL_INFO" },
	{ PR_BILATERAL_INFO_ERROR, "PR_BILATERAL_INFO_ERROR" },
	{ PR_MSG_BODY_ID, "PR_MSG_BODY_ID" },
	{ PR_MSG_BODY_ID_ERROR, "PR_MSG_BODY_ID_ERROR" },
	{ PR_INTERNET_CPID, "PR_INTERNET_CPID" },
	{ PR_INTERNET_CPID_ERROR, "PR_INTERNET_CPID_ERROR" },
	{ PR_AUTO_RESPONSE_SUPPRESS, "PR_AUTO_RESPONSE_SUPPRESS" },
	{ PR_AUTO_RESPONSE_SUPPRESS_ERROR, "PR_AUTO_RESPONSE_SUPPRESS_ERROR" },
	{ PR_ACL_TABLE, "PR_ACL_TABLE" },
	{ PR_ACL_TABLE_ERROR, "PR_ACL_TABLE_ERROR" },
	{ PR_ACL_DATA, "PR_ACL_DATA" },
	{ PR_RULES_TABLE, "PR_RULES_TABLE" },
	{ PR_RULES_TABLE_ERROR, "PR_RULES_TABLE_ERROR" },
	{ PR_RULES_DATA, "PR_RULES_DATA" },
	{ PR_FOLDER_DESIGN_FLAGS, "PR_FOLDER_DESIGN_FLAGS" },
	{ PR_FOLDER_DESIGN_FLAGS_ERROR, "PR_FOLDER_DESIGN_FLAGS_ERROR" },
	{ PR_DELEGATED_BY_RULE, "PR_DELEGATED_BY_RULE" },
	{ PR_DELEGATED_BY_RULE_ERROR, "PR_DELEGATED_BY_RULE_ERROR" },
	{ PR_DESIGN_IN_PROGRESS, "PR_DESIGN_IN_PROGRESS" },
	{ PR_DESIGN_IN_PROGRESS_ERROR, "PR_DESIGN_IN_PROGRESS_ERROR" },
	{ PR_SECURE_ORIGINATION, "PR_SECURE_ORIGINATION" },
	{ PR_SECURE_ORIGINATION_ERROR, "PR_SECURE_ORIGINATION_ERROR" },
	{ PR_PUBLISH_IN_ADDRESS_BOOK, "PR_PUBLISH_IN_ADDRESS_BOOK" },
	{ PR_PUBLISH_IN_ADDRESS_BOOK_ERROR, "PR_PUBLISH_IN_ADDRESS_BOOK_ERROR" },
	{ PR_RESOLVE_METHOD, "PR_RESOLVE_METHOD" },
	{ PR_RESOLVE_METHOD_ERROR, "PR_RESOLVE_METHOD_ERROR" },
	{ PR_ADDRESS_BOOK_DISPLAY_NAME, "PR_ADDRESS_BOOK_DISPLAY_NAME" },
	{ PR_ADDRESS_BOOK_DISPLAY_NAME_UNICODE, "PR_ADDRESS_BOOK_DISPLAY_NAME_UNICODE" },
	{ PR_ADDRESS_BOOK_DISPLAY_NAME_ERROR, "PR_ADDRESS_BOOK_DISPLAY_NAME_ERROR" },
	{ PR_EFORMS_LOCALE_ID, "PR_EFORMS_LOCALE_ID" },
	{ PR_EFORMS_LOCALE_ID_ERROR, "PR_EFORMS_LOCALE_ID_ERROR" },
	{ PR_HAS_DAMS, "PR_HAS_DAMS" },
	{ PR_HAS_DAMS_ERROR, "PR_HAS_DAMS_ERROR" },
	{ PR_DEFERRED_SEND_NUMBER, "PR_DEFERRED_SEND_NUMBER" },
	{ PR_DEFERRED_SEND_NUMBER_ERROR, "PR_DEFERRED_SEND_NUMBER_ERROR" },
	{ PR_DEFERRED_SEND_UNITS, "PR_DEFERRED_SEND_UNITS" },
	{ PR_DEFERRED_SEND_UNITS_ERROR, "PR_DEFERRED_SEND_UNITS_ERROR" },
	{ PR_EXPIRY_NUMBER, "PR_EXPIRY_NUMBER" },
	{ PR_EXPIRY_NUMBER_ERROR, "PR_EXPIRY_NUMBER_ERROR" },
	{ PR_EXPIRY_UNITS, "PR_EXPIRY_UNITS" },
	{ PR_EXPIRY_UNITS_ERROR, "PR_EXPIRY_UNITS_ERROR" },
	{ PR_DEFERRED_SEND_TIME, "PR_DEFERRED_SEND_TIME" },
	{ PR_DEFERRED_SEND_TIME_ERROR, "PR_DEFERRED_SEND_TIME_ERROR" },
	{ PR_CONFLICT_ENTRYID, "PR_CONFLICT_ENTRYID" },
	{ PR_CONFLICT_ENTRYID_ERROR, "PR_CONFLICT_ENTRYID_ERROR" },
	{ PR_MESSAGE_LOCALE_ID, "PR_MESSAGE_LOCALE_ID" },
	{ PR_MESSAGE_LOCALE_ID_ERROR, "PR_MESSAGE_LOCALE_ID_ERROR" },
	{ PR_RULE_TRIGGER_HISTORY, "PR_RULE_TRIGGER_HISTORY" },
	{ PR_RULE_TRIGGER_HISTORY_ERROR, "PR_RULE_TRIGGER_HISTORY_ERROR" },
	{ PR_MOVE_TO_STORE_ENTRYID, "PR_MOVE_TO_STORE_ENTRYID" },
	{ PR_MOVE_TO_STORE_ENTRYID_ERROR, "PR_MOVE_TO_STORE_ENTRYID_ERROR" },
	{ PR_MOVE_TO_FOLDER_ENTRYID, "PR_MOVE_TO_FOLDER_ENTRYID" },
	{ PR_MOVE_TO_FOLDER_ENTRYID_ERROR, "PR_MOVE_TO_FOLDER_ENTRYID_ERROR" },
	{ PR_STORAGE_QUOTA_LIMIT, "PR_STORAGE_QUOTA_LIMIT" },
	{ PR_STORAGE_QUOTA_LIMIT_ERROR, "PR_STORAGE_QUOTA_LIMIT_ERROR" },
	{ PR_EXCESS_STORAGE_USED, "PR_EXCESS_STORAGE_USED" },
	{ PR_EXCESS_STORAGE_USED_ERROR, "PR_EXCESS_STORAGE_USED_ERROR" },
	{ PR_SVR_GENERATING_QUOTA_MSG, "PR_SVR_GENERATING_QUOTA_MSG" },
	{ PR_SVR_GENERATING_QUOTA_MSG_UNICODE, "PR_SVR_GENERATING_QUOTA_MSG_UNICODE" },
	{ PR_SVR_GENERATING_QUOTA_MSG_ERROR, "PR_SVR_GENERATING_QUOTA_MSG_ERROR" },
	{ PR_CREATOR_NAME, "PR_CREATOR_NAME" },
	{ PR_CREATOR_NAME_UNICODE, "PR_CREATOR_NAME_UNICODE" },
	{ PR_CREATOR_NAME_ERROR, "PR_CREATOR_NAME_ERROR" },
	{ PR_CREATOR_ENTRYID, "PR_CREATOR_ENTRYID" },
	{ PR_CREATOR_ENTRYID_ERROR, "PR_CREATOR_ENTRYID_ERROR" },
	{ PR_LAST_MODIFIER_NAME, "PR_LAST_MODIFIER_NAME" },
	{ PR_LAST_MODIFIER_NAME_UNICODE, "PR_LAST_MODIFIER_NAME_UNICODE" },
	{ PR_LAST_MODIFIER_NAME_ERROR, "PR_LAST_MODIFIER_NAME_ERROR" },
	{ PR_LAST_MODIFIER_ENTRYID, "PR_LAST_MODIFIER_ENTRYID" },
	{ PR_LAST_MODIFIER_ENTRYID_ERROR, "PR_LAST_MODIFIER_ENTRYID_ERROR" },
	{ PR_REPLY_RECIPIENT_SMTP_PROXIES, "PR_REPLY_RECIPIENT_SMTP_PROXIES" },
	{ PR_REPLY_RECIPIENT_SMTP_PROXIES_UNICODE, "PR_REPLY_RECIPIENT_SMTP_PROXIES_UNICODE" },
	{ PR_REPLY_RECIPIENT_SMTP_PROXIES_ERROR, "PR_REPLY_RECIPIENT_SMTP_PROXIES_ERROR" },
	{ PR_MESSAGE_CODEPAGE, "PR_MESSAGE_CODEPAGE" },
	{ PR_MESSAGE_CODEPAGE_ERROR, "PR_MESSAGE_CODEPAGE_ERROR" },
	{ PR_EXTENDED_ACL_DATA, "PR_EXTENDED_ACL_DATA" },
	{ PR_EXTENDED_ACL_DATA_ERROR, "PR_EXTENDED_ACL_DATA_ERROR" },
	{ PR_FROM_I_HAVE, "PR_FROM_I_HAVE" },
	{ PR_FROM_I_HAVE_ERROR, "PR_FROM_I_HAVE_ERROR" },
	{ PR_NEW_ATTACH, "PR_NEW_ATTACH" },
	{ PR_NEW_ATTACH_ERROR, "PR_NEW_ATTACH_ERROR" },
	{ PR_START_EMBED, "PR_START_EMBED" },
	{ PR_START_EMBED_ERROR, "PR_START_EMBED_ERROR" },
	{ PR_END_EMBED, "PR_END_EMBED" },
	{ PR_END_EMBED_ERROR, "PR_END_EMBED_ERROR" },
	{ PR_START_RECIP, "PR_START_RECIP" },
	{ PR_START_RECIP_ERROR, "PR_START_RECIP_ERROR" },
	{ PR_END_RECIP, "PR_END_RECIP" },
	{ PR_END_RECIP_ERROR, "PR_END_RECIP_ERROR" },
	{ PR_END_CC_RECIP, "PR_END_CC_RECIP" },
	{ PR_END_CC_RECIP_ERROR, "PR_END_CC_RECIP_ERROR" },
	{ PR_END_BCC_RECIP, "PR_END_BCC_RECIP" },
	{ PR_END_BCC_RECIP_ERROR, "PR_END_BCC_RECIP_ERROR" },
	{ PR_END_P1_RECIP, "PR_END_P1_RECIP" },
	{ PR_END_P1_RECIP_ERROR, "PR_END_P1_RECIP_ERROR" },
	{ PR_START_TOP_FLD, "PR_START_TOP_FLD" },
	{ PR_START_TOP_FLD_ERROR, "PR_START_TOP_FLD_ERROR" },
	{ PR_START_SUB_FLD, "PR_START_SUB_FLD" },
	{ PR_START_SUB_FLD_ERROR, "PR_START_SUB_FLD_ERROR" },
	{ PR_END_FOLDER, "PR_END_FOLDER" },
	{ PR_END_FOLDER_ERROR, "PR_END_FOLDER_ERROR" },
	{ PR_START_MESSAGE, "PR_START_MESSAGE" },
	{ PR_START_MESSAGE_ERROR, "PR_START_MESSAGE_ERROR" },
	{ PR_END_MESSAGE, "PR_END_MESSAGE" },
	{ PR_END_MESSAGE_ERROR, "PR_END_MESSAGE_ERROR" },
	{ PR_END_ATTACH, "PR_END_ATTACH" },
	{ PR_END_ATTACH_ERROR, "PR_END_ATTACH_ERROR" },
	{ PR_EC_WARNING, "PR_EC_WARNING" },
	{ PR_EC_WARNING_ERROR, "PR_EC_WARNING_ERROR" },
	{ PR_START_FAI_MSG, "PR_START_FAI_MSG" },
	{ PR_START_FAI_MSG_ERROR, "PR_START_FAI_MSG_ERROR" },
	{ PR_NEW_FX_FOLDER, "PR_NEW_FX_FOLDER" },
	{ PR_NEW_FX_FOLDER_ERROR, "PR_NEW_FX_FOLDER_ERROR" },
	{ PR_INCR_SYNC_CHG, "PR_INCR_SYNC_CHG" },
	{ PR_INCR_SYNC_CHG_ERROR, "PR_INCR_SYNC_CHG_ERROR" },
	{ PR_INCR_SYNC_DEL, "PR_INCR_SYNC_DEL" },
	{ PR_INCR_SYNC_DEL_ERROR, "PR_INCR_SYNC_DEL_ERROR" },
	{ PR_INCR_SYNC_END, "PR_INCR_SYNC_END" },
	{ PR_INCR_SYNC_END_ERROR, "PR_INCR_SYNC_END_ERROR" },
	{ PR_INCR_SYNC_MSG, "PR_INCR_SYNC_MSG" },
	{ PR_INCR_SYNC_MSG_ERROR, "PR_INCR_SYNC_MSG_ERROR" },
	{ PR_FX_DEL_PROP, "PR_FX_DEL_PROP" },
	{ PR_FX_DEL_PROP_ERROR, "PR_FX_DEL_PROP_ERROR" },
	{ PR_IDSET_GIVEN, "PR_IDSET_GIVEN" },
	{ PR_IDSET_GIVEN_ERROR, "PR_IDSET_GIVEN_ERROR" },
	{ PR_SENDER_FLAGS, "PR_SENDER_FLAGS" },
	{ PR_SENDER_FLAGS_ERROR, "PR_SENDER_FLAGS_ERROR" },
	{ PR_SENT_REPRESENTING_FLAGS, "PR_SENT_REPRESENTING_FLAGS" },
	{ PR_SENT_REPRESENTING_FLAGS_ERROR, "PR_SENT_REPRESENTING_FLAGS_ERROR" },
	{ PR_RCVD_BY_FLAGS, "PR_RCVD_BY_FLAGS" },
	{ PR_RCVD_BY_FLAGS_ERROR, "PR_RCVD_BY_FLAGS_ERROR" },
	{ PR_RCVD_REPRESENTING_FLAGS, "PR_RCVD_REPRESENTING_FLAGS" },
	{ PR_RCVD_REPRESENTING_FLAGS_ERROR, "PR_RCVD_REPRESENTING_FLAGS_ERROR" },
	{ PR_ORIGINAL_SENDER_FLAGS, "PR_ORIGINAL_SENDER_FLAGS" },
	{ PR_ORIGINAL_SENDER_FLAGS_ERROR, "PR_ORIGINAL_SENDER_FLAGS_ERROR" },
	{ PR_ORIGINAL_SENT_REPRESENTING_FLAGS, "PR_ORIGINAL_SENT_REPRESENTING_FLAGS" },
	{ PR_ORIGINAL_SENT_REPRESENTING_FLAGS_ERROR, "PR_ORIGINAL_SENT_REPRESENTING_FLAGS_ERROR" },
	{ PR_REPORT_FLAGS, "PR_REPORT_FLAGS" },
	{ PR_REPORT_FLAGS_ERROR, "PR_REPORT_FLAGS_ERROR" },
	{ PR_READ_RECEIPT_FLAGS, "PR_READ_RECEIPT_FLAGS" },
	{ PR_READ_RECEIPT_FLAGS_ERROR, "PR_READ_RECEIPT_FLAGS_ERROR" },
	{ PR_SOFT_DELETES, "PR_SOFT_DELETES" },
	{ PR_SOFT_DELETES_ERROR, "PR_SOFT_DELETES_ERROR" },
	{ PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT, "PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT" },
	{ PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT_ERROR, "PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT_ERROR" },
	{ PR_SENDER_SIMPLE_DISP_NAME, "PR_SENDER_SIMPLE_DISP_NAME" },
	{ PR_SENDER_SIMPLE_DISP_NAME_UNICODE, "PR_SENDER_SIMPLE_DISP_NAME_UNICODE" },
	{ PR_SENDER_SIMPLE_DISP_NAME_ERROR, "PR_SENDER_SIMPLE_DISP_NAME_ERROR" },
	{ PR_SENT_REPRESENTING_SIMPLE_DISP_NAME, "PR_SENT_REPRESENTING_SIMPLE_DISP_NAME" },
	{ PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_UNICODE, "PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_UNICODE" },
	{ PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_ERROR, "PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_ERROR" },
	{ PR_CREATOR_SIMPLE_DISP_NAME, "PR_CREATOR_SIMPLE_DISP_NAME" },
	{ PR_CREATOR_SIMPLE_DISP_NAME_UNICODE, "PR_CREATOR_SIMPLE_DISP_NAME_UNICODE" },
	{ PR_CREATOR_SIMPLE_DISP_NAME_ERROR, "PR_CREATOR_SIMPLE_DISP_NAME_ERROR" },
	{ PR_ORG_ADDR_TYPE, "PR_ORG_ADDR_TYPE" },
	{ PR_ORG_ADDR_TYPE_UNICODE, "PR_ORG_ADDR_TYPE_UNICODE" },
	{ PR_ORG_ADDR_TYPE_ERROR, "PR_ORG_ADDR_TYPE_ERROR" },
	{ PR_ORG_EMAIL_ADDR, "PR_ORG_EMAIL_ADDR" },
	{ PR_ORG_EMAIL_ADDR_UNICODE, "PR_ORG_EMAIL_ADDR_UNICODE" },
	{ PR_ORG_EMAIL_ADDR_ERROR, "PR_ORG_EMAIL_ADDR_ERROR" },
	{ PR_CREATOR_FLAGS, "PR_CREATOR_FLAGS" },
	{ PR_CREATOR_FLAGS_ERROR, "PR_CREATOR_FLAGS_ERROR" },
	{ PR_MODIFIER_FLAGS, "PR_MODIFIER_FLAGS" },
	{ PR_MODIFIER_FLAGS_ERROR, "PR_MODIFIER_FLAGS_ERROR" },
	{ PR_ORIGINATOR_FLAGS, "PR_ORIGINATOR_FLAGS" },
	{ PR_ORIGINATOR_FLAGS_ERROR, "PR_ORIGINATOR_FLAGS_ERROR" },
	{ PR_REPORT_DESTINATION_FLAGS, "PR_REPORT_DESTINATION_FLAGS" },
	{ PR_REPORT_DESTINATION_FLAGS_ERROR, "PR_REPORT_DESTINATION_FLAGS_ERROR" },
	{ PR_ORIGINAL_AUTHOR_FLAGS, "PR_ORIGINAL_AUTHOR_FLAGS" },
	{ PR_ORIGINAL_AUTHOR_FLAGS_ERROR, "PR_ORIGINAL_AUTHOR_FLAGS_ERROR" },
	{ PR_ORIGINATOR_SEARCH_KEY, "PR_ORIGINATOR_SEARCH_KEY" },
	{ PR_ORIGINATOR_SEARCH_KEY_ERROR, "PR_ORIGINATOR_SEARCH_KEY_ERROR" },
	{ PR_REPORT_DESTINATION_SEARCH_KEY, "PR_REPORT_DESTINATION_SEARCH_KEY" },
	{ PR_REPORT_DESTINATION_SEARCH_KEY_ERROR, "PR_REPORT_DESTINATION_SEARCH_KEY_ERROR" },
	{ PR_ER_FLAG, "PR_ER_FLAG" },
	{ PR_ER_FLAG_ERROR, "PR_ER_FLAG_ERROR" },
	{ PR_INTERNET_SUBJECT, "PR_INTERNET_SUBJECT" },
	{ PR_INTERNET_SUBJECT_ERROR, "PR_INTERNET_SUBJECT_ERROR" },
	{ PR_INTERNET_SENT_REPRESENTING_NAME, "PR_INTERNET_SENT_REPRESENTING_NAME" },
	{ PR_INTERNET_SENT_REPRESENTING_NAME_ERROR, "PR_INTERNET_SENT_REPRESENTING_NAME_ERROR" },
	{ PR_INET_MAIL_OVERRIDE_FORMAT, "PR_INET_MAIL_OVERRIDE_FORMAT" },
	{ PR_INET_MAIL_OVERRIDE_FORMAT_ERROR, "PR_INET_MAIL_OVERRIDE_FORMAT_ERROR" },
	{ PR_MSG_EDITOR_FORMAT, "PR_MSG_EDITOR_FORMAT" },
	{ PR_MSG_EDITOR_FORMAT_ERROR, "PR_MSG_EDITOR_FORMAT_ERROR" },
	{ PR_DOTSTUFF_STATE, "PR_DOTSTUFF_STATE" },
	{ PR_DOTSTUFF_STATE_ERROR, "PR_DOTSTUFF_STATE_ERROR" },
	{ PR_RULE_SERVER_RULE_ID, "PR_RULE_SERVER_RULE_ID" },
	{ PR_RULE_SERVER_RULE_ID_ERROR, "PR_RULE_SERVER_RULE_ID_ERROR" },
	{ PR_REPLY_TEMPLATE_ID, "PR_REPLY_TEMPLATE_ID" },
	{ PR_REPLY_TEMPLATE_ID_ERROR, "PR_REPLY_TEMPLATE_ID_ERROR" },
	{ PR_SOURCE_KEY, "PR_SOURCE_KEY" },
	{ PR_SOURCE_KEY_ERROR, "PR_SOURCE_KEY_ERROR" },
	{ PR_PARENT_SOURCE_KEY, "PR_PARENT_SOURCE_KEY" },
	{ PR_PARENT_SOURCE_KEY_ERROR, "PR_PARENT_SOURCE_KEY_ERROR" },
	{ PR_CHANGE_KEY, "PR_CHANGE_KEY" },
	{ PR_CHANGE_KEY_ERROR, "PR_CHANGE_KEY_ERROR" },
	{ PR_PREDECESSOR_CHANGE_LIST, "PR_PREDECESSOR_CHANGE_LIST" },
	{ PR_PREDECESSOR_CHANGE_LIST_ERROR, "PR_PREDECESSOR_CHANGE_LIST_ERROR" },
	{ PR_SYNCHRONIZE_FLAGS, "PR_SYNCHRONIZE_FLAGS" },
	{ PR_SYNCHRONIZE_FLAGS_ERROR, "PR_SYNCHRONIZE_FLAGS_ERROR" },
	{ PR_AUTO_ADD_NEW_SUBS, "PR_AUTO_ADD_NEW_SUBS" },
	{ PR_AUTO_ADD_NEW_SUBS_ERROR, "PR_AUTO_ADD_NEW_SUBS_ERROR" },
	{ PR_NEW_SUBS_GET_AUTO_ADD, "PR_NEW_SUBS_GET_AUTO_ADD" },
	{ PR_NEW_SUBS_GET_AUTO_ADD_ERROR, "PR_NEW_SUBS_GET_AUTO_ADD_ERROR" },
	{ PR_MESSAGE_SITE_NAME, "PR_MESSAGE_SITE_NAME" },
	{ PR_MESSAGE_SITE_NAME_UNICODE, "PR_MESSAGE_SITE_NAME_UNICODE" },
	{ PR_MESSAGE_SITE_NAME_ERROR, "PR_MESSAGE_SITE_NAME_ERROR" },
	{ PR_MESSAGE_PROCESSED, "PR_MESSAGE_PROCESSED" },
	{ PR_MESSAGE_PROCESSED_ERROR, "PR_MESSAGE_PROCESSED_ERROR" },
	{ PR_RULE_MSG_STATE, "PR_RULE_MSG_STATE" },
	{ PR_RULE_MSG_STATE_ERROR, "PR_RULE_MSG_STATE_ERROR" },
	{ PR_RULE_MSG_USER_FLAGS, "PR_RULE_MSG_USER_FLAGS" },
	{ PR_RULE_MSG_USER_FLAGS_ERROR, "PR_RULE_MSG_USER_FLAGS_ERROR" },
	{ PR_RULE_MSG_PROVIDER, "PR_RULE_MSG_PROVIDER" },
	{ PR_RULE_MSG_PROVIDER_UNICODE, "PR_RULE_MSG_PROVIDER_UNICODE" },
	{ PR_RULE_MSG_PROVIDER_ERROR, "PR_RULE_MSG_PROVIDER_ERROR" },
	{ PR_RULE_MSG_NAME, "PR_RULE_MSG_NAME" },
	{ PR_RULE_MSG_NAME_UNICODE, "PR_RULE_MSG_NAME_UNICODE" },
	{ PR_RULE_MSG_NAME_ERROR, "PR_RULE_MSG_NAME_ERROR" },
	{ PR_RULE_MSG_LEVEL, "PR_RULE_MSG_LEVEL" },
	{ PR_RULE_MSG_LEVEL_ERROR, "PR_RULE_MSG_LEVEL_ERROR" },
	{ PR_RULE_MSG_PROVIDER_DATA, "PR_RULE_MSG_PROVIDER_DATA" },
	{ PR_RULE_MSG_PROVIDER_DATA_ERROR, "PR_RULE_MSG_PROVIDER_DATA_ERROR" },
	{ PR_RULE_MSG_ACTIONS, "PR_RULE_MSG_ACTIONS" },
	{ PR_RULE_MSG_ACTIONS_ERROR, "PR_RULE_MSG_ACTIONS_ERROR" },
	{ PR_RULE_MSG_CONDITION, "PR_RULE_MSG_CONDITION" },
	{ PR_RULE_MSG_CONDITION_ERROR, "PR_RULE_MSG_CONDITION_ERROR" },
	{ PR_RULE_MSG_CONDITION_LCID, "PR_RULE_MSG_CONDITION_LCID" },
	{ PR_RULE_MSG_CONDITION_LCID_ERROR, "PR_RULE_MSG_CONDITION_LCID_ERROR" },
	{ PR_RULE_MSG_VERSION, "PR_RULE_MSG_VERSION" },
	{ PR_RULE_MSG_VERSION_ERROR, "PR_RULE_MSG_VERSION_ERROR" },
	{ PR_RULE_MSG_SEQUENCE, "PR_RULE_MSG_SEQUENCE" },
	{ PR_RULE_MSG_SEQUENCE_ERROR, "PR_RULE_MSG_SEQUENCE_ERROR" },
	{ PR_PREVENT_MSG_CREATE, "PR_PREVENT_MSG_CREATE" },
	{ PR_PREVENT_MSG_CREATE_ERROR, "PR_PREVENT_MSG_CREATE_ERROR" },
	{ PR_IMAP_INTERNAL_DATE, "PR_IMAP_INTERNAL_DATE" },
	{ PR_IMAP_INTERNAL_DATE_ERROR, "PR_IMAP_INTERNAL_DATE_ERROR" },
	{ PR_PROFILE_VERSION, "PR_PROFILE_VERSION" },
	{ PR_PROFILE_VERSION_ERROR, "PR_PROFILE_VERSION_ERROR" },
	{ PR_PROFILE_CONFIG_FLAGS, "PR_PROFILE_CONFIG_FLAGS" },
	{ PR_PROFILE_CONFIG_FLAGS_ERROR, "PR_PROFILE_CONFIG_FLAGS_ERROR" },
	{ PR_PROFILE_HOME_SERVER, "PR_PROFILE_HOME_SERVER" },
	{ PR_PROFILE_HOME_SERVER_UNICODE, "PR_PROFILE_HOME_SERVER_UNICODE" },
	{ PR_PROFILE_HOME_SERVER_ERROR, "PR_PROFILE_HOME_SERVER_ERROR" },
	{ PR_PROFILE_USER, "PR_PROFILE_USER" },
	{ PR_PROFILE_USER_UNICODE, "PR_PROFILE_USER_UNICODE" },
	{ PR_PROFILE_USER_ERROR, "PR_PROFILE_USER_ERROR" },
	{ PR_PROFILE_CONNECT_FLAGS, "PR_PROFILE_CONNECT_FLAGS" },
	{ PR_PROFILE_CONNECT_FLAGS_ERROR, "PR_PROFILE_CONNECT_FLAGS_ERROR" },
	{ PR_PROFILE_TRANSPORT_FLAGS, "PR_PROFILE_TRANSPORT_FLAGS" },
	{ PR_PROFILE_TRANSPORT_FLAGS_ERROR, "PR_PROFILE_TRANSPORT_FLAGS_ERROR" },
	{ PR_PROFILE_UI_STATE, "PR_PROFILE_UI_STATE" },
	{ PR_PROFILE_UI_STATE_ERROR, "PR_PROFILE_UI_STATE_ERROR" },
	{ PR_PROFILE_UNRESOLVED_NAME, "PR_PROFILE_UNRESOLVED_NAME" },
	{ PR_PROFILE_UNRESOLVED_NAME_UNICODE, "PR_PROFILE_UNRESOLVED_NAME_UNICODE" },
	{ PR_PROFILE_UNRESOLVED_NAME_ERROR, "PR_PROFILE_UNRESOLVED_NAME_ERROR" },
	{ PR_PROFILE_UNRESOLVED_SERVER, "PR_PROFILE_UNRESOLVED_SERVER" },
	{ PR_PROFILE_UNRESOLVED_SERVER_UNICODE, "PR_PROFILE_UNRESOLVED_SERVER_UNICODE" },
	{ PR_PROFILE_UNRESOLVED_SERVER_ERROR, "PR_PROFILE_UNRESOLVED_SERVER_ERROR" },
	{ PR_PROFILE_OPEN_FLAGS, "PR_PROFILE_OPEN_FLAGS" },
	{ PR_PROFILE_OPEN_FLAGS_ERROR, "PR_PROFILE_OPEN_FLAGS_ERROR" },
	{ PR_PROFILE_BINDING_ORDER, "PR_PROFILE_BINDING_ORDER" },
	{ PR_PROFILE_BINDING_ORDER_UNICODE, "PR_PROFILE_BINDING_ORDER_UNICODE" },
	{ PR_PROFILE_TYPE, "PR_PROFILE_TYPE" },
	{ PR_PROFILE_TYPE_ERROR, "PR_PROFILE_TYPE_ERROR" },
	{ PR_PROFILE_MAILBOX, "PR_PROFILE_MAILBOX" },
	{ PR_PROFILE_MAILBOX_UNICODE, "PR_PROFILE_MAILBOX_UNICODE" },
	{ PR_PROFILE_MAILBOX_ERROR, "PR_PROFILE_MAILBOX_ERROR" },
	{ PR_PROFILE_SERVER, "PR_PROFILE_SERVER" },
	{ PR_PROFILE_SERVER_UNICODE, "PR_PROFILE_SERVER_UNICODE" },
	{ PR_PROFILE_SERVER_ERROR, "PR_PROFILE_SERVER_ERROR" },
	{ PR_PROFILE_MAX_RESTRICT, "PR_PROFILE_MAX_RESTRICT" },
	{ PR_PROFILE_MAX_RESTRICT_ERROR, "PR_PROFILE_MAX_RESTRICT_ERROR" },
	{ PR_PROFILE_AB_FILES_PATH, "PR_PROFILE_AB_FILES_PATH" },
	{ PR_PROFILE_AB_FILES_PATH_UNICODE, "PR_PROFILE_AB_FILES_PATH_UNICODE" },
	{ PR_PROFILE_AB_FILES_PATH_ERROR, "PR_PROFILE_AB_FILES_PATH_ERROR" },
	{ PR_PROFILE_FAVFLD_DISPLAY_NAME, "PR_PROFILE_FAVFLD_DISPLAY_NAME" },
	{ PR_PROFILE_FAVFLD_DISPLAY_NAME_UNICODE, "PR_PROFILE_FAVFLD_DISPLAY_NAME_UNICODE" },
	{ PR_PROFILE_FAVFLD_DISPLAY_NAME_ERROR, "PR_PROFILE_FAVFLD_DISPLAY_NAME_ERROR" },
	{ PR_PROFILE_OFFLINE_STORE_PATH, "PR_PROFILE_OFFLINE_STORE_PATH" },
	{ PR_PROFILE_OFFLINE_STORE_PATH_UNICODE, "PR_PROFILE_OFFLINE_STORE_PATH_UNICODE" },
	{ PR_PROFILE_OFFLINE_STORE_PATH_ERROR, "PR_PROFILE_OFFLINE_STORE_PATH_ERROR" },
	{ PR_PROFILE_OFFLINE_INFO, "PR_PROFILE_OFFLINE_INFO" },
	{ PR_PROFILE_OFFLINE_INFO_ERROR, "PR_PROFILE_OFFLINE_INFO_ERROR" },
	{ PR_PROFILE_HOME_SERVER_DN, "PR_PROFILE_HOME_SERVER_DN" },
	{ PR_PROFILE_HOME_SERVER_DN_UNICODE, "PR_PROFILE_HOME_SERVER_DN_UNICODE" },
	{ PR_PROFILE_HOME_SERVER_DN_ERROR, "PR_PROFILE_HOME_SERVER_DN_ERROR" },
	{ PR_PROFILE_HOME_SERVER_ADDRS, "PR_PROFILE_HOME_SERVER_ADDRS" },
	{ PR_PROFILE_HOME_SERVER_ADDRS_UNICODE, "PR_PROFILE_HOME_SERVER_ADDRS_UNICODE" },
	{ PR_PROFILE_HOME_SERVER_ADDRS_ERROR, "PR_PROFILE_HOME_SERVER_ADDRS_ERROR" },
	{ PR_PROFILE_SERVER_DN, "PR_PROFILE_SERVER_DN" },
	{ PR_PROFILE_SERVER_DN_UNICODE, "PR_PROFILE_SERVER_DN_UNICODE" },
	{ PR_PROFILE_SERVER_DN_ERROR, "PR_PROFILE_SERVER_DN_ERROR" },
	{ PR_PROFILE_FAVFLD_COMMENT, "PR_PROFILE_FAVFLD_COMMENT" },
	{ PR_PROFILE_FAVFLD_COMMENT_UNICODE, "PR_PROFILE_FAVFLD_COMMENT_UNICODE" },
	{ PR_PROFILE_FAVFLD_COMMENT_ERROR, "PR_PROFILE_FAVFLD_COMMENT_ERROR" },
	{ PR_PROFILE_ALLPUB_DISPLAY_NAME, "PR_PROFILE_ALLPUB_DISPLAY_NAME" },
	{ PR_PROFILE_ALLPUB_DISPLAY_NAME_UNICODE, "PR_PROFILE_ALLPUB_DISPLAY_NAME_UNICODE" },
	{ PR_PROFILE_ALLPUB_DISPLAY_NAME_ERROR, "PR_PROFILE_ALLPUB_DISPLAY_NAME_ERROR" },
	{ PR_PROFILE_ALLPUB_COMMENT, "PR_PROFILE_ALLPUB_COMMENT" },
	{ PR_PROFILE_ALLPUB_COMMENT_UNICODE, "PR_PROFILE_ALLPUB_COMMENT_UNICODE" },
	{ PR_PROFILE_ALLPUB_COMMENT_ERROR, "PR_PROFILE_ALLPUB_COMMENT_ERROR" },
	{ PR_DISABLE_WINSOCK, "PR_DISABLE_WINSOCK" },
	{ PR_DISABLE_WINSOCK_ERROR, "PR_DISABLE_WINSOCK_ERROR" },
	{ PR_IN_TRANSIT, "PR_IN_TRANSIT" },
	{ PR_PROFILE_AUTH_PACKAGE, "PR_PROFILE_AUTH_PACKAGE" },
	{ PR_PROFILE_AUTH_PACKAGE_ERROR, "PR_PROFILE_AUTH_PACKAGE_ERROR" },
	{ PR_USER_ENTRYID, "PR_USER_ENTRYID" },
	{ PR_USER_NAME, "PR_USER_NAME" },
	{ PR_USER_NAME_UNICODE, "PR_USER_NAME_UNICODE" },
	{ PR_USER_NAME_ERROR, "PR_USER_NAME_ERROR" },
	{ PR_MAILBOX_OWNER_ENTRYID, "PR_MAILBOX_OWNER_ENTRYID" },
	{ PR_MAILBOX_OWNER_ENTRYID_ERROR, "PR_MAILBOX_OWNER_ENTRYID_ERROR" },
	{ PR_MAILBOX_OWNER_NAME, "PR_MAILBOX_OWNER_NAME" },
	{ PR_MAILBOX_OWNER_NAME_UNICODE, "PR_MAILBOX_OWNER_NAME_UNICODE" },
	{ PR_MAILBOX_OWNER_NAME_ERROR, "PR_MAILBOX_OWNER_NAME_ERROR" },
	{ PR_OOF_STATE, "PR_OOF_STATE" },
	{ PR_OOF_STATE_ERROR, "PR_OOF_STATE_ERROR" },
	{ PR_SCHEDULE_FOLDER_ENTRYID, "PR_SCHEDULE_FOLDER_ENTRYID" },
	{ PR_SCHEDULE_FOLDER_ENTRYID_ERROR, "PR_SCHEDULE_FOLDER_ENTRYID_ERROR" },
	{ PR_IPM_DAF_ENTRYID, "PR_IPM_DAF_ENTRYID" },
	{ PR_IPM_DAF_ENTRYID_ERROR, "PR_IPM_DAF_ENTRYID_ERROR" },
	{ PR_NON_IPM_SUBTREE_ENTRYID, "PR_NON_IPM_SUBTREE_ENTRYID" },
	{ PR_NON_IPM_SUBTREE_ENTRYID_ERROR, "PR_NON_IPM_SUBTREE_ENTRYID_ERROR" },
	{ PR_EFORMS_REGISTRY_ENTRYID, "PR_EFORMS_REGISTRY_ENTRYID" },
	{ PR_EFORMS_REGISTRY_ENTRYID_ERROR, "PR_EFORMS_REGISTRY_ENTRYID_ERROR" },
	{ PR_SPLUS_FREE_BUSY_ENTRYID, "PR_SPLUS_FREE_BUSY_ENTRYID" },
	{ PR_SPLUS_FREE_BUSY_ENTRYID_ERROR, "PR_SPLUS_FREE_BUSY_ENTRYID_ERROR" },
	{ PR_HIERARCHY_SERVER, "PR_HIERARCHY_SERVER" },
	{ PR_HIERARCHY_SERVER_UNICODE, "PR_HIERARCHY_SERVER_UNICODE" },
	{ PR_HIERARCHY_SERVER_ERROR, "PR_HIERARCHY_SERVER_ERROR" },
	{ PR_OFFLINE_ADDRBOOK_ENTRYID, "PR_OFFLINE_ADDRBOOK_ENTRYID" },
	{ PR_EFORMS_FOR_LOCALE_ENTRYID, "PR_EFORMS_FOR_LOCALE_ENTRYID" },
	{ PR_EFORMS_FOR_LOCALE_ENTRYID_ERROR, "PR_EFORMS_FOR_LOCALE_ENTRYID_ERROR" },
	{ PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID, "PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID" },
	{ PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID_ERROR, "PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID_ERROR" },
	{ PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID, "PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID" },
	{ PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID_ERROR, "PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID_ERROR" },
	{ PR_OFFLINE_MESSAGE_ENTRYID, "PR_OFFLINE_MESSAGE_ENTRYID" },
	{ PR_OFFLINE_MESSAGE_ENTRYID_ERROR, "PR_OFFLINE_MESSAGE_ENTRYID_ERROR" },
	{ PR_GW_MTSIN_ENTRYID, "PR_GW_MTSIN_ENTRYID" },
	{ PR_GW_MTSIN_ENTRYID_ERROR, "PR_GW_MTSIN_ENTRYID_ERROR" },
	{ PR_GW_MTSOUT_ENTRYID, "PR_GW_MTSOUT_ENTRYID" },
	{ PR_GW_MTSOUT_ENTRYID_ERROR, "PR_GW_MTSOUT_ENTRYID_ERROR" },
	{ PR_TRANSFER_ENABLED, "PR_TRANSFER_ENABLED" },
	{ PR_TRANSFER_ENABLED_ERROR, "PR_TRANSFER_ENABLED_ERROR" },
	{ PR_TEST_LINE_SPEED, "PR_TEST_LINE_SPEED" },
	{ PR_TEST_LINE_SPEED_ERROR, "PR_TEST_LINE_SPEED_ERROR" },
	{ PR_HIERARCHY_SYNCHRONIZER, "PR_HIERARCHY_SYNCHRONIZER" },
	{ PR_HIERARCHY_SYNCHRONIZER_ERROR, "PR_HIERARCHY_SYNCHRONIZER_ERROR" },
	{ PR_CONTENTS_SYNCHRONIZER, "PR_CONTENTS_SYNCHRONIZER" },
	{ PR_CONTENTS_SYNCHRONIZER_ERROR, "PR_CONTENTS_SYNCHRONIZER_ERROR" },
	{ PR_COLLECTOR, "PR_COLLECTOR" },
	{ PR_COLLECTOR_ERROR, "PR_COLLECTOR_ERROR" },
	{ PR_FAST_TRANSFER, "PR_FAST_TRANSFER" },
	{ PR_FAST_TRANSFER_ERROR, "PR_FAST_TRANSFER_ERROR" },
	{ PR_IPM_FAVORITES_ENTRYID, "PR_IPM_FAVORITES_ENTRYID" },
	{ PR_IPM_FAVORITES_ENTRYID_ERROR, "PR_IPM_FAVORITES_ENTRYID_ERROR" },
	{ PR_IPM_PUBLIC_FOLDERS_ENTRYID, "PR_IPM_PUBLIC_FOLDERS_ENTRYID" },
	{ PR_IPM_PUBLIC_FOLDERS_ENTRYID_ERROR, "PR_IPM_PUBLIC_FOLDERS_ENTRYID_ERROR" },
	{ PR_STORE_OFFLINE, "PR_STORE_OFFLINE" },
	{ PR_STORE_OFFLINE_ERROR, "PR_STORE_OFFLINE_ERROR" },
	{ PR_CHANGE_ADVISOR, "PR_CHANGE_ADVISOR" },
	{ PR_CHANGE_ADVISOR_ERROR, "PR_CHANGE_ADVISOR_ERROR" },
	{ PR_FAVORITES_DEFAULT_NAME, "PR_FAVORITES_DEFAULT_NAME" },
	{ PR_FAVORITES_DEFAULT_NAME_UNICODE, "PR_FAVORITES_DEFAULT_NAME_UNICODE" },
	{ PR_FAVORITES_DEFAULT_NAME_ERROR, "PR_FAVORITES_DEFAULT_NAME_ERROR" },
	{ PR_SYS_CONFIG_FOLDER_ENTRYID, "PR_SYS_CONFIG_FOLDER_ENTRYID" },
	{ PR_SYS_CONFIG_FOLDER_ENTRYID_ERROR, "PR_SYS_CONFIG_FOLDER_ENTRYID_ERROR" },
	{ PR_CHANGE_NOTIFICATION_GUID, "PR_CHANGE_NOTIFICATION_GUID" },
	{ PR_CHANGE_NOTIFICATION_GUID_ERROR, "PR_CHANGE_NOTIFICATION_GUID_ERROR" },
	{ PR_FOLDER_CHILD_COUNT, "PR_FOLDER_CHILD_COUNT" },
	{ PR_FOLDER_CHILD_COUNT_ERROR, "PR_FOLDER_CHILD_COUNT_ERROR" },
	{ PR_RIGHTS, "PR_RIGHTS" },
	{ PR_RIGHTS_ERROR, "PR_RIGHTS_ERROR" },
	{ PR_HAS_RULES, "PR_HAS_RULES" },
	{ PR_HAS_RULES_ERROR, "PR_HAS_RULES_ERROR" },
	{ PR_ADDRESS_BOOK_ENTRYID, "PR_ADDRESS_BOOK_ENTRYID" },
	{ PR_ADDRESS_BOOK_ENTRYID_ERROR, "PR_ADDRESS_BOOK_ENTRYID_ERROR" },
	{ PR_PUBLIC_FOLDER_ENTRYID, "PR_PUBLIC_FOLDER_ENTRYID" },
	{ PR_PUBLIC_FOLDER_ENTRYID_ERROR, "PR_PUBLIC_FOLDER_ENTRYID_ERROR" },
	{ PR_OFFLINE_FLAGS, "PR_OFFLINE_FLAGS" },
	{ PR_OFFLINE_FLAGS_ERROR, "PR_OFFLINE_FLAGS_ERROR" },
	{ PR_HIERARCHY_CHANGE_NUM, "PR_HIERARCHY_CHANGE_NUM" },
	{ PR_HIERARCHY_CHANGE_NUM_ERROR, "PR_HIERARCHY_CHANGE_NUM_ERROR" },
	{ PR_HAS_MODERATOR_RULES, "PR_HAS_MODERATOR_RULES" },
	{ PR_HAS_MODERATOR_RULES_ERROR, "PR_HAS_MODERATOR_RULES_ERROR" },
	{ PR_DELETED_MSG_COUNT, "PR_DELETED_MSG_COUNT" },
	{ PR_DELETED_MSG_COUNT_ERROR, "PR_DELETED_MSG_COUNT_ERROR" },
	{ PR_DELETED_FOLDER_COUNT, "PR_DELETED_FOLDER_COUNT" },
	{ PR_DELETED_FOLDER_COUNT_ERROR, "PR_DELETED_FOLDER_COUNT_ERROR" },
	{ PR_OLDEST_DELETED_ON, "PR_OLDEST_DELETED_ON" },
	{ PR_OLDEST_DELETED_ON_ERROR, "PR_OLDEST_DELETED_ON_ERROR" },
	{ PR_DELETED_ASSOC_MSG_COUNT, "PR_DELETED_ASSOC_MSG_COUNT" },
	{ PR_DELETED_ASSOC_MSG_COUNT_ERROR, "PR_DELETED_ASSOC_MSG_COUNT_ERROR" },
	{ PR_REPLICA_SERVER, "PR_REPLICA_SERVER" },
	{ PR_REPLICA_SERVER_UNICODE, "PR_REPLICA_SERVER_UNICODE" },
	{ PR_REPLICA_SERVER_ERROR, "PR_REPLICA_SERVER_ERROR" },
	{ PR_CLIENT_ACTIONS, "PR_CLIENT_ACTIONS" },
	{ PR_CLIENT_ACTIONS_ERROR, "PR_CLIENT_ACTIONS_ERROR" },
	{ PR_DAM_ORIGINAL_ENTRYID, "PR_DAM_ORIGINAL_ENTRYID" },
	{ PR_DAM_ORIGINAL_ENTRYID_ERROR, "PR_DAM_ORIGINAL_ENTRYID_ERROR" },
	{ PR_DAM_BACK_PATCHED, "PR_DAM_BACK_PATCHED" },
	{ PR_DAM_BACK_PATCHED_ERROR, "PR_DAM_BACK_PATCHED_ERROR" },
	{ PR_RULE_ERROR, "PR_RULE_ERROR" },
	{ PR_RULE_ERROR_ERROR, "PR_RULE_ERROR_ERROR" },
	{ PR_RULE_ACTION_TYPE, "PR_RULE_ACTION_TYPE" },
	{ PR_RULE_ACTION_TYPE_ERROR, "PR_RULE_ACTION_TYPE_ERROR" },
	{ PR_HAS_NAMED_PROPERTIES, "PR_HAS_NAMED_PROPERTIES" },
	{ PR_HAS_NAMED_PROPERTIES_ERROR, "PR_HAS_NAMED_PROPERTIES_ERROR" },
	{ PR_REPLICA_VERSION, "PR_REPLICA_VERSION" },
	{ PR_REPLICA_VERSION_ERROR, "PR_REPLICA_VERSION_ERROR" },
	{ PR_RULE_ACTION_NUMBER, "PR_RULE_ACTION_NUMBER" },
	{ PR_RULE_ACTION_NUMBER_ERROR, "PR_RULE_ACTION_NUMBER_ERROR" },
	{ PR_RULE_FOLDER_ENTRYID, "PR_RULE_FOLDER_ENTRYID" },
	{ PR_RULE_FOLDER_ENTRYID_ERROR, "PR_RULE_FOLDER_ENTRYID_ERROR" },
	{ PR_ACTIVE_USER_ENTRYID, "PR_ACTIVE_USER_ENTRYID" },
	{ PR_ACTIVE_USER_ENTRYID_ERROR, "PR_ACTIVE_USER_ENTRYID_ERROR" },
	{ PR_0X400_ENVELOPE_TYPE, "PR_0X400_ENVELOPE_TYPE" },
	{ PR_0X400_ENVELOPE_TYPE_ERROR, "PR_0X400_ENVELOPE_TYPE_ERROR" },
	{ PR_MSG_FOLD_TIME, "PR_MSG_FOLD_TIME" },
	{ PR_MSG_FOLD_TIME_ERROR, "PR_MSG_FOLD_TIME_ERROR" },
	{ PR_ICS_CHANGE_KEY, "PR_ICS_CHANGE_KEY" },
	{ PR_ICS_CHANGE_KEY_ERROR, "PR_ICS_CHANGE_KEY_ERROR" },
	{ PR_GW_ADMIN_OPERATIONS, "PR_GW_ADMIN_OPERATIONS" },
	{ PR_GW_ADMIN_OPERATIONS_ERROR, "PR_GW_ADMIN_OPERATIONS_ERROR" },
	{ PR_INTERNET_CONTENT, "PR_INTERNET_CONTENT" },
	{ PR_INTERNET_CONTENT_ERROR, "PR_INTERNET_CONTENT_ERROR" },
	{ PR_HAS_ATTACH_FROM_IMAIL, "PR_HAS_ATTACH_FROM_IMAIL" },
	{ PR_HAS_ATTACH_FROM_IMAIL_ERROR, "PR_HAS_ATTACH_FROM_IMAIL_ERROR" },
	{ PR_ORIGINATOR_NAME, "PR_ORIGINATOR_NAME" },
	{ PR_ORIGINATOR_NAME_UNICODE, "PR_ORIGINATOR_NAME_UNICODE" },
	{ PR_ORIGINATOR_NAME_ERROR, "PR_ORIGINATOR_NAME_ERROR" },
	{ PR_ORIGINATOR_ADDR, "PR_ORIGINATOR_ADDR" },
	{ PR_ORIGINATOR_ADDR_UNICODE, "PR_ORIGINATOR_ADDR_UNICODE" },
	{ PR_ORIGINATOR_ADDR_ERROR, "PR_ORIGINATOR_ADDR_ERROR" },
	{ PR_ORIGINATOR_ADDRTYPE, "PR_ORIGINATOR_ADDRTYPE" },
	{ PR_ORIGINATOR_ADDRTYPE_UNICODE, "PR_ORIGINATOR_ADDRTYPE_UNICODE" },
	{ PR_ORIGINATOR_ADDRTYPE_ERROR, "PR_ORIGINATOR_ADDRTYPE_ERROR" },
	{ PR_ORIGINATOR_ENTRYID, "PR_ORIGINATOR_ENTRYID" },
	{ PR_ORIGINATOR_ENTRYID_ERROR, "PR_ORIGINATOR_ENTRYID_ERROR" },
	{ PR_ARRIVAL_TIME, "PR_ARRIVAL_TIME" },
	{ PR_ARRIVAL_TIME_ERROR, "PR_ARRIVAL_TIME_ERROR" },
	{ PR_TRACE_INFO, "PR_TRACE_INFO" },
	{ PR_TRACE_INFO_ERROR, "PR_TRACE_INFO_ERROR" },
	{ PR_SUBJECT_TRACE_INFO, "PR_SUBJECT_TRACE_INFO" },
	{ PR_SUBJECT_TRACE_INFO_ERROR, "PR_SUBJECT_TRACE_INFO_ERROR" },
	{ PR_RECIPIENT_NUMBER, "PR_RECIPIENT_NUMBER" },
	{ PR_RECIPIENT_NUMBER_ERROR, "PR_RECIPIENT_NUMBER_ERROR" },
	{ PR_MTS_SUBJECT_ID, "PR_MTS_SUBJECT_ID" },
	{ PR_MTS_SUBJECT_ID_ERROR, "PR_MTS_SUBJECT_ID_ERROR" },
	{ PR_REPORT_DESTINATION_NAME, "PR_REPORT_DESTINATION_NAME" },
	{ PR_REPORT_DESTINATION_NAME_UNICODE, "PR_REPORT_DESTINATION_NAME_UNICODE" },
	{ PR_REPORT_DESTINATION_NAME_ERROR, "PR_REPORT_DESTINATION_NAME_ERROR" },
	{ PR_REPORT_DESTINATION_ENTRYID, "PR_REPORT_DESTINATION_ENTRYID" },
	{ PR_REPORT_DESTINATION_ENTRYID_ERROR, "PR_REPORT_DESTINATION_ENTRYID_ERROR" },
	{ PR_CONTENT_SEARCH_KEY, "PR_CONTENT_SEARCH_KEY" },
	{ PR_CONTENT_SEARCH_KEY_ERROR, "PR_CONTENT_SEARCH_KEY_ERROR" },
	{ PR_FOREIGN_ID, "PR_FOREIGN_ID" },
	{ PR_FOREIGN_ID_ERROR, "PR_FOREIGN_ID_ERROR" },
	{ PR_FOREIGN_REPORT_ID, "PR_FOREIGN_REPORT_ID" },
	{ PR_FOREIGN_REPORT_ID_ERROR, "PR_FOREIGN_REPORT_ID_ERROR" },
	{ PR_FOREIGN_SUBJECT_ID, "PR_FOREIGN_SUBJECT_ID" },
	{ PR_FOREIGN_SUBJECT_ID_ERROR, "PR_FOREIGN_SUBJECT_ID_ERROR" },
	{ PR_INTERNAL_TRACE_INFO, "PR_INTERNAL_TRACE_INFO" },
	{ PR_INTERNAL_TRACE_INFO_ERROR, "PR_INTERNAL_TRACE_INFO_ERROR" },
	{ PR_IN_CONFLICT, "PR_IN_CONFLICT" },
	{ PR_IN_CONFLICT_ERROR, "PR_IN_CONFLICT_ERROR" },
	{ PR_LONGTERM_ENTRYID_FROM_TABLE, "PR_LONGTERM_ENTRYID_FROM_TABLE" },
	{ PR_LONGTERM_ENTRYID_FROM_TABLE_ERROR, "PR_LONGTERM_ENTRYID_FROM_TABLE_ERROR" },
	{ PR_MEMBER_ID, "PR_MEMBER_ID" },
	{ PR_MEMBER_ID_ERROR, "PR_MEMBER_ID_ERROR" },
	{ PR_MEMBER_NAME, "PR_MEMBER_NAME" },
	{ PR_MEMBER_NAME_UNICODE, "PR_MEMBER_NAME_UNICODE" },
	{ PR_MEMBER_NAME_ERROR, "PR_MEMBER_NAME_ERROR" },
	{ PR_MEMBER_RIGHTS, "PR_MEMBER_RIGHTS" },
	{ PR_MEMBER_RIGHTS_ERROR, "PR_MEMBER_RIGHTS_ERROR" },
	{ PR_RULE_ID, "PR_RULE_ID" },
	{ PR_RULE_ID_ERROR, "PR_RULE_ID_ERROR" },
	{ PR_RULE_IDS, "PR_RULE_IDS" },
	{ PR_RULE_IDS_ERROR, "PR_RULE_IDS_ERROR" },
	{ PR_RULE_SEQUENCE, "PR_RULE_SEQUENCE" },
	{ PR_RULE_SEQUENCE_ERROR, "PR_RULE_SEQUENCE_ERROR" },
	{ PR_RULE_STATE, "PR_RULE_STATE" },
	{ PR_RULE_STATE_ERROR, "PR_RULE_STATE_ERROR" },
	{ PR_RULE_USER_FLAGS, "PR_RULE_USER_FLAGS" },
	{ PR_RULE_USER_FLAGS_ERROR, "PR_RULE_USER_FLAGS_ERROR" },
	{ PR_RULE_CONDITION, "PR_RULE_CONDITION" },
	{ PR_RULE_CONDITION_ERROR, "PR_RULE_CONDITION_ERROR" },
	{ PR_PROFILE_MOAB, "PR_PROFILE_MOAB" },
	{ PR_PROFILE_MOAB_UNICODE, "PR_PROFILE_MOAB_UNICODE" },
	{ PR_PROFILE_MOAB_ERROR, "PR_PROFILE_MOAB_ERROR" },
	{ PR_PROFILE_MOAB_GUID, "PR_PROFILE_MOAB_GUID" },
	{ PR_PROFILE_MOAB_GUID_UNICODE, "PR_PROFILE_MOAB_GUID_UNICODE" },
	{ PR_PROFILE_MOAB_GUID_ERROR, "PR_PROFILE_MOAB_GUID_ERROR" },
	{ PR_PROFILE_MOAB_SEQ, "PR_PROFILE_MOAB_SEQ" },
	{ PR_PROFILE_MOAB_SEQ_ERROR, "PR_PROFILE_MOAB_SEQ_ERROR" },
	{ PR_IMPLIED_RESTRICTIONS, "PR_IMPLIED_RESTRICTIONS" },
	{ PR_IMPLIED_RESTRICTIONS_ERROR, "PR_IMPLIED_RESTRICTIONS_ERROR" },
	{ PR_RULE_ACTIONS, "PR_RULE_ACTIONS" },
	{ PR_RULE_ACTIONS_ERROR, "PR_RULE_ACTIONS_ERROR" },
	{ PR_RULE_PROVIDER, "PR_RULE_PROVIDER" },
	{ PR_RULE_PROVIDER_UNICODE, "PR_RULE_PROVIDER_UNICODE" },
	{ PR_RULE_PROVIDER_ERROR, "PR_RULE_PROVIDER_ERROR" },
	{ PR_RULE_NAME, "PR_RULE_NAME" },
	{ PR_RULE_NAME_UNICODE, "PR_RULE_NAME_UNICODE" },
	{ PR_RULE_NAME_ERROR, "PR_RULE_NAME_ERROR" },
	{ PR_RULE_LEVEL, "PR_RULE_LEVEL" },
	{ PR_RULE_LEVEL_ERROR, "PR_RULE_LEVEL_ERROR" },
	{ PR_RULE_PROVIDER_DATA, "PR_RULE_PROVIDER_DATA" },
	{ PR_RULE_PROVIDER_DATA_ERROR, "PR_RULE_PROVIDER_DATA_ERROR" },
	{ PR_LAST_FULL_BACKUP, "PR_LAST_FULL_BACKUP" },
	{ PR_LAST_FULL_BACKUP_ERROR, "PR_LAST_FULL_BACKUP_ERROR" },
	{ PR_PROFILE_ADDR_INFO, "PR_PROFILE_ADDR_INFO" },
	{ PR_PROFILE_ADDR_INFO_ERROR, "PR_PROFILE_ADDR_INFO_ERROR" },
	{ PR_PROFILE_OPTIONS_DATA, "PR_PROFILE_OPTIONS_DATA" },
	{ PR_PROFILE_OPTIONS_DATA_ERROR, "PR_PROFILE_OPTIONS_DATA_ERROR" },
	{ PR_EVENTS_ROOT_FOLDER_ENTRYID, "PR_EVENTS_ROOT_FOLDER_ENTRYID" },
	{ PR_EVENTS_ROOT_FOLDER_ENTRYID_ERROR, "PR_EVENTS_ROOT_FOLDER_ENTRYID_ERROR" },
	{ PR_INBOUND_NEWSFEED_DN, "PR_INBOUND_NEWSFEED_DN" },
	{ PR_INBOUND_NEWSFEED_DN_UNICODE, "PR_INBOUND_NEWSFEED_DN_UNICODE" },
	{ PR_INBOUND_NEWSFEED_DN_ERROR, "PR_INBOUND_NEWSFEED_DN_ERROR" },
	{ PR_OUTBOUND_NEWSFEED_DN, "PR_OUTBOUND_NEWSFEED_DN" },
	{ PR_OUTBOUND_NEWSFEED_DN_UNICODE, "PR_OUTBOUND_NEWSFEED_DN_UNICODE" },
	{ PR_OUTBOUND_NEWSFEED_DN_ERROR, "PR_OUTBOUND_NEWSFEED_DN_ERROR" },
	{ PR_DELETED_ON, "PR_DELETED_ON" },
	{ PR_DELETED_ON_ERROR, "PR_DELETED_ON_ERROR" },
	{ PR_REPLICATION_STYLE, "PR_REPLICATION_STYLE" },
	{ PR_REPLICATION_STYLE_ERROR, "PR_REPLICATION_STYLE_ERROR" },
	{ PR_REPLICATION_SCHEDULE, "PR_REPLICATION_SCHEDULE" },
	{ PR_REPLICATION_SCHEDULE_ERROR, "PR_REPLICATION_SCHEDULE_ERROR" },
	{ PR_REPLICATION_MESSAGE_PRIORITY, "PR_REPLICATION_MESSAGE_PRIORITY" },
	{ PR_REPLICATION_MESSAGE_PRIORITY_ERROR, "PR_REPLICATION_MESSAGE_PRIORITY_ERROR" },
	{ PR_OVERALL_MSG_AGE_LIMIT, "PR_OVERALL_MSG_AGE_LIMIT" },
	{ PR_OVERALL_MSG_AGE_LIMIT_ERROR, "PR_OVERALL_MSG_AGE_LIMIT_ERROR" },
	{ PR_REPLICATION_ALWAYS_INTERVAL, "PR_REPLICATION_ALWAYS_INTERVAL" },
	{ PR_REPLICATION_ALWAYS_INTERVAL_ERROR, "PR_REPLICATION_ALWAYS_INTERVAL_ERROR" },
	{ PR_REPLICATION_MSG_SIZE, "PR_REPLICATION_MSG_SIZE" },
	{ PR_REPLICATION_MSG_SIZE_ERROR, "PR_REPLICATION_MSG_SIZE_ERROR" },
	{ PR_IS_NEWSGROUP_ANCHOR, "PR_IS_NEWSGROUP_ANCHOR" },
	{ PR_IS_NEWSGROUP_ANCHOR_ERROR, "PR_IS_NEWSGROUP_ANCHOR_ERROR" },
	{ PR_IS_NEWSGROUP, "PR_IS_NEWSGROUP" },
	{ PR_IS_NEWSGROUP_ERROR, "PR_IS_NEWSGROUP_ERROR" },
	{ PR_REPLICA_LIST, "PR_REPLICA_LIST" },
	{ PR_REPLICA_LIST_ERROR, "PR_REPLICA_LIST_ERROR" },
	{ PR_OVERALL_AGE_LIMIT, "PR_OVERALL_AGE_LIMIT" },
	{ PR_OVERALL_AGE_LIMIT_ERROR, "PR_OVERALL_AGE_LIMIT_ERROR" },
	{ PR_INTERNET_CHARSET, "PR_INTERNET_CHARSET" },
	{ PR_INTERNET_CHARSET_UNICODE, "PR_INTERNET_CHARSET_UNICODE" },
	{ PR_INTERNET_CHARSET_ERROR, "PR_INTERNET_CHARSET_ERROR" },
	{ PR_DELETED_MESSAGE_SIZE_EXTENDED, "PR_DELETED_MESSAGE_SIZE_EXTENDED" },
	{ PR_DELETED_MESSAGE_SIZE_EXTENDED_ERROR, "PR_DELETED_MESSAGE_SIZE_EXTENDED_ERROR" },
	{ PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED, "PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED" },
	{ PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED_ERROR, "PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED_ERROR" },
	{ PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED, "PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED" },
	{ PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED_ERROR, "PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED_ERROR" },
	{ PR_SECURE_IN_SITE, "PR_SECURE_IN_SITE" },
	{ PR_SECURE_IN_SITE_ERROR, "PR_SECURE_IN_SITE_ERROR" },
	{ PR_NT_USER_NAME, "PR_NT_USER_NAME" },
	{ PR_NT_USER_NAME_UNICODE, "PR_NT_USER_NAME_UNICODE" },
	{ PR_NT_USER_NAME_ERROR, "PR_NT_USER_NAME_ERROR" },
	{ PR_LOCALE_ID, "PR_LOCALE_ID" },
	{ PR_LOCALE_ID_ERROR, "PR_LOCALE_ID_ERROR" },
	{ PR_LAST_LOGON_TIME, "PR_LAST_LOGON_TIME" },
	{ PR_LAST_LOGON_TIME_ERROR, "PR_LAST_LOGON_TIME_ERROR" },
	{ PR_LAST_LOGOFF_TIME, "PR_LAST_LOGOFF_TIME" },
	{ PR_LAST_LOGOFF_TIME_ERROR, "PR_LAST_LOGOFF_TIME_ERROR" },
	{ PR_STORAGE_LIMIT_INFORMATION, "PR_STORAGE_LIMIT_INFORMATION" },
	{ PR_STORAGE_LIMIT_INFORMATION_ERROR, "PR_STORAGE_LIMIT_INFORMATION_ERROR" },
	{ PR_NEWSGROUP_COMPONENT, "PR_NEWSGROUP_COMPONENT" },
	{ PR_NEWSGROUP_COMPONENT_UNICODE, "PR_NEWSGROUP_COMPONENT_UNICODE" },
	{ PR_NEWSGROUP_COMPONENT_ERROR, "PR_NEWSGROUP_COMPONENT_ERROR" },
	{ PR_NEWSFEED_INFO, "PR_NEWSFEED_INFO" },
	{ PR_NEWSFEED_INFO_ERROR, "PR_NEWSFEED_INFO_ERROR" },
	{ PR_INTERNET_NEWSGROUP_NAME, "PR_INTERNET_NEWSGROUP_NAME" },
	{ PR_INTERNET_NEWSGROUP_NAME_UNICODE, "PR_INTERNET_NEWSGROUP_NAME_UNICODE" },
	{ PR_INTERNET_NEWSGROUP_NAME_ERROR, "PR_INTERNET_NEWSGROUP_NAME_ERROR" },
	{ PR_FOLDER_FLAGS, "PR_FOLDER_FLAGS" },
	{ PR_FOLDER_FLAGS_ERROR, "PR_FOLDER_FLAGS_ERROR" },
	{ PR_LAST_ACCESS_TIME, "PR_LAST_ACCESS_TIME" },
	{ PR_LAST_ACCESS_TIME_ERROR, "PR_LAST_ACCESS_TIME_ERROR" },
	{ PR_RESTRICTION_COUNT, "PR_RESTRICTION_COUNT" },
	{ PR_RESTRICTION_COUNT_ERROR, "PR_RESTRICTION_COUNT_ERROR" },
	{ PR_CATEG_COUNT, "PR_CATEG_COUNT" },
	{ PR_CATEG_COUNT_ERROR, "PR_CATEG_COUNT_ERROR" },
	{ PR_CACHED_COLUMN_COUNT, "PR_CACHED_COLUMN_COUNT" },
	{ PR_CACHED_COLUMN_COUNT_ERROR, "PR_CACHED_COLUMN_COUNT_ERROR" },
	{ PR_NORMAL_MSG_W_ATTACH_COUNT, "PR_NORMAL_MSG_W_ATTACH_COUNT" },
	{ PR_NORMAL_MSG_W_ATTACH_COUNT_ERROR, "PR_NORMAL_MSG_W_ATTACH_COUNT_ERROR" },
	{ PR_ASSOC_MSG_W_ATTACH_COUNT, "PR_ASSOC_MSG_W_ATTACH_COUNT" },
	{ PR_ASSOC_MSG_W_ATTACH_COUNT_ERROR, "PR_ASSOC_MSG_W_ATTACH_COUNT_ERROR" },
	{ PR_RECIPIENT_ON_NORMAL_MSG_COUNT, "PR_RECIPIENT_ON_NORMAL_MSG_COUNT" },
	{ PR_RECIPIENT_ON_NORMAL_MSG_COUNT_ERROR, "PR_RECIPIENT_ON_NORMAL_MSG_COUNT_ERROR" },
	{ PR_RECIPIENT_ON_ASSOC_MSG_COUNT, "PR_RECIPIENT_ON_ASSOC_MSG_COUNT" },
	{ PR_RECIPIENT_ON_ASSOC_MSG_COUNT_ERROR, "PR_RECIPIENT_ON_ASSOC_MSG_COUNT_ERROR" },
	{ PR_ATTACH_ON_NORMAL_MSG_COUNT, "PR_ATTACH_ON_NORMAL_MSG_COUNT" },
	{ PR_ATTACH_ON_NORMAL_MSG_COUNT_ERROR, "PR_ATTACH_ON_NORMAL_MSG_COUNT_ERROR" },
	{ PR_ATTACH_ON_ASSOC_MSG_COUNT, "PR_ATTACH_ON_ASSOC_MSG_COUNT" },
	{ PR_ATTACH_ON_ASSOC_MSG_COUNT_ERROR, "PR_ATTACH_ON_ASSOC_MSG_COUNT_ERROR" },
	{ PR_NORMAL_MESSAGE_SIZE, "PR_NORMAL_MESSAGE_SIZE" },
	{ PR_NORMAL_MESSAGE_SIZE_ERROR, "PR_NORMAL_MESSAGE_SIZE_ERROR" },
	{ PR_NORMAL_MESSAGE_SIZE_EXTENDED, "PR_NORMAL_MESSAGE_SIZE_EXTENDED" },
	{ PR_ASSOC_MESSAGE_SIZE, "PR_ASSOC_MESSAGE_SIZE" },
	{ PR_ASSOC_MESSAGE_SIZE_ERROR, "PR_ASSOC_MESSAGE_SIZE_ERROR" },
	{ PR_ASSOC_MESSAGE_SIZE_EXTENDED, "PR_ASSOC_MESSAGE_SIZE_EXTENDED" },
	{ PR_FOLDER_PATHNAME, "PR_FOLDER_PATHNAME" },
	{ PR_FOLDER_PATHNAME_UNICODE, "PR_FOLDER_PATHNAME_UNICODE" },
	{ PR_FOLDER_PATHNAME_ERROR, "PR_FOLDER_PATHNAME_ERROR" },
	{ PR_OWNER_COUNT, "PR_OWNER_COUNT" },
	{ PR_OWNER_COUNT_ERROR, "PR_OWNER_COUNT_ERROR" },
	{ PR_CONTACT_COUNT, "PR_CONTACT_COUNT" },
	{ PR_CONTACT_COUNT_ERROR, "PR_CONTACT_COUNT_ERROR" },
	{ PR_CODE_PAGE_ID, "PR_CODE_PAGE_ID" },
	{ PR_CODE_PAGE_ID_ERROR, "PR_CODE_PAGE_ID_ERROR" },
	{ PR_RETENTION_AGE_LIMIT, "PR_RETENTION_AGE_LIMIT" },
	{ PR_RETENTION_AGE_LIMIT_ERROR, "PR_RETENTION_AGE_LIMIT_ERROR" },
	{ PR_DISABLE_PERUSER_READ, "PR_DISABLE_PERUSER_READ" },
	{ PR_DISABLE_PERUSER_READ_ERROR, "PR_DISABLE_PERUSER_READ_ERROR" },
	{ PR_INTERNET_PARSE_STATE, "PR_INTERNET_PARSE_STATE" },
	{ PR_INTERNET_PARSE_STATE_ERROR, "PR_INTERNET_PARSE_STATE_ERROR" },
	{ PR_INTERNET_MESSAGE_INFO, "PR_INTERNET_MESSAGE_INFO" },
	{ PR_INTERNET_MESSAGE_INFO_ERROR, "PR_INTERNET_MESSAGE_INFO_ERROR" },
	{ PR_PST_PATH, "PR_PST_PATH" },
	{ PR_PST_PATH_UNICODE, "PR_PST_PATH_UNICODE" },
	{ PR_PST_PATH_ERROR, "PR_PST_PATH_ERROR" },
	{ PR_PST_REMEMBER_PW, "PR_PST_REMEMBER_PW" },
	{ PR_PST_REMEMBER_PW_ERROR, "PR_PST_REMEMBER_PW_ERROR" },
	{ PR_OST_ENCRYPTION, "PR_OST_ENCRYPTION" },
	{ PR_OST_ENCRYPTION_ERROR, "PR_OST_ENCRYPTION_ERROR" },
	{ PR_PST_PW_SZ_OLD, "PR_PST_PW_SZ_OLD" },
	{ PR_PST_PW_SZ_OLD_UNICODE, "PR_PST_PW_SZ_OLD_UNICODE" },
	{ PR_PST_PW_SZ_OLD_ERROR, "PR_PST_PW_SZ_OLD_ERROR" },
	{ PR_PST_PW_SZ_NEW, "PR_PST_PW_SZ_NEW" },
	{ PR_PST_PW_SZ_NEW_UNICODE, "PR_PST_PW_SZ_NEW_UNICODE" },
	{ PR_PST_PW_SZ_NEW_ERROR, "PR_PST_PW_SZ_NEW_ERROR" },
	{ PR_SORT_LOCALE_ID, "PR_SORT_LOCALE_ID" },
	{ PR_SORT_LOCALE_ID_ERROR, "PR_SORT_LOCALE_ID_ERROR" },
	{ PR_URL_NAME, "PR_URL_NAME" },
	{ PR_URL_NAME_UNICODE, "PR_URL_NAME_UNICODE" },
	{ PR_URL_NAME_ERROR, "PR_URL_NAME_ERROR" },
	{ PR_LOCAL_COMMIT_TIME, "PR_LOCAL_COMMIT_TIME" },
	{ PR_LOCAL_COMMIT_TIME_ERROR, "PR_LOCAL_COMMIT_TIME_ERROR" },
	{ PR_LOCAL_COMMIT_TIME_MAX, "PR_LOCAL_COMMIT_TIME_MAX" },
	{ PR_LOCAL_COMMIT_TIME_MAX_ERROR, "PR_LOCAL_COMMIT_TIME_MAX_ERROR" },
	{ PR_DELETED_COUNT_TOTAL, "PR_DELETED_COUNT_TOTAL" },
	{ PR_DELETED_COUNT_TOTAL_ERROR, "PR_DELETED_COUNT_TOTAL_ERROR" },
	{ PR_AUTO_RESET, "PR_AUTO_RESET" },
	{ PR_AUTO_RESET_ERROR, "PR_AUTO_RESET_ERROR" },
	{ PR_URL_COMP_NAME_HASH, "PR_URL_COMP_NAME_HASH" },
	{ PR_URL_COMP_NAME_HASH_ERROR, "PR_URL_COMP_NAME_HASH_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_2, "PR_MSG_FOLDER_TEMPLATE_RES_2" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_2_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_2_ERROR" },
	{ PR_RANK, "PR_RANK" },
	{ PR_RANK_ERROR, "PR_RANK_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_4, "PR_MSG_FOLDER_TEMPLATE_RES_4" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_4_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_4_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_5, "PR_MSG_FOLDER_TEMPLATE_RES_5" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_5_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_5_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_6, "PR_MSG_FOLDER_TEMPLATE_RES_6" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_6_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_6_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_7, "PR_MSG_FOLDER_TEMPLATE_RES_7" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_7_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_7_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_8, "PR_MSG_FOLDER_TEMPLATE_RES_8" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_8_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_8_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_9, "PR_MSG_FOLDER_TEMPLATE_RES_9" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_9_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_9_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_10, "PR_MSG_FOLDER_TEMPLATE_RES_10" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_10_UNICODE, "PR_MSG_FOLDER_TEMPLATE_RES_10_UNICODE" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_10_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_10_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_11, "PR_MSG_FOLDER_TEMPLATE_RES_11" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_11_UNICODE, "PR_MSG_FOLDER_TEMPLATE_RES_11_UNICODE" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_11_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_11_ERROR" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_12, "PR_MSG_FOLDER_TEMPLATE_RES_12" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_12_UNICODE, "PR_MSG_FOLDER_TEMPLATE_RES_12_UNICODE" },
	{ PR_MSG_FOLDER_TEMPLATE_RES_12_ERROR, "PR_MSG_FOLDER_TEMPLATE_RES_12_ERROR" },
	{ PR_PF_PLATINUM_HOME_MDB, "PR_PF_PLATINUM_HOME_MDB" },
	{ PR_PF_PLATINUM_HOME_MDB_ERROR, "PR_PF_PLATINUM_HOME_MDB_ERROR" },
	{ PR_PF_PROXY_REQUIRED, "PR_PF_PROXY_REQUIRED" },
	{ PR_PF_PROXY_REQUIRED_ERROR, "PR_PF_PROXY_REQUIRED_ERROR" },
	{ PR_INTERNET_FREE_DOC_INFO, "PR_INTERNET_FREE_DOC_INFO" },
	{ PR_INTERNET_FREE_DOC_INFO_ERROR, "PR_INTERNET_FREE_DOC_INFO_ERROR" },
	{ PR_PF_OVER_HARD_QUOTA_LIMIT, "PR_PF_OVER_HARD_QUOTA_LIMIT" },
	{ PR_PF_OVER_HARD_QUOTA_LIMIT_ERROR, "PR_PF_OVER_HARD_QUOTA_LIMIT_ERROR" },
	{ PR_PF_MSG_SIZE_LIMIT, "PR_PF_MSG_SIZE_LIMIT" },
	{ PR_PF_MSG_SIZE_LIMIT_ERROR, "PR_PF_MSG_SIZE_LIMIT_ERROR" },
	{ PR_CONNECTION_MODULUS, "PR_CONNECTION_MODULUS" },
	{ PR_CONNECTION_MODULUS_ERROR, "PR_CONNECTION_MODULUS_ERROR" },
	{ PR_DELIVER_TO_DN, "PR_DELIVER_TO_DN" },
	{ PR_DELIVER_TO_DN_UNICODE, "PR_DELIVER_TO_DN_UNICODE" },
	{ PR_DELIVER_TO_DN_ERROR, "PR_DELIVER_TO_DN_ERROR" },
	{ PR_MIME_SIZE, "PR_MIME_SIZE" },
	{ PR_MIME_SIZE_ERROR, "PR_MIME_SIZE_ERROR" },
	{ PR_FILE_SIZE_EXTENDED, "PR_FILE_SIZE_EXTENDED" },
	{ PR_FILE_SIZE_EXTENDED_ERROR, "PR_FILE_SIZE_EXTENDED_ERROR" },
	{ PR_FID, "PR_FID" },
	{ PR_FID_ERROR, "PR_FID_ERROR" },
	{ PR_PARENT_FID, "PR_PARENT_FID" },
	{ PR_PARENT_FID_ERROR, "PR_PARENT_FID_ERROR" },
	{ PR_MID, "PR_MID" },
	{ PR_MID_ERROR, "PR_MID_ERROR" },
	{ PR_CATEG_ID, "PR_CATEG_ID" },
	{ PR_CATEG_ID_ERROR, "PR_CATEG_ID_ERROR" },
	{ PR_PARENT_CATEG_ID, "PR_PARENT_CATEG_ID" },
	{ PR_PARENT_CATEG_ID_ERROR, "PR_PARENT_CATEG_ID_ERROR" },
	{ PR_INST_ID, "PR_INST_ID" },
	{ PR_INST_ID_ERROR, "PR_INST_ID_ERROR" },
	{ PR_INSTANCE_NUM, "PR_INSTANCE_NUM" },
	{ PR_INSTANCE_NUM_ERROR, "PR_INSTANCE_NUM_ERROR" },
	{ PR_ADDRBOOK_MID, "PR_ADDRBOOK_MID" },
	{ PR_ADDRBOOK_MID_ERROR, "PR_ADDRBOOK_MID_ERROR" },
	{ PR_ICS_NOTIF, "PR_ICS_NOTIF" },
	{ PR_ICS_NOTIF_ERROR, "PR_ICS_NOTIF_ERROR" },
	{ PR_ARTICLE_NUM_NEXT, "PR_ARTICLE_NUM_NEXT" },
	{ PR_ARTICLE_NUM_NEXT_ERROR, "PR_ARTICLE_NUM_NEXT_ERROR" },
	{ PR_IMAP_LAST_ARTICLE_ID, "PR_IMAP_LAST_ARTICLE_ID" },
	{ PR_IMAP_LAST_ARTICLE_ID_ERROR, "PR_IMAP_LAST_ARTICLE_ID_ERROR" },
	{ PR_NOT_822_RENDERABLE, "PR_NOT_822_RENDERABLE" },
	{ PR_NOT_822_RENDERABLE_ERROR, "PR_NOT_822_RENDERABLE_ERROR" },
	{ PR_LTID, "PR_LTID" },
	{ PR_LTID_ERROR, "PR_LTID_ERROR" },
	{ PR_CN_EXPORT, "PR_CN_EXPORT" },
	{ PR_CN_EXPORT_ERROR, "PR_CN_EXPORT_ERROR" },
	{ PR_PCL_EXPORT, "PR_PCL_EXPORT" },
	{ PR_PCL_EXPORT_ERROR, "PR_PCL_EXPORT_ERROR" },
	{ PR_CN_MV_EXPORT, "PR_CN_MV_EXPORT" },
	{ PR_CN_MV_EXPORT_ERROR, "PR_CN_MV_EXPORT_ERROR" },
	{ PR_PF_QUOTA_STYLE, "PR_PF_QUOTA_STYLE" },
	{ PR_PF_QUOTA_STYLE_ERROR, "PR_PF_QUOTA_STYLE_ERROR" },
	{ PR_PF_STORAGE_QUOTA, "PR_PF_STORAGE_QUOTA" },
	{ PR_PF_STORAGE_QUOTA_ERROR, "PR_PF_STORAGE_QUOTA_ERROR" },
	{ PR_SEARCH_FLAGS, "PR_SEARCH_FLAGS" },
	{ PR_SEARCH_FLAGS_ERROR, "PR_SEARCH_FLAGS_ERROR" },
	{ PR_ASSOCIATED, "PR_ASSOCIATED" },
	{ PR_ASSOCIATED_ERROR, "PR_ASSOCIATED_ERROR" },
	{ PR_PROFILE_SECURE_MAILBOX, "PR_PROFILE_SECURE_MAILBOX" },
	{ PR_PROFILE_SECURE_MAILBOX_ERROR, "PR_PROFILE_SECURE_MAILBOX_ERROR" },
	{ PR_MAILBEAT_BOUNCE_SERVER, "PR_MAILBEAT_BOUNCE_SERVER" },
	{ PR_MAILBEAT_BOUNCE_SERVER_UNICODE, "PR_MAILBEAT_BOUNCE_SERVER_UNICODE" },
	{ PR_MAILBEAT_BOUNCE_SERVER_ERROR, "PR_MAILBEAT_BOUNCE_SERVER_ERROR" },
	{ PR_MAILBEAT_REQUEST_SENT, "PR_MAILBEAT_REQUEST_SENT" },
	{ PR_MAILBEAT_REQUEST_SENT_ERROR, "PR_MAILBEAT_REQUEST_SENT_ERROR" },
	{ PR_USENET_SITE_NAME, "PR_USENET_SITE_NAME" },
	{ PR_USENET_SITE_NAME_UNICODE, "PR_USENET_SITE_NAME_UNICODE" },
	{ PR_USENET_SITE_NAME_ERROR, "PR_USENET_SITE_NAME_ERROR" },
	{ PR_MAILBEAT_REQUEST_RECEIVED, "PR_MAILBEAT_REQUEST_RECEIVED" },
	{ PR_MAILBEAT_REQUEST_RECEIVED_ERROR, "PR_MAILBEAT_REQUEST_RECEIVED_ERROR" },
	{ PR_MAILBEAT_REQUEST_PROCESSED, "PR_MAILBEAT_REQUEST_PROCESSED" },
	{ PR_MAILBEAT_REQUEST_PROCESSED_ERROR, "PR_MAILBEAT_REQUEST_PROCESSED_ERROR" },
	{ PR_MAILBEAT_REPLY_SENT, "PR_MAILBEAT_REPLY_SENT" },
	{ PR_MAILBEAT_REPLY_SENT_ERROR, "PR_MAILBEAT_REPLY_SENT_ERROR" },
	{ PR_MAILBEAT_REPLY_SUBMIT, "PR_MAILBEAT_REPLY_SUBMIT" },
	{ PR_MAILBEAT_REPLY_SUBMIT_ERROR, "PR_MAILBEAT_REPLY_SUBMIT_ERROR" },
	{ PR_MAILBEAT_REPLY_RECEIVED, "PR_MAILBEAT_REPLY_RECEIVED" },
	{ PR_MAILBEAT_REPLY_RECEIVED_ERROR, "PR_MAILBEAT_REPLY_RECEIVED_ERROR" },
	{ PR_MAILBEAT_REPLY_PROCESSED, "PR_MAILBEAT_REPLY_PROCESSED" },
	{ PR_MAILBEAT_REPLY_PROCESSED_ERROR, "PR_MAILBEAT_REPLY_PROCESSED_ERROR" },
	{ PR_DELEGATES_DISPLAY_NAMES, "PR_DELEGATES_DISPLAY_NAMES" },
	{ PR_DELEGATES_DISPLAY_NAMES_UNICODE, "PR_DELEGATES_DISPLAY_NAMES_UNICODE" },
	{ PR_DELEGATES_DISPLAY_NAMES_ERROR, "PR_DELEGATES_DISPLAY_NAMES_ERROR" },
	{ PR_DELEGATES_ENTRYIDS, "PR_DELEGATES_ENTRYIDS" },
	{ PR_DELEGATES_ENTRYIDS_ERROR, "PR_DELEGATES_ENTRYIDS_ERROR" },
	{ PR_FREEBUSY_START_RANGE, "PR_FREEBUSY_START_RANGE" },
	{ PR_FREEBUSY_START_RANGE_ERROR, "PR_FREEBUSY_START_RANGE_ERROR" },
	{ PR_FREEBUSY_END_RANGE, "PR_FREEBUSY_END_RANGE" },
	{ PR_FREEBUSY_END_RANGE_ERROR, "PR_FREEBUSY_END_RANGE_ERROR" },
	{ PR_FREEBUSY_EMAIL_ADDRESS, "PR_FREEBUSY_EMAIL_ADDRESS" },
	{ PR_FREEBUSY_EMAIL_ADDRESS_UNICODE, "PR_FREEBUSY_EMAIL_ADDRESS_UNICODE" },
	{ PR_FREEBUSY_EMAIL_ADDRESS_ERROR, "PR_FREEBUSY_EMAIL_ADDRESS_ERROR" },
	{ PR_FREEBUSY_ALL_MONTHS, "PR_FREEBUSY_ALL_MONTHS" },
	{ PR_FREEBUSY_ALL_MONTHS_ERROR, "PR_FREEBUSY_ALL_MONTHS_ERROR" },
	{ PR_FREEBUSY_ALL_EVENTS, "PR_FREEBUSY_ALL_EVENTS" },
	{ PR_FREEBUSY_ALL_EVENTS_ERROR, "PR_FREEBUSY_ALL_EVENTS_ERROR" },
	{ PR_FREEBUSY_TENTATIVE_MONTHS, "PR_FREEBUSY_TENTATIVE_MONTHS" },
	{ PR_FREEBUSY_TENTATIVE_MONTHS_ERROR, "PR_FREEBUSY_TENTATIVE_MONTHS_ERROR" },
	{ PR_FREEBUSY_TENTATIVE_EVENTS, "PR_FREEBUSY_TENTATIVE_EVENTS" },
	{ PR_FREEBUSY_TENTATIVE_EVENTS_ERROR, "PR_FREEBUSY_TENTATIVE_EVENTS_ERROR" },
	{ PR_FREEBUSY_BUSY_MONTHS, "PR_FREEBUSY_BUSY_MONTHS" },
	{ PR_FREEBUSY_BUSY_MONTHS_ERROR, "PR_FREEBUSY_BUSY_MONTHS_ERROR" },
	{ PR_FREEBUSY_BUSY_EVENTS, "PR_FREEBUSY_BUSY_EVENTS" },
	{ PR_FREEBUSY_BUSY_EVENTS_ERROR, "PR_FREEBUSY_BUSY_EVENTS_ERROR" },
	{ PR_FREEBUSY_OOF_MONTHS, "PR_FREEBUSY_OOF_MONTHS" },
	{ PR_FREEBUSY_OOF_MONTHS_ERROR, "PR_FREEBUSY_OOF_MONTHS_ERROR" },
	{ PR_FREEBUSY_OOF_EVENTS, "PR_FREEBUSY_OOF_EVENTS" },
	{ PR_FREEBUSY_OOF_EVENTS_ERROR, "PR_FREEBUSY_OOF_EVENTS_ERROR" },
	{ PR_FREEBUSY_LAST_MODIFIED, "PR_FREEBUSY_LAST_MODIFIED" },
	{ PR_FREEBUSY_LAST_MODIFIED_ERROR, "PR_FREEBUSY_LAST_MODIFIED_ERROR" },
	{ PR_FREEBUSY_NUM_MONTHS, "PR_FREEBUSY_NUM_MONTHS" },
	{ PR_FREEBUSY_NUM_MONTHS_ERROR, "PR_FREEBUSY_NUM_MONTHS_ERROR" },
	{ PR_DELEGATES_SEE_PRIVATE, "PR_DELEGATES_SEE_PRIVATE" },
	{ PR_DELEGATES_SEE_PRIVATE_ERROR, "PR_DELEGATES_SEE_PRIVATE_ERROR" },
	{ PR_PERSONAL_FREEBUSY, "PR_PERSONAL_FREEBUSY" },
	{ PR_PERSONAL_FREEBUSY_ERROR, "PR_PERSONAL_FREEBUSY_ERROR" },
	{ PR_PROCESS_MEETING_REQUESTS, "PR_PROCESS_MEETING_REQUESTS" },
	{ PR_PROCESS_MEETING_REQUESTS_ERROR, "PR_PROCESS_MEETING_REQUESTS_ERROR" },
	{ PR_DECLINE_RECURRING_MEETING_REQUESTS, "PR_DECLINE_RECURRING_MEETING_REQUESTS" },
	{ PR_DECLINE_RECURRING_MEETING_REQUESTS_ERROR, "PR_DECLINE_RECURRING_MEETING_REQUESTS_ERROR" },
	{ PR_DECLINE_CONFLICTING_MEETING_REQUESTS, "PR_DECLINE_CONFLICTING_MEETING_REQUESTS" },
	{ PR_DECLINE_CONFLICTING_MEETING_REQUESTS_ERROR, "PR_DECLINE_CONFLICTING_MEETING_REQUESTS_ERROR" },
	{ PR_VD_BINARY, "PR_VD_BINARY" },
	{ PR_VD_BINARY_ERROR, "PR_VD_BINARY_ERROR" },
	{ PR_VD_STRINGS, "PR_VD_STRINGS" },
	{ PR_VD_STRINGS_UNICODE, "PR_VD_STRINGS_UNICODE" },
	{ PR_VD_STRINGS_ERROR, "PR_VD_STRINGS_ERROR" },
	{ PR_VD_FLAGS, "PR_VD_FLAGS" },
	{ PR_VD_FLAGS_ERROR, "PR_VD_FLAGS_ERROR" },
	{ PR_VD_LINK_TO, "PR_VD_LINK_TO" },
	{ PR_VD_LINK_TO_ERROR, "PR_VD_LINK_TO_ERROR" },
	{ PR_VD_VIEW_FOLDER, "PR_VD_VIEW_FOLDER" },
	{ PR_VD_VIEW_FOLDER_ERROR, "PR_VD_VIEW_FOLDER_ERROR" },
	{ PR_VD_NAME, "PR_VD_NAME" },
	{ PR_VD_NAME_UNICODE, "PR_VD_NAME_UNICODE" },
	{ PR_VD_NAME_ERROR, "PR_VD_NAME_ERROR" },
	{ PR_VD_VERSION, "PR_VD_VERSION" },
	{ PR_VD_VERSION_ERROR, "PR_VD_VERSION_ERROR" },
	{ PR_FAV_DISPLAY_NAME_A, "PR_FAV_DISPLAY_NAME_A" },
	{ PR_FAV_DISPLAY_NAME_A_UNICODE, "PR_FAV_DISPLAY_NAME_A_UNICODE" },
	{ PR_FAV_DISPLAY_NAME_A_ERROR, "PR_FAV_DISPLAY_NAME_A_ERROR" },
	{ PR_FAV_PUBLIC_SOURCE_KEY, "PR_FAV_PUBLIC_SOURCE_KEY" },
	{ PR_FAV_PUBLIC_SOURCE_KEY_ERROR, "PR_FAV_PUBLIC_SOURCE_KEY_ERROR" },
	{ PR_OST_OSTID, "PR_OST_OSTID" },
	{ PR_OST_OSTID_ERROR, "PR_OST_OSTID_ERROR" },
	{ PR_STORE_SLOWLINK, "PR_STORE_SLOWLINK" },
	{ PR_STORE_SLOWLINK_ERROR, "PR_STORE_SLOWLINK_ERROR" },
	{ PR_FAV_AUTOSUBFOLDERS, "PR_FAV_AUTOSUBFOLDERS" },
	{ PR_FAV_AUTOSUBFOLDERS_ERROR, "PR_FAV_AUTOSUBFOLDERS_ERROR" },
	{ PR_FAV_PARENT_SOURCE_KEY, "PR_FAV_PARENT_SOURCE_KEY" },
	{ PR_FAV_PARENT_SOURCE_KEY_ERROR, "PR_FAV_PARENT_SOURCE_KEY_ERROR" },
	{ PR_FAV_LEVEL_MASK, "PR_FAV_LEVEL_MASK" },
	{ PR_FAV_LEVEL_MASK_ERROR, "PR_FAV_LEVEL_MASK_ERROR" },
	{ PR_FAV_INHERIT_AUTO, "PR_FAV_INHERIT_AUTO" },
	{ PR_FAV_INHERIT_AUTO_ERROR, "PR_FAV_INHERIT_AUTO_ERROR" },
	{ PR_FAV_DEL_SUBS, "PR_FAV_DEL_SUBS" },
	{ PR_FAV_DEL_SUBS_ERROR, "PR_FAV_DEL_SUBS_ERROR" },
	{ PR_ATTACHMENT_LINKID, "PR_ATTACHMENT_LINKID" },
	{ PR_ATTACHMENT_LINKID_ERROR, "PR_ATTACHMENT_LINKID_ERROR" },
	{ PR_EXCEPTION_STARTTIME, "PR_EXCEPTION_STARTTIME" },
	{ PR_EXCEPTION_STARTTIME_ERROR, "PR_EXCEPTION_STARTTIME_ERROR" },
	{ PR_EXCEPTION_ENDTIME, "PR_EXCEPTION_ENDTIME" },
	{ PR_EXCEPTION_ENDTIME_ERROR, "PR_EXCEPTION_ENDTIME_ERROR" },
	{ PR_ATTACHMENT_FLAGS, "PR_ATTACHMENT_FLAGS" },
	{ PR_ATTACHMENT_FLAGS_ERROR, "PR_ATTACHMENT_FLAGS_ERROR" },
	{ PR_ATTACHMENT_HIDDEN, "PR_ATTACHMENT_HIDDEN" },
	{ PR_ATTACHMENT_HIDDEN_ERROR, "PR_ATTACHMENT_HIDDEN_ERROR" },
	{ PR_EMS_AB_DISPLAY_NAME_OVERRIDE, "PR_EMS_AB_DISPLAY_NAME_OVERRIDE" },
	{ PR_EMS_AB_DISPLAY_NAME_OVERRIDE_ERROR, "PR_EMS_AB_DISPLAY_NAME_OVERRIDE_ERROR" },
	{ PR_EMS_AB_CA_CERTIFICATE, "PR_EMS_AB_CA_CERTIFICATE" },
	{ PR_EMS_AB_CA_CERTIFICATE_ERROR, "PR_EMS_AB_CA_CERTIFICATE_ERROR" },
	{ PR_EMS_AB_FOLDER_PATHNAME, "PR_EMS_AB_FOLDER_PATHNAME" },
	{ PR_EMS_AB_FOLDER_PATHNAME_UNICODE, "PR_EMS_AB_FOLDER_PATHNAME_UNICODE" },
	{ PR_EMS_AB_FOLDER_PATHNAME_ERROR, "PR_EMS_AB_FOLDER_PATHNAME_ERROR" },
	{ PR_EMS_AB_MANAGER, "PR_EMS_AB_MANAGER" },
	{ PR_EMS_AB_MANAGER_ERROR, "PR_EMS_AB_MANAGER_ERROR" },
	{ PR_EMS_AB_MANAGER_T, "PR_EMS_AB_MANAGER_T" },
	{ PR_EMS_AB_MANAGER_T_UNICODE, "PR_EMS_AB_MANAGER_T_UNICODE" },
	{ PR_EMS_AB_HOME_MDB_O, "PR_EMS_AB_HOME_MDB_O" },
	{ PR_EMS_AB_HOME_MDB_O_ERROR, "PR_EMS_AB_HOME_MDB_O_ERROR" },
	{ PR_EMS_AB_HOME_MDB, "PR_EMS_AB_HOME_MDB" },
	{ PR_EMS_AB_HOME_MDB_UNICODE, "PR_EMS_AB_HOME_MDB_UNICODE" },
	{ PR_EMS_AB_HOME_MTA_O, "PR_EMS_AB_HOME_MTA_O" },
	{ PR_EMS_AB_HOME_MTA_O_ERROR, "PR_EMS_AB_HOME_MTA_O_ERROR" },
	{ PR_EMS_AB_HOME_MTA, "PR_EMS_AB_HOME_MTA" },
	{ PR_EMS_AB_HOME_MTA_UNICODE, "PR_EMS_AB_HOME_MTA_UNICODE" },
	{ PR_EMS_AB_IS_MEMBER_OF_DL, "PR_EMS_AB_IS_MEMBER_OF_DL" },
	{ PR_EMS_AB_IS_MEMBER_OF_DL_ERROR, "PR_EMS_AB_IS_MEMBER_OF_DL_ERROR" },
	{ PR_EMS_AB_IS_MEMBER_OF_DL_T, "PR_EMS_AB_IS_MEMBER_OF_DL_T" },
	{ PR_EMS_AB_IS_MEMBER_OF_DL_T_UNICODE, "PR_EMS_AB_IS_MEMBER_OF_DL_T_UNICODE" },
	{ PR_EMS_AB_MEMBER, "PR_EMS_AB_MEMBER" },
	{ PR_EMS_AB_MEMBER_ERROR, "PR_EMS_AB_MEMBER_ERROR" },
	{ PR_EMS_AB_MEMBER_T, "PR_EMS_AB_MEMBER_T" },
	{ PR_EMS_AB_MEMBER_T_UNICODE, "PR_EMS_AB_MEMBER_T_UNICODE" },
	{ PR_EMS_AB_AUTOREPLY_MESSAGE, "PR_EMS_AB_AUTOREPLY_MESSAGE" },
	{ PR_EMS_AB_AUTOREPLY_MESSAGE_UNICODE, "PR_EMS_AB_AUTOREPLY_MESSAGE_UNICODE" },
	{ PR_EMS_AB_AUTOREPLY_MESSAGE_ERROR, "PR_EMS_AB_AUTOREPLY_MESSAGE_ERROR" },
	{ PR_EMS_AB_AUTOREPLY, "PR_EMS_AB_AUTOREPLY" },
	{ PR_EMS_AB_AUTOREPLY_ERROR, "PR_EMS_AB_AUTOREPLY_ERROR" },
	{ PR_EMS_AB_OWNER_O, "PR_EMS_AB_OWNER_O" },
	{ PR_EMS_AB_OWNER_O_ERROR, "PR_EMS_AB_OWNER_O_ERROR" },
	{ PR_EMS_AB_OWNER, "PR_EMS_AB_OWNER" },
	{ PR_EMS_AB_OWNER_UNICODE, "PR_EMS_AB_OWNER_UNICODE" },
	{ PR_EMS_AB_KM_SERVER_O, "PR_EMS_AB_KM_SERVER_O" },
	{ PR_EMS_AB_KM_SERVER_O_ERROR, "PR_EMS_AB_KM_SERVER_O_ERROR" },
	{ PR_EMS_AB_KM_SERVER, "PR_EMS_AB_KM_SERVER" },
	{ PR_EMS_AB_KM_SERVER_UNICODE, "PR_EMS_AB_KM_SERVER_UNICODE" },
	{ PR_EMS_AB_REPORTS, "PR_EMS_AB_REPORTS" },
	{ PR_EMS_AB_REPORTS_ERROR, "PR_EMS_AB_REPORTS_ERROR" },
	{ PR_EMS_AB_PROXY_ADDRESSES, "PR_EMS_AB_PROXY_ADDRESSES" },
	{ PR_EMS_AB_PROXY_ADDRESSES_UNICODE, "PR_EMS_AB_PROXY_ADDRESSES_UNICODE" },
	{ PR_EMS_AB_PROXY_ADDRESSES_ERROR, "PR_EMS_AB_PROXY_ADDRESSES_ERROR" },
	{ PR_EMS_AB_HELP_DATA32, "PR_EMS_AB_HELP_DATA32" },
	{ PR_EMS_AB_HELP_DATA32_ERROR, "PR_EMS_AB_HELP_DATA32_ERROR" },
	{ PR_EMS_AB_TARGET_ADDRESS, "PR_EMS_AB_TARGET_ADDRESS" },
	{ PR_EMS_AB_TARGET_ADDRESS_UNICODE, "PR_EMS_AB_TARGET_ADDRESS_UNICODE" },
	{ PR_EMS_AB_TARGET_ADDRESS_ERROR, "PR_EMS_AB_TARGET_ADDRESS_ERROR" },
	{ PR_EMS_AB_TELEPHONE_NUMBER, "PR_EMS_AB_TELEPHONE_NUMBER" },
	{ PR_EMS_AB_TELEPHONE_NUMBER_UNICODE, "PR_EMS_AB_TELEPHONE_NUMBER_UNICODE" },
	{ PR_EMS_AB_TELEPHONE_NUMBER_ERROR, "PR_EMS_AB_TELEPHONE_NUMBER_ERROR" },
	{ PR_EMS_AB_NT_SECURITY_DESCRIPTOR, "PR_EMS_AB_NT_SECURITY_DESCRIPTOR" },
	{ PR_EMS_AB_NT_SECURITY_DESCRIPTOR_ERROR, "PR_EMS_AB_NT_SECURITY_DESCRIPTOR_ERROR" },
	{ PR_EMS_AB_HOME_MDB_BL_O, "PR_EMS_AB_HOME_MDB_BL_O" },
	{ PR_EMS_AB_HOME_MDB_BL_O_ERROR, "PR_EMS_AB_HOME_MDB_BL_O_ERROR" },
	{ PR_EMS_AB_HOME_MDB_BL, "PR_EMS_AB_HOME_MDB_BL" },
	{ PR_EMS_AB_HOME_MDB_BL_UNICODE, "PR_EMS_AB_HOME_MDB_BL_UNICODE" },
	{ PR_EMS_AB_PUBLIC_DELEGATES, "PR_EMS_AB_PUBLIC_DELEGATES" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_ERROR, "PR_EMS_AB_PUBLIC_DELEGATES_ERROR" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_T, "PR_EMS_AB_PUBLIC_DELEGATES_T" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_T_UNICODE, "PR_EMS_AB_PUBLIC_DELEGATES_T_UNICODE" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_ERROR, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_ERROR" },
	{ PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE, "PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE" },
	{ PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_ERROR, "PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_ERROR" },
	{ PR_EMS_AB_ADDRESS_SYNTAX, "PR_EMS_AB_ADDRESS_SYNTAX" },
	{ PR_EMS_AB_ADDRESS_SYNTAX_ERROR, "PR_EMS_AB_ADDRESS_SYNTAX_ERROR" },
	{ PR_EMS_AB_BUSINESS_ROLES, "PR_EMS_AB_BUSINESS_ROLES" },
	{ PR_EMS_AB_BUSINESS_ROLES_ERROR, "PR_EMS_AB_BUSINESS_ROLES_ERROR" },
	{ PR_EMS_AB_OWNER_BL_O, "PR_EMS_AB_OWNER_BL_O" },
	{ PR_EMS_AB_OWNER_BL_O_ERROR, "PR_EMS_AB_OWNER_BL_O_ERROR" },
	{ PR_EMS_AB_OWNER_BL, "PR_EMS_AB_OWNER_BL" },
	{ PR_EMS_AB_OWNER_BL_UNICODE, "PR_EMS_AB_OWNER_BL_UNICODE" },
	{ PR_EMS_AB_CROSS_CERTIFICATE_PAIR, "PR_EMS_AB_CROSS_CERTIFICATE_PAIR" },
	{ PR_EMS_AB_CROSS_CERTIFICATE_PAIR_ERROR, "PR_EMS_AB_CROSS_CERTIFICATE_PAIR_ERROR" },
	{ PR_EMS_AB_AUTHORITY_REVOCATION_LIST, "PR_EMS_AB_AUTHORITY_REVOCATION_LIST" },
	{ PR_EMS_AB_AUTHORITY_REVOCATION_LIST_ERROR, "PR_EMS_AB_AUTHORITY_REVOCATION_LIST_ERROR" },
	{ PR_EMS_AB_ASSOC_NT_ACCOUNT, "PR_EMS_AB_ASSOC_NT_ACCOUNT" },
	{ PR_EMS_AB_ASSOC_NT_ACCOUNT_ERROR, "PR_EMS_AB_ASSOC_NT_ACCOUNT_ERROR" },
	{ PR_EMS_AB_EXPIRATION_TIME, "PR_EMS_AB_EXPIRATION_TIME" },
	{ PR_EMS_AB_EXPIRATION_TIME_ERROR, "PR_EMS_AB_EXPIRATION_TIME_ERROR" },
	{ PR_EMS_AB_USN_CHANGED, "PR_EMS_AB_USN_CHANGED" },
	{ PR_EMS_AB_USN_CHANGED_ERROR, "PR_EMS_AB_USN_CHANGED_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_1, "PR_EMS_AB_EXTENSION_ATTRIBUTE_1" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_1_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_1_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_1_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_1_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_2, "PR_EMS_AB_EXTENSION_ATTRIBUTE_2" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_2_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_2_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_2_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_2_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_3, "PR_EMS_AB_EXTENSION_ATTRIBUTE_3" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_3_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_3_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_3_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_3_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_4, "PR_EMS_AB_EXTENSION_ATTRIBUTE_4" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_4_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_4_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_4_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_4_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_5, "PR_EMS_AB_EXTENSION_ATTRIBUTE_5" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_5_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_5_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_5_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_5_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_6, "PR_EMS_AB_EXTENSION_ATTRIBUTE_6" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_6_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_6_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_6_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_6_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_7, "PR_EMS_AB_EXTENSION_ATTRIBUTE_7" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_7_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_7_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_7_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_7_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_8, "PR_EMS_AB_EXTENSION_ATTRIBUTE_8" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_8_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_8_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_8_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_8_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_9, "PR_EMS_AB_EXTENSION_ATTRIBUTE_9" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_9_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_9_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_9_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_9_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_10, "PR_EMS_AB_EXTENSION_ATTRIBUTE_10" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_10_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_10_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_10_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_10_ERROR" },
	{ PR_EMS_AB_SECURITY_PROTOCOL, "PR_EMS_AB_SECURITY_PROTOCOL" },
	{ PR_EMS_AB_SECURITY_PROTOCOL_ERROR, "PR_EMS_AB_SECURITY_PROTOCOL_ERROR" },
	{ PR_EMS_AB_PF_CONTACTS_O, "PR_EMS_AB_PF_CONTACTS_O" },
	{ PR_EMS_AB_PF_CONTACTS_O_ERROR, "PR_EMS_AB_PF_CONTACTS_O_ERROR" },
	{ PR_EMS_AB_PF_CONTACTS, "PR_EMS_AB_PF_CONTACTS" },
	{ PR_EMS_AB_PF_CONTACTS_UNICODE, "PR_EMS_AB_PF_CONTACTS_UNICODE" },
	{ PR_EMS_AB_HELP_DATA16, "PR_EMS_AB_HELP_DATA16" },
	{ PR_EMS_AB_HELP_DATA16_ERROR, "PR_EMS_AB_HELP_DATA16_ERROR" },
	{ PR_EMS_AB_HELP_FILE_NAME, "PR_EMS_AB_HELP_FILE_NAME" },
	{ PR_EMS_AB_HELP_FILE_NAME_UNICODE, "PR_EMS_AB_HELP_FILE_NAME_UNICODE" },
	{ PR_EMS_AB_HELP_FILE_NAME_ERROR, "PR_EMS_AB_HELP_FILE_NAME_ERROR" },
	{ PR_EMS_AB_OBJ_DIST_NAME_O, "PR_EMS_AB_OBJ_DIST_NAME_O" },
	{ PR_EMS_AB_OBJ_DIST_NAME_O_ERROR, "PR_EMS_AB_OBJ_DIST_NAME_O_ERROR" },
	{ PR_EMS_AB_OBJ_DIST_NAME, "PR_EMS_AB_OBJ_DIST_NAME" },
	{ PR_EMS_AB_OBJ_DIST_NAME_UNICODE, "PR_EMS_AB_OBJ_DIST_NAME_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_UNICODE, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_ERROR, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_ERROR" },
	{ PR_EMS_AB_AUTOREPLY_SUBJECT, "PR_EMS_AB_AUTOREPLY_SUBJECT" },
	{ PR_EMS_AB_AUTOREPLY_SUBJECT_UNICODE, "PR_EMS_AB_AUTOREPLY_SUBJECT_UNICODE" },
	{ PR_EMS_AB_AUTOREPLY_SUBJECT_ERROR, "PR_EMS_AB_AUTOREPLY_SUBJECT_ERROR" },
	{ PR_EMS_AB_HOME_PUBLIC_SERVER_O, "PR_EMS_AB_HOME_PUBLIC_SERVER_O" },
	{ PR_EMS_AB_HOME_PUBLIC_SERVER_O_ERROR, "PR_EMS_AB_HOME_PUBLIC_SERVER_O_ERROR" },
	{ PR_EMS_AB_HOME_PUBLIC_SERVER, "PR_EMS_AB_HOME_PUBLIC_SERVER" },
	{ PR_EMS_AB_HOME_PUBLIC_SERVER_UNICODE, "PR_EMS_AB_HOME_PUBLIC_SERVER_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_NA, "PR_EMS_AB_ENCRYPT_ALG_LIST_NA" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_NA_UNICODE, "PR_EMS_AB_ENCRYPT_ALG_LIST_NA_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_NA_ERROR, "PR_EMS_AB_ENCRYPT_ALG_LIST_NA_ERROR" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER, "PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_UNICODE, "PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_ERROR, "PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_ERROR" },
	{ PR_EMS_AB_IMPORTED_FROM, "PR_EMS_AB_IMPORTED_FROM" },
	{ PR_EMS_AB_IMPORTED_FROM_UNICODE, "PR_EMS_AB_IMPORTED_FROM_UNICODE" },
	{ PR_EMS_AB_IMPORTED_FROM_ERROR, "PR_EMS_AB_IMPORTED_FROM_ERROR" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_UNICODE, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_UNICODE" },
	{ PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_ERROR, "PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_ERROR" },
	{ PR_EMS_AB_ACCESS_CATEGORY, "PR_EMS_AB_ACCESS_CATEGORY" },
	{ PR_EMS_AB_ACCESS_CATEGORY_ERROR, "PR_EMS_AB_ACCESS_CATEGORY_ERROR" },
	{ PR_EMS_AB_ACTIVATION_SCHEDULE, "PR_EMS_AB_ACTIVATION_SCHEDULE" },
	{ PR_EMS_AB_ACTIVATION_SCHEDULE_ERROR, "PR_EMS_AB_ACTIVATION_SCHEDULE_ERROR" },
	{ PR_EMS_AB_ACTIVATION_STYLE, "PR_EMS_AB_ACTIVATION_STYLE" },
	{ PR_EMS_AB_ACTIVATION_STYLE_ERROR, "PR_EMS_AB_ACTIVATION_STYLE_ERROR" },
	{ PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS, "PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS" },
	{ PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS_ERROR, "PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS_ERROR" },
	{ PR_EMS_AB_ADDRESS_TYPE, "PR_EMS_AB_ADDRESS_TYPE" },
	{ PR_EMS_AB_ADDRESS_TYPE_UNICODE, "PR_EMS_AB_ADDRESS_TYPE_UNICODE" },
	{ PR_EMS_AB_ADDRESS_TYPE_ERROR, "PR_EMS_AB_ADDRESS_TYPE_ERROR" },
	{ PR_EMS_AB_ADMD, "PR_EMS_AB_ADMD" },
	{ PR_EMS_AB_ADMD_UNICODE, "PR_EMS_AB_ADMD_UNICODE" },
	{ PR_EMS_AB_ADMD_ERROR, "PR_EMS_AB_ADMD_ERROR" },
	{ PR_EMS_AB_ADMIN_DESCRIPTION, "PR_EMS_AB_ADMIN_DESCRIPTION" },
	{ PR_EMS_AB_ADMIN_DESCRIPTION_UNICODE, "PR_EMS_AB_ADMIN_DESCRIPTION_UNICODE" },
	{ PR_EMS_AB_ADMIN_DESCRIPTION_ERROR, "PR_EMS_AB_ADMIN_DESCRIPTION_ERROR" },
	{ PR_EMS_AB_ADMIN_DISPLAY_NAME, "PR_EMS_AB_ADMIN_DISPLAY_NAME" },
	{ PR_EMS_AB_ADMIN_DISPLAY_NAME_UNICODE, "PR_EMS_AB_ADMIN_DISPLAY_NAME_UNICODE" },
	{ PR_EMS_AB_ADMIN_DISPLAY_NAME_ERROR, "PR_EMS_AB_ADMIN_DISPLAY_NAME_ERROR" },
	{ PR_EMS_AB_ADMIN_EXTENSION_DLL, "PR_EMS_AB_ADMIN_EXTENSION_DLL" },
	{ PR_EMS_AB_ADMIN_EXTENSION_DLL_UNICODE, "PR_EMS_AB_ADMIN_EXTENSION_DLL_UNICODE" },
	{ PR_EMS_AB_ADMIN_EXTENSION_DLL_ERROR, "PR_EMS_AB_ADMIN_EXTENSION_DLL_ERROR" },
	{ PR_EMS_AB_ALIASED_OBJECT_NAME_O, "PR_EMS_AB_ALIASED_OBJECT_NAME_O" },
	{ PR_EMS_AB_ALIASED_OBJECT_NAME_O_ERROR, "PR_EMS_AB_ALIASED_OBJECT_NAME_O_ERROR" },
	{ PR_EMS_AB_ALIASED_OBJECT_NAME, "PR_EMS_AB_ALIASED_OBJECT_NAME" },
	{ PR_EMS_AB_ALIASED_OBJECT_NAME_UNICODE, "PR_EMS_AB_ALIASED_OBJECT_NAME_UNICODE" },
	{ PR_EMS_AB_ALT_RECIPIENT_O, "PR_EMS_AB_ALT_RECIPIENT_O" },
	{ PR_EMS_AB_ALT_RECIPIENT_O_ERROR, "PR_EMS_AB_ALT_RECIPIENT_O_ERROR" },
	{ PR_EMS_AB_ALT_RECIPIENT, "PR_EMS_AB_ALT_RECIPIENT" },
	{ PR_EMS_AB_ALT_RECIPIENT_UNICODE, "PR_EMS_AB_ALT_RECIPIENT_UNICODE" },
	{ PR_EMS_AB_ALT_RECIPIENT_BL_O, "PR_EMS_AB_ALT_RECIPIENT_BL_O" },
	{ PR_EMS_AB_ALT_RECIPIENT_BL_O_ERROR, "PR_EMS_AB_ALT_RECIPIENT_BL_O_ERROR" },
	{ PR_EMS_AB_ALT_RECIPIENT_BL, "PR_EMS_AB_ALT_RECIPIENT_BL" },
	{ PR_EMS_AB_ALT_RECIPIENT_BL_UNICODE, "PR_EMS_AB_ALT_RECIPIENT_BL_UNICODE" },
	{ PR_EMS_AB_ANCESTOR_ID, "PR_EMS_AB_ANCESTOR_ID" },
	{ PR_EMS_AB_ANCESTOR_ID_ERROR, "PR_EMS_AB_ANCESTOR_ID_ERROR" },
	{ PR_EMS_AB_ASSOC_REMOTE_DXA_O, "PR_EMS_AB_ASSOC_REMOTE_DXA_O" },
	{ PR_EMS_AB_ASSOC_REMOTE_DXA_O_ERROR, "PR_EMS_AB_ASSOC_REMOTE_DXA_O_ERROR" },
	{ PR_EMS_AB_ASSOC_REMOTE_DXA, "PR_EMS_AB_ASSOC_REMOTE_DXA" },
	{ PR_EMS_AB_ASSOC_REMOTE_DXA_UNICODE, "PR_EMS_AB_ASSOC_REMOTE_DXA_UNICODE" },
	{ PR_EMS_AB_ASSOCIATION_LIFETIME, "PR_EMS_AB_ASSOCIATION_LIFETIME" },
	{ PR_EMS_AB_ASSOCIATION_LIFETIME_ERROR, "PR_EMS_AB_ASSOCIATION_LIFETIME_ERROR" },
	{ PR_EMS_AB_AUTH_ORIG_BL_O, "PR_EMS_AB_AUTH_ORIG_BL_O" },
	{ PR_EMS_AB_AUTH_ORIG_BL_O_ERROR, "PR_EMS_AB_AUTH_ORIG_BL_O_ERROR" },
	{ PR_EMS_AB_AUTH_ORIG_BL, "PR_EMS_AB_AUTH_ORIG_BL" },
	{ PR_EMS_AB_AUTH_ORIG_BL_UNICODE, "PR_EMS_AB_AUTH_ORIG_BL_UNICODE" },
	{ PR_EMS_AB_AUTHORIZED_DOMAIN, "PR_EMS_AB_AUTHORIZED_DOMAIN" },
	{ PR_EMS_AB_AUTHORIZED_DOMAIN_UNICODE, "PR_EMS_AB_AUTHORIZED_DOMAIN_UNICODE" },
	{ PR_EMS_AB_AUTHORIZED_DOMAIN_ERROR, "PR_EMS_AB_AUTHORIZED_DOMAIN_ERROR" },
	{ PR_EMS_AB_AUTHORIZED_PASSWORD, "PR_EMS_AB_AUTHORIZED_PASSWORD" },
	{ PR_EMS_AB_AUTHORIZED_PASSWORD_ERROR, "PR_EMS_AB_AUTHORIZED_PASSWORD_ERROR" },
	{ PR_EMS_AB_AUTHORIZED_USER, "PR_EMS_AB_AUTHORIZED_USER" },
	{ PR_EMS_AB_AUTHORIZED_USER_UNICODE, "PR_EMS_AB_AUTHORIZED_USER_UNICODE" },
	{ PR_EMS_AB_AUTHORIZED_USER_ERROR, "PR_EMS_AB_AUTHORIZED_USER_ERROR" },
	{ PR_EMS_AB_BUSINESS_CATEGORY, "PR_EMS_AB_BUSINESS_CATEGORY" },
	{ PR_EMS_AB_BUSINESS_CATEGORY_UNICODE, "PR_EMS_AB_BUSINESS_CATEGORY_UNICODE" },
	{ PR_EMS_AB_BUSINESS_CATEGORY_ERROR, "PR_EMS_AB_BUSINESS_CATEGORY_ERROR" },
	{ PR_EMS_AB_CAN_CREATE_PF_O, "PR_EMS_AB_CAN_CREATE_PF_O" },
	{ PR_EMS_AB_CAN_CREATE_PF_O_ERROR, "PR_EMS_AB_CAN_CREATE_PF_O_ERROR" },
	{ PR_EMS_AB_CAN_CREATE_PF, "PR_EMS_AB_CAN_CREATE_PF" },
	{ PR_EMS_AB_CAN_CREATE_PF_UNICODE, "PR_EMS_AB_CAN_CREATE_PF_UNICODE" },
	{ PR_EMS_AB_CAN_CREATE_PF_BL_O, "PR_EMS_AB_CAN_CREATE_PF_BL_O" },
	{ PR_EMS_AB_CAN_CREATE_PF_BL_O_ERROR, "PR_EMS_AB_CAN_CREATE_PF_BL_O_ERROR" },
	{ PR_EMS_AB_CAN_CREATE_PF_BL, "PR_EMS_AB_CAN_CREATE_PF_BL" },
	{ PR_EMS_AB_CAN_CREATE_PF_BL_UNICODE, "PR_EMS_AB_CAN_CREATE_PF_BL_UNICODE" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_O, "PR_EMS_AB_CAN_CREATE_PF_DL_O" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_O_ERROR, "PR_EMS_AB_CAN_CREATE_PF_DL_O_ERROR" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL, "PR_EMS_AB_CAN_CREATE_PF_DL" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_UNICODE, "PR_EMS_AB_CAN_CREATE_PF_DL_UNICODE" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_BL_O, "PR_EMS_AB_CAN_CREATE_PF_DL_BL_O" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_BL_O_ERROR, "PR_EMS_AB_CAN_CREATE_PF_DL_BL_O_ERROR" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_BL, "PR_EMS_AB_CAN_CREATE_PF_DL_BL" },
	{ PR_EMS_AB_CAN_CREATE_PF_DL_BL_UNICODE, "PR_EMS_AB_CAN_CREATE_PF_DL_BL_UNICODE" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_O, "PR_EMS_AB_CAN_NOT_CREATE_PF_O" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_O_ERROR, "PR_EMS_AB_CAN_NOT_CREATE_PF_O_ERROR" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF, "PR_EMS_AB_CAN_NOT_CREATE_PF" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_UNICODE, "PR_EMS_AB_CAN_NOT_CREATE_PF_UNICODE" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O, "PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O_ERROR, "PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O_ERROR" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_BL, "PR_EMS_AB_CAN_NOT_CREATE_PF_BL" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_BL_UNICODE, "PR_EMS_AB_CAN_NOT_CREATE_PF_BL_UNICODE" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O_ERROR, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O_ERROR" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_UNICODE, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_UNICODE" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O_ERROR, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O_ERROR" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL" },
	{ PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_UNICODE, "PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_UNICODE" },
	{ PR_EMS_AB_CAN_PRESERVE_DNS, "PR_EMS_AB_CAN_PRESERVE_DNS" },
	{ PR_EMS_AB_CAN_PRESERVE_DNS_ERROR, "PR_EMS_AB_CAN_PRESERVE_DNS_ERROR" },
	{ PR_EMS_AB_CLOCK_ALERT_OFFSET, "PR_EMS_AB_CLOCK_ALERT_OFFSET" },
	{ PR_EMS_AB_CLOCK_ALERT_OFFSET_ERROR, "PR_EMS_AB_CLOCK_ALERT_OFFSET_ERROR" },
	{ PR_EMS_AB_CLOCK_ALERT_REPAIR, "PR_EMS_AB_CLOCK_ALERT_REPAIR" },
	{ PR_EMS_AB_CLOCK_ALERT_REPAIR_ERROR, "PR_EMS_AB_CLOCK_ALERT_REPAIR_ERROR" },
	{ PR_EMS_AB_CLOCK_WARNING_OFFSET, "PR_EMS_AB_CLOCK_WARNING_OFFSET" },
	{ PR_EMS_AB_CLOCK_WARNING_OFFSET_ERROR, "PR_EMS_AB_CLOCK_WARNING_OFFSET_ERROR" },
	{ PR_EMS_AB_CLOCK_WARNING_REPAIR, "PR_EMS_AB_CLOCK_WARNING_REPAIR" },
	{ PR_EMS_AB_CLOCK_WARNING_REPAIR_ERROR, "PR_EMS_AB_CLOCK_WARNING_REPAIR_ERROR" },
	{ PR_EMS_AB_COMPUTER_NAME, "PR_EMS_AB_COMPUTER_NAME" },
	{ PR_EMS_AB_COMPUTER_NAME_UNICODE, "PR_EMS_AB_COMPUTER_NAME_UNICODE" },
	{ PR_EMS_AB_COMPUTER_NAME_ERROR, "PR_EMS_AB_COMPUTER_NAME_ERROR" },
	{ PR_EMS_AB_CONNECTED_DOMAINS, "PR_EMS_AB_CONNECTED_DOMAINS" },
	{ PR_EMS_AB_CONNECTED_DOMAINS_UNICODE, "PR_EMS_AB_CONNECTED_DOMAINS_UNICODE" },
	{ PR_EMS_AB_CONNECTED_DOMAINS_ERROR, "PR_EMS_AB_CONNECTED_DOMAINS_ERROR" },
	{ PR_EMS_AB_CONTAINER_INFO, "PR_EMS_AB_CONTAINER_INFO" },
	{ PR_EMS_AB_CONTAINER_INFO_ERROR, "PR_EMS_AB_CONTAINER_INFO_ERROR" },
	{ PR_EMS_AB_COST, "PR_EMS_AB_COST" },
	{ PR_EMS_AB_COST_ERROR, "PR_EMS_AB_COST_ERROR" },
	{ PR_EMS_AB_COUNTRY_NAME, "PR_EMS_AB_COUNTRY_NAME" },
	{ PR_EMS_AB_COUNTRY_NAME_UNICODE, "PR_EMS_AB_COUNTRY_NAME_UNICODE" },
	{ PR_EMS_AB_COUNTRY_NAME_ERROR, "PR_EMS_AB_COUNTRY_NAME_ERROR" },
	{ PR_EMS_AB_DELIV_CONT_LENGTH, "PR_EMS_AB_DELIV_CONT_LENGTH" },
	{ PR_EMS_AB_DELIV_CONT_LENGTH_ERROR, "PR_EMS_AB_DELIV_CONT_LENGTH_ERROR" },
	{ PR_EMS_AB_DELIV_EITS, "PR_EMS_AB_DELIV_EITS" },
	{ PR_EMS_AB_DELIV_EITS_ERROR, "PR_EMS_AB_DELIV_EITS_ERROR" },
	{ PR_EMS_AB_DELIV_EXT_CONT_TYPES, "PR_EMS_AB_DELIV_EXT_CONT_TYPES" },
	{ PR_EMS_AB_DELIV_EXT_CONT_TYPES_ERROR, "PR_EMS_AB_DELIV_EXT_CONT_TYPES_ERROR" },
	{ PR_EMS_AB_DELIVER_AND_REDIRECT, "PR_EMS_AB_DELIVER_AND_REDIRECT" },
	{ PR_EMS_AB_DELIVER_AND_REDIRECT_ERROR, "PR_EMS_AB_DELIVER_AND_REDIRECT_ERROR" },
	{ PR_EMS_AB_DELIVERY_MECHANISM, "PR_EMS_AB_DELIVERY_MECHANISM" },
	{ PR_EMS_AB_DELIVERY_MECHANISM_ERROR, "PR_EMS_AB_DELIVERY_MECHANISM_ERROR" },
	{ PR_EMS_AB_DESCRIPTION, "PR_EMS_AB_DESCRIPTION" },
	{ PR_EMS_AB_DESCRIPTION_UNICODE, "PR_EMS_AB_DESCRIPTION_UNICODE" },
	{ PR_EMS_AB_DESCRIPTION_ERROR, "PR_EMS_AB_DESCRIPTION_ERROR" },
	{ PR_EMS_AB_DESTINATION_INDICATOR, "PR_EMS_AB_DESTINATION_INDICATOR" },
	{ PR_EMS_AB_DESTINATION_INDICATOR_UNICODE, "PR_EMS_AB_DESTINATION_INDICATOR_UNICODE" },
	{ PR_EMS_AB_DESTINATION_INDICATOR_ERROR, "PR_EMS_AB_DESTINATION_INDICATOR_ERROR" },
	{ PR_EMS_AB_DIAGNOSTIC_REG_KEY, "PR_EMS_AB_DIAGNOSTIC_REG_KEY" },
	{ PR_EMS_AB_DIAGNOSTIC_REG_KEY_UNICODE, "PR_EMS_AB_DIAGNOSTIC_REG_KEY_UNICODE" },
	{ PR_EMS_AB_DIAGNOSTIC_REG_KEY_ERROR, "PR_EMS_AB_DIAGNOSTIC_REG_KEY_ERROR" },
	{ PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O, "PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O" },
	{ PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O_ERROR, "PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O_ERROR" },
	{ PR_EMS_AB_DL_MEM_REJECT_PERMS_BL, "PR_EMS_AB_DL_MEM_REJECT_PERMS_BL" },
	{ PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_UNICODE, "PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_UNICODE" },
	{ PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O, "PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O" },
	{ PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O_ERROR, "PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O_ERROR" },
	{ PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL, "PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL" },
	{ PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_UNICODE, "PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_UNICODE" },
	{ PR_EMS_AB_DL_MEMBER_RULE, "PR_EMS_AB_DL_MEMBER_RULE" },
	{ PR_EMS_AB_DL_MEMBER_RULE_ERROR, "PR_EMS_AB_DL_MEMBER_RULE_ERROR" },
	{ PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O, "PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O" },
	{ PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O_ERROR, "PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O_ERROR" },
	{ PR_EMS_AB_DOMAIN_DEF_ALT_RECIP, "PR_EMS_AB_DOMAIN_DEF_ALT_RECIP" },
	{ PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_UNICODE, "PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_UNICODE" },
	{ PR_EMS_AB_DOMAIN_NAME, "PR_EMS_AB_DOMAIN_NAME" },
	{ PR_EMS_AB_DOMAIN_NAME_UNICODE, "PR_EMS_AB_DOMAIN_NAME_UNICODE" },
	{ PR_EMS_AB_DOMAIN_NAME_ERROR, "PR_EMS_AB_DOMAIN_NAME_ERROR" },
	{ PR_EMS_AB_DSA_SIGNATURE, "PR_EMS_AB_DSA_SIGNATURE" },
	{ PR_EMS_AB_DSA_SIGNATURE_ERROR, "PR_EMS_AB_DSA_SIGNATURE_ERROR" },
	{ PR_EMS_AB_DXA_ADMIN_COPY, "PR_EMS_AB_DXA_ADMIN_COPY" },
	{ PR_EMS_AB_DXA_ADMIN_COPY_ERROR, "PR_EMS_AB_DXA_ADMIN_COPY_ERROR" },
	{ PR_EMS_AB_DXA_ADMIN_FORWARD, "PR_EMS_AB_DXA_ADMIN_FORWARD" },
	{ PR_EMS_AB_DXA_ADMIN_FORWARD_ERROR, "PR_EMS_AB_DXA_ADMIN_FORWARD_ERROR" },
	{ PR_EMS_AB_DXA_ADMIN_UPDATE, "PR_EMS_AB_DXA_ADMIN_UPDATE" },
	{ PR_EMS_AB_DXA_ADMIN_UPDATE_ERROR, "PR_EMS_AB_DXA_ADMIN_UPDATE_ERROR" },
	{ PR_EMS_AB_DXA_APPEND_REQCN, "PR_EMS_AB_DXA_APPEND_REQCN" },
	{ PR_EMS_AB_DXA_APPEND_REQCN_ERROR, "PR_EMS_AB_DXA_APPEND_REQCN_ERROR" },
	{ PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O, "PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O" },
	{ PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O_ERROR, "PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O_ERROR" },
	{ PR_EMS_AB_DXA_CONF_CONTAINER_LIST, "PR_EMS_AB_DXA_CONF_CONTAINER_LIST" },
	{ PR_EMS_AB_DXA_CONF_CONTAINER_LIST_UNICODE, "PR_EMS_AB_DXA_CONF_CONTAINER_LIST_UNICODE" },
	{ PR_EMS_AB_DXA_CONF_REQ_TIME, "PR_EMS_AB_DXA_CONF_REQ_TIME" },
	{ PR_EMS_AB_DXA_CONF_REQ_TIME_ERROR, "PR_EMS_AB_DXA_CONF_REQ_TIME_ERROR" },
	{ PR_EMS_AB_DXA_CONF_SEQ, "PR_EMS_AB_DXA_CONF_SEQ" },
	{ PR_EMS_AB_DXA_CONF_SEQ_UNICODE, "PR_EMS_AB_DXA_CONF_SEQ_UNICODE" },
	{ PR_EMS_AB_DXA_CONF_SEQ_ERROR, "PR_EMS_AB_DXA_CONF_SEQ_ERROR" },
	{ PR_EMS_AB_DXA_CONF_SEQ_USN, "PR_EMS_AB_DXA_CONF_SEQ_USN" },
	{ PR_EMS_AB_DXA_CONF_SEQ_USN_ERROR, "PR_EMS_AB_DXA_CONF_SEQ_USN_ERROR" },
	{ PR_EMS_AB_DXA_EXCHANGE_OPTIONS, "PR_EMS_AB_DXA_EXCHANGE_OPTIONS" },
	{ PR_EMS_AB_DXA_EXCHANGE_OPTIONS_ERROR, "PR_EMS_AB_DXA_EXCHANGE_OPTIONS_ERROR" },
	{ PR_EMS_AB_DXA_EXPORT_NOW, "PR_EMS_AB_DXA_EXPORT_NOW" },
	{ PR_EMS_AB_DXA_EXPORT_NOW_ERROR, "PR_EMS_AB_DXA_EXPORT_NOW_ERROR" },
	{ PR_EMS_AB_DXA_FLAGS, "PR_EMS_AB_DXA_FLAGS" },
	{ PR_EMS_AB_DXA_FLAGS_ERROR, "PR_EMS_AB_DXA_FLAGS_ERROR" },
	{ PR_EMS_AB_DXA_IMP_SEQ, "PR_EMS_AB_DXA_IMP_SEQ" },
	{ PR_EMS_AB_DXA_IMP_SEQ_UNICODE, "PR_EMS_AB_DXA_IMP_SEQ_UNICODE" },
	{ PR_EMS_AB_DXA_IMP_SEQ_ERROR, "PR_EMS_AB_DXA_IMP_SEQ_ERROR" },
	{ PR_EMS_AB_DXA_IMP_SEQ_TIME, "PR_EMS_AB_DXA_IMP_SEQ_TIME" },
	{ PR_EMS_AB_DXA_IMP_SEQ_TIME_ERROR, "PR_EMS_AB_DXA_IMP_SEQ_TIME_ERROR" },
	{ PR_EMS_AB_DXA_IMP_SEQ_USN, "PR_EMS_AB_DXA_IMP_SEQ_USN" },
	{ PR_EMS_AB_DXA_IMP_SEQ_USN_ERROR, "PR_EMS_AB_DXA_IMP_SEQ_USN_ERROR" },
	{ PR_EMS_AB_DXA_IMPORT_NOW, "PR_EMS_AB_DXA_IMPORT_NOW" },
	{ PR_EMS_AB_DXA_IMPORT_NOW_ERROR, "PR_EMS_AB_DXA_IMPORT_NOW_ERROR" },
	{ PR_EMS_AB_DXA_IN_TEMPLATE_MAP, "PR_EMS_AB_DXA_IN_TEMPLATE_MAP" },
	{ PR_EMS_AB_DXA_IN_TEMPLATE_MAP_UNICODE, "PR_EMS_AB_DXA_IN_TEMPLATE_MAP_UNICODE" },
	{ PR_EMS_AB_DXA_IN_TEMPLATE_MAP_ERROR, "PR_EMS_AB_DXA_IN_TEMPLATE_MAP_ERROR" },
	{ PR_EMS_AB_DXA_LOCAL_ADMIN_O, "PR_EMS_AB_DXA_LOCAL_ADMIN_O" },
	{ PR_EMS_AB_DXA_LOCAL_ADMIN_O_ERROR, "PR_EMS_AB_DXA_LOCAL_ADMIN_O_ERROR" },
	{ PR_EMS_AB_DXA_LOCAL_ADMIN, "PR_EMS_AB_DXA_LOCAL_ADMIN" },
	{ PR_EMS_AB_DXA_LOCAL_ADMIN_UNICODE, "PR_EMS_AB_DXA_LOCAL_ADMIN_UNICODE" },
	{ PR_EMS_AB_DXA_LOGGING_LEVEL, "PR_EMS_AB_DXA_LOGGING_LEVEL" },
	{ PR_EMS_AB_DXA_LOGGING_LEVEL_ERROR, "PR_EMS_AB_DXA_LOGGING_LEVEL_ERROR" },
	{ PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE, "PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE" },
	{ PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_UNICODE, "PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_UNICODE" },
	{ PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_ERROR, "PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_ERROR" },
	{ PR_EMS_AB_DXA_OUT_TEMPLATE_MAP, "PR_EMS_AB_DXA_OUT_TEMPLATE_MAP" },
	{ PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_UNICODE, "PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_UNICODE" },
	{ PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_ERROR, "PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_ERROR" },
	{ PR_EMS_AB_DXA_PASSWORD, "PR_EMS_AB_DXA_PASSWORD" },
	{ PR_EMS_AB_DXA_PASSWORD_UNICODE, "PR_EMS_AB_DXA_PASSWORD_UNICODE" },
	{ PR_EMS_AB_DXA_PASSWORD_ERROR, "PR_EMS_AB_DXA_PASSWORD_ERROR" },
	{ PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS, "PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS" },
	{ PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS_ERROR, "PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS_ERROR" },
	{ PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY, "PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY" },
	{ PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY_ERROR, "PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY_ERROR" },
	{ PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY, "PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY" },
	{ PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY_ERROR, "PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY_ERROR" },
	{ PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O, "PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O" },
	{ PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O_ERROR, "PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O_ERROR" },
	{ PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES, "PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES" },
	{ PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_UNICODE, "PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_UNICODE" },
	{ PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY, "PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY" },
	{ PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY_ERROR, "PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY_ERROR" },
	{ PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS, "PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS" },
	{ PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS_ERROR, "PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS_ERROR" },
	{ PR_EMS_AB_DXA_PREV_TYPES, "PR_EMS_AB_DXA_PREV_TYPES" },
	{ PR_EMS_AB_DXA_PREV_TYPES_ERROR, "PR_EMS_AB_DXA_PREV_TYPES_ERROR" },
	{ PR_EMS_AB_DXA_RECIPIENT_CP, "PR_EMS_AB_DXA_RECIPIENT_CP" },
	{ PR_EMS_AB_DXA_RECIPIENT_CP_UNICODE, "PR_EMS_AB_DXA_RECIPIENT_CP_UNICODE" },
	{ PR_EMS_AB_DXA_RECIPIENT_CP_ERROR, "PR_EMS_AB_DXA_RECIPIENT_CP_ERROR" },
	{ PR_EMS_AB_DXA_REMOTE_CLIENT_O, "PR_EMS_AB_DXA_REMOTE_CLIENT_O" },
	{ PR_EMS_AB_DXA_REMOTE_CLIENT_O_ERROR, "PR_EMS_AB_DXA_REMOTE_CLIENT_O_ERROR" },
	{ PR_EMS_AB_DXA_REMOTE_CLIENT, "PR_EMS_AB_DXA_REMOTE_CLIENT" },
	{ PR_EMS_AB_DXA_REMOTE_CLIENT_UNICODE, "PR_EMS_AB_DXA_REMOTE_CLIENT_UNICODE" },
	{ PR_EMS_AB_DXA_REQ_SEQ, "PR_EMS_AB_DXA_REQ_SEQ" },
	{ PR_EMS_AB_DXA_REQ_SEQ_UNICODE, "PR_EMS_AB_DXA_REQ_SEQ_UNICODE" },
	{ PR_EMS_AB_DXA_REQ_SEQ_ERROR, "PR_EMS_AB_DXA_REQ_SEQ_ERROR" },
	{ PR_EMS_AB_DXA_REQ_SEQ_TIME, "PR_EMS_AB_DXA_REQ_SEQ_TIME" },
	{ PR_EMS_AB_DXA_REQ_SEQ_TIME_ERROR, "PR_EMS_AB_DXA_REQ_SEQ_TIME_ERROR" },
	{ PR_EMS_AB_DXA_REQ_SEQ_USN, "PR_EMS_AB_DXA_REQ_SEQ_USN" },
	{ PR_EMS_AB_DXA_REQ_SEQ_USN_ERROR, "PR_EMS_AB_DXA_REQ_SEQ_USN_ERROR" },
	{ PR_EMS_AB_DXA_REQNAME, "PR_EMS_AB_DXA_REQNAME" },
	{ PR_EMS_AB_DXA_REQNAME_UNICODE, "PR_EMS_AB_DXA_REQNAME_UNICODE" },
	{ PR_EMS_AB_DXA_REQNAME_ERROR, "PR_EMS_AB_DXA_REQNAME_ERROR" },
	{ PR_EMS_AB_DXA_SVR_SEQ, "PR_EMS_AB_DXA_SVR_SEQ" },
	{ PR_EMS_AB_DXA_SVR_SEQ_UNICODE, "PR_EMS_AB_DXA_SVR_SEQ_UNICODE" },
	{ PR_EMS_AB_DXA_SVR_SEQ_ERROR, "PR_EMS_AB_DXA_SVR_SEQ_ERROR" },
	{ PR_EMS_AB_DXA_SVR_SEQ_TIME, "PR_EMS_AB_DXA_SVR_SEQ_TIME" },
	{ PR_EMS_AB_DXA_SVR_SEQ_TIME_ERROR, "PR_EMS_AB_DXA_SVR_SEQ_TIME_ERROR" },
	{ PR_EMS_AB_DXA_SVR_SEQ_USN, "PR_EMS_AB_DXA_SVR_SEQ_USN" },
	{ PR_EMS_AB_DXA_SVR_SEQ_USN_ERROR, "PR_EMS_AB_DXA_SVR_SEQ_USN_ERROR" },
	{ PR_EMS_AB_DXA_TASK, "PR_EMS_AB_DXA_TASK" },
	{ PR_EMS_AB_DXA_TASK_ERROR, "PR_EMS_AB_DXA_TASK_ERROR" },
	{ PR_EMS_AB_DXA_TEMPLATE_OPTIONS, "PR_EMS_AB_DXA_TEMPLATE_OPTIONS" },
	{ PR_EMS_AB_DXA_TEMPLATE_OPTIONS_ERROR, "PR_EMS_AB_DXA_TEMPLATE_OPTIONS_ERROR" },
	{ PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP, "PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP" },
	{ PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP_ERROR, "PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP_ERROR" },
	{ PR_EMS_AB_DXA_TYPES, "PR_EMS_AB_DXA_TYPES" },
	{ PR_EMS_AB_DXA_TYPES_ERROR, "PR_EMS_AB_DXA_TYPES_ERROR" },
	{ PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O, "PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O" },
	{ PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O_ERROR, "PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O_ERROR" },
	{ PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST, "PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST" },
	{ PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_UNICODE, "PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_UNICODE" },
	{ PR_EMS_AB_ENCAPSULATION_METHOD, "PR_EMS_AB_ENCAPSULATION_METHOD" },
	{ PR_EMS_AB_ENCAPSULATION_METHOD_ERROR, "PR_EMS_AB_ENCAPSULATION_METHOD_ERROR" },
	{ PR_EMS_AB_ENCRYPT, "PR_EMS_AB_ENCRYPT" },
	{ PR_EMS_AB_ENCRYPT_ERROR, "PR_EMS_AB_ENCRYPT_ERROR" },
	{ PR_EMS_AB_EXPAND_DLS_LOCALLY, "PR_EMS_AB_EXPAND_DLS_LOCALLY" },
	{ PR_EMS_AB_EXPAND_DLS_LOCALLY_ERROR, "PR_EMS_AB_EXPAND_DLS_LOCALLY_ERROR" },
	{ PR_EMS_AB_EXPORT_CONTAINERS_O, "PR_EMS_AB_EXPORT_CONTAINERS_O" },
	{ PR_EMS_AB_EXPORT_CONTAINERS_O_ERROR, "PR_EMS_AB_EXPORT_CONTAINERS_O_ERROR" },
	{ PR_EMS_AB_EXPORT_CONTAINERS, "PR_EMS_AB_EXPORT_CONTAINERS" },
	{ PR_EMS_AB_EXPORT_CONTAINERS_UNICODE, "PR_EMS_AB_EXPORT_CONTAINERS_UNICODE" },
	{ PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS, "PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS" },
	{ PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS_ERROR, "PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS_ERROR" },
	{ PR_EMS_AB_EXTENDED_CHARS_ALLOWED, "PR_EMS_AB_EXTENDED_CHARS_ALLOWED" },
	{ PR_EMS_AB_EXTENDED_CHARS_ALLOWED_ERROR, "PR_EMS_AB_EXTENDED_CHARS_ALLOWED_ERROR" },
	{ PR_EMS_AB_EXTENSION_DATA, "PR_EMS_AB_EXTENSION_DATA" },
	{ PR_EMS_AB_EXTENSION_DATA_ERROR, "PR_EMS_AB_EXTENSION_DATA_ERROR" },
	{ PR_EMS_AB_EXTENSION_NAME, "PR_EMS_AB_EXTENSION_NAME" },
	{ PR_EMS_AB_EXTENSION_NAME_UNICODE, "PR_EMS_AB_EXTENSION_NAME_UNICODE" },
	{ PR_EMS_AB_EXTENSION_NAME_ERROR, "PR_EMS_AB_EXTENSION_NAME_ERROR" },
	{ PR_EMS_AB_EXTENSION_NAME_INHERITED, "PR_EMS_AB_EXTENSION_NAME_INHERITED" },
	{ PR_EMS_AB_EXTENSION_NAME_INHERITED_UNICODE, "PR_EMS_AB_EXTENSION_NAME_INHERITED_UNICODE" },
	{ PR_EMS_AB_EXTENSION_NAME_INHERITED_ERROR, "PR_EMS_AB_EXTENSION_NAME_INHERITED_ERROR" },
	{ PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER, "PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER" },
	{ PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER_ERROR, "PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER_ERROR" },
	{ PR_EMS_AB_FILE_VERSION, "PR_EMS_AB_FILE_VERSION" },
	{ PR_EMS_AB_FILE_VERSION_ERROR, "PR_EMS_AB_FILE_VERSION_ERROR" },
	{ PR_EMS_AB_FILTER_LOCAL_ADDRESSES, "PR_EMS_AB_FILTER_LOCAL_ADDRESSES" },
	{ PR_EMS_AB_FILTER_LOCAL_ADDRESSES_ERROR, "PR_EMS_AB_FILTER_LOCAL_ADDRESSES_ERROR" },
	{ PR_EMS_AB_FOLDERS_CONTAINER_O, "PR_EMS_AB_FOLDERS_CONTAINER_O" },
	{ PR_EMS_AB_FOLDERS_CONTAINER_O_ERROR, "PR_EMS_AB_FOLDERS_CONTAINER_O_ERROR" },
	{ PR_EMS_AB_FOLDERS_CONTAINER, "PR_EMS_AB_FOLDERS_CONTAINER" },
	{ PR_EMS_AB_FOLDERS_CONTAINER_UNICODE, "PR_EMS_AB_FOLDERS_CONTAINER_UNICODE" },
	{ PR_EMS_AB_GARBAGE_COLL_PERIOD, "PR_EMS_AB_GARBAGE_COLL_PERIOD" },
	{ PR_EMS_AB_GARBAGE_COLL_PERIOD_ERROR, "PR_EMS_AB_GARBAGE_COLL_PERIOD_ERROR" },
	{ PR_EMS_AB_GATEWAY_LOCAL_CRED, "PR_EMS_AB_GATEWAY_LOCAL_CRED" },
	{ PR_EMS_AB_GATEWAY_LOCAL_CRED_UNICODE, "PR_EMS_AB_GATEWAY_LOCAL_CRED_UNICODE" },
	{ PR_EMS_AB_GATEWAY_LOCAL_CRED_ERROR, "PR_EMS_AB_GATEWAY_LOCAL_CRED_ERROR" },
	{ PR_EMS_AB_GATEWAY_LOCAL_DESIG, "PR_EMS_AB_GATEWAY_LOCAL_DESIG" },
	{ PR_EMS_AB_GATEWAY_LOCAL_DESIG_UNICODE, "PR_EMS_AB_GATEWAY_LOCAL_DESIG_UNICODE" },
	{ PR_EMS_AB_GATEWAY_LOCAL_DESIG_ERROR, "PR_EMS_AB_GATEWAY_LOCAL_DESIG_ERROR" },
	{ PR_EMS_AB_GATEWAY_PROXY, "PR_EMS_AB_GATEWAY_PROXY" },
	{ PR_EMS_AB_GATEWAY_PROXY_UNICODE, "PR_EMS_AB_GATEWAY_PROXY_UNICODE" },
	{ PR_EMS_AB_GATEWAY_PROXY_ERROR, "PR_EMS_AB_GATEWAY_PROXY_ERROR" },
	{ PR_EMS_AB_GATEWAY_ROUTING_TREE, "PR_EMS_AB_GATEWAY_ROUTING_TREE" },
	{ PR_EMS_AB_GATEWAY_ROUTING_TREE_ERROR, "PR_EMS_AB_GATEWAY_ROUTING_TREE_ERROR" },
	{ PR_EMS_AB_GWART_LAST_MODIFIED, "PR_EMS_AB_GWART_LAST_MODIFIED" },
	{ PR_EMS_AB_GWART_LAST_MODIFIED_ERROR, "PR_EMS_AB_GWART_LAST_MODIFIED_ERROR" },
	{ PR_EMS_AB_HAS_FULL_REPLICA_NCS_O, "PR_EMS_AB_HAS_FULL_REPLICA_NCS_O" },
	{ PR_EMS_AB_HAS_FULL_REPLICA_NCS_O_ERROR, "PR_EMS_AB_HAS_FULL_REPLICA_NCS_O_ERROR" },
	{ PR_EMS_AB_HAS_FULL_REPLICA_NCS, "PR_EMS_AB_HAS_FULL_REPLICA_NCS" },
	{ PR_EMS_AB_HAS_FULL_REPLICA_NCS_UNICODE, "PR_EMS_AB_HAS_FULL_REPLICA_NCS_UNICODE" },
	{ PR_EMS_AB_HAS_MASTER_NCS_O, "PR_EMS_AB_HAS_MASTER_NCS_O" },
	{ PR_EMS_AB_HAS_MASTER_NCS_O_ERROR, "PR_EMS_AB_HAS_MASTER_NCS_O_ERROR" },
	{ PR_EMS_AB_HAS_MASTER_NCS, "PR_EMS_AB_HAS_MASTER_NCS" },
	{ PR_EMS_AB_HAS_MASTER_NCS_UNICODE, "PR_EMS_AB_HAS_MASTER_NCS_UNICODE" },
	{ PR_EMS_AB_HEURISTICS, "PR_EMS_AB_HEURISTICS" },
	{ PR_EMS_AB_HEURISTICS_ERROR, "PR_EMS_AB_HEURISTICS_ERROR" },
	{ PR_EMS_AB_HIDE_DL_MEMBERSHIP, "PR_EMS_AB_HIDE_DL_MEMBERSHIP" },
	{ PR_EMS_AB_HIDE_DL_MEMBERSHIP_ERROR, "PR_EMS_AB_HIDE_DL_MEMBERSHIP_ERROR" },
	{ PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK, "PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK" },
	{ PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK_ERROR, "PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK_ERROR" },
	{ PR_EMS_AB_IMPORT_CONTAINER_O, "PR_EMS_AB_IMPORT_CONTAINER_O" },
	{ PR_EMS_AB_IMPORT_CONTAINER_O_ERROR, "PR_EMS_AB_IMPORT_CONTAINER_O_ERROR" },
	{ PR_EMS_AB_IMPORT_CONTAINER, "PR_EMS_AB_IMPORT_CONTAINER" },
	{ PR_EMS_AB_IMPORT_CONTAINER_UNICODE, "PR_EMS_AB_IMPORT_CONTAINER_UNICODE" },
	{ PR_EMS_AB_IMPORT_SENSITIVITY, "PR_EMS_AB_IMPORT_SENSITIVITY" },
	{ PR_EMS_AB_IMPORT_SENSITIVITY_ERROR, "PR_EMS_AB_IMPORT_SENSITIVITY_ERROR" },
	{ PR_EMS_AB_INBOUND_SITES_O, "PR_EMS_AB_INBOUND_SITES_O" },
	{ PR_EMS_AB_INBOUND_SITES_O_ERROR, "PR_EMS_AB_INBOUND_SITES_O_ERROR" },
	{ PR_EMS_AB_INBOUND_SITES, "PR_EMS_AB_INBOUND_SITES" },
	{ PR_EMS_AB_INBOUND_SITES_UNICODE, "PR_EMS_AB_INBOUND_SITES_UNICODE" },
	{ PR_EMS_AB_INSTANCE_TYPE, "PR_EMS_AB_INSTANCE_TYPE" },
	{ PR_EMS_AB_INSTANCE_TYPE_ERROR, "PR_EMS_AB_INSTANCE_TYPE_ERROR" },
	{ PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER, "PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER" },
	{ PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_UNICODE, "PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_UNICODE" },
	{ PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_ERROR, "PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_ERROR" },
	{ PR_EMS_AB_INVOCATION_ID, "PR_EMS_AB_INVOCATION_ID" },
	{ PR_EMS_AB_INVOCATION_ID_ERROR, "PR_EMS_AB_INVOCATION_ID_ERROR" },
	{ PR_EMS_AB_IS_DELETED, "PR_EMS_AB_IS_DELETED" },
	{ PR_EMS_AB_IS_DELETED_ERROR, "PR_EMS_AB_IS_DELETED_ERROR" },
	{ PR_EMS_AB_IS_SINGLE_VALUED, "PR_EMS_AB_IS_SINGLE_VALUED" },
	{ PR_EMS_AB_IS_SINGLE_VALUED_ERROR, "PR_EMS_AB_IS_SINGLE_VALUED_ERROR" },
	{ PR_EMS_AB_KCC_STATUS, "PR_EMS_AB_KCC_STATUS" },
	{ PR_EMS_AB_KCC_STATUS_ERROR, "PR_EMS_AB_KCC_STATUS_ERROR" },
	{ PR_EMS_AB_KNOWLEDGE_INFORMATION, "PR_EMS_AB_KNOWLEDGE_INFORMATION" },
	{ PR_EMS_AB_KNOWLEDGE_INFORMATION_UNICODE, "PR_EMS_AB_KNOWLEDGE_INFORMATION_UNICODE" },
	{ PR_EMS_AB_KNOWLEDGE_INFORMATION_ERROR, "PR_EMS_AB_KNOWLEDGE_INFORMATION_ERROR" },
	{ PR_EMS_AB_LINE_WRAP, "PR_EMS_AB_LINE_WRAP" },
	{ PR_EMS_AB_LINE_WRAP_ERROR, "PR_EMS_AB_LINE_WRAP_ERROR" },
	{ PR_EMS_AB_LINK_ID, "PR_EMS_AB_LINK_ID" },
	{ PR_EMS_AB_LINK_ID_ERROR, "PR_EMS_AB_LINK_ID_ERROR" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD, "PR_EMS_AB_LOCAL_BRIDGE_HEAD" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD_UNICODE, "PR_EMS_AB_LOCAL_BRIDGE_HEAD_UNICODE" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD_ERROR, "PR_EMS_AB_LOCAL_BRIDGE_HEAD_ERROR" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS, "PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_UNICODE, "PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_UNICODE" },
	{ PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_ERROR, "PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_ERROR" },
	{ PR_EMS_AB_LOCAL_INITIAL_TURN, "PR_EMS_AB_LOCAL_INITIAL_TURN" },
	{ PR_EMS_AB_LOCAL_INITIAL_TURN_ERROR, "PR_EMS_AB_LOCAL_INITIAL_TURN_ERROR" },
	{ PR_EMS_AB_LOCAL_SCOPE_O, "PR_EMS_AB_LOCAL_SCOPE_O" },
	{ PR_EMS_AB_LOCAL_SCOPE_O_ERROR, "PR_EMS_AB_LOCAL_SCOPE_O_ERROR" },
	{ PR_EMS_AB_LOCAL_SCOPE, "PR_EMS_AB_LOCAL_SCOPE" },
	{ PR_EMS_AB_LOCAL_SCOPE_UNICODE, "PR_EMS_AB_LOCAL_SCOPE_UNICODE" },
	{ PR_EMS_AB_LOG_FILENAME, "PR_EMS_AB_LOG_FILENAME" },
	{ PR_EMS_AB_LOG_FILENAME_UNICODE, "PR_EMS_AB_LOG_FILENAME_UNICODE" },
	{ PR_EMS_AB_LOG_FILENAME_ERROR, "PR_EMS_AB_LOG_FILENAME_ERROR" },
	{ PR_EMS_AB_LOG_ROLLOVER_INTERVAL, "PR_EMS_AB_LOG_ROLLOVER_INTERVAL" },
	{ PR_EMS_AB_LOG_ROLLOVER_INTERVAL_ERROR, "PR_EMS_AB_LOG_ROLLOVER_INTERVAL_ERROR" },
	{ PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY, "PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY" },
	{ PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY_ERROR, "PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY_ERROR" },
	{ PR_EMS_AB_MAPI_DISPLAY_TYPE, "PR_EMS_AB_MAPI_DISPLAY_TYPE" },
	{ PR_EMS_AB_MAPI_DISPLAY_TYPE_ERROR, "PR_EMS_AB_MAPI_DISPLAY_TYPE_ERROR" },
	{ PR_EMS_AB_MAPI_ID, "PR_EMS_AB_MAPI_ID" },
	{ PR_EMS_AB_MAPI_ID_ERROR, "PR_EMS_AB_MAPI_ID_ERROR" },
	{ PR_EMS_AB_MDB_BACKOFF_INTERVAL, "PR_EMS_AB_MDB_BACKOFF_INTERVAL" },
	{ PR_EMS_AB_MDB_BACKOFF_INTERVAL_ERROR, "PR_EMS_AB_MDB_BACKOFF_INTERVAL_ERROR" },
	{ PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD, "PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD" },
	{ PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD_ERROR, "PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD_ERROR" },
	{ PR_EMS_AB_MDB_OVER_QUOTA_LIMIT, "PR_EMS_AB_MDB_OVER_QUOTA_LIMIT" },
	{ PR_EMS_AB_MDB_OVER_QUOTA_LIMIT_ERROR, "PR_EMS_AB_MDB_OVER_QUOTA_LIMIT_ERROR" },
	{ PR_EMS_AB_MDB_STORAGE_QUOTA, "PR_EMS_AB_MDB_STORAGE_QUOTA" },
	{ PR_EMS_AB_MDB_STORAGE_QUOTA_ERROR, "PR_EMS_AB_MDB_STORAGE_QUOTA_ERROR" },
	{ PR_EMS_AB_MDB_UNREAD_LIMIT, "PR_EMS_AB_MDB_UNREAD_LIMIT" },
	{ PR_EMS_AB_MDB_UNREAD_LIMIT_ERROR, "PR_EMS_AB_MDB_UNREAD_LIMIT_ERROR" },
	{ PR_EMS_AB_MDB_USE_DEFAULTS, "PR_EMS_AB_MDB_USE_DEFAULTS" },
	{ PR_EMS_AB_MDB_USE_DEFAULTS_ERROR, "PR_EMS_AB_MDB_USE_DEFAULTS_ERROR" },
	{ PR_EMS_AB_MESSAGE_TRACKING_ENABLED, "PR_EMS_AB_MESSAGE_TRACKING_ENABLED" },
	{ PR_EMS_AB_MESSAGE_TRACKING_ENABLED_ERROR, "PR_EMS_AB_MESSAGE_TRACKING_ENABLED_ERROR" },
	{ PR_EMS_AB_MONITOR_CLOCK, "PR_EMS_AB_MONITOR_CLOCK" },
	{ PR_EMS_AB_MONITOR_CLOCK_ERROR, "PR_EMS_AB_MONITOR_CLOCK_ERROR" },
	{ PR_EMS_AB_MONITOR_SERVERS, "PR_EMS_AB_MONITOR_SERVERS" },
	{ PR_EMS_AB_MONITOR_SERVERS_ERROR, "PR_EMS_AB_MONITOR_SERVERS_ERROR" },
	{ PR_EMS_AB_MONITOR_SERVICES, "PR_EMS_AB_MONITOR_SERVICES" },
	{ PR_EMS_AB_MONITOR_SERVICES_ERROR, "PR_EMS_AB_MONITOR_SERVICES_ERROR" },
	{ PR_EMS_AB_MONITORED_CONFIGURATIONS_O, "PR_EMS_AB_MONITORED_CONFIGURATIONS_O" },
	{ PR_EMS_AB_MONITORED_CONFIGURATIONS_O_ERROR, "PR_EMS_AB_MONITORED_CONFIGURATIONS_O_ERROR" },
	{ PR_EMS_AB_MONITORED_CONFIGURATIONS, "PR_EMS_AB_MONITORED_CONFIGURATIONS" },
	{ PR_EMS_AB_MONITORED_CONFIGURATIONS_UNICODE, "PR_EMS_AB_MONITORED_CONFIGURATIONS_UNICODE" },
	{ PR_EMS_AB_MONITORED_SERVERS_O, "PR_EMS_AB_MONITORED_SERVERS_O" },
	{ PR_EMS_AB_MONITORED_SERVERS_O_ERROR, "PR_EMS_AB_MONITORED_SERVERS_O_ERROR" },
	{ PR_EMS_AB_MONITORED_SERVERS, "PR_EMS_AB_MONITORED_SERVERS" },
	{ PR_EMS_AB_MONITORED_SERVERS_UNICODE, "PR_EMS_AB_MONITORED_SERVERS_UNICODE" },
	{ PR_EMS_AB_MONITORED_SERVICES, "PR_EMS_AB_MONITORED_SERVICES" },
	{ PR_EMS_AB_MONITORED_SERVICES_UNICODE, "PR_EMS_AB_MONITORED_SERVICES_UNICODE" },
	{ PR_EMS_AB_MONITORED_SERVICES_ERROR, "PR_EMS_AB_MONITORED_SERVICES_ERROR" },
	{ PR_EMS_AB_MONITORING_ALERT_DELAY, "PR_EMS_AB_MONITORING_ALERT_DELAY" },
	{ PR_EMS_AB_MONITORING_ALERT_DELAY_ERROR, "PR_EMS_AB_MONITORING_ALERT_DELAY_ERROR" },
	{ PR_EMS_AB_MONITORING_ALERT_UNITS, "PR_EMS_AB_MONITORING_ALERT_UNITS" },
	{ PR_EMS_AB_MONITORING_ALERT_UNITS_ERROR, "PR_EMS_AB_MONITORING_ALERT_UNITS_ERROR" },
	{ PR_EMS_AB_MONITORING_AVAILABILITY_STYLE, "PR_EMS_AB_MONITORING_AVAILABILITY_STYLE" },
	{ PR_EMS_AB_MONITORING_AVAILABILITY_STYLE_ERROR, "PR_EMS_AB_MONITORING_AVAILABILITY_STYLE_ERROR" },
	{ PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW, "PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW" },
	{ PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW_ERROR, "PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW_ERROR" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O, "PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O_ERROR, "PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O_ERROR" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_MAIL, "PR_EMS_AB_MONITORING_CACHED_VIA_MAIL" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_UNICODE, "PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_UNICODE" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O, "PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O_ERROR, "PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O_ERROR" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_RPC, "PR_EMS_AB_MONITORING_CACHED_VIA_RPC" },
	{ PR_EMS_AB_MONITORING_CACHED_VIA_RPC_UNICODE, "PR_EMS_AB_MONITORING_CACHED_VIA_RPC_UNICODE" },
	{ PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE, "PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE" },
	{ PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE_ERROR, "PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE_ERROR" },
	{ PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL, "PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL" },
	{ PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL_ERROR, "PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL_ERROR" },
	{ PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS, "PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS" },
	{ PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS_ERROR, "PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS_ERROR" },
	{ PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL, "PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL" },
	{ PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL_ERROR, "PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL_ERROR" },
	{ PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS, "PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS" },
	{ PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS_ERROR, "PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS_ERROR" },
	{ PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL, "PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL" },
	{ PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL_ERROR, "PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL_ERROR" },
	{ PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS, "PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS" },
	{ PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS_ERROR, "PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS_ERROR" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_O, "PR_EMS_AB_MONITORING_RECIPIENTS_O" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_O_ERROR, "PR_EMS_AB_MONITORING_RECIPIENTS_O_ERROR" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS, "PR_EMS_AB_MONITORING_RECIPIENTS" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_UNICODE, "PR_EMS_AB_MONITORING_RECIPIENTS_UNICODE" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O, "PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O_ERROR, "PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O_ERROR" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_NDR, "PR_EMS_AB_MONITORING_RECIPIENTS_NDR" },
	{ PR_EMS_AB_MONITORING_RECIPIENTS_NDR_UNICODE, "PR_EMS_AB_MONITORING_RECIPIENTS_NDR_UNICODE" },
	{ PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL, "PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL" },
	{ PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL_ERROR, "PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL_ERROR" },
	{ PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS, "PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS" },
	{ PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS_ERROR, "PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS_ERROR" },
	{ PR_EMS_AB_MONITORING_WARNING_DELAY, "PR_EMS_AB_MONITORING_WARNING_DELAY" },
	{ PR_EMS_AB_MONITORING_WARNING_DELAY_ERROR, "PR_EMS_AB_MONITORING_WARNING_DELAY_ERROR" },
	{ PR_EMS_AB_MONITORING_WARNING_UNITS, "PR_EMS_AB_MONITORING_WARNING_UNITS" },
	{ PR_EMS_AB_MONITORING_WARNING_UNITS_ERROR, "PR_EMS_AB_MONITORING_WARNING_UNITS_ERROR" },
	{ PR_EMS_AB_MTA_LOCAL_CRED, "PR_EMS_AB_MTA_LOCAL_CRED" },
	{ PR_EMS_AB_MTA_LOCAL_CRED_UNICODE, "PR_EMS_AB_MTA_LOCAL_CRED_UNICODE" },
	{ PR_EMS_AB_MTA_LOCAL_CRED_ERROR, "PR_EMS_AB_MTA_LOCAL_CRED_ERROR" },
	{ PR_EMS_AB_MTA_LOCAL_DESIG, "PR_EMS_AB_MTA_LOCAL_DESIG" },
	{ PR_EMS_AB_MTA_LOCAL_DESIG_UNICODE, "PR_EMS_AB_MTA_LOCAL_DESIG_UNICODE" },
	{ PR_EMS_AB_MTA_LOCAL_DESIG_ERROR, "PR_EMS_AB_MTA_LOCAL_DESIG_ERROR" },
	{ PR_EMS_AB_N_ADDRESS, "PR_EMS_AB_N_ADDRESS" },
	{ PR_EMS_AB_N_ADDRESS_ERROR, "PR_EMS_AB_N_ADDRESS_ERROR" },
	{ PR_EMS_AB_N_ADDRESS_TYPE, "PR_EMS_AB_N_ADDRESS_TYPE" },
	{ PR_EMS_AB_N_ADDRESS_TYPE_ERROR, "PR_EMS_AB_N_ADDRESS_TYPE_ERROR" },
	{ PR_EMS_AB_NT_MACHINE_NAME, "PR_EMS_AB_NT_MACHINE_NAME" },
	{ PR_EMS_AB_NT_MACHINE_NAME_UNICODE, "PR_EMS_AB_NT_MACHINE_NAME_UNICODE" },
	{ PR_EMS_AB_NT_MACHINE_NAME_ERROR, "PR_EMS_AB_NT_MACHINE_NAME_ERROR" },
	{ PR_EMS_AB_NUM_OF_OPEN_RETRIES, "PR_EMS_AB_NUM_OF_OPEN_RETRIES" },
	{ PR_EMS_AB_NUM_OF_OPEN_RETRIES_ERROR, "PR_EMS_AB_NUM_OF_OPEN_RETRIES_ERROR" },
	{ PR_EMS_AB_NUM_OF_TRANSFER_RETRIES, "PR_EMS_AB_NUM_OF_TRANSFER_RETRIES" },
	{ PR_EMS_AB_NUM_OF_TRANSFER_RETRIES_ERROR, "PR_EMS_AB_NUM_OF_TRANSFER_RETRIES_ERROR" },
	{ PR_EMS_AB_OBJECT_CLASS_CATEGORY, "PR_EMS_AB_OBJECT_CLASS_CATEGORY" },
	{ PR_EMS_AB_OBJECT_CLASS_CATEGORY_ERROR, "PR_EMS_AB_OBJECT_CLASS_CATEGORY_ERROR" },
	{ PR_EMS_AB_OBJECT_VERSION, "PR_EMS_AB_OBJECT_VERSION" },
	{ PR_EMS_AB_OBJECT_VERSION_ERROR, "PR_EMS_AB_OBJECT_VERSION_ERROR" },
	{ PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O, "PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O" },
	{ PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O_ERROR, "PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O_ERROR" },
	{ PR_EMS_AB_OFF_LINE_AB_CONTAINERS, "PR_EMS_AB_OFF_LINE_AB_CONTAINERS" },
	{ PR_EMS_AB_OFF_LINE_AB_CONTAINERS_UNICODE, "PR_EMS_AB_OFF_LINE_AB_CONTAINERS_UNICODE" },
	{ PR_EMS_AB_OFF_LINE_AB_SCHEDULE, "PR_EMS_AB_OFF_LINE_AB_SCHEDULE" },
	{ PR_EMS_AB_OFF_LINE_AB_SCHEDULE_ERROR, "PR_EMS_AB_OFF_LINE_AB_SCHEDULE_ERROR" },
	{ PR_EMS_AB_OFF_LINE_AB_SERVER_O, "PR_EMS_AB_OFF_LINE_AB_SERVER_O" },
	{ PR_EMS_AB_OFF_LINE_AB_SERVER_O_ERROR, "PR_EMS_AB_OFF_LINE_AB_SERVER_O_ERROR" },
	{ PR_EMS_AB_OFF_LINE_AB_SERVER, "PR_EMS_AB_OFF_LINE_AB_SERVER" },
	{ PR_EMS_AB_OFF_LINE_AB_SERVER_UNICODE, "PR_EMS_AB_OFF_LINE_AB_SERVER_UNICODE" },
	{ PR_EMS_AB_OFF_LINE_AB_STYLE, "PR_EMS_AB_OFF_LINE_AB_STYLE" },
	{ PR_EMS_AB_OFF_LINE_AB_STYLE_ERROR, "PR_EMS_AB_OFF_LINE_AB_STYLE_ERROR" },
	{ PR_EMS_AB_OID_TYPE, "PR_EMS_AB_OID_TYPE" },
	{ PR_EMS_AB_OID_TYPE_ERROR, "PR_EMS_AB_OID_TYPE_ERROR" },
	{ PR_EMS_AB_OM_OBJECT_CLASS, "PR_EMS_AB_OM_OBJECT_CLASS" },
	{ PR_EMS_AB_OM_OBJECT_CLASS_ERROR, "PR_EMS_AB_OM_OBJECT_CLASS_ERROR" },
	{ PR_EMS_AB_OM_SYNTAX, "PR_EMS_AB_OM_SYNTAX" },
	{ PR_EMS_AB_OM_SYNTAX_ERROR, "PR_EMS_AB_OM_SYNTAX_ERROR" },
	{ PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR, "PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR" },
	{ PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR_ERROR, "PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR_ERROR" },
	{ PR_EMS_AB_OPEN_RETRY_INTERVAL, "PR_EMS_AB_OPEN_RETRY_INTERVAL" },
	{ PR_EMS_AB_OPEN_RETRY_INTERVAL_ERROR, "PR_EMS_AB_OPEN_RETRY_INTERVAL_ERROR" },
	{ PR_EMS_AB_ORGANIZATION_NAME, "PR_EMS_AB_ORGANIZATION_NAME" },
	{ PR_EMS_AB_ORGANIZATION_NAME_UNICODE, "PR_EMS_AB_ORGANIZATION_NAME_UNICODE" },
	{ PR_EMS_AB_ORGANIZATION_NAME_ERROR, "PR_EMS_AB_ORGANIZATION_NAME_ERROR" },
	{ PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME, "PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME" },
	{ PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_UNICODE, "PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_UNICODE" },
	{ PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_ERROR, "PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_ERROR" },
	{ PR_EMS_AB_ORIGINAL_DISPLAY_TABLE, "PR_EMS_AB_ORIGINAL_DISPLAY_TABLE" },
	{ PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_ERROR, "PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_ERROR" },
	{ PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS, "PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS" },
	{ PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS_ERROR, "PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS_ERROR" },
	{ PR_EMS_AB_OUTBOUND_SITES_O, "PR_EMS_AB_OUTBOUND_SITES_O" },
	{ PR_EMS_AB_OUTBOUND_SITES_O_ERROR, "PR_EMS_AB_OUTBOUND_SITES_O_ERROR" },
	{ PR_EMS_AB_OUTBOUND_SITES, "PR_EMS_AB_OUTBOUND_SITES" },
	{ PR_EMS_AB_OUTBOUND_SITES_UNICODE, "PR_EMS_AB_OUTBOUND_SITES_UNICODE" },
	{ PR_EMS_AB_P_SELECTOR, "PR_EMS_AB_P_SELECTOR" },
	{ PR_EMS_AB_P_SELECTOR_ERROR, "PR_EMS_AB_P_SELECTOR_ERROR" },
	{ PR_EMS_AB_P_SELECTOR_INBOUND, "PR_EMS_AB_P_SELECTOR_INBOUND" },
	{ PR_EMS_AB_P_SELECTOR_INBOUND_ERROR, "PR_EMS_AB_P_SELECTOR_INBOUND_ERROR" },
	{ PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE, "PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE" },
	{ PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE_ERROR, "PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE_ERROR" },
	{ PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE, "PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE" },
	{ PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE_ERROR, "PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE_ERROR" },
	{ PR_EMS_AB_PERIOD_REP_SYNC_TIMES, "PR_EMS_AB_PERIOD_REP_SYNC_TIMES" },
	{ PR_EMS_AB_PERIOD_REP_SYNC_TIMES_ERROR, "PR_EMS_AB_PERIOD_REP_SYNC_TIMES_ERROR" },
	{ PR_EMS_AB_PERIOD_REPL_STAGGER, "PR_EMS_AB_PERIOD_REPL_STAGGER" },
	{ PR_EMS_AB_PERIOD_REPL_STAGGER_ERROR, "PR_EMS_AB_PERIOD_REPL_STAGGER_ERROR" },
	{ PR_EMS_AB_POSTAL_ADDRESS, "PR_EMS_AB_POSTAL_ADDRESS" },
	{ PR_EMS_AB_POSTAL_ADDRESS_ERROR, "PR_EMS_AB_POSTAL_ADDRESS_ERROR" },
	{ PR_EMS_AB_PREFERRED_DELIVERY_METHOD, "PR_EMS_AB_PREFERRED_DELIVERY_METHOD" },
	{ PR_EMS_AB_PREFERRED_DELIVERY_METHOD_ERROR, "PR_EMS_AB_PREFERRED_DELIVERY_METHOD_ERROR" },
	{ PR_EMS_AB_PRMD, "PR_EMS_AB_PRMD" },
	{ PR_EMS_AB_PRMD_UNICODE, "PR_EMS_AB_PRMD_UNICODE" },
	{ PR_EMS_AB_PRMD_ERROR, "PR_EMS_AB_PRMD_ERROR" },
	{ PR_EMS_AB_PROXY_GENERATOR_DLL, "PR_EMS_AB_PROXY_GENERATOR_DLL" },
	{ PR_EMS_AB_PROXY_GENERATOR_DLL_UNICODE, "PR_EMS_AB_PROXY_GENERATOR_DLL_UNICODE" },
	{ PR_EMS_AB_PROXY_GENERATOR_DLL_ERROR, "PR_EMS_AB_PROXY_GENERATOR_DLL_ERROR" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_BL_O, "PR_EMS_AB_PUBLIC_DELEGATES_BL_O" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_BL_O_ERROR, "PR_EMS_AB_PUBLIC_DELEGATES_BL_O_ERROR" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_BL, "PR_EMS_AB_PUBLIC_DELEGATES_BL" },
	{ PR_EMS_AB_PUBLIC_DELEGATES_BL_UNICODE, "PR_EMS_AB_PUBLIC_DELEGATES_BL_UNICODE" },
	{ PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE, "PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE" },
	{ PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE_ERROR, "PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE_ERROR" },
	{ PR_EMS_AB_QUOTA_NOTIFICATION_STYLE, "PR_EMS_AB_QUOTA_NOTIFICATION_STYLE" },
	{ PR_EMS_AB_QUOTA_NOTIFICATION_STYLE_ERROR, "PR_EMS_AB_QUOTA_NOTIFICATION_STYLE_ERROR" },
	{ PR_EMS_AB_RANGE_LOWER, "PR_EMS_AB_RANGE_LOWER" },
	{ PR_EMS_AB_RANGE_LOWER_ERROR, "PR_EMS_AB_RANGE_LOWER_ERROR" },
	{ PR_EMS_AB_RANGE_UPPER, "PR_EMS_AB_RANGE_UPPER" },
	{ PR_EMS_AB_RANGE_UPPER_ERROR, "PR_EMS_AB_RANGE_UPPER_ERROR" },
	{ PR_EMS_AB_RAS_CALLBACK_NUMBER, "PR_EMS_AB_RAS_CALLBACK_NUMBER" },
	{ PR_EMS_AB_RAS_CALLBACK_NUMBER_UNICODE, "PR_EMS_AB_RAS_CALLBACK_NUMBER_UNICODE" },
	{ PR_EMS_AB_RAS_CALLBACK_NUMBER_ERROR, "PR_EMS_AB_RAS_CALLBACK_NUMBER_ERROR" },
	{ PR_EMS_AB_RAS_PHONE_NUMBER, "PR_EMS_AB_RAS_PHONE_NUMBER" },
	{ PR_EMS_AB_RAS_PHONE_NUMBER_UNICODE, "PR_EMS_AB_RAS_PHONE_NUMBER_UNICODE" },
	{ PR_EMS_AB_RAS_PHONE_NUMBER_ERROR, "PR_EMS_AB_RAS_PHONE_NUMBER_ERROR" },
	{ PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME, "PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME" },
	{ PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_UNICODE, "PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_UNICODE" },
	{ PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_ERROR, "PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_ERROR" },
	{ PR_EMS_AB_RAS_REMOTE_SRVR_NAME, "PR_EMS_AB_RAS_REMOTE_SRVR_NAME" },
	{ PR_EMS_AB_RAS_REMOTE_SRVR_NAME_UNICODE, "PR_EMS_AB_RAS_REMOTE_SRVR_NAME_UNICODE" },
	{ PR_EMS_AB_RAS_REMOTE_SRVR_NAME_ERROR, "PR_EMS_AB_RAS_REMOTE_SRVR_NAME_ERROR" },
	{ PR_EMS_AB_REGISTERED_ADDRESS, "PR_EMS_AB_REGISTERED_ADDRESS" },
	{ PR_EMS_AB_REGISTERED_ADDRESS_ERROR, "PR_EMS_AB_REGISTERED_ADDRESS_ERROR" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD, "PR_EMS_AB_REMOTE_BRIDGE_HEAD" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD_UNICODE, "PR_EMS_AB_REMOTE_BRIDGE_HEAD_UNICODE" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD_ERROR, "PR_EMS_AB_REMOTE_BRIDGE_HEAD_ERROR" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS, "PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_UNICODE, "PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_UNICODE" },
	{ PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_ERROR, "PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_ERROR" },
	{ PR_EMS_AB_REMOTE_OUT_BH_SERVER_O, "PR_EMS_AB_REMOTE_OUT_BH_SERVER_O" },
	{ PR_EMS_AB_REMOTE_OUT_BH_SERVER_O_ERROR, "PR_EMS_AB_REMOTE_OUT_BH_SERVER_O_ERROR" },
	{ PR_EMS_AB_REMOTE_OUT_BH_SERVER, "PR_EMS_AB_REMOTE_OUT_BH_SERVER" },
	{ PR_EMS_AB_REMOTE_OUT_BH_SERVER_UNICODE, "PR_EMS_AB_REMOTE_OUT_BH_SERVER_UNICODE" },
	{ PR_EMS_AB_REMOTE_SITE_O, "PR_EMS_AB_REMOTE_SITE_O" },
	{ PR_EMS_AB_REMOTE_SITE_O_ERROR, "PR_EMS_AB_REMOTE_SITE_O_ERROR" },
	{ PR_EMS_AB_REMOTE_SITE, "PR_EMS_AB_REMOTE_SITE" },
	{ PR_EMS_AB_REMOTE_SITE_UNICODE, "PR_EMS_AB_REMOTE_SITE_UNICODE" },
	{ PR_EMS_AB_REPLICATION_SENSITIVITY, "PR_EMS_AB_REPLICATION_SENSITIVITY" },
	{ PR_EMS_AB_REPLICATION_SENSITIVITY_ERROR, "PR_EMS_AB_REPLICATION_SENSITIVITY_ERROR" },
	{ PR_EMS_AB_REPLICATION_STAGGER, "PR_EMS_AB_REPLICATION_STAGGER" },
	{ PR_EMS_AB_REPLICATION_STAGGER_ERROR, "PR_EMS_AB_REPLICATION_STAGGER_ERROR" },
	{ PR_EMS_AB_REPORT_TO_ORIGINATOR, "PR_EMS_AB_REPORT_TO_ORIGINATOR" },
	{ PR_EMS_AB_REPORT_TO_ORIGINATOR_ERROR, "PR_EMS_AB_REPORT_TO_ORIGINATOR_ERROR" },
	{ PR_EMS_AB_REPORT_TO_OWNER, "PR_EMS_AB_REPORT_TO_OWNER" },
	{ PR_EMS_AB_REPORT_TO_OWNER_ERROR, "PR_EMS_AB_REPORT_TO_OWNER_ERROR" },
	{ PR_EMS_AB_REQ_SEQ, "PR_EMS_AB_REQ_SEQ" },
	{ PR_EMS_AB_REQ_SEQ_ERROR, "PR_EMS_AB_REQ_SEQ_ERROR" },
	{ PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O, "PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O" },
	{ PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O_ERROR, "PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O_ERROR" },
	{ PR_EMS_AB_RESPONSIBLE_LOCAL_DXA, "PR_EMS_AB_RESPONSIBLE_LOCAL_DXA" },
	{ PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_UNICODE, "PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_UNICODE" },
	{ PR_EMS_AB_RID_SERVER_O, "PR_EMS_AB_RID_SERVER_O" },
	{ PR_EMS_AB_RID_SERVER_O_ERROR, "PR_EMS_AB_RID_SERVER_O_ERROR" },
	{ PR_EMS_AB_RID_SERVER, "PR_EMS_AB_RID_SERVER" },
	{ PR_EMS_AB_RID_SERVER_UNICODE, "PR_EMS_AB_RID_SERVER_UNICODE" },
	{ PR_EMS_AB_ROLE_OCCUPANT_O, "PR_EMS_AB_ROLE_OCCUPANT_O" },
	{ PR_EMS_AB_ROLE_OCCUPANT_O_ERROR, "PR_EMS_AB_ROLE_OCCUPANT_O_ERROR" },
	{ PR_EMS_AB_ROLE_OCCUPANT, "PR_EMS_AB_ROLE_OCCUPANT" },
	{ PR_EMS_AB_ROLE_OCCUPANT_UNICODE, "PR_EMS_AB_ROLE_OCCUPANT_UNICODE" },
	{ PR_EMS_AB_ROUTING_LIST, "PR_EMS_AB_ROUTING_LIST" },
	{ PR_EMS_AB_ROUTING_LIST_UNICODE, "PR_EMS_AB_ROUTING_LIST_UNICODE" },
	{ PR_EMS_AB_ROUTING_LIST_ERROR, "PR_EMS_AB_ROUTING_LIST_ERROR" },
	{ PR_EMS_AB_RTS_CHECKPOINT_SIZE, "PR_EMS_AB_RTS_CHECKPOINT_SIZE" },
	{ PR_EMS_AB_RTS_CHECKPOINT_SIZE_ERROR, "PR_EMS_AB_RTS_CHECKPOINT_SIZE_ERROR" },
	{ PR_EMS_AB_RTS_RECOVERY_TIMEOUT, "PR_EMS_AB_RTS_RECOVERY_TIMEOUT" },
	{ PR_EMS_AB_RTS_RECOVERY_TIMEOUT_ERROR, "PR_EMS_AB_RTS_RECOVERY_TIMEOUT_ERROR" },
	{ PR_EMS_AB_RTS_WINDOW_SIZE, "PR_EMS_AB_RTS_WINDOW_SIZE" },
	{ PR_EMS_AB_RTS_WINDOW_SIZE_ERROR, "PR_EMS_AB_RTS_WINDOW_SIZE_ERROR" },
	{ PR_EMS_AB_RUNS_ON_O, "PR_EMS_AB_RUNS_ON_O" },
	{ PR_EMS_AB_RUNS_ON_O_ERROR, "PR_EMS_AB_RUNS_ON_O_ERROR" },
	{ PR_EMS_AB_RUNS_ON, "PR_EMS_AB_RUNS_ON" },
	{ PR_EMS_AB_RUNS_ON_UNICODE, "PR_EMS_AB_RUNS_ON_UNICODE" },
	{ PR_EMS_AB_S_SELECTOR, "PR_EMS_AB_S_SELECTOR" },
	{ PR_EMS_AB_S_SELECTOR_ERROR, "PR_EMS_AB_S_SELECTOR_ERROR" },
	{ PR_EMS_AB_S_SELECTOR_INBOUND, "PR_EMS_AB_S_SELECTOR_INBOUND" },
	{ PR_EMS_AB_S_SELECTOR_INBOUND_ERROR, "PR_EMS_AB_S_SELECTOR_INBOUND_ERROR" },
	{ PR_EMS_AB_SEARCH_FLAGS, "PR_EMS_AB_SEARCH_FLAGS" },
	{ PR_EMS_AB_SEARCH_FLAGS_ERROR, "PR_EMS_AB_SEARCH_FLAGS_ERROR" },
	{ PR_EMS_AB_SEARCH_GUIDE, "PR_EMS_AB_SEARCH_GUIDE" },
	{ PR_EMS_AB_SEARCH_GUIDE_ERROR, "PR_EMS_AB_SEARCH_GUIDE_ERROR" },
	{ PR_EMS_AB_SEE_ALSO_O, "PR_EMS_AB_SEE_ALSO_O" },
	{ PR_EMS_AB_SEE_ALSO_O_ERROR, "PR_EMS_AB_SEE_ALSO_O_ERROR" },
	{ PR_EMS_AB_SEE_ALSO, "PR_EMS_AB_SEE_ALSO" },
	{ PR_EMS_AB_SEE_ALSO_UNICODE, "PR_EMS_AB_SEE_ALSO_UNICODE" },
	{ PR_EMS_AB_SERIAL_NUMBER, "PR_EMS_AB_SERIAL_NUMBER" },
	{ PR_EMS_AB_SERIAL_NUMBER_UNICODE, "PR_EMS_AB_SERIAL_NUMBER_UNICODE" },
	{ PR_EMS_AB_SERIAL_NUMBER_ERROR, "PR_EMS_AB_SERIAL_NUMBER_ERROR" },
	{ PR_EMS_AB_SERVICE_ACTION_FIRST, "PR_EMS_AB_SERVICE_ACTION_FIRST" },
	{ PR_EMS_AB_SERVICE_ACTION_FIRST_ERROR, "PR_EMS_AB_SERVICE_ACTION_FIRST_ERROR" },
	{ PR_EMS_AB_SERVICE_ACTION_OTHER, "PR_EMS_AB_SERVICE_ACTION_OTHER" },
	{ PR_EMS_AB_SERVICE_ACTION_OTHER_ERROR, "PR_EMS_AB_SERVICE_ACTION_OTHER_ERROR" },
	{ PR_EMS_AB_SERVICE_ACTION_SECOND, "PR_EMS_AB_SERVICE_ACTION_SECOND" },
	{ PR_EMS_AB_SERVICE_ACTION_SECOND_ERROR, "PR_EMS_AB_SERVICE_ACTION_SECOND_ERROR" },
	{ PR_EMS_AB_SERVICE_RESTART_DELAY, "PR_EMS_AB_SERVICE_RESTART_DELAY" },
	{ PR_EMS_AB_SERVICE_RESTART_DELAY_ERROR, "PR_EMS_AB_SERVICE_RESTART_DELAY_ERROR" },
	{ PR_EMS_AB_SERVICE_RESTART_MESSAGE, "PR_EMS_AB_SERVICE_RESTART_MESSAGE" },
	{ PR_EMS_AB_SERVICE_RESTART_MESSAGE_UNICODE, "PR_EMS_AB_SERVICE_RESTART_MESSAGE_UNICODE" },
	{ PR_EMS_AB_SERVICE_RESTART_MESSAGE_ERROR, "PR_EMS_AB_SERVICE_RESTART_MESSAGE_ERROR" },
	{ PR_EMS_AB_SESSION_DISCONNECT_TIMER, "PR_EMS_AB_SESSION_DISCONNECT_TIMER" },
	{ PR_EMS_AB_SESSION_DISCONNECT_TIMER_ERROR, "PR_EMS_AB_SESSION_DISCONNECT_TIMER_ERROR" },
	{ PR_EMS_AB_SITE_AFFINITY, "PR_EMS_AB_SITE_AFFINITY" },
	{ PR_EMS_AB_SITE_AFFINITY_UNICODE, "PR_EMS_AB_SITE_AFFINITY_UNICODE" },
	{ PR_EMS_AB_SITE_AFFINITY_ERROR, "PR_EMS_AB_SITE_AFFINITY_ERROR" },
	{ PR_EMS_AB_SITE_PROXY_SPACE, "PR_EMS_AB_SITE_PROXY_SPACE" },
	{ PR_EMS_AB_SITE_PROXY_SPACE_UNICODE, "PR_EMS_AB_SITE_PROXY_SPACE_UNICODE" },
	{ PR_EMS_AB_SITE_PROXY_SPACE_ERROR, "PR_EMS_AB_SITE_PROXY_SPACE_ERROR" },
	{ PR_EMS_AB_SPACE_LAST_COMPUTED, "PR_EMS_AB_SPACE_LAST_COMPUTED" },
	{ PR_EMS_AB_SPACE_LAST_COMPUTED_ERROR, "PR_EMS_AB_SPACE_LAST_COMPUTED_ERROR" },
	{ PR_EMS_AB_STREET_ADDRESS, "PR_EMS_AB_STREET_ADDRESS" },
	{ PR_EMS_AB_STREET_ADDRESS_UNICODE, "PR_EMS_AB_STREET_ADDRESS_UNICODE" },
	{ PR_EMS_AB_STREET_ADDRESS_ERROR, "PR_EMS_AB_STREET_ADDRESS_ERROR" },
	{ PR_EMS_AB_SUB_REFS_O, "PR_EMS_AB_SUB_REFS_O" },
	{ PR_EMS_AB_SUB_REFS_O_ERROR, "PR_EMS_AB_SUB_REFS_O_ERROR" },
	{ PR_EMS_AB_SUB_REFS, "PR_EMS_AB_SUB_REFS" },
	{ PR_EMS_AB_SUB_REFS_UNICODE, "PR_EMS_AB_SUB_REFS_UNICODE" },
	{ PR_EMS_AB_SUBMISSION_CONT_LENGTH, "PR_EMS_AB_SUBMISSION_CONT_LENGTH" },
	{ PR_EMS_AB_SUBMISSION_CONT_LENGTH_ERROR, "PR_EMS_AB_SUBMISSION_CONT_LENGTH_ERROR" },
	{ PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT, "PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT" },
	{ PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT_ERROR, "PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT_ERROR" },
	{ PR_EMS_AB_SUPPORTING_STACK_O, "PR_EMS_AB_SUPPORTING_STACK_O" },
	{ PR_EMS_AB_SUPPORTING_STACK_O_ERROR, "PR_EMS_AB_SUPPORTING_STACK_O_ERROR" },
	{ PR_EMS_AB_SUPPORTING_STACK, "PR_EMS_AB_SUPPORTING_STACK" },
	{ PR_EMS_AB_SUPPORTING_STACK_UNICODE, "PR_EMS_AB_SUPPORTING_STACK_UNICODE" },
	{ PR_EMS_AB_SUPPORTING_STACK_BL_O, "PR_EMS_AB_SUPPORTING_STACK_BL_O" },
	{ PR_EMS_AB_SUPPORTING_STACK_BL_O_ERROR, "PR_EMS_AB_SUPPORTING_STACK_BL_O_ERROR" },
	{ PR_EMS_AB_SUPPORTING_STACK_BL, "PR_EMS_AB_SUPPORTING_STACK_BL" },
	{ PR_EMS_AB_SUPPORTING_STACK_BL_UNICODE, "PR_EMS_AB_SUPPORTING_STACK_BL_UNICODE" },
	{ PR_EMS_AB_T_SELECTOR, "PR_EMS_AB_T_SELECTOR" },
	{ PR_EMS_AB_T_SELECTOR_ERROR, "PR_EMS_AB_T_SELECTOR_ERROR" },
	{ PR_EMS_AB_T_SELECTOR_INBOUND, "PR_EMS_AB_T_SELECTOR_INBOUND" },
	{ PR_EMS_AB_T_SELECTOR_INBOUND_ERROR, "PR_EMS_AB_T_SELECTOR_INBOUND_ERROR" },
	{ PR_EMS_AB_TARGET_MTAS, "PR_EMS_AB_TARGET_MTAS" },
	{ PR_EMS_AB_TARGET_MTAS_UNICODE, "PR_EMS_AB_TARGET_MTAS_UNICODE" },
	{ PR_EMS_AB_TARGET_MTAS_ERROR, "PR_EMS_AB_TARGET_MTAS_ERROR" },
	{ PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER, "PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER" },
	{ PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER_ERROR, "PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER_ERROR" },
	{ PR_EMS_AB_TEMP_ASSOC_THRESHOLD, "PR_EMS_AB_TEMP_ASSOC_THRESHOLD" },
	{ PR_EMS_AB_TEMP_ASSOC_THRESHOLD_ERROR, "PR_EMS_AB_TEMP_ASSOC_THRESHOLD_ERROR" },
	{ PR_EMS_AB_TOMBSTONE_LIFETIME, "PR_EMS_AB_TOMBSTONE_LIFETIME" },
	{ PR_EMS_AB_TOMBSTONE_LIFETIME_ERROR, "PR_EMS_AB_TOMBSTONE_LIFETIME_ERROR" },
	{ PR_EMS_AB_TRACKING_LOG_PATH_NAME, "PR_EMS_AB_TRACKING_LOG_PATH_NAME" },
	{ PR_EMS_AB_TRACKING_LOG_PATH_NAME_UNICODE, "PR_EMS_AB_TRACKING_LOG_PATH_NAME_UNICODE" },
	{ PR_EMS_AB_TRACKING_LOG_PATH_NAME_ERROR, "PR_EMS_AB_TRACKING_LOG_PATH_NAME_ERROR" },
	{ PR_EMS_AB_TRANS_RETRY_MINS, "PR_EMS_AB_TRANS_RETRY_MINS" },
	{ PR_EMS_AB_TRANS_RETRY_MINS_ERROR, "PR_EMS_AB_TRANS_RETRY_MINS_ERROR" },
	{ PR_EMS_AB_TRANS_TIMEOUT_MINS, "PR_EMS_AB_TRANS_TIMEOUT_MINS" },
	{ PR_EMS_AB_TRANS_TIMEOUT_MINS_ERROR, "PR_EMS_AB_TRANS_TIMEOUT_MINS_ERROR" },
	{ PR_EMS_AB_TRANSFER_RETRY_INTERVAL, "PR_EMS_AB_TRANSFER_RETRY_INTERVAL" },
	{ PR_EMS_AB_TRANSFER_RETRY_INTERVAL_ERROR, "PR_EMS_AB_TRANSFER_RETRY_INTERVAL_ERROR" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT, "PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT_ERROR, "PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT_ERROR" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL, "PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL_ERROR, "PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL_ERROR" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_URGENT, "PR_EMS_AB_TRANSFER_TIMEOUT_URGENT" },
	{ PR_EMS_AB_TRANSFER_TIMEOUT_URGENT_ERROR, "PR_EMS_AB_TRANSFER_TIMEOUT_URGENT_ERROR" },
	{ PR_EMS_AB_TRANSLATION_TABLE_USED, "PR_EMS_AB_TRANSLATION_TABLE_USED" },
	{ PR_EMS_AB_TRANSLATION_TABLE_USED_ERROR, "PR_EMS_AB_TRANSLATION_TABLE_USED_ERROR" },
	{ PR_EMS_AB_TRANSPORT_EXPEDITED_DATA, "PR_EMS_AB_TRANSPORT_EXPEDITED_DATA" },
	{ PR_EMS_AB_TRANSPORT_EXPEDITED_DATA_ERROR, "PR_EMS_AB_TRANSPORT_EXPEDITED_DATA_ERROR" },
	{ PR_EMS_AB_TRUST_LEVEL, "PR_EMS_AB_TRUST_LEVEL" },
	{ PR_EMS_AB_TRUST_LEVEL_ERROR, "PR_EMS_AB_TRUST_LEVEL_ERROR" },
	{ PR_EMS_AB_TURN_REQUEST_THRESHOLD, "PR_EMS_AB_TURN_REQUEST_THRESHOLD" },
	{ PR_EMS_AB_TURN_REQUEST_THRESHOLD_ERROR, "PR_EMS_AB_TURN_REQUEST_THRESHOLD_ERROR" },
	{ PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY, "PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY" },
	{ PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY_ERROR, "PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY_ERROR" },
	{ PR_EMS_AB_UNAUTH_ORIG_BL_O, "PR_EMS_AB_UNAUTH_ORIG_BL_O" },
	{ PR_EMS_AB_UNAUTH_ORIG_BL_O_ERROR, "PR_EMS_AB_UNAUTH_ORIG_BL_O_ERROR" },
	{ PR_EMS_AB_UNAUTH_ORIG_BL, "PR_EMS_AB_UNAUTH_ORIG_BL" },
	{ PR_EMS_AB_UNAUTH_ORIG_BL_UNICODE, "PR_EMS_AB_UNAUTH_ORIG_BL_UNICODE" },
	{ PR_EMS_AB_USER_PASSWORD, "PR_EMS_AB_USER_PASSWORD" },
	{ PR_EMS_AB_USER_PASSWORD_ERROR, "PR_EMS_AB_USER_PASSWORD_ERROR" },
	{ PR_EMS_AB_USN_CREATED, "PR_EMS_AB_USN_CREATED" },
	{ PR_EMS_AB_USN_CREATED_ERROR, "PR_EMS_AB_USN_CREATED_ERROR" },
	{ PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED, "PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED" },
	{ PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED_ERROR, "PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED_ERROR" },
	{ PR_EMS_AB_USN_LAST_OBJ_REM, "PR_EMS_AB_USN_LAST_OBJ_REM" },
	{ PR_EMS_AB_USN_LAST_OBJ_REM_ERROR, "PR_EMS_AB_USN_LAST_OBJ_REM_ERROR" },
	{ PR_EMS_AB_USN_SOURCE, "PR_EMS_AB_USN_SOURCE" },
	{ PR_EMS_AB_USN_SOURCE_ERROR, "PR_EMS_AB_USN_SOURCE_ERROR" },
	{ PR_EMS_AB_X121_ADDRESS, "PR_EMS_AB_X121_ADDRESS" },
	{ PR_EMS_AB_X121_ADDRESS_UNICODE, "PR_EMS_AB_X121_ADDRESS_UNICODE" },
	{ PR_EMS_AB_X121_ADDRESS_ERROR, "PR_EMS_AB_X121_ADDRESS_ERROR" },
	{ PR_EMS_AB_X25_CALL_USER_DATA_INCOMING, "PR_EMS_AB_X25_CALL_USER_DATA_INCOMING" },
	{ PR_EMS_AB_X25_CALL_USER_DATA_INCOMING_ERROR, "PR_EMS_AB_X25_CALL_USER_DATA_INCOMING_ERROR" },
	{ PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING, "PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING" },
	{ PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING_ERROR, "PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING_ERROR" },
	{ PR_EMS_AB_X25_FACILITIES_DATA_INCOMING, "PR_EMS_AB_X25_FACILITIES_DATA_INCOMING" },
	{ PR_EMS_AB_X25_FACILITIES_DATA_INCOMING_ERROR, "PR_EMS_AB_X25_FACILITIES_DATA_INCOMING_ERROR" },
	{ PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING, "PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING" },
	{ PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING_ERROR, "PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING_ERROR" },
	{ PR_EMS_AB_X25_LEASED_LINE_PORT, "PR_EMS_AB_X25_LEASED_LINE_PORT" },
	{ PR_EMS_AB_X25_LEASED_LINE_PORT_ERROR, "PR_EMS_AB_X25_LEASED_LINE_PORT_ERROR" },
	{ PR_EMS_AB_X25_LEASED_OR_SWITCHED, "PR_EMS_AB_X25_LEASED_OR_SWITCHED" },
	{ PR_EMS_AB_X25_LEASED_OR_SWITCHED_ERROR, "PR_EMS_AB_X25_LEASED_OR_SWITCHED_ERROR" },
	{ PR_EMS_AB_X25_REMOTE_MTA_PHONE, "PR_EMS_AB_X25_REMOTE_MTA_PHONE" },
	{ PR_EMS_AB_X25_REMOTE_MTA_PHONE_UNICODE, "PR_EMS_AB_X25_REMOTE_MTA_PHONE_UNICODE" },
	{ PR_EMS_AB_X25_REMOTE_MTA_PHONE_ERROR, "PR_EMS_AB_X25_REMOTE_MTA_PHONE_ERROR" },
	{ PR_EMS_AB_X400_ATTACHMENT_TYPE, "PR_EMS_AB_X400_ATTACHMENT_TYPE" },
	{ PR_EMS_AB_X400_ATTACHMENT_TYPE_ERROR, "PR_EMS_AB_X400_ATTACHMENT_TYPE_ERROR" },
	{ PR_EMS_AB_X400_SELECTOR_SYNTAX, "PR_EMS_AB_X400_SELECTOR_SYNTAX" },
	{ PR_EMS_AB_X400_SELECTOR_SYNTAX_ERROR, "PR_EMS_AB_X400_SELECTOR_SYNTAX_ERROR" },
	{ PR_EMS_AB_X500_ACCESS_CONTROL_LIST, "PR_EMS_AB_X500_ACCESS_CONTROL_LIST" },
	{ PR_EMS_AB_X500_ACCESS_CONTROL_LIST_ERROR, "PR_EMS_AB_X500_ACCESS_CONTROL_LIST_ERROR" },
	{ PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT, "PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT" },
	{ PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT_ERROR, "PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT_ERROR" },
	{ PR_EMS_AB_XMIT_TIMEOUT_NORMAL, "PR_EMS_AB_XMIT_TIMEOUT_NORMAL" },
	{ PR_EMS_AB_XMIT_TIMEOUT_NORMAL_ERROR, "PR_EMS_AB_XMIT_TIMEOUT_NORMAL_ERROR" },
	{ PR_EMS_AB_XMIT_TIMEOUT_URGENT, "PR_EMS_AB_XMIT_TIMEOUT_URGENT" },
	{ PR_EMS_AB_XMIT_TIMEOUT_URGENT_ERROR, "PR_EMS_AB_XMIT_TIMEOUT_URGENT_ERROR" },
	{ PR_EMS_AB_SITE_FOLDER_GUID, "PR_EMS_AB_SITE_FOLDER_GUID" },
	{ PR_EMS_AB_SITE_FOLDER_GUID_ERROR, "PR_EMS_AB_SITE_FOLDER_GUID_ERROR" },
	{ PR_EMS_AB_SITE_FOLDER_SERVER_O, "PR_EMS_AB_SITE_FOLDER_SERVER_O" },
	{ PR_EMS_AB_SITE_FOLDER_SERVER_O_ERROR, "PR_EMS_AB_SITE_FOLDER_SERVER_O_ERROR" },
	{ PR_EMS_AB_SITE_FOLDER_SERVER, "PR_EMS_AB_SITE_FOLDER_SERVER" },
	{ PR_EMS_AB_SITE_FOLDER_SERVER_UNICODE, "PR_EMS_AB_SITE_FOLDER_SERVER_UNICODE" },
	{ PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE, "PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE" },
	{ PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE_ERROR, "PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE_ERROR" },
	{ PR_EMS_AB_MAXIMUM_OBJECT_ID, "PR_EMS_AB_MAXIMUM_OBJECT_ID" },
	{ PR_EMS_AB_MAXIMUM_OBJECT_ID_ERROR, "PR_EMS_AB_MAXIMUM_OBJECT_ID_ERROR" },
	{ PR_EMS_AB_NETWORK_ADDRESS, "PR_EMS_AB_NETWORK_ADDRESS" },
	{ PR_EMS_AB_NETWORK_ADDRESS_UNICODE, "PR_EMS_AB_NETWORK_ADDRESS_UNICODE" },
	{ PR_EMS_AB_NETWORK_ADDRESS_ERROR, "PR_EMS_AB_NETWORK_ADDRESS_ERROR" },
	{ PR_EMS_AB_LDAP_DISPLAY_NAME, "PR_EMS_AB_LDAP_DISPLAY_NAME" },
	{ PR_EMS_AB_LDAP_DISPLAY_NAME_UNICODE, "PR_EMS_AB_LDAP_DISPLAY_NAME_UNICODE" },
	{ PR_EMS_AB_LDAP_DISPLAY_NAME_ERROR, "PR_EMS_AB_LDAP_DISPLAY_NAME_ERROR" },
	{ PR_EMS_AB_SCHEMA_FLAGS, "PR_EMS_AB_SCHEMA_FLAGS" },
	{ PR_EMS_AB_SCHEMA_FLAGS_ERROR, "PR_EMS_AB_SCHEMA_FLAGS_ERROR" },
	{ PR_EMS_AB_BRIDGEHEAD_SERVERS_O, "PR_EMS_AB_BRIDGEHEAD_SERVERS_O" },
	{ PR_EMS_AB_BRIDGEHEAD_SERVERS_O_ERROR, "PR_EMS_AB_BRIDGEHEAD_SERVERS_O_ERROR" },
	{ PR_EMS_AB_BRIDGEHEAD_SERVERS, "PR_EMS_AB_BRIDGEHEAD_SERVERS" },
	{ PR_EMS_AB_BRIDGEHEAD_SERVERS_UNICODE, "PR_EMS_AB_BRIDGEHEAD_SERVERS_UNICODE" },
	{ PR_EMS_AB_WWW_HOME_PAGE, "PR_EMS_AB_WWW_HOME_PAGE" },
	{ PR_EMS_AB_WWW_HOME_PAGE_UNICODE, "PR_EMS_AB_WWW_HOME_PAGE_UNICODE" },
	{ PR_EMS_AB_WWW_HOME_PAGE_ERROR, "PR_EMS_AB_WWW_HOME_PAGE_ERROR" },
	{ PR_EMS_AB_NNTP_CONTENT_FORMAT, "PR_EMS_AB_NNTP_CONTENT_FORMAT" },
	{ PR_EMS_AB_NNTP_CONTENT_FORMAT_UNICODE, "PR_EMS_AB_NNTP_CONTENT_FORMAT_UNICODE" },
	{ PR_EMS_AB_NNTP_CONTENT_FORMAT_ERROR, "PR_EMS_AB_NNTP_CONTENT_FORMAT_ERROR" },
	{ PR_EMS_AB_POP_CONTENT_FORMAT, "PR_EMS_AB_POP_CONTENT_FORMAT" },
	{ PR_EMS_AB_POP_CONTENT_FORMAT_UNICODE, "PR_EMS_AB_POP_CONTENT_FORMAT_UNICODE" },
	{ PR_EMS_AB_POP_CONTENT_FORMAT_ERROR, "PR_EMS_AB_POP_CONTENT_FORMAT_ERROR" },
	{ PR_EMS_AB_LANGUAGE, "PR_EMS_AB_LANGUAGE" },
	{ PR_EMS_AB_LANGUAGE_ERROR, "PR_EMS_AB_LANGUAGE_ERROR" },
	{ PR_EMS_AB_POP_CHARACTER_SET, "PR_EMS_AB_POP_CHARACTER_SET" },
	{ PR_EMS_AB_POP_CHARACTER_SET_UNICODE, "PR_EMS_AB_POP_CHARACTER_SET_UNICODE" },
	{ PR_EMS_AB_POP_CHARACTER_SET_ERROR, "PR_EMS_AB_POP_CHARACTER_SET_ERROR" },
	{ PR_EMS_AB_USN_INTERSITE, "PR_EMS_AB_USN_INTERSITE" },
	{ PR_EMS_AB_USN_INTERSITE_ERROR, "PR_EMS_AB_USN_INTERSITE_ERROR" },
	{ PR_EMS_AB_SUB_SITE, "PR_EMS_AB_SUB_SITE" },
	{ PR_EMS_AB_SUB_SITE_UNICODE, "PR_EMS_AB_SUB_SITE_UNICODE" },
	{ PR_EMS_AB_SUB_SITE_ERROR, "PR_EMS_AB_SUB_SITE_ERROR" },
	{ PR_EMS_AB_SCHEMA_VERSION, "PR_EMS_AB_SCHEMA_VERSION" },
	{ PR_EMS_AB_SCHEMA_VERSION_ERROR, "PR_EMS_AB_SCHEMA_VERSION_ERROR" },
	{ PR_EMS_AB_NNTP_CHARACTER_SET, "PR_EMS_AB_NNTP_CHARACTER_SET" },
	{ PR_EMS_AB_NNTP_CHARACTER_SET_UNICODE, "PR_EMS_AB_NNTP_CHARACTER_SET_UNICODE" },
	{ PR_EMS_AB_NNTP_CHARACTER_SET_ERROR, "PR_EMS_AB_NNTP_CHARACTER_SET_ERROR" },
	{ PR_EMS_AB_USE_SERVER_VALUES, "PR_EMS_AB_USE_SERVER_VALUES" },
	{ PR_EMS_AB_USE_SERVER_VALUES_ERROR, "PR_EMS_AB_USE_SERVER_VALUES_ERROR" },
	{ PR_EMS_AB_ENABLED_PROTOCOLS, "PR_EMS_AB_ENABLED_PROTOCOLS" },
	{ PR_EMS_AB_ENABLED_PROTOCOLS_ERROR, "PR_EMS_AB_ENABLED_PROTOCOLS_ERROR" },
	{ PR_EMS_AB_CONNECTION_LIST_FILTER, "PR_EMS_AB_CONNECTION_LIST_FILTER" },
	{ PR_EMS_AB_CONNECTION_LIST_FILTER_ERROR, "PR_EMS_AB_CONNECTION_LIST_FILTER_ERROR" },
	{ PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES, "PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES" },
	{ PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_UNICODE, "PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_UNICODE" },
	{ PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_ERROR, "PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_ERROR" },
	{ PR_EMS_AB_CHARACTER_SET_LIST, "PR_EMS_AB_CHARACTER_SET_LIST" },
	{ PR_EMS_AB_CHARACTER_SET_LIST_UNICODE, "PR_EMS_AB_CHARACTER_SET_LIST_UNICODE" },
	{ PR_EMS_AB_CHARACTER_SET_LIST_ERROR, "PR_EMS_AB_CHARACTER_SET_LIST_ERROR" },
	{ PR_EMS_AB_USE_SITE_VALUES, "PR_EMS_AB_USE_SITE_VALUES" },
	{ PR_EMS_AB_USE_SITE_VALUES_ERROR, "PR_EMS_AB_USE_SITE_VALUES_ERROR" },
	{ PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES, "PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES" },
	{ PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_UNICODE, "PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_UNICODE" },
	{ PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_ERROR, "PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_ERROR" },
	{ PR_EMS_AB_CHARACTER_SET, "PR_EMS_AB_CHARACTER_SET" },
	{ PR_EMS_AB_CHARACTER_SET_UNICODE, "PR_EMS_AB_CHARACTER_SET_UNICODE" },
	{ PR_EMS_AB_CHARACTER_SET_ERROR, "PR_EMS_AB_CHARACTER_SET_ERROR" },
	{ PR_EMS_AB_CONTENT_TYPE, "PR_EMS_AB_CONTENT_TYPE" },
	{ PR_EMS_AB_CONTENT_TYPE_ERROR, "PR_EMS_AB_CONTENT_TYPE_ERROR" },
	{ PR_EMS_AB_ANONYMOUS_ACCESS, "PR_EMS_AB_ANONYMOUS_ACCESS" },
	{ PR_EMS_AB_ANONYMOUS_ACCESS_ERROR, "PR_EMS_AB_ANONYMOUS_ACCESS_ERROR" },
	{ PR_EMS_AB_CONTROL_MSG_FOLDER_ID, "PR_EMS_AB_CONTROL_MSG_FOLDER_ID" },
	{ PR_EMS_AB_CONTROL_MSG_FOLDER_ID_ERROR, "PR_EMS_AB_CONTROL_MSG_FOLDER_ID_ERROR" },
	{ PR_EMS_AB_USENET_SITE_NAME, "PR_EMS_AB_USENET_SITE_NAME" },
	{ PR_EMS_AB_USENET_SITE_NAME_UNICODE, "PR_EMS_AB_USENET_SITE_NAME_UNICODE" },
	{ PR_EMS_AB_USENET_SITE_NAME_ERROR, "PR_EMS_AB_USENET_SITE_NAME_ERROR" },
	{ PR_EMS_AB_CONTROL_MSG_RULES, "PR_EMS_AB_CONTROL_MSG_RULES" },
	{ PR_EMS_AB_CONTROL_MSG_RULES_ERROR, "PR_EMS_AB_CONTROL_MSG_RULES_ERROR" },
	{ PR_EMS_AB_AVAILABLE_DISTRIBUTIONS, "PR_EMS_AB_AVAILABLE_DISTRIBUTIONS" },
	{ PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_UNICODE, "PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_UNICODE" },
	{ PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_ERROR, "PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_ERROR" },
	{ PR_EMS_AB_OUTBOUND_HOST, "PR_EMS_AB_OUTBOUND_HOST" },
	{ PR_EMS_AB_OUTBOUND_HOST_ERROR, "PR_EMS_AB_OUTBOUND_HOST_ERROR" },
	{ PR_EMS_AB_INBOUND_HOST, "PR_EMS_AB_INBOUND_HOST" },
	{ PR_EMS_AB_INBOUND_HOST_UNICODE, "PR_EMS_AB_INBOUND_HOST_UNICODE" },
	{ PR_EMS_AB_INBOUND_HOST_ERROR, "PR_EMS_AB_INBOUND_HOST_ERROR" },
	{ PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT, "PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT" },
	{ PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT_ERROR, "PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT_ERROR" },
	{ PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT, "PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT" },
	{ PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT_ERROR, "PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT_ERROR" },
	{ PR_EMS_AB_SEND_TNEF, "PR_EMS_AB_SEND_TNEF" },
	{ PR_EMS_AB_SEND_TNEF_ERROR, "PR_EMS_AB_SEND_TNEF_ERROR" },
	{ PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM, "PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM" },
	{ PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM_ERROR, "PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM_ERROR" },
	{ PR_EMS_AB_INBOUND_NEWSFEED, "PR_EMS_AB_INBOUND_NEWSFEED" },
	{ PR_EMS_AB_INBOUND_NEWSFEED_UNICODE, "PR_EMS_AB_INBOUND_NEWSFEED_UNICODE" },
	{ PR_EMS_AB_INBOUND_NEWSFEED_ERROR, "PR_EMS_AB_INBOUND_NEWSFEED_ERROR" },
	{ PR_EMS_AB_NEWSFEED_TYPE, "PR_EMS_AB_NEWSFEED_TYPE" },
	{ PR_EMS_AB_NEWSFEED_TYPE_ERROR, "PR_EMS_AB_NEWSFEED_TYPE_ERROR" },
	{ PR_EMS_AB_OUTBOUND_NEWSFEED, "PR_EMS_AB_OUTBOUND_NEWSFEED" },
	{ PR_EMS_AB_OUTBOUND_NEWSFEED_UNICODE, "PR_EMS_AB_OUTBOUND_NEWSFEED_UNICODE" },
	{ PR_EMS_AB_OUTBOUND_NEWSFEED_ERROR, "PR_EMS_AB_OUTBOUND_NEWSFEED_ERROR" },
	{ PR_EMS_AB_NEWSGROUP_LIST, "PR_EMS_AB_NEWSGROUP_LIST" },
	{ PR_EMS_AB_NEWSGROUP_LIST_ERROR, "PR_EMS_AB_NEWSGROUP_LIST_ERROR" },
	{ PR_EMS_AB_NNTP_DISTRIBUTIONS, "PR_EMS_AB_NNTP_DISTRIBUTIONS" },
	{ PR_EMS_AB_NNTP_DISTRIBUTIONS_UNICODE, "PR_EMS_AB_NNTP_DISTRIBUTIONS_UNICODE" },
	{ PR_EMS_AB_NNTP_DISTRIBUTIONS_ERROR, "PR_EMS_AB_NNTP_DISTRIBUTIONS_ERROR" },
	{ PR_EMS_AB_NEWSGROUP, "PR_EMS_AB_NEWSGROUP" },
	{ PR_EMS_AB_NEWSGROUP_UNICODE, "PR_EMS_AB_NEWSGROUP_UNICODE" },
	{ PR_EMS_AB_NEWSGROUP_ERROR, "PR_EMS_AB_NEWSGROUP_ERROR" },
	{ PR_EMS_AB_MODERATOR, "PR_EMS_AB_MODERATOR" },
	{ PR_EMS_AB_MODERATOR_UNICODE, "PR_EMS_AB_MODERATOR_UNICODE" },
	{ PR_EMS_AB_MODERATOR_ERROR, "PR_EMS_AB_MODERATOR_ERROR" },
	{ PR_EMS_AB_AUTHENTICATION_TO_USE, "PR_EMS_AB_AUTHENTICATION_TO_USE" },
	{ PR_EMS_AB_AUTHENTICATION_TO_USE_UNICODE, "PR_EMS_AB_AUTHENTICATION_TO_USE_UNICODE" },
	{ PR_EMS_AB_AUTHENTICATION_TO_USE_ERROR, "PR_EMS_AB_AUTHENTICATION_TO_USE_ERROR" },
	{ PR_EMS_AB_HTTP_PUB_GAL, "PR_EMS_AB_HTTP_PUB_GAL" },
	{ PR_EMS_AB_HTTP_PUB_GAL_ERROR, "PR_EMS_AB_HTTP_PUB_GAL_ERROR" },
	{ PR_EMS_AB_HTTP_PUB_GAL_LIMIT, "PR_EMS_AB_HTTP_PUB_GAL_LIMIT" },
	{ PR_EMS_AB_HTTP_PUB_GAL_LIMIT_ERROR, "PR_EMS_AB_HTTP_PUB_GAL_LIMIT_ERROR" },
	{ PR_EMS_AB_HTTP_PUB_PF, "PR_EMS_AB_HTTP_PUB_PF" },
	{ PR_EMS_AB_HTTP_PUB_PF_ERROR, "PR_EMS_AB_HTTP_PUB_PF_ERROR" },
	{ PR_EMS_AB_X500_RDN, "PR_EMS_AB_X500_RDN" },
	{ PR_EMS_AB_X500_RDN_UNICODE, "PR_EMS_AB_X500_RDN_UNICODE" },
	{ PR_EMS_AB_X500_RDN_ERROR, "PR_EMS_AB_X500_RDN_ERROR" },
	{ PR_EMS_AB_X500_NC, "PR_EMS_AB_X500_NC" },
	{ PR_EMS_AB_X500_NC_UNICODE, "PR_EMS_AB_X500_NC_UNICODE" },
	{ PR_EMS_AB_X500_NC_ERROR, "PR_EMS_AB_X500_NC_ERROR" },
	{ PR_EMS_AB_REFERRAL_LIST, "PR_EMS_AB_REFERRAL_LIST" },
	{ PR_EMS_AB_REFERRAL_LIST_UNICODE, "PR_EMS_AB_REFERRAL_LIST_UNICODE" },
	{ PR_EMS_AB_REFERRAL_LIST_ERROR, "PR_EMS_AB_REFERRAL_LIST_ERROR" },
	{ PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG, "PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG" },
	{ PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG_ERROR, "PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG_ERROR" },
	{ PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O, "PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O" },
	{ PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O_ERROR, "PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O_ERROR" },
	{ PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP, "PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP" },
	{ PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_UNICODE, "PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_UNICODE" },
	{ PR_EMS_AB_NNTP_NEWSFEEDS_O, "PR_EMS_AB_NNTP_NEWSFEEDS_O" },
	{ PR_EMS_AB_NNTP_NEWSFEEDS_O_ERROR, "PR_EMS_AB_NNTP_NEWSFEEDS_O_ERROR" },
	{ PR_EMS_AB_NNTP_NEWSFEEDS, "PR_EMS_AB_NNTP_NEWSFEEDS" },
	{ PR_EMS_AB_NNTP_NEWSFEEDS_UNICODE, "PR_EMS_AB_NNTP_NEWSFEEDS_UNICODE" },
	{ PR_EMS_AB_ENABLED_PROTOCOL_CFG, "PR_EMS_AB_ENABLED_PROTOCOL_CFG" },
	{ PR_EMS_AB_ENABLED_PROTOCOL_CFG_ERROR, "PR_EMS_AB_ENABLED_PROTOCOL_CFG_ERROR" },
	{ PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES, "PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES" },
	{ PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_UNICODE, "PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_UNICODE" },
	{ PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_ERROR, "PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_ERROR" },
	{ PR_EMS_AB_HTTP_SERVERS, "PR_EMS_AB_HTTP_SERVERS" },
	{ PR_EMS_AB_HTTP_SERVERS_UNICODE, "PR_EMS_AB_HTTP_SERVERS_UNICODE" },
	{ PR_EMS_AB_HTTP_SERVERS_ERROR, "PR_EMS_AB_HTTP_SERVERS_ERROR" },
	{ PR_EMS_AB_MODERATED, "PR_EMS_AB_MODERATED" },
	{ PR_EMS_AB_MODERATED_ERROR, "PR_EMS_AB_MODERATED_ERROR" },
	{ PR_EMS_AB_RAS_ACCOUNT, "PR_EMS_AB_RAS_ACCOUNT" },
	{ PR_EMS_AB_RAS_ACCOUNT_UNICODE, "PR_EMS_AB_RAS_ACCOUNT_UNICODE" },
	{ PR_EMS_AB_RAS_ACCOUNT_ERROR, "PR_EMS_AB_RAS_ACCOUNT_ERROR" },
	{ PR_EMS_AB_RAS_PASSWORD, "PR_EMS_AB_RAS_PASSWORD" },
	{ PR_EMS_AB_RAS_PASSWORD_ERROR, "PR_EMS_AB_RAS_PASSWORD_ERROR" },
	{ PR_EMS_AB_INCOMING_PASSWORD, "PR_EMS_AB_INCOMING_PASSWORD" },
	{ PR_EMS_AB_INCOMING_PASSWORD_ERROR, "PR_EMS_AB_INCOMING_PASSWORD_ERROR" },
	{ PR_EMS_AB_OUTBOUND_HOST_TYPE, "PR_EMS_AB_OUTBOUND_HOST_TYPE" },
	{ PR_EMS_AB_OUTBOUND_HOST_TYPE_ERROR, "PR_EMS_AB_OUTBOUND_HOST_TYPE_ERROR" },
	{ PR_EMS_AB_PROXY_GENERATION_ENABLED, "PR_EMS_AB_PROXY_GENERATION_ENABLED" },
	{ PR_EMS_AB_PROXY_GENERATION_ENABLED_ERROR, "PR_EMS_AB_PROXY_GENERATION_ENABLED_ERROR" },
	{ PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID, "PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID" },
	{ PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID_ERROR, "PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID_ERROR" },
	{ PR_EMS_AB_CONNECTION_TYPE, "PR_EMS_AB_CONNECTION_TYPE" },
	{ PR_EMS_AB_CONNECTION_TYPE_ERROR, "PR_EMS_AB_CONNECTION_TYPE_ERROR" },
	{ PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE, "PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE" },
	{ PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE_ERROR, "PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE_ERROR" },
	{ PR_EMS_AB_PORT_NUMBER, "PR_EMS_AB_PORT_NUMBER" },
	{ PR_EMS_AB_PORT_NUMBER_ERROR, "PR_EMS_AB_PORT_NUMBER_ERROR" },
	{ PR_EMS_AB_PROTOCOL_SETTINGS, "PR_EMS_AB_PROTOCOL_SETTINGS" },
	{ PR_EMS_AB_PROTOCOL_SETTINGS_UNICODE, "PR_EMS_AB_PROTOCOL_SETTINGS_UNICODE" },
	{ PR_EMS_AB_PROTOCOL_SETTINGS_ERROR, "PR_EMS_AB_PROTOCOL_SETTINGS_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_1, "PR_EMS_AB_GROUP_BY_ATTR_1" },
	{ PR_EMS_AB_GROUP_BY_ATTR_1_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_1_UNICODE" },
	{ PR_EMS_AB_GROUP_BY_ATTR_1_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_1_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_2, "PR_EMS_AB_GROUP_BY_ATTR_2" },
	{ PR_EMS_AB_GROUP_BY_ATTR_2_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_2_UNICODE" },
	{ PR_EMS_AB_GROUP_BY_ATTR_2_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_2_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_3, "PR_EMS_AB_GROUP_BY_ATTR_3" },
	{ PR_EMS_AB_GROUP_BY_ATTR_3_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_3_UNICODE" },
	{ PR_EMS_AB_GROUP_BY_ATTR_3_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_3_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_4, "PR_EMS_AB_GROUP_BY_ATTR_4" },
	{ PR_EMS_AB_GROUP_BY_ATTR_4_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_4_UNICODE" },
	{ PR_EMS_AB_GROUP_BY_ATTR_4_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_4_ERROR" },
	{ PR_EMS_AB_VIEW_SITE, "PR_EMS_AB_VIEW_SITE" },
	{ PR_EMS_AB_VIEW_SITE_UNICODE, "PR_EMS_AB_VIEW_SITE_UNICODE" },
	{ PR_EMS_AB_VIEW_SITE_ERROR, "PR_EMS_AB_VIEW_SITE_ERROR" },
	{ PR_EMS_AB_VIEW_CONTAINER_1, "PR_EMS_AB_VIEW_CONTAINER_1" },
	{ PR_EMS_AB_VIEW_CONTAINER_1_UNICODE, "PR_EMS_AB_VIEW_CONTAINER_1_UNICODE" },
	{ PR_EMS_AB_VIEW_CONTAINER_1_ERROR, "PR_EMS_AB_VIEW_CONTAINER_1_ERROR" },
	{ PR_EMS_AB_VIEW_CONTAINER_2, "PR_EMS_AB_VIEW_CONTAINER_2" },
	{ PR_EMS_AB_VIEW_CONTAINER_2_UNICODE, "PR_EMS_AB_VIEW_CONTAINER_2_UNICODE" },
	{ PR_EMS_AB_VIEW_CONTAINER_2_ERROR, "PR_EMS_AB_VIEW_CONTAINER_2_ERROR" },
	{ PR_EMS_AB_VIEW_CONTAINER_3, "PR_EMS_AB_VIEW_CONTAINER_3" },
	{ PR_EMS_AB_VIEW_CONTAINER_3_UNICODE, "PR_EMS_AB_VIEW_CONTAINER_3_UNICODE" },
	{ PR_EMS_AB_VIEW_CONTAINER_3_ERROR, "PR_EMS_AB_VIEW_CONTAINER_3_ERROR" },
	{ PR_EMS_AB_PROMO_EXPIRATION, "PR_EMS_AB_PROMO_EXPIRATION" },
	{ PR_EMS_AB_PROMO_EXPIRATION_ERROR, "PR_EMS_AB_PROMO_EXPIRATION_ERROR" },
	{ PR_EMS_AB_DISABLED_GATEWAY_PROXY, "PR_EMS_AB_DISABLED_GATEWAY_PROXY" },
	{ PR_EMS_AB_DISABLED_GATEWAY_PROXY_UNICODE, "PR_EMS_AB_DISABLED_GATEWAY_PROXY_UNICODE" },
	{ PR_EMS_AB_DISABLED_GATEWAY_PROXY_ERROR, "PR_EMS_AB_DISABLED_GATEWAY_PROXY_ERROR" },
	{ PR_EMS_AB_COMPROMISED_KEY_LIST, "PR_EMS_AB_COMPROMISED_KEY_LIST" },
	{ PR_EMS_AB_COMPROMISED_KEY_LIST_ERROR, "PR_EMS_AB_COMPROMISED_KEY_LIST_ERROR" },
	{ PR_EMS_AB_INSADMIN_O, "PR_EMS_AB_INSADMIN_O" },
	{ PR_EMS_AB_INSADMIN_O_ERROR, "PR_EMS_AB_INSADMIN_O_ERROR" },
	{ PR_EMS_AB_INSADMIN, "PR_EMS_AB_INSADMIN" },
	{ PR_EMS_AB_INSADMIN_UNICODE, "PR_EMS_AB_INSADMIN_UNICODE" },
	{ PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT, "PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT" },
	{ PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT_ERROR, "PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT_ERROR" },
	{ PR_EMS_AB_OBJ_VIEW_CONTAINERS_O, "PR_EMS_AB_OBJ_VIEW_CONTAINERS_O" },
	{ PR_EMS_AB_OBJ_VIEW_CONTAINERS_O_ERROR, "PR_EMS_AB_OBJ_VIEW_CONTAINERS_O_ERROR" },
	{ PR_EMS_AB_OBJ_VIEW_CONTAINERS, "PR_EMS_AB_OBJ_VIEW_CONTAINERS" },
	{ PR_EMS_AB_OBJ_VIEW_CONTAINERS_UNICODE, "PR_EMS_AB_OBJ_VIEW_CONTAINERS_UNICODE" },
	{ PR_EMS_AB_VIEW_FLAGS, "PR_EMS_AB_VIEW_FLAGS" },
	{ PR_EMS_AB_VIEW_FLAGS_ERROR, "PR_EMS_AB_VIEW_FLAGS_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_UNICODE" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O_ERROR, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O_ERROR" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN" },
	{ PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_UNICODE, "PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_UNICODE" },
	{ PR_EMS_AB_VIEW_DEFINITION, "PR_EMS_AB_VIEW_DEFINITION" },
	{ PR_EMS_AB_VIEW_DEFINITION_ERROR, "PR_EMS_AB_VIEW_DEFINITION_ERROR" },
	{ PR_EMS_AB_MIME_TYPES, "PR_EMS_AB_MIME_TYPES" },
	{ PR_EMS_AB_MIME_TYPES_ERROR, "PR_EMS_AB_MIME_TYPES_ERROR" },
	{ PR_EMS_AB_LDAP_SEARCH_CFG, "PR_EMS_AB_LDAP_SEARCH_CFG" },
	{ PR_EMS_AB_LDAP_SEARCH_CFG_ERROR, "PR_EMS_AB_LDAP_SEARCH_CFG_ERROR" },
	{ PR_EMS_AB_INBOUND_DN_O, "PR_EMS_AB_INBOUND_DN_O" },
	{ PR_EMS_AB_INBOUND_DN_O_ERROR, "PR_EMS_AB_INBOUND_DN_O_ERROR" },
	{ PR_EMS_AB_INBOUND_DN, "PR_EMS_AB_INBOUND_DN" },
	{ PR_EMS_AB_INBOUND_DN_UNICODE, "PR_EMS_AB_INBOUND_DN_UNICODE" },
	{ PR_EMS_AB_INBOUND_NEWSFEED_TYPE, "PR_EMS_AB_INBOUND_NEWSFEED_TYPE" },
	{ PR_EMS_AB_INBOUND_NEWSFEED_TYPE_ERROR, "PR_EMS_AB_INBOUND_NEWSFEED_TYPE_ERROR" },
	{ PR_EMS_AB_INBOUND_ACCEPT_ALL, "PR_EMS_AB_INBOUND_ACCEPT_ALL" },
	{ PR_EMS_AB_INBOUND_ACCEPT_ALL_ERROR, "PR_EMS_AB_INBOUND_ACCEPT_ALL_ERROR" },
	{ PR_EMS_AB_ENABLED, "PR_EMS_AB_ENABLED" },
	{ PR_EMS_AB_ENABLED_ERROR, "PR_EMS_AB_ENABLED_ERROR" },
	{ PR_EMS_AB_PRESERVE_INTERNET_CONTENT, "PR_EMS_AB_PRESERVE_INTERNET_CONTENT" },
	{ PR_EMS_AB_PRESERVE_INTERNET_CONTENT_ERROR, "PR_EMS_AB_PRESERVE_INTERNET_CONTENT_ERROR" },
	{ PR_EMS_AB_DISABLE_DEFERRED_COMMIT, "PR_EMS_AB_DISABLE_DEFERRED_COMMIT" },
	{ PR_EMS_AB_DISABLE_DEFERRED_COMMIT_ERROR, "PR_EMS_AB_DISABLE_DEFERRED_COMMIT_ERROR" },
	{ PR_EMS_AB_CLIENT_ACCESS_ENABLED, "PR_EMS_AB_CLIENT_ACCESS_ENABLED" },
	{ PR_EMS_AB_CLIENT_ACCESS_ENABLED_ERROR, "PR_EMS_AB_CLIENT_ACCESS_ENABLED_ERROR" },
	{ PR_EMS_AB_REQUIRE_SSL, "PR_EMS_AB_REQUIRE_SSL" },
	{ PR_EMS_AB_REQUIRE_SSL_ERROR, "PR_EMS_AB_REQUIRE_SSL_ERROR" },
	{ PR_EMS_AB_ANONYMOUS_ACCOUNT, "PR_EMS_AB_ANONYMOUS_ACCOUNT" },
	{ PR_EMS_AB_ANONYMOUS_ACCOUNT_UNICODE, "PR_EMS_AB_ANONYMOUS_ACCOUNT_UNICODE" },
	{ PR_EMS_AB_ANONYMOUS_ACCOUNT_ERROR, "PR_EMS_AB_ANONYMOUS_ACCOUNT_ERROR" },
	{ PR_EMS_AB_CERTIFICATE_CHAIN_V3, "PR_EMS_AB_CERTIFICATE_CHAIN_V3" },
	{ PR_EMS_AB_CERTIFICATE_CHAIN_V3_ERROR, "PR_EMS_AB_CERTIFICATE_CHAIN_V3_ERROR" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3_ERROR, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3_ERROR" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1" },
	{ PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1_ERROR, "PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1_ERROR" },
	{ PR_EMS_AB_CROSS_CERTIFICATE_CRL, "PR_EMS_AB_CROSS_CERTIFICATE_CRL" },
	{ PR_EMS_AB_CROSS_CERTIFICATE_CRL_ERROR, "PR_EMS_AB_CROSS_CERTIFICATE_CRL_ERROR" },
	{ PR_EMS_AB_SEND_EMAIL_MESSAGE, "PR_EMS_AB_SEND_EMAIL_MESSAGE" },
	{ PR_EMS_AB_SEND_EMAIL_MESSAGE_ERROR, "PR_EMS_AB_SEND_EMAIL_MESSAGE_ERROR" },
	{ PR_EMS_AB_ENABLE_COMPATIBILITY, "PR_EMS_AB_ENABLE_COMPATIBILITY" },
	{ PR_EMS_AB_ENABLE_COMPATIBILITY_ERROR, "PR_EMS_AB_ENABLE_COMPATIBILITY_ERROR" },
	{ PR_EMS_AB_SMIME_ALG_LIST_NA, "PR_EMS_AB_SMIME_ALG_LIST_NA" },
	{ PR_EMS_AB_SMIME_ALG_LIST_NA_UNICODE, "PR_EMS_AB_SMIME_ALG_LIST_NA_UNICODE" },
	{ PR_EMS_AB_SMIME_ALG_LIST_NA_ERROR, "PR_EMS_AB_SMIME_ALG_LIST_NA_ERROR" },
	{ PR_EMS_AB_SMIME_ALG_LIST_OTHER, "PR_EMS_AB_SMIME_ALG_LIST_OTHER" },
	{ PR_EMS_AB_SMIME_ALG_LIST_OTHER_UNICODE, "PR_EMS_AB_SMIME_ALG_LIST_OTHER_UNICODE" },
	{ PR_EMS_AB_SMIME_ALG_LIST_OTHER_ERROR, "PR_EMS_AB_SMIME_ALG_LIST_OTHER_ERROR" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_NA, "PR_EMS_AB_SMIME_ALG_SELECTED_NA" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_NA_UNICODE, "PR_EMS_AB_SMIME_ALG_SELECTED_NA_UNICODE" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_NA_ERROR, "PR_EMS_AB_SMIME_ALG_SELECTED_NA_ERROR" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_OTHER, "PR_EMS_AB_SMIME_ALG_SELECTED_OTHER" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_UNICODE, "PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_UNICODE" },
	{ PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_ERROR, "PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_ERROR" },
	{ PR_EMS_AB_DEFAULT_MESSAGE_FORMAT, "PR_EMS_AB_DEFAULT_MESSAGE_FORMAT" },
	{ PR_EMS_AB_DEFAULT_MESSAGE_FORMAT_ERROR, "PR_EMS_AB_DEFAULT_MESSAGE_FORMAT_ERROR" },
	{ PR_EMS_AB_TYPE, "PR_EMS_AB_TYPE" },
	{ PR_EMS_AB_TYPE_UNICODE, "PR_EMS_AB_TYPE_UNICODE" },
	{ PR_EMS_AB_TYPE_ERROR, "PR_EMS_AB_TYPE_ERROR" },
	{ PR_EMS_AB_DO_OAB_VERSION, "PR_EMS_AB_DO_OAB_VERSION" },
	{ PR_EMS_AB_DO_OAB_VERSION_ERROR, "PR_EMS_AB_DO_OAB_VERSION_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID, "PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID" },
	{ PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID_ERROR, "PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_USER_ID, "PR_EMS_AB_VOICE_MAIL_USER_ID" },
	{ PR_EMS_AB_VOICE_MAIL_USER_ID_UNICODE, "PR_EMS_AB_VOICE_MAIL_USER_ID_UNICODE" },
	{ PR_EMS_AB_VOICE_MAIL_USER_ID_ERROR, "PR_EMS_AB_VOICE_MAIL_USER_ID_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_PASSWORD, "PR_EMS_AB_VOICE_MAIL_PASSWORD" },
	{ PR_EMS_AB_VOICE_MAIL_PASSWORD_UNICODE, "PR_EMS_AB_VOICE_MAIL_PASSWORD_UNICODE" },
	{ PR_EMS_AB_VOICE_MAIL_PASSWORD_ERROR, "PR_EMS_AB_VOICE_MAIL_PASSWORD_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_RECORDED_NAME, "PR_EMS_AB_VOICE_MAIL_RECORDED_NAME" },
	{ PR_EMS_AB_VOICE_MAIL_RECORDED_NAME_ERROR, "PR_EMS_AB_VOICE_MAIL_RECORDED_NAME_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_GREETINGS, "PR_EMS_AB_VOICE_MAIL_GREETINGS" },
	{ PR_EMS_AB_VOICE_MAIL_GREETINGS_UNICODE, "PR_EMS_AB_VOICE_MAIL_GREETINGS_UNICODE" },
	{ PR_EMS_AB_VOICE_MAIL_GREETINGS_ERROR, "PR_EMS_AB_VOICE_MAIL_GREETINGS_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_FLAGS, "PR_EMS_AB_VOICE_MAIL_FLAGS" },
	{ PR_EMS_AB_VOICE_MAIL_FLAGS_ERROR, "PR_EMS_AB_VOICE_MAIL_FLAGS_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_VOLUME, "PR_EMS_AB_VOICE_MAIL_VOLUME" },
	{ PR_EMS_AB_VOICE_MAIL_VOLUME_ERROR, "PR_EMS_AB_VOICE_MAIL_VOLUME_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_SPEED, "PR_EMS_AB_VOICE_MAIL_SPEED" },
	{ PR_EMS_AB_VOICE_MAIL_SPEED_ERROR, "PR_EMS_AB_VOICE_MAIL_SPEED_ERROR" },
	{ PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH, "PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH" },
	{ PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH_ERROR, "PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH_ERROR" },
	{ PR_EMS_AB_DISPLAY_NAME_SUFFIX, "PR_EMS_AB_DISPLAY_NAME_SUFFIX" },
	{ PR_EMS_AB_DISPLAY_NAME_SUFFIX_UNICODE, "PR_EMS_AB_DISPLAY_NAME_SUFFIX_UNICODE" },
	{ PR_EMS_AB_DISPLAY_NAME_SUFFIX_ERROR, "PR_EMS_AB_DISPLAY_NAME_SUFFIX_ERROR" },
	{ PR_EMS_AB_ATTRIBUTE_CERTIFICATE, "PR_EMS_AB_ATTRIBUTE_CERTIFICATE" },
	{ PR_EMS_AB_ATTRIBUTE_CERTIFICATE_ERROR, "PR_EMS_AB_ATTRIBUTE_CERTIFICATE_ERROR" },
	{ PR_EMS_AB_DELTA_REVOCATION_LIST, "PR_EMS_AB_DELTA_REVOCATION_LIST" },
	{ PR_EMS_AB_DELTA_REVOCATION_LIST_ERROR, "PR_EMS_AB_DELTA_REVOCATION_LIST_ERROR" },
	{ PR_EMS_AB_SECURITY_POLICY, "PR_EMS_AB_SECURITY_POLICY" },
	{ PR_EMS_AB_SECURITY_POLICY_ERROR, "PR_EMS_AB_SECURITY_POLICY_ERROR" },
	{ PR_EMS_AB_SUPPORT_SMIME_SIGNATURES, "PR_EMS_AB_SUPPORT_SMIME_SIGNATURES" },
	{ PR_EMS_AB_SUPPORT_SMIME_SIGNATURES_ERROR, "PR_EMS_AB_SUPPORT_SMIME_SIGNATURES_ERROR" },
	{ PR_EMS_AB_DELEGATE_USER, "PR_EMS_AB_DELEGATE_USER" },
	{ PR_EMS_AB_DELEGATE_USER_ERROR, "PR_EMS_AB_DELEGATE_USER_ERROR" },
	{ PR_EMS_AB_LIST_PUBLIC_FOLDERS, "PR_EMS_AB_LIST_PUBLIC_FOLDERS" },
	{ PR_EMS_AB_LIST_PUBLIC_FOLDERS_ERROR, "PR_EMS_AB_LIST_PUBLIC_FOLDERS_ERROR" },
	{ PR_EMS_AB_LABELEDURI, "PR_EMS_AB_LABELEDURI" },
	{ PR_EMS_AB_LABELEDURI_UNICODE, "PR_EMS_AB_LABELEDURI_UNICODE" },
	{ PR_EMS_AB_LABELEDURI_ERROR, "PR_EMS_AB_LABELEDURI_ERROR" },
	{ PR_EMS_AB_RETURN_EXACT_MSG_SIZE, "PR_EMS_AB_RETURN_EXACT_MSG_SIZE" },
	{ PR_EMS_AB_RETURN_EXACT_MSG_SIZE_ERROR, "PR_EMS_AB_RETURN_EXACT_MSG_SIZE_ERROR" },
	{ PR_EMS_AB_GENERATION_QUALIFIER, "PR_EMS_AB_GENERATION_QUALIFIER" },
	{ PR_EMS_AB_GENERATION_QUALIFIER_UNICODE, "PR_EMS_AB_GENERATION_QUALIFIER_UNICODE" },
	{ PR_EMS_AB_GENERATION_QUALIFIER_ERROR, "PR_EMS_AB_GENERATION_QUALIFIER_ERROR" },
	{ PR_EMS_AB_HOUSE_IDENTIFIER, "PR_EMS_AB_HOUSE_IDENTIFIER" },
	{ PR_EMS_AB_HOUSE_IDENTIFIER_UNICODE, "PR_EMS_AB_HOUSE_IDENTIFIER_UNICODE" },
	{ PR_EMS_AB_HOUSE_IDENTIFIER_ERROR, "PR_EMS_AB_HOUSE_IDENTIFIER_ERROR" },
	{ PR_EMS_AB_SUPPORTED_ALGORITHMS, "PR_EMS_AB_SUPPORTED_ALGORITHMS" },
	{ PR_EMS_AB_SUPPORTED_ALGORITHMS_ERROR, "PR_EMS_AB_SUPPORTED_ALGORITHMS_ERROR" },
	{ PR_EMS_AB_DMD_NAME, "PR_EMS_AB_DMD_NAME" },
	{ PR_EMS_AB_DMD_NAME_UNICODE, "PR_EMS_AB_DMD_NAME_UNICODE" },
	{ PR_EMS_AB_DMD_NAME_ERROR, "PR_EMS_AB_DMD_NAME_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_11, "PR_EMS_AB_EXTENSION_ATTRIBUTE_11" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_11_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_11_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_11_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_11_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_12, "PR_EMS_AB_EXTENSION_ATTRIBUTE_12" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_12_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_12_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_12_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_12_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_13, "PR_EMS_AB_EXTENSION_ATTRIBUTE_13" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_13_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_13_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_13_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_13_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_14, "PR_EMS_AB_EXTENSION_ATTRIBUTE_14" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_14_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_14_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_14_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_14_ERROR" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_15, "PR_EMS_AB_EXTENSION_ATTRIBUTE_15" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_15_UNICODE, "PR_EMS_AB_EXTENSION_ATTRIBUTE_15_UNICODE" },
	{ PR_EMS_AB_EXTENSION_ATTRIBUTE_15_ERROR, "PR_EMS_AB_EXTENSION_ATTRIBUTE_15_ERROR" },
	{ PR_EMS_AB_REPLICATED_OBJECT_VERSION, "PR_EMS_AB_REPLICATED_OBJECT_VERSION" },
	{ PR_EMS_AB_REPLICATED_OBJECT_VERSION_ERROR, "PR_EMS_AB_REPLICATED_OBJECT_VERSION_ERROR" },
	{ PR_EMS_AB_MAIL_DROP, "PR_EMS_AB_MAIL_DROP" },
	{ PR_EMS_AB_MAIL_DROP_UNICODE, "PR_EMS_AB_MAIL_DROP_UNICODE" },
	{ PR_EMS_AB_MAIL_DROP_ERROR, "PR_EMS_AB_MAIL_DROP_ERROR" },
	{ PR_EMS_AB_FORWARDING_ADDRESS, "PR_EMS_AB_FORWARDING_ADDRESS" },
	{ PR_EMS_AB_FORWARDING_ADDRESS_UNICODE, "PR_EMS_AB_FORWARDING_ADDRESS_UNICODE" },
	{ PR_EMS_AB_FORWARDING_ADDRESS_ERROR, "PR_EMS_AB_FORWARDING_ADDRESS_ERROR" },
	{ PR_EMS_AB_FORM_DATA, "PR_EMS_AB_FORM_DATA" },
	{ PR_EMS_AB_FORM_DATA_ERROR, "PR_EMS_AB_FORM_DATA_ERROR" },
	{ PR_EMS_AB_OWA_SERVER, "PR_EMS_AB_OWA_SERVER" },
	{ PR_EMS_AB_OWA_SERVER_UNICODE, "PR_EMS_AB_OWA_SERVER_UNICODE" },
	{ PR_EMS_AB_OWA_SERVER_ERROR, "PR_EMS_AB_OWA_SERVER_ERROR" },
	{ PR_EMS_AB_EMPLOYEE_NUMBER, "PR_EMS_AB_EMPLOYEE_NUMBER" },
	{ PR_EMS_AB_EMPLOYEE_NUMBER_UNICODE, "PR_EMS_AB_EMPLOYEE_NUMBER_UNICODE" },
	{ PR_EMS_AB_EMPLOYEE_NUMBER_ERROR, "PR_EMS_AB_EMPLOYEE_NUMBER_ERROR" },
	{ PR_EMS_AB_TELEPHONE_PERSONAL_PAGER, "PR_EMS_AB_TELEPHONE_PERSONAL_PAGER" },
	{ PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_UNICODE, "PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_UNICODE" },
	{ PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_ERROR, "PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_ERROR" },
	{ PR_EMS_AB_EMPLOYEE_TYPE, "PR_EMS_AB_EMPLOYEE_TYPE" },
	{ PR_EMS_AB_EMPLOYEE_TYPE_UNICODE, "PR_EMS_AB_EMPLOYEE_TYPE_UNICODE" },
	{ PR_EMS_AB_EMPLOYEE_TYPE_ERROR, "PR_EMS_AB_EMPLOYEE_TYPE_ERROR" },
	{ PR_EMS_AB_TAGGED_X509_CERT, "PR_EMS_AB_TAGGED_X509_CERT" },
	{ PR_EMS_AB_TAGGED_X509_CERT_ERROR, "PR_EMS_AB_TAGGED_X509_CERT_ERROR" },
	{ PR_EMS_AB_PERSONAL_TITLE, "PR_EMS_AB_PERSONAL_TITLE" },
	{ PR_EMS_AB_PERSONAL_TITLE_UNICODE, "PR_EMS_AB_PERSONAL_TITLE_UNICODE" },
	{ PR_EMS_AB_PERSONAL_TITLE_ERROR, "PR_EMS_AB_PERSONAL_TITLE_ERROR" },
	{ PR_EMS_AB_LANGUAGE_ISO639, "PR_EMS_AB_LANGUAGE_ISO639" },
	{ PR_EMS_AB_LANGUAGE_ISO639_UNICODE, "PR_EMS_AB_LANGUAGE_ISO639_UNICODE" },
	{ PR_EMS_AB_LANGUAGE_ISO639_ERROR, "PR_EMS_AB_LANGUAGE_ISO639_ERROR" },
	{ PR_EMS_AB_OTHER_RECIPS, "PR_EMS_AB_OTHER_RECIPS" },
	{ PR_EMS_AB_OTHER_RECIPS_ERROR, "PR_EMS_AB_OTHER_RECIPS_ERROR" },
	{ PR_EMS_AB_CHILD_RDNS, "PR_EMS_AB_CHILD_RDNS" },
	{ PR_EMS_AB_CHILD_RDNS_UNICODE, "PR_EMS_AB_CHILD_RDNS_UNICODE" },
	{ PR_EMS_AB_CHILD_RDNS_ERROR, "PR_EMS_AB_CHILD_RDNS_ERROR" },
	{ PR_EMS_AB_HIERARCHY_PATH, "PR_EMS_AB_HIERARCHY_PATH" },
	{ PR_EMS_AB_HIERARCHY_PATH_UNICODE, "PR_EMS_AB_HIERARCHY_PATH_UNICODE" },
	{ PR_EMS_AB_HIERARCHY_PATH_ERROR, "PR_EMS_AB_HIERARCHY_PATH_ERROR" },
	{ PR_EMS_AB_OBJECT_OID, "PR_EMS_AB_OBJECT_OID" },
	{ PR_EMS_AB_OBJECT_OID_ERROR, "PR_EMS_AB_OBJECT_OID_ERROR" },
	{ PR_EMS_AB_IS_MASTER, "PR_EMS_AB_IS_MASTER" },
	{ PR_EMS_AB_IS_MASTER_ERROR, "PR_EMS_AB_IS_MASTER_ERROR" },
	{ PR_EMS_AB_PARENT_ENTRYID, "PR_EMS_AB_PARENT_ENTRYID" },
	{ PR_EMS_AB_PARENT_ENTRYID_ERROR, "PR_EMS_AB_PARENT_ENTRYID_ERROR" },
	{ PR_EMS_AB_CONTAINERID, "PR_EMS_AB_CONTAINERID" },
	{ PR_EMS_AB_CONTAINERID_ERROR, "PR_EMS_AB_CONTAINERID_ERROR" },
	{ PR_EMS_AB_SERVER, "PR_EMS_AB_SERVER" },
	{ PR_EMS_AB_SERVER_UNICODE, "PR_EMS_AB_SERVER_UNICODE" },
	{ PR_EMS_AB_SERVER_ERROR, "PR_EMS_AB_SERVER_ERROR" },
	{ MAPI_PROP_RESERVED, "MAPI_PROP_RESERVED" },
{ 0, NULL }
};
const value_string nspi_MAPISTATUS_vals[] = {
	{ MAPI_E_SUCCESS, "MAPI_E_SUCCESS" },
	{ MAPI_E_NO_SUPPORT, "MAPI_E_NO_SUPPORT" },
	{ MAPI_E_BAD_CHARWIDTH, "MAPI_E_BAD_CHARWIDTH" },
	{ MAPI_E_STRING_TOO_LONG, "MAPI_E_STRING_TOO_LONG" },
	{ MAPI_E_UNKNOWN_FLAGS, "MAPI_E_UNKNOWN_FLAGS" },
	{ MAPI_E_INVALID_ENTRYID, "MAPI_E_INVALID_ENTRYID" },
	{ MAPI_E_INVALID_OBJECT, "MAPI_E_INVALID_OBJECT" },
	{ MAPI_E_OBJECT_CHANGED, "MAPI_E_OBJECT_CHANGED" },
	{ MAPI_E_OBJECT_DELETED, "MAPI_E_OBJECT_DELETED" },
	{ MAPI_E_BUSY, "MAPI_E_BUSY" },
	{ MAPI_E_NOT_ENOUGH_DISK, "MAPI_E_NOT_ENOUGH_DISK" },
	{ MAPI_E_NOT_ENOUGH_RESOURCES, "MAPI_E_NOT_ENOUGH_RESOURCES" },
	{ MAPI_E_NOT_FOUND, "MAPI_E_NOT_FOUND" },
	{ MAPI_E_VERSION, "MAPI_E_VERSION" },
	{ MAPI_E_LOGON_FAILED, "MAPI_E_LOGON_FAILED" },
	{ MAPI_E_SESSION_LIMIT, "MAPI_E_SESSION_LIMIT" },
	{ MAPI_E_USER_CANCEL, "MAPI_E_USER_CANCEL" },
	{ MAPI_E_UNABLE_TO_ABORT, "MAPI_E_UNABLE_TO_ABORT" },
	{ MAPI_E_NETWORK_ERROR, "MAPI_E_NETWORK_ERROR" },
	{ MAPI_E_DISK_ERROR, "MAPI_E_DISK_ERROR" },
	{ MAPI_E_TOO_COMPLEX, "MAPI_E_TOO_COMPLEX" },
	{ MAPI_E_BAD_COLUMN, "MAPI_E_BAD_COLUMN" },
	{ MAPI_E_EXTENDED_ERROR, "MAPI_E_EXTENDED_ERROR" },
	{ MAPI_E_COMPUTED, "MAPI_E_COMPUTED" },
	{ MAPI_E_CORRUPT_DATA, "MAPI_E_CORRUPT_DATA" },
	{ MAPI_E_UNCONFIGURED, "MAPI_E_UNCONFIGURED" },
	{ MAPI_E_FAILONEPROVIDER, "MAPI_E_FAILONEPROVIDER" },
	{ MAPI_E_UNKNOWN_CPID, "MAPI_E_UNKNOWN_CPID" },
	{ MAPI_E_UNKNOWN_LCID, "MAPI_E_UNKNOWN_LCID" },
	{ MAPI_E_PASSWORD_CHANGE_REQUIRED, "MAPI_E_PASSWORD_CHANGE_REQUIRED" },
	{ MAPI_E_PASSWORD_EXPIRED, "MAPI_E_PASSWORD_EXPIRED" },
	{ MAPI_E_INVALID_WORKSTATION_ACCOUNT, "MAPI_E_INVALID_WORKSTATION_ACCOUNT" },
	{ MAPI_E_INVALID_ACCESS_TIME, "MAPI_E_INVALID_ACCESS_TIME" },
	{ MAPI_E_ACCOUNT_DISABLED, "MAPI_E_ACCOUNT_DISABLED" },
	{ MAPI_E_END_OF_SESSION, "MAPI_E_END_OF_SESSION" },
	{ MAPI_E_UNKNOWN_ENTRYID, "MAPI_E_UNKNOWN_ENTRYID" },
	{ MAPI_E_MISSING_REQUIRED_COLUMN, "MAPI_E_MISSING_REQUIRED_COLUMN" },
	{ MAPI_W_NO_SERVICE, "MAPI_W_NO_SERVICE" },
	{ MAPI_E_BAD_VALUE, "MAPI_E_BAD_VALUE" },
	{ MAPI_E_INVALID_TYPE, "MAPI_E_INVALID_TYPE" },
	{ MAPI_E_TYPE_NO_SUPPORT, "MAPI_E_TYPE_NO_SUPPORT" },
	{ MAPI_E_UNEXPECTED_TYPE, "MAPI_E_UNEXPECTED_TYPE" },
	{ MAPI_E_TOO_BIG, "MAPI_E_TOO_BIG" },
	{ MAPI_E_DECLINE_COPY, "MAPI_E_DECLINE_COPY" },
	{ MAPI_E_UNEXPECTED_ID, "MAPI_E_UNEXPECTED_ID" },
	{ MAPI_W_ERRORS_RETURNED, "MAPI_W_ERRORS_RETURNED" },
	{ MAPI_E_UNABLE_TO_COMPLETE, "MAPI_E_UNABLE_TO_COMPLETE" },
	{ MAPI_E_TIMEOUT, "MAPI_E_TIMEOUT" },
	{ MAPI_E_TABLE_EMPTY, "MAPI_E_TABLE_EMPTY" },
	{ MAPI_E_TABLE_TOO_BIG, "MAPI_E_TABLE_TOO_BIG" },
	{ MAPI_E_INVALID_BOOKMARK, "MAPI_E_INVALID_BOOKMARK" },
	{ MAPI_W_POSITION_CHANGED, "MAPI_W_POSITION_CHANGED" },
	{ MAPI_W_APPROX_COUNT, "MAPI_W_APPROX_COUNT" },
	{ MAPI_E_WAIT, "MAPI_E_WAIT" },
	{ MAPI_E_CANCEL, "MAPI_E_CANCEL" },
	{ MAPI_E_NOT_ME, "MAPI_E_NOT_ME" },
	{ MAPI_W_CANCEL_MESSAGE, "MAPI_W_CANCEL_MESSAGE" },
	{ MAPI_E_CORRUPT_STORE, "MAPI_E_CORRUPT_STORE" },
	{ MAPI_E_NOT_IN_QUEUE, "MAPI_E_NOT_IN_QUEUE" },
	{ MAPI_E_NO_SUPPRESS, "MAPI_E_NO_SUPPRESS" },
	{ MAPI_E_COLLISION, "MAPI_E_COLLISION" },
	{ MAPI_E_NOT_INITIALIZED, "MAPI_E_NOT_INITIALIZED" },
	{ MAPI_E_NON_STANDARD, "MAPI_E_NON_STANDARD" },
	{ MAPI_E_NO_RECIPIENTS, "MAPI_E_NO_RECIPIENTS" },
	{ MAPI_E_SUBMITTED, "MAPI_E_SUBMITTED" },
	{ MAPI_E_HAS_FOLDERS, "MAPI_E_HAS_FOLDERS" },
	{ MAPI_E_HAS_MESAGES, "MAPI_E_HAS_MESAGES" },
	{ MAPI_E_FOLDER_CYCLE, "MAPI_E_FOLDER_CYCLE" },
	{ MAPI_W_PARTIAL_COMPLETION, "MAPI_W_PARTIAL_COMPLETION" },
	{ MAPI_E_AMBIGUOUS_RECIP, "MAPI_E_AMBIGUOUS_RECIP" },
	{ MAPI_E_RESERVED, "MAPI_E_RESERVED" },
{ 0, NULL }
};
static int nspi_dissect_element_MAPIUID_ab(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPIUID_ab_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_input_locale_language(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_input_locale_method(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPI_SETTINGS_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPI_SETTINGS_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPI_SETTINGS_service_provider(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPI_SETTINGS_codepage(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPI_SETTINGS_input_locale(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropTagArray_aulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropTagArray_aulPropTag_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropTagArray_aulPropTag__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropTagArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_instance_key_value(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_instance_key_value_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_instance_key_value__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_instance_key_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPINAMEID_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPINAMEID_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPINAMEID_ulKind(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MAPINAMEID_lID(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
const value_string nspi_nspi_RestrictionType_vals[] = {
	{ RES_AND, "RES_AND" },
	{ RES_OR, "RES_OR" },
	{ RES_NOT, "RES_NOT" },
	{ RES_CONTENT, "RES_CONTENT" },
	{ RES_PROPERTY, "RES_PROPERTY" },
	{ RES_COMPAREPROPS, "RES_COMPAREPROPS" },
	{ RES_BITMASK, "RES_BITMASK" },
	{ RES_SIZE, "RES_SIZE" },
	{ RES_EXIST, "RES_EXIST" },
	{ RES_SUBRESTRICTION, "RES_SUBRESTRICTION" },
	{ RES_COMMENT, "RES_COMMENT" },
{ 0, NULL }
};
static int nspi_dissect_element_SPropertyRestriction_relop(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropertyRestriction_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropertyRestriction_lpProp(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropertyRestriction_lpProp_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SAndRestriction_cRes(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SAndRestriction_lpRes(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SAndRestriction_lpRes_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SAndRestriction_lpRes__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRestriction_CTR_resAnd(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRestriction_CTR_resProperty(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRestriction_rt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRestriction_res(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrder_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrder_ulOrder(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_cSorts(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_cCategories(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_cExpanded(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_aSort(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_aSort_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SSortOrderSet_aSort__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NAME_STRING_str(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NAME_STRING_str_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
const value_string nspi_property_types_vals[] = {
	{ PT_UNSPECIFIED, "PT_UNSPECIFIED" },
	{ PT_NULL, "PT_NULL" },
	{ PT_I2, "PT_I2" },
	{ PT_LONG, "PT_LONG" },
	{ PT_R4, "PT_R4" },
	{ PT_DOUBLE, "PT_DOUBLE" },
	{ PT_CURRENCY, "PT_CURRENCY" },
	{ PT_APPTIME, "PT_APPTIME" },
	{ PT_ERROR, "PT_ERROR" },
	{ PT_BOOLEAN, "PT_BOOLEAN" },
	{ PT_OBJECT, "PT_OBJECT" },
	{ PT_I8, "PT_I8" },
	{ PT_STRING8, "PT_STRING8" },
	{ PT_UNICODE, "PT_UNICODE" },
	{ PT_SYSTIME, "PT_SYSTIME" },
	{ PT_CLSID, "PT_CLSID" },
	{ PT_BINARY, "PT_BINARY" },
	{ PT_MV_I2, "PT_MV_I2" },
	{ PT_MV_LONG, "PT_MV_LONG" },
	{ PT_MV_R4, "PT_MV_R4" },
	{ PT_MV_DOUBLE, "PT_MV_DOUBLE" },
	{ PT_MV_CURRENCY, "PT_MV_CURRENCY" },
	{ PT_MV_APPTIME, "PT_MV_APPTIME" },
	{ PT_MV_I8, "PT_MV_I8" },
	{ PT_MV_STRING8, "PT_MV_STRING8" },
	{ PT_MV_TSTRING, "PT_MV_TSTRING" },
	{ PT_MV_UNICODE, "PT_MV_UNICODE" },
	{ PT_MV_SYSTIME, "PT_MV_SYSTIME" },
	{ PT_MV_CLSID, "PT_MV_CLSID" },
	{ PT_MV_BINARY, "PT_MV_BINARY" },
{ 0, NULL }
};
static int nspi_dissect_element_SBinary_cb(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinary_lpb(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinary_lpb_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinary_lpb__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_FILETIME_dwLowDateTime(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_FILETIME_dwHighDateTime(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SShortArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SShortArray_lpi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SShortArray_lpi_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SShortArray_lpi__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_LONG_STRUCT_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_LONG_STRUCT_lpl(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_LONG_STRUCT_lpl_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_LONG_STRUCT_lpl__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_LPSTR_lppszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SLPSTRArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SLPSTRArray_strings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SLPSTRArray_strings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SLPSTRArray_strings__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SLPSTRArray_strings___(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinaryArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinaryArray_lpbin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinaryArray_lpbin_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SBinaryArray_lpbin__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SGuidArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SGuidArray_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SGuidArray_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SGuidArray_lpguid__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_UNICODE_STRUCT_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_UNICODE_STRUCT_lpi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_UNICODE_STRUCT_lpi_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_MV_UNICODE_STRUCT_lpi__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SDateTimeArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SDateTimeArray_lpft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SDateTimeArray_lpft_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SDateTimeArray_lpft__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_i(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_l(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_b(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpszA_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_bin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpszW(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpszW_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_ft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_err(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVl(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVbin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVszW(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_MVft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_null(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_CTR_object(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_dwAlignPad(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SPropValue_value(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRow_ulAdrEntryPad(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRow_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRow_lpProps(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRow_lpProps_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRow_lpProps__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRowSet_cRows(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRowSet_aRow(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_SRowSet_aRow_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_unknown(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_mapiuid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_mapiuid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiBind_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiUnbind_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiUnbind_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiUnbind_status(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_lRows(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_instance_key__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_unknown(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiQueryRows_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_unknown1(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_PropTagArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_PropTagArray_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_unknown2(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_restrictions(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_restrictions_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_unknown3(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetMatches_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_size(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_server_dn(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_server_dn_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_server_dn__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiDNToEph_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_REPL_values(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_REPL_values_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetProps_REPL_values__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_unknown1(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_unknown2(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_unknown2_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);
static int nspi_dissect_element_NspiGetHierarchyInfo_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep);


/* IDL: enum { */
/* IDL: 	PR_ACKNOWLEDGEMENT_MODE=0x00010003, */
/* IDL: 	PR_ACKNOWLEDGEMENT_MODE_ERROR=0x0001000a, */
/* IDL: 	PR_ALTERNATE_RECIPIENT_ALLOWED=0x0002000b, */
/* IDL: 	PR_ALTERNATE_RECIPIENT_ALLOWED_ERROR=0x0002000a, */
/* IDL: 	PR_AUTHORIZING_USERS=0x00030102, */
/* IDL: 	PR_AUTHORIZING_USERS_ERROR=0x0003000a, */
/* IDL: 	PR_AUTO_FORWARD_COMMENT=0x0004001e, */
/* IDL: 	PR_AUTO_FORWARD_COMMENT_UNICODE=0x0004001f, */
/* IDL: 	PR_AUTO_FORWARD_COMMENT_ERROR=0x0004000a, */
/* IDL: 	PR_AUTO_FORWARDED=0x0005000b, */
/* IDL: 	PR_AUTO_FORWARDED_ERROR=0x0005000a, */
/* IDL: 	PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID=0x00060102, */
/* IDL: 	PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID_ERROR=0x0006000a, */
/* IDL: 	PR_CONTENT_CORRELATOR=0x00070102, */
/* IDL: 	PR_CONTENT_CORRELATOR_ERROR=0x0007000a, */
/* IDL: 	PR_CONTENT_IDENTIFIER=0x0008001e, */
/* IDL: 	PR_CONTENT_IDENTIFIER_UNICODE=0x0008001f, */
/* IDL: 	PR_CONTENT_IDENTIFIER_ERROR=0x0008000a, */
/* IDL: 	PR_CONTENT_LENGTH=0x00090003, */
/* IDL: 	PR_CONTENT_LENGTH_ERROR=0x0009000a, */
/* IDL: 	PR_CONTENT_RETURN_REQUESTED=0x000a000b, */
/* IDL: 	PR_CONTENT_RETURN_REQUESTED_ERROR=0x000a000a, */
/* IDL: 	PR_CONVERSATION_KEY=0x000b0102, */
/* IDL: 	PR_CONVERSATION_KEY_ERROR=0x000b000a, */
/* IDL: 	PR_CONVERSION_EITS=0x000c0102, */
/* IDL: 	PR_CONVERSION_EITS_ERROR=0x000c000a, */
/* IDL: 	PR_CONVERSION_WITH_LOSS_PROHIBITED=0x000d000b, */
/* IDL: 	PR_CONVERSION_WITH_LOSS_PROHIBITED_ERROR=0x000d000a, */
/* IDL: 	PR_CONVERTED_EITS=0x000e0102, */
/* IDL: 	PR_CONVERTED_EITS_ERROR=0x000e000a, */
/* IDL: 	PR_DEFERRED_DELIVERY_TIME=0x000f0040, */
/* IDL: 	PR_DEFERRED_DELIVERY_TIME_ERROR=0x000f000a, */
/* IDL: 	PR_DELIVER_TIME=0x00100040, */
/* IDL: 	PR_DELIVER_TIME_ERROR=0x0010000a, */
/* IDL: 	PR_DISCARD_REASON=0x00110003, */
/* IDL: 	PR_DISCARD_REASON_ERROR=0x0011000a, */
/* IDL: 	PR_DISCLOSURE_OF_RECIPIENTS=0x0012000b, */
/* IDL: 	PR_DISCLOSURE_OF_RECIPIENTS_ERROR=0x0012000a, */
/* IDL: 	PR_DL_EXPANSION_HISTORY=0x00130102, */
/* IDL: 	PR_DL_EXPANSION_HISTORY_ERROR=0x0013000a, */
/* IDL: 	PR_DL_EXPANSION_PROHIBITED=0x0014000b, */
/* IDL: 	PR_DL_EXPANSION_PROHIBITED_ERROR=0x0014000a, */
/* IDL: 	PR_EXPIRY_TIME=0x00150040, */
/* IDL: 	PR_EXPIRY_TIME_ERROR=0x0015000a, */
/* IDL: 	PR_IMPLICIT_CONVERSION_PROHIBITED=0x0016000b, */
/* IDL: 	PR_IMPLICIT_CONVERSION_PROHIBITED_ERROR=0x0016000a, */
/* IDL: 	PR_IMPORTANCE=0x00170003, */
/* IDL: 	PR_IMPORTANCE_ERROR=0x0017000a, */
/* IDL: 	PR_IPM_ID=0x00180102, */
/* IDL: 	PR_IPM_ID_ERROR=0x0018000a, */
/* IDL: 	PR_LATEST_DELIVERY_TIME=0x00190040, */
/* IDL: 	PR_LATEST_DELIVERY_TIME_ERROR=0x0019000a, */
/* IDL: 	PR_MESSAGE_CLASS=0x001a001e, */
/* IDL: 	PR_MESSAGE_CLASS_UNICODE=0x001a001f, */
/* IDL: 	PR_MESSAGE_CLASS_ERROR=0x001a000a, */
/* IDL: 	PR_MESSAGE_DELIVERY_ID=0x001b0102, */
/* IDL: 	PR_MESSAGE_DELIVERY_ID_ERROR=0x001b000a, */
/* IDL: 	PR_MESSAGE_SECURITY_LABEL=0x001e0102, */
/* IDL: 	PR_MESSAGE_SECURITY_LABEL_ERROR=0x001e000a, */
/* IDL: 	PR_OBSOLETED_IPMS=0x001f0102, */
/* IDL: 	PR_OBSOLETED_IPMS_ERROR=0x001f000a, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIPIENT_NAME=0x00200102, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIPIENT_NAME_ERROR=0x0020000a, */
/* IDL: 	PR_ORIGINAL_EITS=0x00210102, */
/* IDL: 	PR_ORIGINAL_EITS_ERROR=0x0021000a, */
/* IDL: 	PR_ORIGINATOR_CERTIFICATE=0x00220102, */
/* IDL: 	PR_ORIGINATOR_CERTIFICATE_ERROR=0x0022000a, */
/* IDL: 	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED=0x0023000b, */
/* IDL: 	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED_ERROR=0x0023000a, */
/* IDL: 	PR_ORIGINATOR_RETURN_ADDRESS=0x00240102, */
/* IDL: 	PR_ORIGINATOR_RETURN_ADDRESS_ERROR=0x0024000a, */
/* IDL: 	PR_PARENT_KEY=0x00250102, */
/* IDL: 	PR_PARENT_KEY_ERROR=0x0025000a, */
/* IDL: 	PR_PRIORITY=0x00260003, */
/* IDL: 	PR_PRIORITY_ERROR=0x0026000a, */
/* IDL: 	PR_ORIGIN_CHECK=0x00270102, */
/* IDL: 	PR_ORIGIN_CHECK_ERROR=0x0027000a, */
/* IDL: 	PR_PROOF_OF_SUBMISSION_REQUESTED=0x0028000b, */
/* IDL: 	PR_PROOF_OF_SUBMISSION_REQUESTED_ERROR=0x0028000a, */
/* IDL: 	PR_READ_RECEIPT_REQUESTED=0x0029000b, */
/* IDL: 	PR_READ_RECEIPT_REQUESTED_ERROR=0x0029000a, */
/* IDL: 	PR_RECEIPT_TIME=0x002a0040, */
/* IDL: 	PR_RECEIPT_TIME_ERROR=0x002a000a, */
/* IDL: 	PR_RECIPIENT_REASSIGNMENT_PROHIBITED=0x002b000b, */
/* IDL: 	PR_RECIPIENT_REASSIGNMENT_PROHIBITED_ERROR=0x002b000a, */
/* IDL: 	PR_REDIRECTION_HISTORY=0x002c0102, */
/* IDL: 	PR_REDIRECTION_HISTORY_ERROR=0x002c000a, */
/* IDL: 	PR_RELATED_IPMS=0x002d0102, */
/* IDL: 	PR_RELATED_IPMS_ERROR=0x002d000a, */
/* IDL: 	PR_ORIGINAL_SENSITIVITY=0x002e0003, */
/* IDL: 	PR_ORIGINAL_SENSITIVITY_ERROR=0x002e000a, */
/* IDL: 	PR_LANGUAGES=0x002f001e, */
/* IDL: 	PR_LANGUAGES_UNICODE=0x002f001f, */
/* IDL: 	PR_LANGUAGES_ERROR=0x002f000a, */
/* IDL: 	PR_REPLY_TIME=0x00300040, */
/* IDL: 	PR_REPLY_TIME_ERROR=0x0030000a, */
/* IDL: 	PR_REPORT_TAG=0x00310102, */
/* IDL: 	PR_REPORT_TAG_ERROR=0x0031000a, */
/* IDL: 	PR_REPORT_TIME=0x00320040, */
/* IDL: 	PR_REPORT_TIME_ERROR=0x0032000a, */
/* IDL: 	PR_RETURNED_IPM=0x0033000b, */
/* IDL: 	PR_RETURNED_IPM_ERROR=0x0033000a, */
/* IDL: 	PR_SECURITY=0x00340003, */
/* IDL: 	PR_SECURITY_ERROR=0x0034000a, */
/* IDL: 	PR_INCOMPLETE_COPY=0x0035000b, */
/* IDL: 	PR_INCOMPLETE_COPY_ERROR=0x0035000a, */
/* IDL: 	PR_SENSITIVITY=0x00360003, */
/* IDL: 	PR_SENSITIVITY_ERROR=0x0036000a, */
/* IDL: 	PR_SUBJECT=0x0037001e, */
/* IDL: 	PR_SUBJECT_UNICODE=0x0037001f, */
/* IDL: 	PR_SUBJECT_ERROR=0x0037000a, */
/* IDL: 	PR_SUBJECT_IPM=0x00380102, */
/* IDL: 	PR_SUBJECT_IPM_ERROR=0x0038000a, */
/* IDL: 	PR_CLIENT_SUBMIT_TIME=0x00390040, */
/* IDL: 	PR_CLIENT_SUBMIT_TIME_ERROR=0x0039000a, */
/* IDL: 	PR_REPORT_NAME=0x003a001e, */
/* IDL: 	PR_REPORT_NAME_UNICODE=0x003a001f, */
/* IDL: 	PR_REPORT_NAME_ERROR=0x003a000a, */
/* IDL: 	PR_SENT_REPRESENTING_SEARCH_KEY=0x003b0102, */
/* IDL: 	PR_SENT_REPRESENTING_SEARCH_KEY_ERROR=0x003b000a, */
/* IDL: 	PR_X400_CONTENT_TYPE=0x003c0102, */
/* IDL: 	PR_X400_CONTENT_TYPE_ERROR=0x003c000a, */
/* IDL: 	PR_SUBJECT_PREFIX=0x003d001e, */
/* IDL: 	PR_SUBJECT_PREFIX_UNICODE=0x003d001f, */
/* IDL: 	PR_SUBJECT_PREFIX_ERROR=0x003d000a, */
/* IDL: 	PR_NON_RECEIPT_REASON=0x003e0003, */
/* IDL: 	PR_NON_RECEIPT_REASON_ERROR=0x003e000a, */
/* IDL: 	PR_RECEIVED_BY_ENTRYID=0x003f0102, */
/* IDL: 	PR_RECEIVED_BY_ENTRYID_ERROR=0x003f000a, */
/* IDL: 	PR_RECEIVED_BY_NAME=0x0040001e, */
/* IDL: 	PR_RECEIVED_BY_NAME_UNICODE=0x0040001f, */
/* IDL: 	PR_RECEIVED_BY_NAME_ERROR=0x0040000a, */
/* IDL: 	PR_SENT_REPRESENTING_ENTRYID=0x00410102, */
/* IDL: 	PR_SENT_REPRESENTING_ENTRYID_ERROR=0x0041000a, */
/* IDL: 	PR_SENT_REPRESENTING_NAME=0x0042001e, */
/* IDL: 	PR_SENT_REPRESENTING_NAME_UNICODE=0x0042001f, */
/* IDL: 	PR_SENT_REPRESENTING_NAME_ERROR=0x0042000a, */
/* IDL: 	PR_RCVD_REPRESENTING_ENTRYID=0x00430102, */
/* IDL: 	PR_RCVD_REPRESENTING_ENTRYID_ERROR=0x0043000a, */
/* IDL: 	PR_RCVD_REPRESENTING_NAME=0x0044001e, */
/* IDL: 	PR_RCVD_REPRESENTING_NAME_UNICODE=0x0044001f, */
/* IDL: 	PR_RCVD_REPRESENTING_NAME_ERROR=0x0044000a, */
/* IDL: 	PR_REPORT_ENTRYID=0x00450102, */
/* IDL: 	PR_REPORT_ENTRYID_ERROR=0x0045000a, */
/* IDL: 	PR_READ_RECEIPT_ENTRYID=0x00460102, */
/* IDL: 	PR_READ_RECEIPT_ENTRYID_ERROR=0x0046000a, */
/* IDL: 	PR_MESSAGE_SUBMISSION_ID=0x00470102, */
/* IDL: 	PR_MESSAGE_SUBMISSION_ID_ERROR=0x0047000a, */
/* IDL: 	PR_PROVIDER_SUBMIT_TIME=0x00480040, */
/* IDL: 	PR_PROVIDER_SUBMIT_TIME_ERROR=0x0048000a, */
/* IDL: 	PR_ORIGINAL_SUBJECT=0x0049001e, */
/* IDL: 	PR_ORIGINAL_SUBJECT_UNICODE=0x0049001f, */
/* IDL: 	PR_ORIGINAL_SUBJECT_ERROR=0x0049000a, */
/* IDL: 	PR_DISC_VAL=0x004a000b, */
/* IDL: 	PR_DISC_VAL_ERROR=0x004a000a, */
/* IDL: 	PR_ORIG_MESSAGE_CLASS=0x004b001e, */
/* IDL: 	PR_ORIG_MESSAGE_CLASS_UNICODE=0x004b001f, */
/* IDL: 	PR_ORIG_MESSAGE_CLASS_ERROR=0x004b000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_ENTRYID=0x004c0102, */
/* IDL: 	PR_ORIGINAL_AUTHOR_ENTRYID_ERROR=0x004c000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_NAME=0x004d001e, */
/* IDL: 	PR_ORIGINAL_AUTHOR_NAME_UNICODE=0x004d001f, */
/* IDL: 	PR_ORIGINAL_AUTHOR_NAME_ERROR=0x004d000a, */
/* IDL: 	PR_ORIGINAL_SUBMIT_TIME=0x004e0040, */
/* IDL: 	PR_ORIGINAL_SUBMIT_TIME_ERROR=0x004e000a, */
/* IDL: 	PR_REPLY_RECIPIENT_ENTRIES=0x004f0102, */
/* IDL: 	PR_REPLY_RECIPIENT_ENTRIES_ERROR=0x004f000a, */
/* IDL: 	PR_REPLY_RECIPIENT_NAMES=0x0050001e, */
/* IDL: 	PR_REPLY_RECIPIENT_NAMES_UNICODE=0x0050001f, */
/* IDL: 	PR_REPLY_RECIPIENT_NAMES_ERROR=0x0050000a, */
/* IDL: 	PR_RECEIVED_BY_SEARCH_KEY=0x00510102, */
/* IDL: 	PR_RECEIVED_BY_SEARCH_KEY_ERROR=0x0051000a, */
/* IDL: 	PR_RCVD_REPRESENTING_SEARCH_KEY=0x00520102, */
/* IDL: 	PR_RCVD_REPRESENTING_SEARCH_KEY_ERROR=0x0052000a, */
/* IDL: 	PR_READ_RECEIPT_SEARCH_KEY=0x00530102, */
/* IDL: 	PR_READ_RECEIPT_SEARCH_KEY_ERROR=0x0053000a, */
/* IDL: 	PR_REPORT_SEARCH_KEY=0x00540102, */
/* IDL: 	PR_REPORT_SEARCH_KEY_ERROR=0x0054000a, */
/* IDL: 	PR_ORIGINAL_DELIVERY_TIME=0x00550040, */
/* IDL: 	PR_ORIGINAL_DELIVERY_TIME_ERROR=0x0055000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_SEARCH_KEY=0x00560102, */
/* IDL: 	PR_ORIGINAL_AUTHOR_SEARCH_KEY_ERROR=0x0056000a, */
/* IDL: 	PR_MESSAGE_TO_ME=0x0057000b, */
/* IDL: 	PR_MESSAGE_TO_ME_ERROR=0x0057000a, */
/* IDL: 	PR_MESSAGE_CC_ME=0x0058000b, */
/* IDL: 	PR_MESSAGE_CC_ME_ERROR=0x0058000a, */
/* IDL: 	PR_MESSAGE_RECIP_ME=0x0059000b, */
/* IDL: 	PR_MESSAGE_RECIP_ME_ERROR=0x0059000a, */
/* IDL: 	PR_ORIGINAL_SENDER_NAME=0x005a001e, */
/* IDL: 	PR_ORIGINAL_SENDER_NAME_UNICODE=0x005a001f, */
/* IDL: 	PR_ORIGINAL_SENDER_NAME_ERROR=0x005a000a, */
/* IDL: 	PR_ORIGINAL_SENDER_ENTRYID=0x005b0102, */
/* IDL: 	PR_ORIGINAL_SENDER_ENTRYID_ERROR=0x005b000a, */
/* IDL: 	PR_ORIGINAL_SENDER_SEARCH_KEY=0x005c0102, */
/* IDL: 	PR_ORIGINAL_SENDER_SEARCH_KEY_ERROR=0x005c000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_NAME=0x005d001e, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_NAME_UNICODE=0x005d001f, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_NAME_ERROR=0x005d000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_ENTRYID=0x005e0102, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_ENTRYID_ERROR=0x005e000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY=0x005f0102, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY_ERROR=0x005f000a, */
/* IDL: 	PR_START_DATE=0x00600040, */
/* IDL: 	PR_START_DATE_ERROR=0x0060000a, */
/* IDL: 	PR_END_DATE=0x00610040, */
/* IDL: 	PR_END_DATE_ERROR=0x0061000a, */
/* IDL: 	PR_OWNER_APPT_ID=0x00620003, */
/* IDL: 	PR_OWNER_APPT_ID_ERROR=0x0062000a, */
/* IDL: 	PR_RESPONSE_REQUESTED=0x0063000b, */
/* IDL: 	PR_RESPONSE_REQUESTED_ERROR=0x0063000a, */
/* IDL: 	PR_SENT_REPRESENTING_ADDRTYPE=0x0064001e, */
/* IDL: 	PR_SENT_REPRESENTING_ADDRTYPE_UNICODE=0x0064001f, */
/* IDL: 	PR_SENT_REPRESENTING_ADDRTYPE_ERROR=0x0064000a, */
/* IDL: 	PR_SENT_REPRESENTING_EMAIL_ADDRESS=0x0065001e, */
/* IDL: 	PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE=0x0065001f, */
/* IDL: 	PR_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR=0x0065000a, */
/* IDL: 	PR_ORIGINAL_SENDER_ADDRTYPE=0x0066001e, */
/* IDL: 	PR_ORIGINAL_SENDER_ADDRTYPE_UNICODE=0x0066001f, */
/* IDL: 	PR_ORIGINAL_SENDER_ADDRTYPE_ERROR=0x0066000a, */
/* IDL: 	PR_ORIGINAL_SENDER_EMAIL_ADDRESS=0x0067001e, */
/* IDL: 	PR_ORIGINAL_SENDER_EMAIL_ADDRESS_UNICODE=0x0067001f, */
/* IDL: 	PR_ORIGINAL_SENDER_EMAIL_ADDRESS_ERROR=0x0067000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE=0x0068001e, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_UNICODE=0x0068001f, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_ERROR=0x0068000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS=0x0069001e, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE=0x0069001f, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_ERROR=0x0069000a, */
/* IDL: 	PR_CONVERSATION_TOPIC=0x0070001e, */
/* IDL: 	PR_CONVERSATION_TOPIC_UNICODE=0x0070001f, */
/* IDL: 	PR_CONVERSATION_TOPIC_ERROR=0x0070000a, */
/* IDL: 	PR_CONVERSATION_INDEX=0x00710102, */
/* IDL: 	PR_CONVERSATION_INDEX_ERROR=0x0071000a, */
/* IDL: 	PR_ORIGINAL_DISPLAY_BCC=0x0072001e, */
/* IDL: 	PR_ORIGINAL_DISPLAY_BCC_UNICODE=0x0072001f, */
/* IDL: 	PR_ORIGINAL_DISPLAY_BCC_ERROR=0x0072000a, */
/* IDL: 	PR_ORIGINAL_DISPLAY_CC=0x0073001e, */
/* IDL: 	PR_ORIGINAL_DISPLAY_CC_UNICODE=0x0073001f, */
/* IDL: 	PR_ORIGINAL_DISPLAY_CC_ERROR=0x0073000a, */
/* IDL: 	PR_ORIGINAL_DISPLAY_TO=0x0074001e, */
/* IDL: 	PR_ORIGINAL_DISPLAY_TO_UNICODE=0x0074001f, */
/* IDL: 	PR_ORIGINAL_DISPLAY_TO_ERROR=0x0074000a, */
/* IDL: 	PR_RECEIVED_BY_ADDRTYPE=0x0075001e, */
/* IDL: 	PR_RECEIVED_BY_ADDRTYPE_UNICODE=0x0075001f, */
/* IDL: 	PR_RECEIVED_BY_ADDRTYPE_ERROR=0x0075000a, */
/* IDL: 	PR_RECEIVED_BY_EMAIL_ADDRESS=0x0076001e, */
/* IDL: 	PR_RECEIVED_BY_EMAIL_ADDRESS_UNICODE=0x0076001f, */
/* IDL: 	PR_RECEIVED_BY_EMAIL_ADDRESS_ERROR=0x0076000a, */
/* IDL: 	PR_RCVD_REPRESENTING_ADDRTYPE=0x0077001e, */
/* IDL: 	PR_RCVD_REPRESENTING_ADDRTYPE_UNICODE=0x0077001f, */
/* IDL: 	PR_RCVD_REPRESENTING_ADDRTYPE_ERROR=0x0077000a, */
/* IDL: 	PR_RCVD_REPRESENTING_EMAIL_ADDRESS=0x0078001e, */
/* IDL: 	PR_RCVD_REPRESENTING_EMAIL_ADDRESS_UNICODE=0x0078001f, */
/* IDL: 	PR_RCVD_REPRESENTING_EMAIL_ADDRESS_ERROR=0x0078000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_ADDRTYPE=0x0079001e, */
/* IDL: 	PR_ORIGINAL_AUTHOR_ADDRTYPE_UNICODE=0x0079001f, */
/* IDL: 	PR_ORIGINAL_AUTHOR_ADDRTYPE_ERROR=0x0079000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS=0x007a001e, */
/* IDL: 	PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_UNICODE=0x007a001f, */
/* IDL: 	PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_ERROR=0x007a000a, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE=0x007b001e, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_UNICODE=0x007b001f, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_ERROR=0x007b000a, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS=0x007c001e, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_UNICODE=0x007c001f, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_ERROR=0x007c000a, */
/* IDL: 	PR_TRANSPORT_MESSAGE_HEADERS=0x007d001e, */
/* IDL: 	PR_TRANSPORT_MESSAGE_HEADERS_UNICODE=0x007d001f, */
/* IDL: 	PR_TRANSPORT_MESSAGE_HEADERS_ERROR=0x007d000a, */
/* IDL: 	PR_DELEGATION=0x007e0102, */
/* IDL: 	PR_DELEGATION_ERROR=0x007e000a, */
/* IDL: 	PR_TNEF_CORRELATION_KEY=0x007f0102, */
/* IDL: 	PR_TNEF_CORRELATION_KEY_ERROR=0x007f000a, */
/* IDL: 	PR_CONTENT_INTEGRITY_CHECK=0x0c000102, */
/* IDL: 	PR_CONTENT_INTEGRITY_CHECK_ERROR=0x0c00000a, */
/* IDL: 	PR_EXPLICIT_CONVERSION=0x0c010003, */
/* IDL: 	PR_EXPLICIT_CONVERSION_ERROR=0x0c01000a, */
/* IDL: 	PR_IPM_RETURN_REQUESTED=0x0c02000b, */
/* IDL: 	PR_IPM_RETURN_REQUESTED_ERROR=0x0c02000a, */
/* IDL: 	PR_MESSAGE_TOKEN=0x0c030102, */
/* IDL: 	PR_MESSAGE_TOKEN_ERROR=0x0c03000a, */
/* IDL: 	PR_NDR_REASON_CODE=0x0c040003, */
/* IDL: 	PR_NDR_REASON_CODE_ERROR=0x0c04000a, */
/* IDL: 	PR_NDR_DIAG_CODE=0x0c050003, */
/* IDL: 	PR_NDR_DIAG_CODE_ERROR=0x0c05000a, */
/* IDL: 	PR_NON_RECEIPT_NOTIFICATION_REQUESTED=0x0c06000b, */
/* IDL: 	PR_NON_RECEIPT_NOTIFICATION_REQUESTED_ERROR=0x0c06000a, */
/* IDL: 	PR_DELIVERY_POINT=0x0c070003, */
/* IDL: 	PR_DELIVERY_POINT_ERROR=0x0c07000a, */
/* IDL: 	PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED=0x0c08000b, */
/* IDL: 	PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED_ERROR=0x0c08000a, */
/* IDL: 	PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT=0x0c090102, */
/* IDL: 	PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT_ERROR=0x0c09000a, */
/* IDL: 	PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY=0x0c0a000b, */
/* IDL: 	PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY_ERROR=0x0c0a000a, */
/* IDL: 	PR_PHYSICAL_DELIVERY_MODE=0x0c0b0003, */
/* IDL: 	PR_PHYSICAL_DELIVERY_MODE_ERROR=0x0c0b000a, */
/* IDL: 	PR_PHYSICAL_DELIVERY_REPORT_REQUEST=0x0c0c0003, */
/* IDL: 	PR_PHYSICAL_DELIVERY_REPORT_REQUEST_ERROR=0x0c0c000a, */
/* IDL: 	PR_PHYSICAL_FORWARDING_ADDRESS=0x0c0d0102, */
/* IDL: 	PR_PHYSICAL_FORWARDING_ADDRESS_ERROR=0x0c0d000a, */
/* IDL: 	PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED=0x0c0e000b, */
/* IDL: 	PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED_ERROR=0x0c0e000a, */
/* IDL: 	PR_PHYSICAL_FORWARDING_PROHIBITED=0x0c0f000b, */
/* IDL: 	PR_PHYSICAL_FORWARDING_PROHIBITED_ERROR=0x0c0f000a, */
/* IDL: 	PR_PHYSICAL_RENDITION_ATTRIBUTES=0x0c100102, */
/* IDL: 	PR_PHYSICAL_RENDITION_ATTRIBUTES_ERROR=0x0c10000a, */
/* IDL: 	PR_PROOF_OF_DELIVERY=0x0c110102, */
/* IDL: 	PR_PROOF_OF_DELIVERY_ERROR=0x0c11000a, */
/* IDL: 	PR_PROOF_OF_DELIVERY_REQUESTED=0x0c12000b, */
/* IDL: 	PR_PROOF_OF_DELIVERY_REQUESTED_ERROR=0x0c12000a, */
/* IDL: 	PR_RECIPIENT_CERTIFICATE=0x0c130102, */
/* IDL: 	PR_RECIPIENT_CERTIFICATE_ERROR=0x0c13000a, */
/* IDL: 	PR_RECIPIENT_NUMBER_FOR_ADVICE=0x0c14001e, */
/* IDL: 	PR_RECIPIENT_NUMBER_FOR_ADVICE_UNICODE=0x0c14001f, */
/* IDL: 	PR_RECIPIENT_NUMBER_FOR_ADVICE_ERROR=0x0c14000a, */
/* IDL: 	PR_RECIPIENT_TYPE=0x0c150003, */
/* IDL: 	PR_RECIPIENT_TYPE_ERROR=0x0c15000a, */
/* IDL: 	PR_REGISTERED_MAIL_TYPE=0x0c160003, */
/* IDL: 	PR_REGISTERED_MAIL_TYPE_ERROR=0x0c16000a, */
/* IDL: 	PR_REPLY_REQUESTED=0x0c17000b, */
/* IDL: 	PR_REPLY_REQUESTED_ERROR=0x0c17000a, */
/* IDL: 	PR_REQUESTED_DELIVERY_METHOD=0x0c180003, */
/* IDL: 	PR_REQUESTED_DELIVERY_METHOD_ERROR=0x0c18000a, */
/* IDL: 	PR_SENDER_ENTRYID=0x0c190102, */
/* IDL: 	PR_SENDER_ENTRYID_ERROR=0x0c19000a, */
/* IDL: 	PR_SENDER_NAME=0x0c1a001e, */
/* IDL: 	PR_SENDER_NAME_UNICODE=0x0c1a001f, */
/* IDL: 	PR_SENDER_NAME_ERROR=0x0c1a000a, */
/* IDL: 	PR_SUPPLEMENTARY_INFO=0x0c1b001e, */
/* IDL: 	PR_SUPPLEMENTARY_INFO_UNICODE=0x0c1b001f, */
/* IDL: 	PR_SUPPLEMENTARY_INFO_ERROR=0x0c1b000a, */
/* IDL: 	PR_TYPE_OF_MTS_USER=0x0c1c0003, */
/* IDL: 	PR_TYPE_OF_MTS_USER_ERROR=0x0c1c000a, */
/* IDL: 	PR_SENDER_SEARCH_KEY=0x0c1d0102, */
/* IDL: 	PR_SENDER_SEARCH_KEY_ERROR=0x0c1d000a, */
/* IDL: 	PR_SENDER_ADDRTYPE=0x0c1e001e, */
/* IDL: 	PR_SENDER_ADDRTYPE_UNICODE=0x0c1e001f, */
/* IDL: 	PR_SENDER_ADDRTYPE_ERROR=0x0c1e000a, */
/* IDL: 	PR_SENDER_EMAIL_ADDRESS=0x0c1f001e, */
/* IDL: 	PR_SENDER_EMAIL_ADDRESS_UNICODE=0x0c1f001f, */
/* IDL: 	PR_SENDER_EMAIL_ADDRESS_ERROR=0x0c1f000a, */
/* IDL: 	PR_CURRENT_VERSION=0x0e000014, */
/* IDL: 	PR_CURRENT_VERSION_ERROR=0x0e00000a, */
/* IDL: 	PR_DELETE_AFTER_SUBMIT=0x0e01000b, */
/* IDL: 	PR_DELETE_AFTER_SUBMIT_ERROR=0x0e01000a, */
/* IDL: 	PR_DISPLAY_BCC=0x0e02001e, */
/* IDL: 	PR_DISPLAY_BCC_UNICODE=0x0e02001f, */
/* IDL: 	PR_DISPLAY_BCC_ERROR=0x0e02000a, */
/* IDL: 	PR_DISPLAY_CC=0x0e03001e, */
/* IDL: 	PR_DISPLAY_CC_UNICODE=0x0e03001f, */
/* IDL: 	PR_DISPLAY_CC_ERROR=0x0e03000a, */
/* IDL: 	PR_DISPLAY_TO=0x0e04001e, */
/* IDL: 	PR_DISPLAY_TO_UNICODE=0x0e04001f, */
/* IDL: 	PR_DISPLAY_TO_ERROR=0x0e04000a, */
/* IDL: 	PR_PARENT_DISPLAY=0x0e05001e, */
/* IDL: 	PR_PARENT_DISPLAY_UNICODE=0x0e05001f, */
/* IDL: 	PR_PARENT_DISPLAY_ERROR=0x0e05000a, */
/* IDL: 	PR_MESSAGE_DELIVERY_TIME=0x0e060040, */
/* IDL: 	PR_MESSAGE_DELIVERY_TIME_ERROR=0x0e06000a, */
/* IDL: 	PR_MESSAGE_FLAGS=0x0e070003, */
/* IDL: 	PR_MESSAGE_FLAGS_ERROR=0x0e07000a, */
/* IDL: 	PR_MESSAGE_SIZE=0x0e080003, */
/* IDL: 	PR_MESSAGE_SIZE_ERROR=0x0e08000a, */
/* IDL: 	PR_MESSAGE_SIZE_EXTENDED=0x0e080014, */
/* IDL: 	PR_PARENT_ENTRYID=0x0e090102, */
/* IDL: 	PR_PARENT_ENTRYID_ERROR=0x0e09000a, */
/* IDL: 	PR_SENTMAIL_ENTRYID=0x0e0a0102, */
/* IDL: 	PR_SENTMAIL_ENTRYID_ERROR=0x0e0a000a, */
/* IDL: 	PR_CORRELATE=0x0e0c000b, */
/* IDL: 	PR_CORRELATE_ERROR=0x0e0c000a, */
/* IDL: 	PR_CORRELATE_MTSID=0x0e0d0102, */
/* IDL: 	PR_CORRELATE_MTSID_ERROR=0x0e0d000a, */
/* IDL: 	PR_DISCRETE_VALUES=0x0e0e000b, */
/* IDL: 	PR_DISCRETE_VALUES_ERROR=0x0e0e000a, */
/* IDL: 	PR_RESPONSIBILITY=0x0e0f000b, */
/* IDL: 	PR_RESPONSIBILITY_ERROR=0x0e0f000a, */
/* IDL: 	PR_SPOOLER_STATUS=0x0e100003, */
/* IDL: 	PR_SPOOLER_STATUS_ERROR=0x0e10000a, */
/* IDL: 	PR_TRANSPORT_STATUS=0x0e110003, */
/* IDL: 	PR_TRANSPORT_STATUS_ERROR=0x0e11000a, */
/* IDL: 	PR_MESSAGE_RECIPIENTS=0x0e12000d, */
/* IDL: 	PR_MESSAGE_RECIPIENTS_ERROR=0x0e12000a, */
/* IDL: 	PR_MESSAGE_ATTACHMENTS=0x0e13000d, */
/* IDL: 	PR_MESSAGE_ATTACHMENTS_ERROR=0x0e13000a, */
/* IDL: 	PR_SUBMIT_FLAGS=0x0e140003, */
/* IDL: 	PR_SUBMIT_FLAGS_ERROR=0x0e14000a, */
/* IDL: 	PR_RECIPIENT_STATUS=0x0e150003, */
/* IDL: 	PR_RECIPIENT_STATUS_ERROR=0x0e15000a, */
/* IDL: 	PR_TRANSPORT_KEY=0x0e160003, */
/* IDL: 	PR_TRANSPORT_KEY_ERROR=0x0e16000a, */
/* IDL: 	PR_MSG_STATUS=0x0e170003, */
/* IDL: 	PR_MSG_STATUS_ERROR=0x0e17000a, */
/* IDL: 	PR_MESSAGE_DOWNLOAD_TIME=0x0e180003, */
/* IDL: 	PR_MESSAGE_DOWNLOAD_TIME_ERROR=0x0e18000a, */
/* IDL: 	PR_CREATION_VERSION=0x0e190014, */
/* IDL: 	PR_CREATION_VERSION_ERROR=0x0e19000a, */
/* IDL: 	PR_MODIFY_VERSION=0x0e1a0014, */
/* IDL: 	PR_MODIFY_VERSION_ERROR=0x0e1a000a, */
/* IDL: 	PR_HASATTACH=0x0e1b000b, */
/* IDL: 	PR_HASATTACH_ERROR=0x0e1b000a, */
/* IDL: 	PR_BODY_CRC=0x0e1c0003, */
/* IDL: 	PR_BODY_CRC_ERROR=0x0e1c000a, */
/* IDL: 	PR_NORMALIZED_SUBJECT=0x0e1d001e, */
/* IDL: 	PR_NORMALIZED_SUBJECT_UNICODE=0x0e1d001f, */
/* IDL: 	PR_NORMALIZED_SUBJECT_ERROR=0x0e1d000a, */
/* IDL: 	PR_RTF_IN_SYNC=0x0e1f000b, */
/* IDL: 	PR_RTF_IN_SYNC_ERROR=0x0e1f000a, */
/* IDL: 	PR_ATTACH_SIZE=0x0e200003, */
/* IDL: 	PR_ATTACH_SIZE_ERROR=0x0e20000a, */
/* IDL: 	PR_ATTACH_NUM=0x0e210003, */
/* IDL: 	PR_ATTACH_NUM_ERROR=0x0e21000a, */
/* IDL: 	PR_PREPROCESS=0x0e22000b, */
/* IDL: 	PR_PREPROCESS_ERROR=0x0e22000a, */
/* IDL: 	PR_INTERNET_ARTICLE_NUMBER=0x0e230003, */
/* IDL: 	PR_INTERNET_ARTICLE_NUMBER_ERROR=0x0e23000a, */
/* IDL: 	PR_NEWSGROUP_NAME=0x0e24001e, */
/* IDL: 	PR_NEWSGROUP_NAME_UNICODE=0x0e24001f, */
/* IDL: 	PR_NEWSGROUP_NAME_ERROR=0x0e24000a, */
/* IDL: 	PR_ORIGINATING_MTA_CERTIFICATE=0x0e250102, */
/* IDL: 	PR_ORIGINATING_MTA_CERTIFICATE_ERROR=0x0e25000a, */
/* IDL: 	PR_PROOF_OF_SUBMISSION=0x0e260102, */
/* IDL: 	PR_PROOF_OF_SUBMISSION_ERROR=0x0e26000a, */
/* IDL: 	PR_NT_SECURITY_DESCRIPTOR=0x0e270102, */
/* IDL: 	PR_NT_SECURITY_DESCRIPTOR_ERROR=0x0e27000a, */
/* IDL: 	PR_CREATOR_SID=0x0e580102, */
/* IDL: 	PR_CREATOR_SID_ERROR=0x0e58000a, */
/* IDL: 	PR_LAST_MODIFIER_SID=0x0e590102, */
/* IDL: 	PR_LAST_MODIFIER_SID_ERROR=0x0e59000a, */
/* IDL: 	PR_MIME_HANDLER_CLASSIDS=0x0e5e0048, */
/* IDL: 	PR_MIME_HANDLER_CLASSIDS_ERROR=0x0e5e000a, */
/* IDL: 	PR_URL_COMP_NAME_POSTFIX=0x0e610003, */
/* IDL: 	PR_URL_COMP_NAME_POSTFIX_ERROR=0x0e61000a, */
/* IDL: 	PR_URL_COMP_NAME_SET=0x0e62000b, */
/* IDL: 	PR_URL_COMP_NAME_SET_ERROR=0x0e62000a, */
/* IDL: 	PR_SUBFOLDER_CT=0x0e630003, */
/* IDL: 	PR_SUBFOLDER_CT_ERROR=0x0e63000a, */
/* IDL: 	PR_DELETED_SUBFOLDER_CT=0x0e640003, */
/* IDL: 	PR_DELETED_SUBFOLDER_CT_ERROR=0x0e64000a, */
/* IDL: 	PR_DELETE_TIME=0x0e660040, */
/* IDL: 	PR_DELETE_TIME_ERROR=0x0e66000a, */
/* IDL: 	PR_AGE_LIMIT=0x0e670102, */
/* IDL: 	PR_AGE_LIMIT_ERROR=0x0e67000a, */
/* IDL: 	PR_TRUST_SENDER=0x0e790003, */
/* IDL: 	PR_TRUST_SENDER_ERROR=0x0e79000a, */
/* IDL: 	PR_ATTACH_VIRUS_SCAN_INFO=0x0e960102, */
/* IDL: 	PR_ATTACH_VIRUS_SCAN_INFO_ERROR=0x0e96000a, */
/* IDL: 	PR_ACCESS=0x0ff40003, */
/* IDL: 	PR_ACCESS_ERROR=0x0ff4000a, */
/* IDL: 	PR_ROW_TYPE=0x0ff50003, */
/* IDL: 	PR_ROW_TYPE_ERROR=0x0ff5000a, */
/* IDL: 	PR_INSTANCE_KEY=0x0ff60102, */
/* IDL: 	PR_INSTANCE_KEY_ERROR=0x0ff6000a, */
/* IDL: 	PR_ACCESS_LEVEL=0x0ff70003, */
/* IDL: 	PR_ACCESS_LEVEL_ERROR=0x0ff7000a, */
/* IDL: 	PR_MAPPING_SIGNATURE=0x0ff80102, */
/* IDL: 	PR_MAPPING_SIGNATURE_ERROR=0x0ff8000a, */
/* IDL: 	PR_RECORD_KEY=0x0ff90102, */
/* IDL: 	PR_RECORD_KEY_ERROR=0x0ff9000a, */
/* IDL: 	PR_STORE_RECORD_KEY=0x0ffa0102, */
/* IDL: 	PR_STORE_RECORD_KEY_ERROR=0x0ffa000a, */
/* IDL: 	PR_STORE_ENTRYID=0x0ffb0102, */
/* IDL: 	PR_STORE_ENTRYID_ERROR=0x0ffb000a, */
/* IDL: 	PR_MINI_ICON=0x0ffc0102, */
/* IDL: 	PR_MINI_ICON_ERROR=0x0ffc000a, */
/* IDL: 	PR_ICON=0x0ffd0102, */
/* IDL: 	PR_ICON_ERROR=0x0ffd000a, */
/* IDL: 	PR_OBJECT_TYPE=0x0ffe0003, */
/* IDL: 	PR_OBJECT_TYPE_ERROR=0x0ffe000a, */
/* IDL: 	PR_ENTRYID=0x0fff0102, */
/* IDL: 	PR_ENTRYID_ERROR=0x0fff000a, */
/* IDL: 	PR_BODY=0x1000001e, */
/* IDL: 	PR_BODY_UNICODE=0x1000001f, */
/* IDL: 	PR_BODY_ERROR=0x1000000a, */
/* IDL: 	PR_REPORT_TEXT=0x1001001e, */
/* IDL: 	PR_REPORT_TEXT_UNICODE=0x1001001f, */
/* IDL: 	PR_REPORT_TEXT_ERROR=0x1001000a, */
/* IDL: 	PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY=0x10020102, */
/* IDL: 	PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY_ERROR=0x1002000a, */
/* IDL: 	PR_REPORTING_DL_NAME=0x10030102, */
/* IDL: 	PR_REPORTING_DL_NAME_ERROR=0x1003000a, */
/* IDL: 	PR_REPORTING_MTA_CERTIFICATE=0x10040102, */
/* IDL: 	PR_REPORTING_MTA_CERTIFICATE_ERROR=0x1004000a, */
/* IDL: 	PR_RTF_SYNC_BODY_CRC=0x10060003, */
/* IDL: 	PR_RTF_SYNC_BODY_CRC_ERROR=0x1006000a, */
/* IDL: 	PR_RTF_SYNC_BODY_COUNT=0x10070003, */
/* IDL: 	PR_RTF_SYNC_BODY_COUNT_ERROR=0x1007000a, */
/* IDL: 	PR_RTF_SYNC_BODY_TAG=0x1008001e, */
/* IDL: 	PR_RTF_SYNC_BODY_TAG_UNICODE=0x1008001f, */
/* IDL: 	PR_RTF_SYNC_BODY_TAG_ERROR=0x1008000a, */
/* IDL: 	PR_RTF_COMPRESSED=0x10090102, */
/* IDL: 	PR_RTF_COMPRESSED_ERROR=0x1009000a, */
/* IDL: 	PR_RTF_SYNC_PREFIX_COUNT=0x10100003, */
/* IDL: 	PR_RTF_SYNC_PREFIX_COUNT_ERROR=0x1010000a, */
/* IDL: 	PR_RTF_SYNC_TRAILING_COUNT=0x10110003, */
/* IDL: 	PR_RTF_SYNC_TRAILING_COUNT_ERROR=0x1011000a, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_ENTRYID=0x10120102, */
/* IDL: 	PR_ORIGINALLY_INTENDED_RECIP_ENTRYID_ERROR=0x1012000a, */
/* IDL: 	PR_HTML=0x10130102, */
/* IDL: 	PR_HTML_ERROR=0x1013000a, */
/* IDL: 	PR_INTERNET_APPROVED=0x1030001e, */
/* IDL: 	PR_INTERNET_APPROVED_UNICODE=0x1030001f, */
/* IDL: 	PR_INTERNET_APPROVED_ERROR=0x1030000a, */
/* IDL: 	PR_INTERNET_CONTROL=0x1031001e, */
/* IDL: 	PR_INTERNET_CONTROL_UNICODE=0x1031001f, */
/* IDL: 	PR_INTERNET_CONTROL_ERROR=0x1031000a, */
/* IDL: 	PR_INTERNET_DISTRIBUTION=0x1032001e, */
/* IDL: 	PR_INTERNET_DISTRIBUTION_UNICODE=0x1032001f, */
/* IDL: 	PR_INTERNET_DISTRIBUTION_ERROR=0x1032000a, */
/* IDL: 	PR_INTERNET_FOLLOWUP_TO=0x1033001e, */
/* IDL: 	PR_INTERNET_FOLLOWUP_TO_UNICODE=0x1033001f, */
/* IDL: 	PR_INTERNET_FOLLOWUP_TO_ERROR=0x1033000a, */
/* IDL: 	PR_INTERNET_LINES=0x10340003, */
/* IDL: 	PR_INTERNET_LINES_ERROR=0x1034000a, */
/* IDL: 	PR_INTERNET_MESSAGE_ID=0x1035001e, */
/* IDL: 	PR_INTERNET_MESSAGE_ID_UNICODE=0x1035001f, */
/* IDL: 	PR_INTERNET_MESSAGE_ID_ERROR=0x1035000a, */
/* IDL: 	PR_INTERNET_NEWSGROUPS=0x1036001e, */
/* IDL: 	PR_INTERNET_NEWSGROUPS_UNICODE=0x1036001f, */
/* IDL: 	PR_INTERNET_NEWSGROUPS_ERROR=0x1036000a, */
/* IDL: 	PR_INTERNET_ORGANIZATION=0x1037001e, */
/* IDL: 	PR_INTERNET_ORGANIZATION_UNICODE=0x1037001f, */
/* IDL: 	PR_INTERNET_ORGANIZATION_ERROR=0x1037000a, */
/* IDL: 	PR_INTERNET_NNTP_PATH=0x1038001e, */
/* IDL: 	PR_INTERNET_NNTP_PATH_UNICODE=0x1038001f, */
/* IDL: 	PR_INTERNET_NNTP_PATH_ERROR=0x1038000a, */
/* IDL: 	PR_INTERNET_REFERENCES=0x1039001e, */
/* IDL: 	PR_INTERNET_REFERENCES_UNICODE=0x1039001f, */
/* IDL: 	PR_INTERNET_REFERENCES_ERROR=0x1039000a, */
/* IDL: 	PR_SUPERSEDES=0x103a001e, */
/* IDL: 	PR_SUPERSEDES_UNICODE=0x103a001f, */
/* IDL: 	PR_SUPERSEDES_ERROR=0x103a000a, */
/* IDL: 	PR_POST_FOLDER_ENTRIES=0x103b0102, */
/* IDL: 	PR_POST_FOLDER_ENTRIES_ERROR=0x103b000a, */
/* IDL: 	PR_POST_FOLDER_NAMES=0x103c001e, */
/* IDL: 	PR_POST_FOLDER_NAMES_UNICODE=0x103c001f, */
/* IDL: 	PR_POST_FOLDER_NAMES_ERROR=0x103c000a, */
/* IDL: 	PR_POST_REPLY_FOLDER_ENTRIES=0x103d0102, */
/* IDL: 	PR_POST_REPLY_FOLDER_ENTRIES_ERROR=0x103d000a, */
/* IDL: 	PR_POST_REPLY_FOLDER_NAMES=0x103e001e, */
/* IDL: 	PR_POST_REPLY_FOLDER_NAMES_UNICODE=0x103e001f, */
/* IDL: 	PR_POST_REPLY_FOLDER_NAMES_ERROR=0x103e000a, */
/* IDL: 	PR_POST_REPLY_DENIED=0x103f0102, */
/* IDL: 	PR_POST_REPLY_DENIED_ERROR=0x103f000a, */
/* IDL: 	PR_NNTP_XREF=0x1040001e, */
/* IDL: 	PR_NNTP_XREF_UNICODE=0x1040001f, */
/* IDL: 	PR_NNTP_XREF_ERROR=0x1040000a, */
/* IDL: 	PR_INTERNET_PRECEDENCE=0x1041001e, */
/* IDL: 	PR_INTERNET_PRECEDENCE_UNICODE=0x1041001f, */
/* IDL: 	PR_INTERNET_PRECEDENCE_ERROR=0x1041000a, */
/* IDL: 	PR_IN_REPLY_TO_ID=0x1042001e, */
/* IDL: 	PR_IN_REPLY_TO_ID_UNICODE=0x1042001f, */
/* IDL: 	PR_IN_REPLY_TO_ID_ERROR=0x1042000a, */
/* IDL: 	PR_LIST_HELP=0x1043001e, */
/* IDL: 	PR_LIST_HELP_UNICODE=0x1043001f, */
/* IDL: 	PR_LIST_HELP_ERROR=0x1043000a, */
/* IDL: 	PR_LIST_SUBSCRIBE=0x1044001e, */
/* IDL: 	PR_LIST_SUBSCRIBE_UNICODE=0x1044001f, */
/* IDL: 	PR_LIST_SUBSCRIBE_ERROR=0x1044000a, */
/* IDL: 	PR_LIST_UNSUBSCRIBE=0x1045001e, */
/* IDL: 	PR_LIST_UNSUBSCRIBE_UNICODE=0x1045001f, */
/* IDL: 	PR_LIST_UNSUBSCRIBE_ERROR=0x1045000a, */
/* IDL: 	PR_ACTION=0x10800003, */
/* IDL: 	PR_ACTION_ERROR=0x1080000a, */
/* IDL: 	PR_ACTION_FLAG=0x10810003, */
/* IDL: 	PR_ACTION_FLAG_ERROR=0x1081000a, */
/* IDL: 	PR_ACTION_DATE=0x10820040, */
/* IDL: 	PR_ACTION_DATE_ERROR=0x1082000a, */
/* IDL: 	PR_FLAG_STATUS=0x10900003, */
/* IDL: 	PR_FLAG_STATUS_ERROR=0x1090000a, */
/* IDL: 	PR_FLAG_COMPLETE=0x10910040, */
/* IDL: 	PR_FLAG_COMPLETE_ERROR=0x1091000a, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA=0x10c00102, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA_ERROR=0x10c0000a, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA_2=0x10c10003, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA_2_ERROR=0x10c1000a, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA_3=0x10c20102, */
/* IDL: 	PR_SMTP_TEMP_TBL_DATA_3_ERROR=0x10c2000a, */
/* IDL: 	PR_CAL_START_TIME=0x10c30040, */
/* IDL: 	PR_CAL_START_TIME_ERROR=0x10c3000a, */
/* IDL: 	PR_CAL_END_TIME=0x10c40040, */
/* IDL: 	PR_CAL_END_TIME_ERROR=0x10c4000a, */
/* IDL: 	PR_CAL_RECURRING_ID=0x10c50040, */
/* IDL: 	PR_CAL_RECURRING_ID_ERROR=0x10c5000a, */
/* IDL: 	PR_DAV_SUBMIT_DATA=0x10c6001e, */
/* IDL: 	PR_DAV_SUBMIT_DATA_UNICODE=0x10c6001f, */
/* IDL: 	PR_DAV_SUBMIT_DATA_ERROR=0x10c6000a, */
/* IDL: 	PR_CDO_EXPANSION_INDEX=0x10c70003, */
/* IDL: 	PR_CDO_EXPANSION_INDEX_ERROR=0x10c7000a, */
/* IDL: 	PR_IFS_INTERNAL_DATA=0x10c80102, */
/* IDL: 	PR_IFS_INTERNAL_DATA_ERROR=0x10c8000a, */
/* IDL: 	PR_CAL_REMINDER_NEXT_TIME=0x10ca0040, */
/* IDL: 	PR_CAL_REMINDER_NEXT_TIME_ERROR=0x10ca000a, */
/* IDL: 	PR_OWA_URL=0x10f1001e, */
/* IDL: 	PR_OWA_URL_UNICODE=0x10f1001f, */
/* IDL: 	PR_OWA_URL_ERROR=0x10f1000a, */
/* IDL: 	PR_DISABLE_FULL_FIDELITY=0x10f2000b, */
/* IDL: 	PR_DISABLE_FULL_FIDELITY_ERROR=0x10f2000a, */
/* IDL: 	PR_URL_COMP_NAME=0x10f3001e, */
/* IDL: 	PR_URL_COMP_NAME_UNICODE=0x10f3001f, */
/* IDL: 	PR_URL_COMP_NAME_ERROR=0x10f3000a, */
/* IDL: 	PR_ATTR_HIDDEN=0x10f4000b, */
/* IDL: 	PR_ATTR_HIDDEN_ERROR=0x10f4000a, */
/* IDL: 	PR_ATTR_SYSTEM=0x10f5000b, */
/* IDL: 	PR_ATTR_SYSTEM_ERROR=0x10f5000a, */
/* IDL: 	PR_ATTR_READONLY=0x10f6000b, */
/* IDL: 	PR_ATTR_READONLY_ERROR=0x10f6000a, */
/* IDL: 	PR_P1_CONTENT=0x11000102, */
/* IDL: 	PR_P1_CONTENT_ERROR=0x1100000a, */
/* IDL: 	PR_P1_CONTENT_TYPE=0x11010102, */
/* IDL: 	PR_P1_CONTENT_TYPE_ERROR=0x1101000a, */
/* IDL: 	PR_ROWID=0x30000003, */
/* IDL: 	PR_ROWID_ERROR=0x3000000a, */
/* IDL: 	PR_DISPLAY_NAME=0x3001001e, */
/* IDL: 	PR_DISPLAY_NAME_UNICODE=0x3001001f, */
/* IDL: 	PR_DISPLAY_NAME_ERROR=0x3001000a, */
/* IDL: 	PR_ADDRTYPE=0x3002001e, */
/* IDL: 	PR_ADDRTYPE_UNICODE=0x3002001f, */
/* IDL: 	PR_ADDRTYPE_ERROR=0x3002000a, */
/* IDL: 	PR_EMAIL_ADDRESS=0x3003001e, */
/* IDL: 	PR_EMAIL_ADDRESS_UNICODE=0x3003001f, */
/* IDL: 	PR_EMAIL_ADDRESS_ERROR=0x3003000a, */
/* IDL: 	PR_COMMENT=0x3004001e, */
/* IDL: 	PR_COMMENT_UNICODE=0x3004001f, */
/* IDL: 	PR_COMMENT_ERROR=0x3004000a, */
/* IDL: 	PR_DEPTH=0x30050003, */
/* IDL: 	PR_DEPTH_ERROR=0x3005000a, */
/* IDL: 	PR_PROVIDER_DISPLAY=0x3006001e, */
/* IDL: 	PR_PROVIDER_DISPLAY_UNICODE=0x3006001f, */
/* IDL: 	PR_PROVIDER_DISPLAY_ERROR=0x3006000a, */
/* IDL: 	PR_CREATION_TIME=0x30070040, */
/* IDL: 	PR_CREATION_TIME_ERROR=0x3007000a, */
/* IDL: 	PR_LAST_MODIFICATION_TIME=0x30080040, */
/* IDL: 	PR_LAST_MODIFICATION_TIME_ERROR=0x3008000a, */
/* IDL: 	PR_RESOURCE_FLAGS=0x30090003, */
/* IDL: 	PR_RESOURCE_FLAGS_ERROR=0x3009000a, */
/* IDL: 	PR_PROVIDER_DLL_NAME=0x300a001e, */
/* IDL: 	PR_PROVIDER_DLL_NAME_UNICODE=0x300a001f, */
/* IDL: 	PR_PROVIDER_DLL_NAME_ERROR=0x300a000a, */
/* IDL: 	PR_SEARCH_KEY=0x300b0102, */
/* IDL: 	PR_SEARCH_KEY_ERROR=0x300b000a, */
/* IDL: 	PR_PROVIDER_UID=0x300c0102, */
/* IDL: 	PR_PROVIDER_UID_ERROR=0x300c000a, */
/* IDL: 	PR_PROVIDER_ORDINAL=0x300d0003, */
/* IDL: 	PR_PROVIDER_ORDINAL_ERROR=0x300d000a, */
/* IDL: 	PR_FORM_VERSION=0x3301001e, */
/* IDL: 	PR_FORM_VERSION_UNICODE=0x3301001f, */
/* IDL: 	PR_FORM_VERSION_ERROR=0x3301000a, */
/* IDL: 	PR_FORM_CLSID=0x33020048, */
/* IDL: 	PR_FORM_CLSID_ERROR=0x3302000a, */
/* IDL: 	PR_FORM_CONTACT_NAME=0x3303001e, */
/* IDL: 	PR_FORM_CONTACT_NAME_UNICODE=0x3303001f, */
/* IDL: 	PR_FORM_CONTACT_NAME_ERROR=0x3303000a, */
/* IDL: 	PR_FORM_CATEGORY=0x3304001e, */
/* IDL: 	PR_FORM_CATEGORY_UNICODE=0x3304001f, */
/* IDL: 	PR_FORM_CATEGORY_ERROR=0x3304000a, */
/* IDL: 	PR_FORM_CATEGORY_SUB=0x3305001e, */
/* IDL: 	PR_FORM_CATEGORY_SUB_UNICODE=0x3305001f, */
/* IDL: 	PR_FORM_CATEGORY_SUB_ERROR=0x3305000a, */
/* IDL: 	PR_FORM_HOST_MAP=0x33061003, */
/* IDL: 	PR_FORM_HOST_MAP_ERROR=0x3306000a, */
/* IDL: 	PR_FORM_HIDDEN=0x3307000b, */
/* IDL: 	PR_FORM_HIDDEN_ERROR=0x3307000a, */
/* IDL: 	PR_FORM_DESIGNER_NAME=0x3308001e, */
/* IDL: 	PR_FORM_DESIGNER_NAME_UNICODE=0x3308001f, */
/* IDL: 	PR_FORM_DESIGNER_NAME_ERROR=0x3308000a, */
/* IDL: 	PR_FORM_DESIGNER_GUID=0x33090048, */
/* IDL: 	PR_FORM_DESIGNER_GUID_ERROR=0x3309000a, */
/* IDL: 	PR_FORM_MESSAGE_BEHAVIOR=0x330a0003, */
/* IDL: 	PR_FORM_MESSAGE_BEHAVIOR_ERROR=0x330a000a, */
/* IDL: 	PR_DEFAULT_STORE=0x3400000b, */
/* IDL: 	PR_DEFAULT_STORE_ERROR=0x3400000a, */
/* IDL: 	PR_STORE_SUPPORT_MASK=0x340d0003, */
/* IDL: 	PR_STORE_SUPPORT_MASK_ERROR=0x340d000a, */
/* IDL: 	PR_STORE_STATE=0x340e0003, */
/* IDL: 	PR_STORE_STATE_ERROR=0x340e000a, */
/* IDL: 	PR_IPM_SUBTREE_SEARCH_KEY=0x34100102, */
/* IDL: 	PR_IPM_SUBTREE_SEARCH_KEY_ERROR=0x3410000a, */
/* IDL: 	PR_IPM_OUTBOX_SEARCH_KEY=0x34110102, */
/* IDL: 	PR_IPM_OUTBOX_SEARCH_KEY_ERROR=0x3411000a, */
/* IDL: 	PR_IPM_WASTEBASKET_SEARCH_KEY=0x34120102, */
/* IDL: 	PR_IPM_WASTEBASKET_SEARCH_KEY_ERROR=0x3412000a, */
/* IDL: 	PR_IPM_SENTMAIL_SEARCH_KEY=0x34130102, */
/* IDL: 	PR_IPM_SENTMAIL_SEARCH_KEY_ERROR=0x3413000a, */
/* IDL: 	PR_MDB_PROVIDER=0x34140102, */
/* IDL: 	PR_MDB_PROVIDER_ERROR=0x3414000a, */
/* IDL: 	PR_RECEIVE_FOLDER_SETTINGS=0x3415000d, */
/* IDL: 	PR_RECEIVE_FOLDER_SETTINGS_ERROR=0x3415000a, */
/* IDL: 	PR_VALID_FOLDER_MASK=0x35df0003, */
/* IDL: 	PR_VALID_FOLDER_MASK_ERROR=0x35df000a, */
/* IDL: 	PR_IPM_SUBTREE_ENTRYID=0x35e00102, */
/* IDL: 	PR_IPM_SUBTREE_ENTRYID_ERROR=0x35e0000a, */
/* IDL: 	PR_IPM_OUTBOX_ENTRYID=0x35e20102, */
/* IDL: 	PR_IPM_OUTBOX_ENTRYID_ERROR=0x35e2000a, */
/* IDL: 	PR_IPM_WASTEBASKET_ENTRYID=0x35e30102, */
/* IDL: 	PR_IPM_WASTEBASKET_ENTRYID_ERROR=0x35e3000a, */
/* IDL: 	PR_IPM_SENTMAIL_ENTRYID=0x35e40102, */
/* IDL: 	PR_IPM_SENTMAIL_ENTRYID_ERROR=0x35e4000a, */
/* IDL: 	PR_VIEWS_ENTRYID=0x35e50102, */
/* IDL: 	PR_VIEWS_ENTRYID_ERROR=0x35e5000a, */
/* IDL: 	PR_COMMON_VIEWS_ENTRYID=0x35e60102, */
/* IDL: 	PR_COMMON_VIEWS_ENTRYID_ERROR=0x35e6000a, */
/* IDL: 	PR_FINDER_ENTRYID=0x35e70102, */
/* IDL: 	PR_FINDER_ENTRYID_ERROR=0x35e7000a, */
/* IDL: 	PR_CONTAINER_FLAGS=0x36000003, */
/* IDL: 	PR_CONTAINER_FLAGS_ERROR=0x3600000a, */
/* IDL: 	PR_FOLDER_TYPE=0x36010003, */
/* IDL: 	PR_FOLDER_TYPE_ERROR=0x3601000a, */
/* IDL: 	PR_CONTENT_COUNT=0x36020003, */
/* IDL: 	PR_CONTENT_COUNT_ERROR=0x3602000a, */
/* IDL: 	PR_CONTENT_UNREAD=0x36030003, */
/* IDL: 	PR_CONTENT_UNREAD_ERROR=0x3603000a, */
/* IDL: 	PR_CREATE_TEMPLATES=0x3604000d, */
/* IDL: 	PR_CREATE_TEMPLATES_ERROR=0x3604000a, */
/* IDL: 	PR_DETAILS_TABLE=0x3605000d, */
/* IDL: 	PR_DETAILS_TABLE_ERROR=0x3605000a, */
/* IDL: 	PR_SEARCH=0x3607000d, */
/* IDL: 	PR_SEARCH_ERROR=0x3607000a, */
/* IDL: 	PR_SELECTABLE=0x3609000b, */
/* IDL: 	PR_SELECTABLE_ERROR=0x3609000a, */
/* IDL: 	PR_SUBFOLDERS=0x360a000b, */
/* IDL: 	PR_SUBFOLDERS_ERROR=0x360a000a, */
/* IDL: 	PR_STATUS=0x360b0003, */
/* IDL: 	PR_STATUS_ERROR=0x360b000a, */
/* IDL: 	PR_ANR=0x360c001e, */
/* IDL: 	PR_ANR_UNICODE=0x360c001f, */
/* IDL: 	PR_ANR_ERROR=0x360c000a, */
/* IDL: 	PR_CONTENTS_SORT_ORDER=0x360d1003, */
/* IDL: 	PR_CONTENTS_SORT_ORDER_ERROR=0x360d000a, */
/* IDL: 	PR_CONTAINER_HIERARCHY=0x360e000d, */
/* IDL: 	PR_CONTAINER_HIERARCHY_ERROR=0x360e000a, */
/* IDL: 	PR_CONTAINER_CONTENTS=0x360f000d, */
/* IDL: 	PR_CONTAINER_CONTENTS_ERROR=0x360f000a, */
/* IDL: 	PR_FOLDER_ASSOCIATED_CONTENTS=0x3610000d, */
/* IDL: 	PR_FOLDER_ASSOCIATED_CONTENTS_ERROR=0x3610000a, */
/* IDL: 	PR_DEF_CREATE_DL=0x36110102, */
/* IDL: 	PR_DEF_CREATE_DL_ERROR=0x3611000a, */
/* IDL: 	PR_DEF_CREATE_MAILUSER=0x36120102, */
/* IDL: 	PR_DEF_CREATE_MAILUSER_ERROR=0x3612000a, */
/* IDL: 	PR_CONTAINER_CLASS=0x3613001e, */
/* IDL: 	PR_CONTAINER_CLASS_UNICODE=0x3613001f, */
/* IDL: 	PR_CONTAINER_CLASS_ERROR=0x3613000a, */
/* IDL: 	PR_CONTAINER_MODIFY_VERSION=0x36140014, */
/* IDL: 	PR_CONTAINER_MODIFY_VERSION_ERROR=0x3614000a, */
/* IDL: 	PR_AB_PROVIDER_ID=0x36150102, */
/* IDL: 	PR_AB_PROVIDER_ID_ERROR=0x3615000a, */
/* IDL: 	PR_DEFAULT_VIEW_ENTRYID=0x36160102, */
/* IDL: 	PR_DEFAULT_VIEW_ENTRYID_ERROR=0x3616000a, */
/* IDL: 	PR_ASSOC_CONTENT_COUNT=0x36170003, */
/* IDL: 	PR_ASSOC_CONTENT_COUNT_ERROR=0x3617000a, */
/* IDL: 	PR_PACKED_NAME_PROPS=0x361c0102, */
/* IDL: 	PR_PACKED_NAME_PROPS_ERROR=0x361c000a, */
/* IDL: 	PR_IPM_APPOINTMENT_ENTRYID=0x36d00102, */
/* IDL: 	PR_IPM_APPOINTMENT_ENTRYID_ERROR=0x36d0000a, */
/* IDL: 	PR_IPM_CONTACT_ENTRYID=0x36d10102, */
/* IDL: 	PR_IPM_CONTACT_ENTRYID_ERROR=0x36d1000a, */
/* IDL: 	PR_IPM_JOURNAL_ENTRYID=0x36d20102, */
/* IDL: 	PR_IPM_JOURNAL_ENTRYID_ERROR=0x36d2000a, */
/* IDL: 	PR_IPM_NOTE_ENTRYID=0x36d30102, */
/* IDL: 	PR_IPM_NOTE_ENTRYID_ERROR=0x36d3000a, */
/* IDL: 	PR_IPM_TASK_ENTRYID=0x36d40102, */
/* IDL: 	PR_IPM_TASK_ENTRYID_ERROR=0x36d4000a, */
/* IDL: 	PR_REMINDERS_ONLINE_ENTRYID=0x36d50102, */
/* IDL: 	PR_REMINDERS_ONLINE_ENTRYID_ERROR=0x36d5000a, */
/* IDL: 	PR_REMINDERS_OFFLINE_ENTRYID=0x36d60102, */
/* IDL: 	PR_REMINDERS_OFFLINE_ENTRYID_ERROR=0x36d6000a, */
/* IDL: 	PR_IPM_DRAFTS_ENTRYID=0x36d70102, */
/* IDL: 	PR_IPM_DRAFTS_ENTRYID_ERROR=0x36d7000a, */
/* IDL: 	PR_OUTLOOK_2003_ENTRYIDS=0x36d81102, */
/* IDL: 	PR_OUTLOOK_2003_ENTRYIDS_ERROR=0x36d8000a, */
/* IDL: 	PR_FOLDER_WEBVIEWINFO=0x36df0102, */
/* IDL: 	PR_FOLDER_WEBVIEWINFO_ERROR=0x36df000a, */
/* IDL: 	PR_FOLDER_XVIEWINFO_E=0x36e00102, */
/* IDL: 	PR_FOLDER_XVIEWINFO_E_ERROR=0x36e0000a, */
/* IDL: 	PR_FOLDER_VIEWS_ONLY=0x36e10003, */
/* IDL: 	PR_FOLDER_VIEWS_ONLY_ERROR=0x36e1000a, */
/* IDL: 	PR_FREEBUSY_ENTRYIDS=0x36e41102, */
/* IDL: 	PR_FREEBUSY_ENTRYIDS_ERROR=0x36e4000a, */
/* IDL: 	PR_DEF_MSG_CLASS=0x36e5001e, */
/* IDL: 	PR_DEF_MSG_CLASS_UNICODE=0x36e5001f, */
/* IDL: 	PR_DEF_MSG_CLASS_ERROR=0x36e5000a, */
/* IDL: 	PR_DEF_FORM_NAME=0x36e6001e, */
/* IDL: 	PR_DEF_FORM_NAME_UNICODE=0x36e6001f, */
/* IDL: 	PR_DEF_FORM_NAME_ERROR=0x36e6000a, */
/* IDL: 	PR_GENERATE_EXCHANGE_VIEWS=0x36e9000b, */
/* IDL: 	PR_GENERATE_EXCHANGE_VIEWS_ERROR=0x36e9000a, */
/* IDL: 	PR_AGING_PERIOD=0x36ec0003, */
/* IDL: 	PR_AGING_PERIOD_ERROR=0x36ec000a, */
/* IDL: 	PR_AGING_GRANULARITY=0x36ee0003, */
/* IDL: 	PR_AGING_GRANULARITY_ERROR=0x36ee000a, */
/* IDL: 	PR_ATTACHMENT_X400_PARAMETERS=0x37000102, */
/* IDL: 	PR_ATTACHMENT_X400_PARAMETERS_ERROR=0x3700000a, */
/* IDL: 	PR_ATTACH_DATA_OBJ=0x3701000d, */
/* IDL: 	PR_ATTACH_DATA_OBJ_ERROR=0x3701000a, */
/* IDL: 	PR_ATTACH_DATA_BIN=0x37010102, */
/* IDL: 	PR_ATTACH_ENCODING=0x37020102, */
/* IDL: 	PR_ATTACH_ENCODING_ERROR=0x3702000a, */
/* IDL: 	PR_ATTACH_EXTENSION=0x3703001e, */
/* IDL: 	PR_ATTACH_EXTENSION_UNICODE=0x3703001f, */
/* IDL: 	PR_ATTACH_EXTENSION_ERROR=0x3703000a, */
/* IDL: 	PR_ATTACH_FILENAME=0x3704001e, */
/* IDL: 	PR_ATTACH_FILENAME_UNICODE=0x3704001f, */
/* IDL: 	PR_ATTACH_FILENAME_ERROR=0x3704000a, */
/* IDL: 	PR_ATTACH_METHOD=0x37050003, */
/* IDL: 	PR_ATTACH_METHOD_ERROR=0x3705000a, */
/* IDL: 	PR_ATTACH_LONG_FILENAME=0x3707001e, */
/* IDL: 	PR_ATTACH_LONG_FILENAME_UNICODE=0x3707001f, */
/* IDL: 	PR_ATTACH_LONG_FILENAME_ERROR=0x3707000a, */
/* IDL: 	PR_ATTACH_PATHNAME=0x3708001e, */
/* IDL: 	PR_ATTACH_PATHNAME_UNICODE=0x3708001f, */
/* IDL: 	PR_ATTACH_PATHNAME_ERROR=0x3708000a, */
/* IDL: 	PR_ATTACH_RENDERING=0x37090102, */
/* IDL: 	PR_ATTACH_RENDERING_ERROR=0x3709000a, */
/* IDL: 	PR_ATTACH_TAG=0x370a0102, */
/* IDL: 	PR_ATTACH_TAG_ERROR=0x370a000a, */
/* IDL: 	PR_RENDERING_POSITION=0x370b0003, */
/* IDL: 	PR_RENDERING_POSITION_ERROR=0x370b000a, */
/* IDL: 	PR_ATTACH_TRANSPORT_NAME=0x370c001e, */
/* IDL: 	PR_ATTACH_TRANSPORT_NAME_UNICODE=0x370c001f, */
/* IDL: 	PR_ATTACH_TRANSPORT_NAME_ERROR=0x370c000a, */
/* IDL: 	PR_ATTACH_LONG_PATHNAME=0x370d001e, */
/* IDL: 	PR_ATTACH_LONG_PATHNAME_UNICODE=0x370d001f, */
/* IDL: 	PR_ATTACH_LONG_PATHNAME_ERROR=0x370d000a, */
/* IDL: 	PR_ATTACH_MIME_TAG=0x370e001e, */
/* IDL: 	PR_ATTACH_MIME_TAG_UNICODE=0x370e001f, */
/* IDL: 	PR_ATTACH_MIME_TAG_ERROR=0x370e000a, */
/* IDL: 	PR_ATTACH_ADDITIONAL_INFO=0x370f0102, */
/* IDL: 	PR_ATTACH_ADDITIONAL_INFO_ERROR=0x370f000a, */
/* IDL: 	PR_ATTACH_CONTENT_ID=0x3712001e, */
/* IDL: 	PR_ATTACH_CONTENT_ID_UNICODE=0x3712001f, */
/* IDL: 	PR_ATTACH_CONTENT_ID_ERROR=0x3712000a, */
/* IDL: 	PR_ATTACH_CONTENT_LOCATION=0x3713001e, */
/* IDL: 	PR_ATTACH_CONTENT_LOCATION_UNICODE=0x3713001f, */
/* IDL: 	PR_ATTACH_CONTENT_LOCATION_ERROR=0x3713000a, */
/* IDL: 	PR_ATTACH_FLAGS=0x37140003, */
/* IDL: 	PR_ATTACH_FLAGS_ERROR=0x3714000a, */
/* IDL: 	PR_ATTACH_CONTENT_DISPOSITION=0x3716001e, */
/* IDL: 	PR_ATTACH_CONTENT_DISPOSITION_UNICODE=0x3716001f, */
/* IDL: 	PR_ATTACH_CONTENT_DISPOSITION_ERROR=0x3716000a, */
/* IDL: 	PR_SYNCEVENT_SUPPRESS_GUID=0x38800102, */
/* IDL: 	PR_SYNCEVENT_SUPPRESS_GUID_ERROR=0x3880000a, */
/* IDL: 	PR_DISPLAY_TYPE=0x39000003, */
/* IDL: 	PR_DISPLAY_TYPE_ERROR=0x3900000a, */
/* IDL: 	PR_TEMPLATEID=0x39020102, */
/* IDL: 	PR_TEMPLATEID_ERROR=0x3902000a, */
/* IDL: 	PR_PRIMARY_CAPABILITY=0x39040102, */
/* IDL: 	PR_PRIMARY_CAPABILITY_ERROR=0x3904000a, */
/* IDL: 	PR_SMTP_ADDRESS=0x39fe001e, */
/* IDL: 	PR_SMTP_ADDRESS_UNICODE=0x39fe001f, */
/* IDL: 	PR_SMTP_ADDRESS_ERROR=0x39fe000a, */
/* IDL: 	PR_7BIT_DISPLAY_NAME=0x39ff001e, */
/* IDL: 	PR_7BIT_DISPLAY_NAME_UNICODE=0x39ff001f, */
/* IDL: 	PR_7BIT_DISPLAY_NAME_ERROR=0x39ff000a, */
/* IDL: 	PR_ACCOUNT=0x3a00001e, */
/* IDL: 	PR_ACCOUNT_UNICODE=0x3a00001f, */
/* IDL: 	PR_ACCOUNT_ERROR=0x3a00000a, */
/* IDL: 	PR_ALTERNATE_RECIPIENT=0x3a010102, */
/* IDL: 	PR_ALTERNATE_RECIPIENT_ERROR=0x3a01000a, */
/* IDL: 	PR_CALLBACK_TELEPHONE_NUMBER=0x3a02001e, */
/* IDL: 	PR_CALLBACK_TELEPHONE_NUMBER_UNICODE=0x3a02001f, */
/* IDL: 	PR_CALLBACK_TELEPHONE_NUMBER_ERROR=0x3a02000a, */
/* IDL: 	PR_CONVERSION_PROHIBITED=0x3a03000b, */
/* IDL: 	PR_CONVERSION_PROHIBITED_ERROR=0x3a03000a, */
/* IDL: 	PR_DISCLOSE_RECIPIENTS=0x3a04000b, */
/* IDL: 	PR_DISCLOSE_RECIPIENTS_ERROR=0x3a04000a, */
/* IDL: 	PR_GENERATION=0x3a05001e, */
/* IDL: 	PR_GENERATION_UNICODE=0x3a05001f, */
/* IDL: 	PR_GENERATION_ERROR=0x3a05000a, */
/* IDL: 	PR_GIVEN_NAME=0x3a06001e, */
/* IDL: 	PR_GIVEN_NAME_UNICODE=0x3a06001f, */
/* IDL: 	PR_GIVEN_NAME_ERROR=0x3a06000a, */
/* IDL: 	PR_GOVERNMENT_ID_NUMBER=0x3a07001e, */
/* IDL: 	PR_GOVERNMENT_ID_NUMBER_UNICODE=0x3a07001f, */
/* IDL: 	PR_GOVERNMENT_ID_NUMBER_ERROR=0x3a07000a, */
/* IDL: 	PR_OFFICE_TELEPHONE_NUMBER=0x3a08001e, */
/* IDL: 	PR_OFFICE_TELEPHONE_NUMBER_UNICODE=0x3a08001f, */
/* IDL: 	PR_OFFICE_TELEPHONE_NUMBER_ERROR=0x3a08000a, */
/* IDL: 	PR_HOME_TELEPHONE_NUMBER=0x3a09001e, */
/* IDL: 	PR_HOME_TELEPHONE_NUMBER_UNICODE=0x3a09001f, */
/* IDL: 	PR_HOME_TELEPHONE_NUMBER_ERROR=0x3a09000a, */
/* IDL: 	PR_INITIALS=0x3a0a001e, */
/* IDL: 	PR_INITIALS_UNICODE=0x3a0a001f, */
/* IDL: 	PR_INITIALS_ERROR=0x3a0a000a, */
/* IDL: 	PR_KEYWORD=0x3a0b001e, */
/* IDL: 	PR_KEYWORD_UNICODE=0x3a0b001f, */
/* IDL: 	PR_KEYWORD_ERROR=0x3a0b000a, */
/* IDL: 	PR_LANGUAGE=0x3a0c001e, */
/* IDL: 	PR_LANGUAGE_UNICODE=0x3a0c001f, */
/* IDL: 	PR_LANGUAGE_ERROR=0x3a0c000a, */
/* IDL: 	PR_LOCATION=0x3a0d001e, */
/* IDL: 	PR_LOCATION_UNICODE=0x3a0d001f, */
/* IDL: 	PR_LOCATION_ERROR=0x3a0d000a, */
/* IDL: 	PR_MAIL_PERMISSION=0x3a0e000b, */
/* IDL: 	PR_MAIL_PERMISSION_ERROR=0x3a0e000a, */
/* IDL: 	PR_MHS_COMMON_NAME=0x3a0f001e, */
/* IDL: 	PR_MHS_COMMON_NAME_UNICODE=0x3a0f001f, */
/* IDL: 	PR_MHS_COMMON_NAME_ERROR=0x3a0f000a, */
/* IDL: 	PR_ORGANIZATIONAL_ID_NUMBER=0x3a10001e, */
/* IDL: 	PR_ORGANIZATIONAL_ID_NUMBER_UNICODE=0x3a10001f, */
/* IDL: 	PR_ORGANIZATIONAL_ID_NUMBER_ERROR=0x3a10000a, */
/* IDL: 	PR_SURNAME=0x3a11001e, */
/* IDL: 	PR_SURNAME_UNICODE=0x3a11001f, */
/* IDL: 	PR_SURNAME_ERROR=0x3a11000a, */
/* IDL: 	PR_ORIGINAL_ENTRYID=0x3a120102, */
/* IDL: 	PR_ORIGINAL_ENTRYID_ERROR=0x3a12000a, */
/* IDL: 	PR_ORIGINAL_DISPLAY_NAME=0x3a13001e, */
/* IDL: 	PR_ORIGINAL_DISPLAY_NAME_UNICODE=0x3a13001f, */
/* IDL: 	PR_ORIGINAL_DISPLAY_NAME_ERROR=0x3a13000a, */
/* IDL: 	PR_ORIGINAL_SEARCH_KEY=0x3a140102, */
/* IDL: 	PR_ORIGINAL_SEARCH_KEY_ERROR=0x3a14000a, */
/* IDL: 	PR_POSTAL_ADDRESS=0x3a15001e, */
/* IDL: 	PR_POSTAL_ADDRESS_UNICODE=0x3a15001f, */
/* IDL: 	PR_POSTAL_ADDRESS_ERROR=0x3a15000a, */
/* IDL: 	PR_COMPANY_NAME=0x3a16001e, */
/* IDL: 	PR_COMPANY_NAME_UNICODE=0x3a16001f, */
/* IDL: 	PR_COMPANY_NAME_ERROR=0x3a16000a, */
/* IDL: 	PR_TITLE=0x3a17001e, */
/* IDL: 	PR_TITLE_UNICODE=0x3a17001f, */
/* IDL: 	PR_TITLE_ERROR=0x3a17000a, */
/* IDL: 	PR_DEPARTMENT_NAME=0x3a18001e, */
/* IDL: 	PR_DEPARTMENT_NAME_UNICODE=0x3a18001f, */
/* IDL: 	PR_DEPARTMENT_NAME_ERROR=0x3a18000a, */
/* IDL: 	PR_OFFICE_LOCATION=0x3a19001e, */
/* IDL: 	PR_OFFICE_LOCATION_UNICODE=0x3a19001f, */
/* IDL: 	PR_OFFICE_LOCATION_ERROR=0x3a19000a, */
/* IDL: 	PR_PRIMARY_TELEPHONE_NUMBER=0x3a1a001e, */
/* IDL: 	PR_PRIMARY_TELEPHONE_NUMBER_UNICODE=0x3a1a001f, */
/* IDL: 	PR_PRIMARY_TELEPHONE_NUMBER_ERROR=0x3a1a000a, */
/* IDL: 	PR_OFFICE2_TELEPHONE_NUMBER=0x3a1b001e, */
/* IDL: 	PR_OFFICE2_TELEPHONE_NUMBER_UNICODE=0x3a1b001f, */
/* IDL: 	PR_OFFICE2_TELEPHONE_NUMBER_ERROR=0x3a1b000a, */
/* IDL: 	PR_MOBILE_TELEPHONE_NUMBER=0x3a1c001e, */
/* IDL: 	PR_MOBILE_TELEPHONE_NUMBER_UNICODE=0x3a1c001f, */
/* IDL: 	PR_MOBILE_TELEPHONE_NUMBER_ERROR=0x3a1c000a, */
/* IDL: 	PR_RADIO_TELEPHONE_NUMBER=0x3a1d001e, */
/* IDL: 	PR_RADIO_TELEPHONE_NUMBER_UNICODE=0x3a1d001f, */
/* IDL: 	PR_RADIO_TELEPHONE_NUMBER_ERROR=0x3a1d000a, */
/* IDL: 	PR_CAR_TELEPHONE_NUMBER=0x3a1e001e, */
/* IDL: 	PR_CAR_TELEPHONE_NUMBER_UNICODE=0x3a1e001f, */
/* IDL: 	PR_CAR_TELEPHONE_NUMBER_ERROR=0x3a1e000a, */
/* IDL: 	PR_OTHER_TELEPHONE_NUMBER=0x3a1f001e, */
/* IDL: 	PR_OTHER_TELEPHONE_NUMBER_UNICODE=0x3a1f001f, */
/* IDL: 	PR_OTHER_TELEPHONE_NUMBER_ERROR=0x3a1f000a, */
/* IDL: 	PR_TRANSMITTABLE_DISPLAY_NAME=0x3a20001e, */
/* IDL: 	PR_TRANSMITTABLE_DISPLAY_NAME_UNICODE=0x3a20001f, */
/* IDL: 	PR_TRANSMITTABLE_DISPLAY_NAME_ERROR=0x3a20000a, */
/* IDL: 	PR_PAGER_TELEPHONE_NUMBER=0x3a21001e, */
/* IDL: 	PR_PAGER_TELEPHONE_NUMBER_UNICODE=0x3a21001f, */
/* IDL: 	PR_PAGER_TELEPHONE_NUMBER_ERROR=0x3a21000a, */
/* IDL: 	PR_USER_CERTIFICATE=0x3a220102, */
/* IDL: 	PR_USER_CERTIFICATE_ERROR=0x3a22000a, */
/* IDL: 	PR_PRIMARY_FAX_NUMBER=0x3a23001e, */
/* IDL: 	PR_PRIMARY_FAX_NUMBER_UNICODE=0x3a23001f, */
/* IDL: 	PR_PRIMARY_FAX_NUMBER_ERROR=0x3a23000a, */
/* IDL: 	PR_HOME_FAX_NUMBER=0x3a25001e, */
/* IDL: 	PR_HOME_FAX_NUMBER_UNICODE=0x3a25001f, */
/* IDL: 	PR_HOME_FAX_NUMBER_ERROR=0x3a25000a, */
/* IDL: 	PR_COUNTRY=0x3a26001e, */
/* IDL: 	PR_COUNTRY_UNICODE=0x3a26001f, */
/* IDL: 	PR_COUNTRY_ERROR=0x3a26000a, */
/* IDL: 	PR_LOCALITY=0x3a27001e, */
/* IDL: 	PR_LOCALITY_UNICODE=0x3a27001f, */
/* IDL: 	PR_LOCALITY_ERROR=0x3a27000a, */
/* IDL: 	PR_STATE_OR_PROVINCE=0x3a28001e, */
/* IDL: 	PR_STATE_OR_PROVINCE_UNICODE=0x3a28001f, */
/* IDL: 	PR_STATE_OR_PROVINCE_ERROR=0x3a28000a, */
/* IDL: 	PR_STREET_ADDRESS=0x3a29001e, */
/* IDL: 	PR_STREET_ADDRESS_UNICODE=0x3a29001f, */
/* IDL: 	PR_STREET_ADDRESS_ERROR=0x3a29000a, */
/* IDL: 	PR_POSTAL_CODE=0x3a2a001e, */
/* IDL: 	PR_POSTAL_CODE_UNICODE=0x3a2a001f, */
/* IDL: 	PR_POSTAL_CODE_ERROR=0x3a2a000a, */
/* IDL: 	PR_POST_OFFICE_BOX=0x3a2b001e, */
/* IDL: 	PR_POST_OFFICE_BOX_UNICODE=0x3a2b001f, */
/* IDL: 	PR_POST_OFFICE_BOX_ERROR=0x3a2b000a, */
/* IDL: 	PR_TELEX_NUMBER=0x3a2c001e, */
/* IDL: 	PR_TELEX_NUMBER_UNICODE=0x3a2c001f, */
/* IDL: 	PR_TELEX_NUMBER_ERROR=0x3a2c000a, */
/* IDL: 	PR_ISDN_NUMBER=0x3a2d001e, */
/* IDL: 	PR_ISDN_NUMBER_UNICODE=0x3a2d001f, */
/* IDL: 	PR_ISDN_NUMBER_ERROR=0x3a2d000a, */
/* IDL: 	PR_ASSISTANT_TELEPHONE_NUMBER=0x3a2e001e, */
/* IDL: 	PR_ASSISTANT_TELEPHONE_NUMBER_UNICODE=0x3a2e001f, */
/* IDL: 	PR_ASSISTANT_TELEPHONE_NUMBER_ERROR=0x3a2e000a, */
/* IDL: 	PR_HOME2_TELEPHONE_NUMBER=0x3a2f001e, */
/* IDL: 	PR_HOME2_TELEPHONE_NUMBER_UNICODE=0x3a2f001f, */
/* IDL: 	PR_HOME2_TELEPHONE_NUMBER_ERROR=0x3a2f000a, */
/* IDL: 	PR_ASSISTANT=0x3a30001e, */
/* IDL: 	PR_ASSISTANT_UNICODE=0x3a30001f, */
/* IDL: 	PR_ASSISTANT_ERROR=0x3a30000a, */
/* IDL: 	PR_SEND_RICH_INFO=0x3a40000b, */
/* IDL: 	PR_SEND_RICH_INFO_ERROR=0x3a40000a, */
/* IDL: 	PR_WEDDING_ANNIVERSARY=0x3a410040, */
/* IDL: 	PR_WEDDING_ANNIVERSARY_ERROR=0x3a41000a, */
/* IDL: 	PR_BIRTHDAY=0x3a420040, */
/* IDL: 	PR_BIRTHDAY_ERROR=0x3a42000a, */
/* IDL: 	PR_HOBBIES=0x3a43001e, */
/* IDL: 	PR_HOBBIES_UNICODE=0x3a43001f, */
/* IDL: 	PR_HOBBIES_ERROR=0x3a43000a, */
/* IDL: 	PR_MIDDLE_NAME=0x3a44001e, */
/* IDL: 	PR_MIDDLE_NAME_UNICODE=0x3a44001f, */
/* IDL: 	PR_MIDDLE_NAME_ERROR=0x3a44000a, */
/* IDL: 	PR_DISPLAY_NAME_PREFIX=0x3a45001e, */
/* IDL: 	PR_DISPLAY_NAME_PREFIX_UNICODE=0x3a45001f, */
/* IDL: 	PR_DISPLAY_NAME_PREFIX_ERROR=0x3a45000a, */
/* IDL: 	PR_PROFESSION=0x3a46001e, */
/* IDL: 	PR_PROFESSION_UNICODE=0x3a46001f, */
/* IDL: 	PR_PROFESSION_ERROR=0x3a46000a, */
/* IDL: 	PR_PREFERRED_BY_NAME=0x3a47001e, */
/* IDL: 	PR_PREFERRED_BY_NAME_UNICODE=0x3a47001f, */
/* IDL: 	PR_PREFERRED_BY_NAME_ERROR=0x3a47000a, */
/* IDL: 	PR_SPOUSE_NAME=0x3a48001e, */
/* IDL: 	PR_SPOUSE_NAME_UNICODE=0x3a48001f, */
/* IDL: 	PR_SPOUSE_NAME_ERROR=0x3a48000a, */
/* IDL: 	PR_COMPUTER_NETWORK_NAME=0x3a49001e, */
/* IDL: 	PR_COMPUTER_NETWORK_NAME_UNICODE=0x3a49001f, */
/* IDL: 	PR_COMPUTER_NETWORK_NAME_ERROR=0x3a49000a, */
/* IDL: 	PR_CUSTOMER_ID=0x3a4a001e, */
/* IDL: 	PR_CUSTOMER_ID_UNICODE=0x3a4a001f, */
/* IDL: 	PR_CUSTOMER_ID_ERROR=0x3a4a000a, */
/* IDL: 	PR_TTYTDD_PHONE_NUMBER=0x3a4b001e, */
/* IDL: 	PR_TTYTDD_PHONE_NUMBER_UNICODE=0x3a4b001f, */
/* IDL: 	PR_TTYTDD_PHONE_NUMBER_ERROR=0x3a4b000a, */
/* IDL: 	PR_FTP_SITE=0x3a4c001e, */
/* IDL: 	PR_FTP_SITE_UNICODE=0x3a4c001f, */
/* IDL: 	PR_FTP_SITE_ERROR=0x3a4c000a, */
/* IDL: 	PR_GENDER=0x3a4d0002, */
/* IDL: 	PR_GENDER_ERROR=0x3a4d000a, */
/* IDL: 	PR_MANAGER_NAME=0x3a4e001e, */
/* IDL: 	PR_MANAGER_NAME_UNICODE=0x3a4e001f, */
/* IDL: 	PR_MANAGER_NAME_ERROR=0x3a4e000a, */
/* IDL: 	PR_NICKNAME=0x3a4f001e, */
/* IDL: 	PR_NICKNAME_UNICODE=0x3a4f001f, */
/* IDL: 	PR_NICKNAME_ERROR=0x3a4f000a, */
/* IDL: 	PR_PERSONAL_HOME_PAGE=0x3a50001e, */
/* IDL: 	PR_PERSONAL_HOME_PAGE_UNICODE=0x3a50001f, */
/* IDL: 	PR_PERSONAL_HOME_PAGE_ERROR=0x3a50000a, */
/* IDL: 	PR_BUSINESS_HOME_PAGE=0x3a51001e, */
/* IDL: 	PR_BUSINESS_HOME_PAGE_UNICODE=0x3a51001f, */
/* IDL: 	PR_BUSINESS_HOME_PAGE_ERROR=0x3a51000a, */
/* IDL: 	PR_CONTACT_VERSION=0x3a520048, */
/* IDL: 	PR_CONTACT_VERSION_ERROR=0x3a52000a, */
/* IDL: 	PR_CONTACT_ENTRYIDS=0x3a531102, */
/* IDL: 	PR_CONTACT_ENTRYIDS_ERROR=0x3a53000a, */
/* IDL: 	PR_CONTACT_ADDRTYPES=0x3a54101e, */
/* IDL: 	PR_CONTACT_ADDRTYPES_UNICODE=0x3a54101f, */
/* IDL: 	PR_CONTACT_ADDRTYPES_ERROR=0x3a54000a, */
/* IDL: 	PR_CONTACT_DEFAULT_ADDRESS_INDEX=0x3a550003, */
/* IDL: 	PR_CONTACT_DEFAULT_ADDRESS_INDEX_ERROR=0x3a55000a, */
/* IDL: 	PR_CONTACT_EMAIL_ADDRESSES=0x3a56101e, */
/* IDL: 	PR_CONTACT_EMAIL_ADDRESSES_UNICODE=0x3a56101f, */
/* IDL: 	PR_CONTACT_EMAIL_ADDRESSES_ERROR=0x3a56000a, */
/* IDL: 	PR_COMPANY_MAIN_PHONE_NUMBER=0x3a57001e, */
/* IDL: 	PR_COMPANY_MAIN_PHONE_NUMBER_UNICODE=0x3a57001f, */
/* IDL: 	PR_COMPANY_MAIN_PHONE_NUMBER_ERROR=0x3a57000a, */
/* IDL: 	PR_CHILDRENS_NAMES=0x3a58101e, */
/* IDL: 	PR_CHILDRENS_NAMES_UNICODE=0x3a58101f, */
/* IDL: 	PR_CHILDRENS_NAMES_ERROR=0x3a58000a, */
/* IDL: 	PR_HOME_ADDRESS_CITY=0x3a59001e, */
/* IDL: 	PR_HOME_ADDRESS_CITY_UNICODE=0x3a59001f, */
/* IDL: 	PR_HOME_ADDRESS_CITY_ERROR=0x3a59000a, */
/* IDL: 	PR_HOME_ADDRESS_COUNTRY=0x3a5a001e, */
/* IDL: 	PR_HOME_ADDRESS_COUNTRY_UNICODE=0x3a5a001f, */
/* IDL: 	PR_HOME_ADDRESS_COUNTRY_ERROR=0x3a5a000a, */
/* IDL: 	PR_HOME_ADDRESS_POSTAL_CODE=0x3a5b001e, */
/* IDL: 	PR_HOME_ADDRESS_POSTAL_CODE_UNICODE=0x3a5b001f, */
/* IDL: 	PR_HOME_ADDRESS_POSTAL_CODE_ERROR=0x3a5b000a, */
/* IDL: 	PR_HOME_ADDRESS_STATE_OR_PROVINCE=0x3a5c001e, */
/* IDL: 	PR_HOME_ADDRESS_STATE_OR_PROVINCE_UNICODE=0x3a5c001f, */
/* IDL: 	PR_HOME_ADDRESS_STATE_OR_PROVINCE_ERROR=0x3a5c000a, */
/* IDL: 	PR_HOME_ADDRESS_STREET=0x3a5d001e, */
/* IDL: 	PR_HOME_ADDRESS_STREET_UNICODE=0x3a5d001f, */
/* IDL: 	PR_HOME_ADDRESS_STREET_ERROR=0x3a5d000a, */
/* IDL: 	PR_HOME_ADDRESS_POST_OFFICE_BOX=0x3a5e001e, */
/* IDL: 	PR_HOME_ADDRESS_POST_OFFICE_BOX_UNICODE=0x3a5e001f, */
/* IDL: 	PR_HOME_ADDRESS_POST_OFFICE_BOX_ERROR=0x3a5e000a, */
/* IDL: 	PR_OTHER_ADDRESS_CITY=0x3a5f001e, */
/* IDL: 	PR_OTHER_ADDRESS_CITY_UNICODE=0x3a5f001f, */
/* IDL: 	PR_OTHER_ADDRESS_CITY_ERROR=0x3a5f000a, */
/* IDL: 	PR_OTHER_ADDRESS_COUNTRY=0x3a60001e, */
/* IDL: 	PR_OTHER_ADDRESS_COUNTRY_UNICODE=0x3a60001f, */
/* IDL: 	PR_OTHER_ADDRESS_COUNTRY_ERROR=0x3a60000a, */
/* IDL: 	PR_OTHER_ADDRESS_POSTAL_CODE=0x3a61001e, */
/* IDL: 	PR_OTHER_ADDRESS_POSTAL_CODE_UNICODE=0x3a61001f, */
/* IDL: 	PR_OTHER_ADDRESS_POSTAL_CODE_ERROR=0x3a61000a, */
/* IDL: 	PR_OTHER_ADDRESS_STATE_OR_PROVINCE=0x3a62001e, */
/* IDL: 	PR_OTHER_ADDRESS_STATE_OR_PROVINCE_UNICODE=0x3a62001f, */
/* IDL: 	PR_OTHER_ADDRESS_STATE_OR_PROVINCE_ERROR=0x3a62000a, */
/* IDL: 	PR_OTHER_ADDRESS_STREET=0x3a63001e, */
/* IDL: 	PR_OTHER_ADDRESS_STREET_UNICODE=0x3a63001f, */
/* IDL: 	PR_OTHER_ADDRESS_STREET_ERROR=0x3a63000a, */
/* IDL: 	PR_OTHER_ADDRESS_POST_OFFICE_BOX=0x3a64001e, */
/* IDL: 	PR_OTHER_ADDRESS_POST_OFFICE_BOX_UNICODE=0x3a64001f, */
/* IDL: 	PR_OTHER_ADDRESS_POST_OFFICE_BOX_ERROR=0x3a64000a, */
/* IDL: 	PR_USER_X509_CERTIFICATE=0x3a701102, */
/* IDL: 	PR_USER_X509_CERTIFICATE_ERROR=0x3a70000a, */
/* IDL: 	PR_SEND_INTERNET_ENCODING=0x3a710003, */
/* IDL: 	PR_SEND_INTERNET_ENCODING_ERROR=0x3a71000a, */
/* IDL: 	PR_STORE_PROVIDERS=0x3d000102, */
/* IDL: 	PR_STORE_PROVIDERS_ERROR=0x3d00000a, */
/* IDL: 	PR_AB_PROVIDERS=0x3d010102, */
/* IDL: 	PR_AB_PROVIDERS_ERROR=0x3d01000a, */
/* IDL: 	PR_TRANSPORT_PROVIDERS=0x3d020102, */
/* IDL: 	PR_TRANSPORT_PROVIDERS_ERROR=0x3d02000a, */
/* IDL: 	PR_DEFAULT_PROFILE=0x3d04000b, */
/* IDL: 	PR_DEFAULT_PROFILE_ERROR=0x3d04000a, */
/* IDL: 	PR_AB_SEARCH_PATH=0x3d051102, */
/* IDL: 	PR_AB_SEARCH_PATH_ERROR=0x3d05000a, */
/* IDL: 	PR_AB_DEFAULT_DIR=0x3d060102, */
/* IDL: 	PR_AB_DEFAULT_DIR_ERROR=0x3d06000a, */
/* IDL: 	PR_AB_DEFAULT_PAB=0x3d070102, */
/* IDL: 	PR_AB_DEFAULT_PAB_ERROR=0x3d07000a, */
/* IDL: 	PR_FILTERING_HOOKS=0x3d080102, */
/* IDL: 	PR_FILTERING_HOOKS_ERROR=0x3d08000a, */
/* IDL: 	PR_SERVICE_NAME=0x3d09001e, */
/* IDL: 	PR_SERVICE_NAME_UNICODE=0x3d09001f, */
/* IDL: 	PR_SERVICE_NAME_ERROR=0x3d09000a, */
/* IDL: 	PR_SERVICE_DLL_NAME=0x3d0a001e, */
/* IDL: 	PR_SERVICE_DLL_NAME_UNICODE=0x3d0a001f, */
/* IDL: 	PR_SERVICE_DLL_NAME_ERROR=0x3d0a000a, */
/* IDL: 	PR_SERVICE_ENTRY_NAME=0x3d0b001e, */
/* IDL: 	PR_SERVICE_ENTRY_NAME_UNICODE=0x3d0b001f, */
/* IDL: 	PR_SERVICE_ENTRY_NAME_ERROR=0x3d0b000a, */
/* IDL: 	PR_SERVICE_UID=0x3d0c0102, */
/* IDL: 	PR_SERVICE_UID_ERROR=0x3d0c000a, */
/* IDL: 	PR_SERVICE_EXTRA_UIDS=0x3d0d0102, */
/* IDL: 	PR_SERVICE_EXTRA_UIDS_ERROR=0x3d0d000a, */
/* IDL: 	PR_SERVICES=0x3d0e0102, */
/* IDL: 	PR_SERVICES_ERROR=0x3d0e000a, */
/* IDL: 	PR_SERVICE_SUPPORT_FILES=0x3d0f101e, */
/* IDL: 	PR_SERVICE_SUPPORT_FILES_UNICODE=0x3d0f101f, */
/* IDL: 	PR_SERVICE_SUPPORT_FILES_ERROR=0x3d0f000a, */
/* IDL: 	PR_SERVICE_DELETE_FILES=0x3d10101e, */
/* IDL: 	PR_SERVICE_DELETE_FILES_UNICODE=0x3d10101f, */
/* IDL: 	PR_SERVICE_DELETE_FILES_ERROR=0x3d10000a, */
/* IDL: 	PR_AB_SEARCH_PATH_UPDATE=0x3d110102, */
/* IDL: 	PR_AB_SEARCH_PATH_UPDATE_ERROR=0x3d11000a, */
/* IDL: 	PR_PROFILE_NAME=0x3d12001e, */
/* IDL: 	PR_PROFILE_NAME_UNICODE=0x3d12001f, */
/* IDL: 	PR_PROFILE_NAME_ERROR=0x3d12000a, */
/* IDL: 	PR_SERVICE_INSTALL_ID=0x3d13001e, */
/* IDL: 	PR_SERVICE_INSTALL_ID_UNICODE=0x3d13001f, */
/* IDL: 	PR_SERVICE_INSTALL_ID_ERROR=0x3d13000a, */
/* IDL: 	PR_ADMIN_SECURITY_DESCRIPTOR=0x3d210102, */
/* IDL: 	PR_ADMIN_SECURITY_DESCRIPTOR_ERROR=0x3d21000a, */
/* IDL: 	PR_IDENTITY_DISPLAY=0x3e00001e, */
/* IDL: 	PR_IDENTITY_DISPLAY_UNICODE=0x3e00001f, */
/* IDL: 	PR_IDENTITY_DISPLAY_ERROR=0x3e00000a, */
/* IDL: 	PR_IDENTITY_ENTRYID=0x3e010102, */
/* IDL: 	PR_IDENTITY_ENTRYID_ERROR=0x3e01000a, */
/* IDL: 	PR_RESOURCE_METHODS=0x3e020003, */
/* IDL: 	PR_RESOURCE_METHODS_ERROR=0x3e02000a, */
/* IDL: 	PR_RESOURCE_TYPE=0x3e030003, */
/* IDL: 	PR_RESOURCE_TYPE_ERROR=0x3e03000a, */
/* IDL: 	PR_STATUS_CODE=0x3e040003, */
/* IDL: 	PR_STATUS_CODE_ERROR=0x3e04000a, */
/* IDL: 	PR_IDENTITY_SEARCH_KEY=0x3e050102, */
/* IDL: 	PR_IDENTITY_SEARCH_KEY_ERROR=0x3e05000a, */
/* IDL: 	PR_OWN_STORE_ENTRYID=0x3e060102, */
/* IDL: 	PR_OWN_STORE_ENTRYID_ERROR=0x3e06000a, */
/* IDL: 	PR_RESOURCE_PATH=0x3e07001e, */
/* IDL: 	PR_RESOURCE_PATH_UNICODE=0x3e07001f, */
/* IDL: 	PR_RESOURCE_PATH_ERROR=0x3e07000a, */
/* IDL: 	PR_STATUS_STRING=0x3e08001e, */
/* IDL: 	PR_STATUS_STRING_UNICODE=0x3e08001f, */
/* IDL: 	PR_STATUS_STRING_ERROR=0x3e08000a, */
/* IDL: 	PR_X400_DEFERRED_DELIVERY_CANCEL=0x3e09000b, */
/* IDL: 	PR_X400_DEFERRED_DELIVERY_CANCEL_ERROR=0x3e09000a, */
/* IDL: 	PR_HEADER_FOLDER_ENTRYID=0x3e0a0102, */
/* IDL: 	PR_HEADER_FOLDER_ENTRYID_ERROR=0x3e0a000a, */
/* IDL: 	PR_REMOTE_PROGRESS=0x3e0b0003, */
/* IDL: 	PR_REMOTE_PROGRESS_ERROR=0x3e0b000a, */
/* IDL: 	PR_REMOTE_PROGRESS_TEXT=0x3e0c001e, */
/* IDL: 	PR_REMOTE_PROGRESS_TEXT_UNICODE=0x3e0c001f, */
/* IDL: 	PR_REMOTE_PROGRESS_TEXT_ERROR=0x3e0c000a, */
/* IDL: 	PR_REMOTE_VALIDATE_OK=0x3e0d000b, */
/* IDL: 	PR_REMOTE_VALIDATE_OK_ERROR=0x3e0d000a, */
/* IDL: 	PR_CONTROL_FLAGS=0x3f000003, */
/* IDL: 	PR_CONTROL_FLAGS_ERROR=0x3f00000a, */
/* IDL: 	PR_CONTROL_STRUCTURE=0x3f010102, */
/* IDL: 	PR_CONTROL_STRUCTURE_ERROR=0x3f01000a, */
/* IDL: 	PR_CONTROL_TYPE=0x3f020003, */
/* IDL: 	PR_CONTROL_TYPE_ERROR=0x3f02000a, */
/* IDL: 	PR_DELTAX=0x3f030003, */
/* IDL: 	PR_DELTAX_ERROR=0x3f03000a, */
/* IDL: 	PR_DELTAY=0x3f040003, */
/* IDL: 	PR_DELTAY_ERROR=0x3f04000a, */
/* IDL: 	PR_XPOS=0x3f050003, */
/* IDL: 	PR_XPOS_ERROR=0x3f05000a, */
/* IDL: 	PR_YPOS=0x3f060003, */
/* IDL: 	PR_YPOS_ERROR=0x3f06000a, */
/* IDL: 	PR_CONTROL_ID=0x3f070102, */
/* IDL: 	PR_CONTROL_ID_ERROR=0x3f07000a, */
/* IDL: 	PR_INITIAL_DETAILS_PANE=0x3f080003, */
/* IDL: 	PR_INITIAL_DETAILS_PANE_ERROR=0x3f08000a, */
/* IDL: 	PR_DID=0x3f800014, */
/* IDL: 	PR_DID_ERROR=0x3f80000a, */
/* IDL: 	PR_SEQID=0x3f810014, */
/* IDL: 	PR_SEQID_ERROR=0x3f81000a, */
/* IDL: 	PR_DRAFTID=0x3f820014, */
/* IDL: 	PR_DRAFTID_ERROR=0x3f82000a, */
/* IDL: 	PR_CHECK_IN_TIME=0x3f830040, */
/* IDL: 	PR_CHECK_IN_TIME_ERROR=0x3f83000a, */
/* IDL: 	PR_CHECK_IN_COMMENT=0x3f84001e, */
/* IDL: 	PR_CHECK_IN_COMMENT_UNICODE=0x3f84001f, */
/* IDL: 	PR_CHECK_IN_COMMENT_ERROR=0x3f84000a, */
/* IDL: 	PR_VERSION_OP_CODE=0x3f850003, */
/* IDL: 	PR_VERSION_OP_CODE_ERROR=0x3f85000a, */
/* IDL: 	PR_VERSION_OP_DATA=0x3f860102, */
/* IDL: 	PR_VERSION_OP_DATA_ERROR=0x3f86000a, */
/* IDL: 	PR_VERSION_SEQUENCE_NUMBER=0x3f870003, */
/* IDL: 	PR_VERSION_SEQUENCE_NUMBER_ERROR=0x3f87000a, */
/* IDL: 	PR_ATTACH_ID=0x3f880014, */
/* IDL: 	PR_ATTACH_ID_ERROR=0x3f88000a, */
/* IDL: 	PR_PKM_DOC_STATUS=0x3f8d001e, */
/* IDL: 	PR_PKM_DOC_STATUS_UNICODE=0x3f8d001f, */
/* IDL: 	PR_PKM_DOC_STATUS_ERROR=0x3f8d000a, */
/* IDL: 	PR_MV_PKM_OPERATION_REQ=0x3f8e101e, */
/* IDL: 	PR_MV_PKM_OPERATION_REQ_UNICODE=0x3f8e101f, */
/* IDL: 	PR_MV_PKM_OPERATION_REQ_ERROR=0x3f8e000a, */
/* IDL: 	PR_PKM_DOC_INTERNAL_STATE=0x3f8f001e, */
/* IDL: 	PR_PKM_DOC_INTERNAL_STATE_UNICODE=0x3f8f001f, */
/* IDL: 	PR_PKM_DOC_INTERNAL_STATE_ERROR=0x3f8f000a, */
/* IDL: 	PR_VERSIONING_FLAGS=0x3f900002, */
/* IDL: 	PR_VERSIONING_FLAGS_ERROR=0x3f90000a, */
/* IDL: 	PR_PKM_LAST_UNAPPROVED_VID=0x3f910102, */
/* IDL: 	PR_PKM_LAST_UNAPPROVED_VID_ERROR=0x3f91000a, */
/* IDL: 	PR_MV_PKM_VERSION_LABELS=0x3f92101e, */
/* IDL: 	PR_MV_PKM_VERSION_LABELS_UNICODE=0x3f92101f, */
/* IDL: 	PR_MV_PKM_VERSION_LABELS_ERROR=0x3f92000a, */
/* IDL: 	PR_MV_PKM_VERSION_STATUS=0x3f93101e, */
/* IDL: 	PR_MV_PKM_VERSION_STATUS_UNICODE=0x3f93101f, */
/* IDL: 	PR_MV_PKM_VERSION_STATUS_ERROR=0x3f93000a, */
/* IDL: 	PR_PKM_INTERNAL_DATA=0x3f940102, */
/* IDL: 	PR_PKM_INTERNAL_DATA_ERROR=0x3f94000a, */
/* IDL: 	PR_LAST_CONFLICT=0x3fc90102, */
/* IDL: 	PR_LAST_CONFLICT_ERROR=0x3fc9000a, */
/* IDL: 	PR_CONFLICT_MSG_KEY=0x3fca0102, */
/* IDL: 	PR_CONFLICT_MSG_KEY_ERROR=0x3fca000a, */
/* IDL: 	PR_REPL_HEADER=0x3fd00102, */
/* IDL: 	PR_REPL_HEADER_ERROR=0x3fd0000a, */
/* IDL: 	PR_REPL_STATUS=0x3fd10102, */
/* IDL: 	PR_REPL_STATUS_ERROR=0x3fd1000a, */
/* IDL: 	PR_REPL_CHANGES=0x3fd20102, */
/* IDL: 	PR_REPL_CHANGES_ERROR=0x3fd2000a, */
/* IDL: 	PR_REPL_RGM=0x3fd30102, */
/* IDL: 	PR_REPL_RGM_ERROR=0x3fd3000a, */
/* IDL: 	PR_RMI=0x3fd40102, */
/* IDL: 	PR_RMI_ERROR=0x3fd4000a, */
/* IDL: 	PR_INTERNAL_POST_REPLY=0x3fd50102, */
/* IDL: 	PR_INTERNAL_POST_REPLY_ERROR=0x3fd5000a, */
/* IDL: 	PR_NTSD_MODIFICATION_TIME=0x3fd60040, */
/* IDL: 	PR_NTSD_MODIFICATION_TIME_ERROR=0x3fd6000a, */
/* IDL: 	PR_PREVIEW_UNREAD=0x3fd8001e, */
/* IDL: 	PR_PREVIEW_UNREAD_UNICODE=0x3fd8001f, */
/* IDL: 	PR_PREVIEW_UNREAD_ERROR=0x3fd8000a, */
/* IDL: 	PR_PREVIEW=0x3fd9001e, */
/* IDL: 	PR_PREVIEW_UNICODE=0x3fd9001f, */
/* IDL: 	PR_PREVIEW_ERROR=0x3fd9000a, */
/* IDL: 	PR_ABSTRACT=0x3fda001e, */
/* IDL: 	PR_ABSTRACT_UNICODE=0x3fda001f, */
/* IDL: 	PR_ABSTRACT_ERROR=0x3fda000a, */
/* IDL: 	PR_DL_REPORT_FLAGS=0x3fdb0003, */
/* IDL: 	PR_DL_REPORT_FLAGS_ERROR=0x3fdb000a, */
/* IDL: 	PR_BILATERAL_INFO=0x3fdc0102, */
/* IDL: 	PR_BILATERAL_INFO_ERROR=0x3fdc000a, */
/* IDL: 	PR_MSG_BODY_ID=0x3fdd0003, */
/* IDL: 	PR_MSG_BODY_ID_ERROR=0x3fdd000a, */
/* IDL: 	PR_INTERNET_CPID=0x3fde0003, */
/* IDL: 	PR_INTERNET_CPID_ERROR=0x3fde000a, */
/* IDL: 	PR_AUTO_RESPONSE_SUPPRESS=0x3fdf0003, */
/* IDL: 	PR_AUTO_RESPONSE_SUPPRESS_ERROR=0x3fdf000a, */
/* IDL: 	PR_ACL_TABLE=0x3fe0000d, */
/* IDL: 	PR_ACL_TABLE_ERROR=0x3fe0000a, */
/* IDL: 	PR_ACL_DATA=0x3fe00102, */
/* IDL: 	PR_RULES_TABLE=0x3fe1000d, */
/* IDL: 	PR_RULES_TABLE_ERROR=0x3fe1000a, */
/* IDL: 	PR_RULES_DATA=0x3fe10102, */
/* IDL: 	PR_FOLDER_DESIGN_FLAGS=0x3fe20003, */
/* IDL: 	PR_FOLDER_DESIGN_FLAGS_ERROR=0x3fe2000a, */
/* IDL: 	PR_DELEGATED_BY_RULE=0x3fe3000b, */
/* IDL: 	PR_DELEGATED_BY_RULE_ERROR=0x3fe3000a, */
/* IDL: 	PR_DESIGN_IN_PROGRESS=0x3fe4000b, */
/* IDL: 	PR_DESIGN_IN_PROGRESS_ERROR=0x3fe4000a, */
/* IDL: 	PR_SECURE_ORIGINATION=0x3fe5000b, */
/* IDL: 	PR_SECURE_ORIGINATION_ERROR=0x3fe5000a, */
/* IDL: 	PR_PUBLISH_IN_ADDRESS_BOOK=0x3fe6000b, */
/* IDL: 	PR_PUBLISH_IN_ADDRESS_BOOK_ERROR=0x3fe6000a, */
/* IDL: 	PR_RESOLVE_METHOD=0x3fe70003, */
/* IDL: 	PR_RESOLVE_METHOD_ERROR=0x3fe7000a, */
/* IDL: 	PR_ADDRESS_BOOK_DISPLAY_NAME=0x3fe8001e, */
/* IDL: 	PR_ADDRESS_BOOK_DISPLAY_NAME_UNICODE=0x3fe8001f, */
/* IDL: 	PR_ADDRESS_BOOK_DISPLAY_NAME_ERROR=0x3fe8000a, */
/* IDL: 	PR_EFORMS_LOCALE_ID=0x3fe90003, */
/* IDL: 	PR_EFORMS_LOCALE_ID_ERROR=0x3fe9000a, */
/* IDL: 	PR_HAS_DAMS=0x3fea000b, */
/* IDL: 	PR_HAS_DAMS_ERROR=0x3fea000a, */
/* IDL: 	PR_DEFERRED_SEND_NUMBER=0x3feb0003, */
/* IDL: 	PR_DEFERRED_SEND_NUMBER_ERROR=0x3feb000a, */
/* IDL: 	PR_DEFERRED_SEND_UNITS=0x3fec0003, */
/* IDL: 	PR_DEFERRED_SEND_UNITS_ERROR=0x3fec000a, */
/* IDL: 	PR_EXPIRY_NUMBER=0x3fed0003, */
/* IDL: 	PR_EXPIRY_NUMBER_ERROR=0x3fed000a, */
/* IDL: 	PR_EXPIRY_UNITS=0x3fee0003, */
/* IDL: 	PR_EXPIRY_UNITS_ERROR=0x3fee000a, */
/* IDL: 	PR_DEFERRED_SEND_TIME=0x3fef0040, */
/* IDL: 	PR_DEFERRED_SEND_TIME_ERROR=0x3fef000a, */
/* IDL: 	PR_CONFLICT_ENTRYID=0x3ff00102, */
/* IDL: 	PR_CONFLICT_ENTRYID_ERROR=0x3ff0000a, */
/* IDL: 	PR_MESSAGE_LOCALE_ID=0x3ff10003, */
/* IDL: 	PR_MESSAGE_LOCALE_ID_ERROR=0x3ff1000a, */
/* IDL: 	PR_RULE_TRIGGER_HISTORY=0x3ff20102, */
/* IDL: 	PR_RULE_TRIGGER_HISTORY_ERROR=0x3ff2000a, */
/* IDL: 	PR_MOVE_TO_STORE_ENTRYID=0x3ff30102, */
/* IDL: 	PR_MOVE_TO_STORE_ENTRYID_ERROR=0x3ff3000a, */
/* IDL: 	PR_MOVE_TO_FOLDER_ENTRYID=0x3ff40102, */
/* IDL: 	PR_MOVE_TO_FOLDER_ENTRYID_ERROR=0x3ff4000a, */
/* IDL: 	PR_STORAGE_QUOTA_LIMIT=0x3ff50003, */
/* IDL: 	PR_STORAGE_QUOTA_LIMIT_ERROR=0x3ff5000a, */
/* IDL: 	PR_EXCESS_STORAGE_USED=0x3ff60003, */
/* IDL: 	PR_EXCESS_STORAGE_USED_ERROR=0x3ff6000a, */
/* IDL: 	PR_SVR_GENERATING_QUOTA_MSG=0x3ff7001e, */
/* IDL: 	PR_SVR_GENERATING_QUOTA_MSG_UNICODE=0x3ff7001f, */
/* IDL: 	PR_SVR_GENERATING_QUOTA_MSG_ERROR=0x3ff7000a, */
/* IDL: 	PR_CREATOR_NAME=0x3ff8001e, */
/* IDL: 	PR_CREATOR_NAME_UNICODE=0x3ff8001f, */
/* IDL: 	PR_CREATOR_NAME_ERROR=0x3ff8000a, */
/* IDL: 	PR_CREATOR_ENTRYID=0x3ff90102, */
/* IDL: 	PR_CREATOR_ENTRYID_ERROR=0x3ff9000a, */
/* IDL: 	PR_LAST_MODIFIER_NAME=0x3ffa001e, */
/* IDL: 	PR_LAST_MODIFIER_NAME_UNICODE=0x3ffa001f, */
/* IDL: 	PR_LAST_MODIFIER_NAME_ERROR=0x3ffa000a, */
/* IDL: 	PR_LAST_MODIFIER_ENTRYID=0x3ffb0102, */
/* IDL: 	PR_LAST_MODIFIER_ENTRYID_ERROR=0x3ffb000a, */
/* IDL: 	PR_REPLY_RECIPIENT_SMTP_PROXIES=0x3ffc001e, */
/* IDL: 	PR_REPLY_RECIPIENT_SMTP_PROXIES_UNICODE=0x3ffc001f, */
/* IDL: 	PR_REPLY_RECIPIENT_SMTP_PROXIES_ERROR=0x3ffc000a, */
/* IDL: 	PR_MESSAGE_CODEPAGE=0x3ffd0003, */
/* IDL: 	PR_MESSAGE_CODEPAGE_ERROR=0x3ffd000a, */
/* IDL: 	PR_EXTENDED_ACL_DATA=0x3ffe0102, */
/* IDL: 	PR_EXTENDED_ACL_DATA_ERROR=0x3ffe000a, */
/* IDL: 	PR_FROM_I_HAVE=0x3fff000b, */
/* IDL: 	PR_FROM_I_HAVE_ERROR=0x3fff000a, */
/* IDL: 	PR_NEW_ATTACH=0x40000003, */
/* IDL: 	PR_NEW_ATTACH_ERROR=0x4000000a, */
/* IDL: 	PR_START_EMBED=0x40010003, */
/* IDL: 	PR_START_EMBED_ERROR=0x4001000a, */
/* IDL: 	PR_END_EMBED=0x40020003, */
/* IDL: 	PR_END_EMBED_ERROR=0x4002000a, */
/* IDL: 	PR_START_RECIP=0x40030003, */
/* IDL: 	PR_START_RECIP_ERROR=0x4003000a, */
/* IDL: 	PR_END_RECIP=0x40040003, */
/* IDL: 	PR_END_RECIP_ERROR=0x4004000a, */
/* IDL: 	PR_END_CC_RECIP=0x40050003, */
/* IDL: 	PR_END_CC_RECIP_ERROR=0x4005000a, */
/* IDL: 	PR_END_BCC_RECIP=0x40060003, */
/* IDL: 	PR_END_BCC_RECIP_ERROR=0x4006000a, */
/* IDL: 	PR_END_P1_RECIP=0x40070003, */
/* IDL: 	PR_END_P1_RECIP_ERROR=0x4007000a, */
/* IDL: 	PR_START_TOP_FLD=0x40090003, */
/* IDL: 	PR_START_TOP_FLD_ERROR=0x4009000a, */
/* IDL: 	PR_START_SUB_FLD=0x400a0003, */
/* IDL: 	PR_START_SUB_FLD_ERROR=0x400a000a, */
/* IDL: 	PR_END_FOLDER=0x400b0003, */
/* IDL: 	PR_END_FOLDER_ERROR=0x400b000a, */
/* IDL: 	PR_START_MESSAGE=0x400c0003, */
/* IDL: 	PR_START_MESSAGE_ERROR=0x400c000a, */
/* IDL: 	PR_END_MESSAGE=0x400d0003, */
/* IDL: 	PR_END_MESSAGE_ERROR=0x400d000a, */
/* IDL: 	PR_END_ATTACH=0x400e0003, */
/* IDL: 	PR_END_ATTACH_ERROR=0x400e000a, */
/* IDL: 	PR_EC_WARNING=0x400f0003, */
/* IDL: 	PR_EC_WARNING_ERROR=0x400f000a, */
/* IDL: 	PR_START_FAI_MSG=0x40100003, */
/* IDL: 	PR_START_FAI_MSG_ERROR=0x4010000a, */
/* IDL: 	PR_NEW_FX_FOLDER=0x40110102, */
/* IDL: 	PR_NEW_FX_FOLDER_ERROR=0x4011000a, */
/* IDL: 	PR_INCR_SYNC_CHG=0x40120003, */
/* IDL: 	PR_INCR_SYNC_CHG_ERROR=0x4012000a, */
/* IDL: 	PR_INCR_SYNC_DEL=0x40130003, */
/* IDL: 	PR_INCR_SYNC_DEL_ERROR=0x4013000a, */
/* IDL: 	PR_INCR_SYNC_END=0x40140003, */
/* IDL: 	PR_INCR_SYNC_END_ERROR=0x4014000a, */
/* IDL: 	PR_INCR_SYNC_MSG=0x40150003, */
/* IDL: 	PR_INCR_SYNC_MSG_ERROR=0x4015000a, */
/* IDL: 	PR_FX_DEL_PROP=0x40160003, */
/* IDL: 	PR_FX_DEL_PROP_ERROR=0x4016000a, */
/* IDL: 	PR_IDSET_GIVEN=0x40170003, */
/* IDL: 	PR_IDSET_GIVEN_ERROR=0x4017000a, */
/* IDL: 	PR_SENDER_FLAGS=0x40190003, */
/* IDL: 	PR_SENDER_FLAGS_ERROR=0x4019000a, */
/* IDL: 	PR_SENT_REPRESENTING_FLAGS=0x401a0003, */
/* IDL: 	PR_SENT_REPRESENTING_FLAGS_ERROR=0x401a000a, */
/* IDL: 	PR_RCVD_BY_FLAGS=0x401b0003, */
/* IDL: 	PR_RCVD_BY_FLAGS_ERROR=0x401b000a, */
/* IDL: 	PR_RCVD_REPRESENTING_FLAGS=0x401c0003, */
/* IDL: 	PR_RCVD_REPRESENTING_FLAGS_ERROR=0x401c000a, */
/* IDL: 	PR_ORIGINAL_SENDER_FLAGS=0x401d0003, */
/* IDL: 	PR_ORIGINAL_SENDER_FLAGS_ERROR=0x401d000a, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_FLAGS=0x401e0003, */
/* IDL: 	PR_ORIGINAL_SENT_REPRESENTING_FLAGS_ERROR=0x401e000a, */
/* IDL: 	PR_REPORT_FLAGS=0x401f0003, */
/* IDL: 	PR_REPORT_FLAGS_ERROR=0x401f000a, */
/* IDL: 	PR_READ_RECEIPT_FLAGS=0x40200003, */
/* IDL: 	PR_READ_RECEIPT_FLAGS_ERROR=0x4020000a, */
/* IDL: 	PR_SOFT_DELETES=0x4021000b, */
/* IDL: 	PR_SOFT_DELETES_ERROR=0x4021000a, */
/* IDL: 	PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT=0x402c0102, */
/* IDL: 	PR_MESSAGE_SUBMISSION_ID_FROM_CLIENT_ERROR=0x402c000a, */
/* IDL: 	PR_SENDER_SIMPLE_DISP_NAME=0x4030001e, */
/* IDL: 	PR_SENDER_SIMPLE_DISP_NAME_UNICODE=0x4030001f, */
/* IDL: 	PR_SENDER_SIMPLE_DISP_NAME_ERROR=0x4030000a, */
/* IDL: 	PR_SENT_REPRESENTING_SIMPLE_DISP_NAME=0x4031001e, */
/* IDL: 	PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_UNICODE=0x4031001f, */
/* IDL: 	PR_SENT_REPRESENTING_SIMPLE_DISP_NAME_ERROR=0x4031000a, */
/* IDL: 	PR_CREATOR_SIMPLE_DISP_NAME=0x4038001e, */
/* IDL: 	PR_CREATOR_SIMPLE_DISP_NAME_UNICODE=0x4038001f, */
/* IDL: 	PR_CREATOR_SIMPLE_DISP_NAME_ERROR=0x4038000a, */
/* IDL: 	PR_ORG_ADDR_TYPE=0x403d001e, */
/* IDL: 	PR_ORG_ADDR_TYPE_UNICODE=0x403d001f, */
/* IDL: 	PR_ORG_ADDR_TYPE_ERROR=0x403d000a, */
/* IDL: 	PR_ORG_EMAIL_ADDR=0x403e001e, */
/* IDL: 	PR_ORG_EMAIL_ADDR_UNICODE=0x403e001f, */
/* IDL: 	PR_ORG_EMAIL_ADDR_ERROR=0x403e000a, */
/* IDL: 	PR_CREATOR_FLAGS=0x40590003, */
/* IDL: 	PR_CREATOR_FLAGS_ERROR=0x4059000a, */
/* IDL: 	PR_MODIFIER_FLAGS=0x405a0003, */
/* IDL: 	PR_MODIFIER_FLAGS_ERROR=0x405a000a, */
/* IDL: 	PR_ORIGINATOR_FLAGS=0x405b0003, */
/* IDL: 	PR_ORIGINATOR_FLAGS_ERROR=0x405b000a, */
/* IDL: 	PR_REPORT_DESTINATION_FLAGS=0x405c0003, */
/* IDL: 	PR_REPORT_DESTINATION_FLAGS_ERROR=0x405c000a, */
/* IDL: 	PR_ORIGINAL_AUTHOR_FLAGS=0x405d0003, */
/* IDL: 	PR_ORIGINAL_AUTHOR_FLAGS_ERROR=0x405d000a, */
/* IDL: 	PR_ORIGINATOR_SEARCH_KEY=0x40610102, */
/* IDL: 	PR_ORIGINATOR_SEARCH_KEY_ERROR=0x4061000a, */
/* IDL: 	PR_REPORT_DESTINATION_SEARCH_KEY=0x40640102, */
/* IDL: 	PR_REPORT_DESTINATION_SEARCH_KEY_ERROR=0x4064000a, */
/* IDL: 	PR_ER_FLAG=0x40650003, */
/* IDL: 	PR_ER_FLAG_ERROR=0x4065000a, */
/* IDL: 	PR_INTERNET_SUBJECT=0x40680102, */
/* IDL: 	PR_INTERNET_SUBJECT_ERROR=0x4068000a, */
/* IDL: 	PR_INTERNET_SENT_REPRESENTING_NAME=0x40690102, */
/* IDL: 	PR_INTERNET_SENT_REPRESENTING_NAME_ERROR=0x4069000a, */
/* IDL: 	PR_INET_MAIL_OVERRIDE_FORMAT=0x59020003, */
/* IDL: 	PR_INET_MAIL_OVERRIDE_FORMAT_ERROR=0x5902000a, */
/* IDL: 	PR_MSG_EDITOR_FORMAT=0x59090003, */
/* IDL: 	PR_MSG_EDITOR_FORMAT_ERROR=0x5909000a, */
/* IDL: 	PR_DOTSTUFF_STATE=0x60010003, */
/* IDL: 	PR_DOTSTUFF_STATE_ERROR=0x6001000a, */
/* IDL: 	PR_RULE_SERVER_RULE_ID=0x65a00014, */
/* IDL: 	PR_RULE_SERVER_RULE_ID_ERROR=0x65a0000a, */
/* IDL: 	PR_REPLY_TEMPLATE_ID=0x65c20102, */
/* IDL: 	PR_REPLY_TEMPLATE_ID_ERROR=0x65c2000a, */
/* IDL: 	PR_SOURCE_KEY=0x65e00102, */
/* IDL: 	PR_SOURCE_KEY_ERROR=0x65e0000a, */
/* IDL: 	PR_PARENT_SOURCE_KEY=0x65e10102, */
/* IDL: 	PR_PARENT_SOURCE_KEY_ERROR=0x65e1000a, */
/* IDL: 	PR_CHANGE_KEY=0x65e20102, */
/* IDL: 	PR_CHANGE_KEY_ERROR=0x65e2000a, */
/* IDL: 	PR_PREDECESSOR_CHANGE_LIST=0x65e30102, */
/* IDL: 	PR_PREDECESSOR_CHANGE_LIST_ERROR=0x65e3000a, */
/* IDL: 	PR_SYNCHRONIZE_FLAGS=0x65e40003, */
/* IDL: 	PR_SYNCHRONIZE_FLAGS_ERROR=0x65e4000a, */
/* IDL: 	PR_AUTO_ADD_NEW_SUBS=0x65e5000b, */
/* IDL: 	PR_AUTO_ADD_NEW_SUBS_ERROR=0x65e5000a, */
/* IDL: 	PR_NEW_SUBS_GET_AUTO_ADD=0x65e6000b, */
/* IDL: 	PR_NEW_SUBS_GET_AUTO_ADD_ERROR=0x65e6000a, */
/* IDL: 	PR_MESSAGE_SITE_NAME=0x65e7001e, */
/* IDL: 	PR_MESSAGE_SITE_NAME_UNICODE=0x65e7001f, */
/* IDL: 	PR_MESSAGE_SITE_NAME_ERROR=0x65e7000a, */
/* IDL: 	PR_MESSAGE_PROCESSED=0x65e8000b, */
/* IDL: 	PR_MESSAGE_PROCESSED_ERROR=0x65e8000a, */
/* IDL: 	PR_RULE_MSG_STATE=0x65e90003, */
/* IDL: 	PR_RULE_MSG_STATE_ERROR=0x65e9000a, */
/* IDL: 	PR_RULE_MSG_USER_FLAGS=0x65ea0003, */
/* IDL: 	PR_RULE_MSG_USER_FLAGS_ERROR=0x65ea000a, */
/* IDL: 	PR_RULE_MSG_PROVIDER=0x65eb001e, */
/* IDL: 	PR_RULE_MSG_PROVIDER_UNICODE=0x65eb001f, */
/* IDL: 	PR_RULE_MSG_PROVIDER_ERROR=0x65eb000a, */
/* IDL: 	PR_RULE_MSG_NAME=0x65ec001e, */
/* IDL: 	PR_RULE_MSG_NAME_UNICODE=0x65ec001f, */
/* IDL: 	PR_RULE_MSG_NAME_ERROR=0x65ec000a, */
/* IDL: 	PR_RULE_MSG_LEVEL=0x65ed0003, */
/* IDL: 	PR_RULE_MSG_LEVEL_ERROR=0x65ed000a, */
/* IDL: 	PR_RULE_MSG_PROVIDER_DATA=0x65ee0102, */
/* IDL: 	PR_RULE_MSG_PROVIDER_DATA_ERROR=0x65ee000a, */
/* IDL: 	PR_RULE_MSG_ACTIONS=0x65ef0102, */
/* IDL: 	PR_RULE_MSG_ACTIONS_ERROR=0x65ef000a, */
/* IDL: 	PR_RULE_MSG_CONDITION=0x65f00102, */
/* IDL: 	PR_RULE_MSG_CONDITION_ERROR=0x65f0000a, */
/* IDL: 	PR_RULE_MSG_CONDITION_LCID=0x65f10003, */
/* IDL: 	PR_RULE_MSG_CONDITION_LCID_ERROR=0x65f1000a, */
/* IDL: 	PR_RULE_MSG_VERSION=0x65f20002, */
/* IDL: 	PR_RULE_MSG_VERSION_ERROR=0x65f2000a, */
/* IDL: 	PR_RULE_MSG_SEQUENCE=0x65f30003, */
/* IDL: 	PR_RULE_MSG_SEQUENCE_ERROR=0x65f3000a, */
/* IDL: 	PR_PREVENT_MSG_CREATE=0x65f4000b, */
/* IDL: 	PR_PREVENT_MSG_CREATE_ERROR=0x65f4000a, */
/* IDL: 	PR_IMAP_INTERNAL_DATE=0x65f50040, */
/* IDL: 	PR_IMAP_INTERNAL_DATE_ERROR=0x65f5000a, */
/* IDL: 	PR_PROFILE_VERSION=0x66000003, */
/* IDL: 	PR_PROFILE_VERSION_ERROR=0x6600000a, */
/* IDL: 	PR_PROFILE_CONFIG_FLAGS=0x66010003, */
/* IDL: 	PR_PROFILE_CONFIG_FLAGS_ERROR=0x6601000a, */
/* IDL: 	PR_PROFILE_HOME_SERVER=0x6602001e, */
/* IDL: 	PR_PROFILE_HOME_SERVER_UNICODE=0x6602001f, */
/* IDL: 	PR_PROFILE_HOME_SERVER_ERROR=0x6602000a, */
/* IDL: 	PR_PROFILE_USER=0x6603001e, */
/* IDL: 	PR_PROFILE_USER_UNICODE=0x6603001f, */
/* IDL: 	PR_PROFILE_USER_ERROR=0x6603000a, */
/* IDL: 	PR_PROFILE_CONNECT_FLAGS=0x66040003, */
/* IDL: 	PR_PROFILE_CONNECT_FLAGS_ERROR=0x6604000a, */
/* IDL: 	PR_PROFILE_TRANSPORT_FLAGS=0x66050003, */
/* IDL: 	PR_PROFILE_TRANSPORT_FLAGS_ERROR=0x6605000a, */
/* IDL: 	PR_PROFILE_UI_STATE=0x66060003, */
/* IDL: 	PR_PROFILE_UI_STATE_ERROR=0x6606000a, */
/* IDL: 	PR_PROFILE_UNRESOLVED_NAME=0x6607001e, */
/* IDL: 	PR_PROFILE_UNRESOLVED_NAME_UNICODE=0x6607001f, */
/* IDL: 	PR_PROFILE_UNRESOLVED_NAME_ERROR=0x6607000a, */
/* IDL: 	PR_PROFILE_UNRESOLVED_SERVER=0x6608001e, */
/* IDL: 	PR_PROFILE_UNRESOLVED_SERVER_UNICODE=0x6608001f, */
/* IDL: 	PR_PROFILE_UNRESOLVED_SERVER_ERROR=0x6608000a, */
/* IDL: 	PR_PROFILE_OPEN_FLAGS=0x66090003, */
/* IDL: 	PR_PROFILE_OPEN_FLAGS_ERROR=0x6609000a, */
/* IDL: 	PR_PROFILE_BINDING_ORDER=0x6609001e, */
/* IDL: 	PR_PROFILE_BINDING_ORDER_UNICODE=0x6609001f, */
/* IDL: 	PR_PROFILE_TYPE=0x660a0003, */
/* IDL: 	PR_PROFILE_TYPE_ERROR=0x660a000a, */
/* IDL: 	PR_PROFILE_MAILBOX=0x660b001e, */
/* IDL: 	PR_PROFILE_MAILBOX_UNICODE=0x660b001f, */
/* IDL: 	PR_PROFILE_MAILBOX_ERROR=0x660b000a, */
/* IDL: 	PR_PROFILE_SERVER=0x660c001e, */
/* IDL: 	PR_PROFILE_SERVER_UNICODE=0x660c001f, */
/* IDL: 	PR_PROFILE_SERVER_ERROR=0x660c000a, */
/* IDL: 	PR_PROFILE_MAX_RESTRICT=0x660d0003, */
/* IDL: 	PR_PROFILE_MAX_RESTRICT_ERROR=0x660d000a, */
/* IDL: 	PR_PROFILE_AB_FILES_PATH=0x660e001e, */
/* IDL: 	PR_PROFILE_AB_FILES_PATH_UNICODE=0x660e001f, */
/* IDL: 	PR_PROFILE_AB_FILES_PATH_ERROR=0x660e000a, */
/* IDL: 	PR_PROFILE_FAVFLD_DISPLAY_NAME=0x660f001e, */
/* IDL: 	PR_PROFILE_FAVFLD_DISPLAY_NAME_UNICODE=0x660f001f, */
/* IDL: 	PR_PROFILE_FAVFLD_DISPLAY_NAME_ERROR=0x660f000a, */
/* IDL: 	PR_PROFILE_OFFLINE_STORE_PATH=0x6610001e, */
/* IDL: 	PR_PROFILE_OFFLINE_STORE_PATH_UNICODE=0x6610001f, */
/* IDL: 	PR_PROFILE_OFFLINE_STORE_PATH_ERROR=0x6610000a, */
/* IDL: 	PR_PROFILE_OFFLINE_INFO=0x66110102, */
/* IDL: 	PR_PROFILE_OFFLINE_INFO_ERROR=0x6611000a, */
/* IDL: 	PR_PROFILE_HOME_SERVER_DN=0x6612001e, */
/* IDL: 	PR_PROFILE_HOME_SERVER_DN_UNICODE=0x6612001f, */
/* IDL: 	PR_PROFILE_HOME_SERVER_DN_ERROR=0x6612000a, */
/* IDL: 	PR_PROFILE_HOME_SERVER_ADDRS=0x6613101e, */
/* IDL: 	PR_PROFILE_HOME_SERVER_ADDRS_UNICODE=0x6613101f, */
/* IDL: 	PR_PROFILE_HOME_SERVER_ADDRS_ERROR=0x6613000a, */
/* IDL: 	PR_PROFILE_SERVER_DN=0x6614001e, */
/* IDL: 	PR_PROFILE_SERVER_DN_UNICODE=0x6614001f, */
/* IDL: 	PR_PROFILE_SERVER_DN_ERROR=0x6614000a, */
/* IDL: 	PR_PROFILE_FAVFLD_COMMENT=0x6615001e, */
/* IDL: 	PR_PROFILE_FAVFLD_COMMENT_UNICODE=0x6615001f, */
/* IDL: 	PR_PROFILE_FAVFLD_COMMENT_ERROR=0x6615000a, */
/* IDL: 	PR_PROFILE_ALLPUB_DISPLAY_NAME=0x6616001e, */
/* IDL: 	PR_PROFILE_ALLPUB_DISPLAY_NAME_UNICODE=0x6616001f, */
/* IDL: 	PR_PROFILE_ALLPUB_DISPLAY_NAME_ERROR=0x6616000a, */
/* IDL: 	PR_PROFILE_ALLPUB_COMMENT=0x6617001e, */
/* IDL: 	PR_PROFILE_ALLPUB_COMMENT_UNICODE=0x6617001f, */
/* IDL: 	PR_PROFILE_ALLPUB_COMMENT_ERROR=0x6617000a, */
/* IDL: 	PR_DISABLE_WINSOCK=0x66180003, */
/* IDL: 	PR_DISABLE_WINSOCK_ERROR=0x6618000a, */
/* IDL: 	PR_IN_TRANSIT=0x6618000b, */
/* IDL: 	PR_PROFILE_AUTH_PACKAGE=0x66190003, */
/* IDL: 	PR_PROFILE_AUTH_PACKAGE_ERROR=0x6619000a, */
/* IDL: 	PR_USER_ENTRYID=0x66190102, */
/* IDL: 	PR_USER_NAME=0x661a001e, */
/* IDL: 	PR_USER_NAME_UNICODE=0x661a001f, */
/* IDL: 	PR_USER_NAME_ERROR=0x661a000a, */
/* IDL: 	PR_MAILBOX_OWNER_ENTRYID=0x661b0102, */
/* IDL: 	PR_MAILBOX_OWNER_ENTRYID_ERROR=0x661b000a, */
/* IDL: 	PR_MAILBOX_OWNER_NAME=0x661c001e, */
/* IDL: 	PR_MAILBOX_OWNER_NAME_UNICODE=0x661c001f, */
/* IDL: 	PR_MAILBOX_OWNER_NAME_ERROR=0x661c000a, */
/* IDL: 	PR_OOF_STATE=0x661d000b, */
/* IDL: 	PR_OOF_STATE_ERROR=0x661d000a, */
/* IDL: 	PR_SCHEDULE_FOLDER_ENTRYID=0x661e0102, */
/* IDL: 	PR_SCHEDULE_FOLDER_ENTRYID_ERROR=0x661e000a, */
/* IDL: 	PR_IPM_DAF_ENTRYID=0x661f0102, */
/* IDL: 	PR_IPM_DAF_ENTRYID_ERROR=0x661f000a, */
/* IDL: 	PR_NON_IPM_SUBTREE_ENTRYID=0x66200102, */
/* IDL: 	PR_NON_IPM_SUBTREE_ENTRYID_ERROR=0x6620000a, */
/* IDL: 	PR_EFORMS_REGISTRY_ENTRYID=0x66210102, */
/* IDL: 	PR_EFORMS_REGISTRY_ENTRYID_ERROR=0x6621000a, */
/* IDL: 	PR_SPLUS_FREE_BUSY_ENTRYID=0x66220102, */
/* IDL: 	PR_SPLUS_FREE_BUSY_ENTRYID_ERROR=0x6622000a, */
/* IDL: 	PR_HIERARCHY_SERVER=0x6623001e, */
/* IDL: 	PR_HIERARCHY_SERVER_UNICODE=0x6623001f, */
/* IDL: 	PR_HIERARCHY_SERVER_ERROR=0x6623000a, */
/* IDL: 	PR_OFFLINE_ADDRBOOK_ENTRYID=0x66230102, */
/* IDL: 	PR_EFORMS_FOR_LOCALE_ENTRYID=0x66240102, */
/* IDL: 	PR_EFORMS_FOR_LOCALE_ENTRYID_ERROR=0x6624000a, */
/* IDL: 	PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID=0x66250102, */
/* IDL: 	PR_FREE_BUSY_FOR_LOCAL_SITE_ENTRYID_ERROR=0x6625000a, */
/* IDL: 	PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID=0x66260102, */
/* IDL: 	PR_ADDRBOOK_FOR_LOCAL_SITE_ENTRYID_ERROR=0x6626000a, */
/* IDL: 	PR_OFFLINE_MESSAGE_ENTRYID=0x66270102, */
/* IDL: 	PR_OFFLINE_MESSAGE_ENTRYID_ERROR=0x6627000a, */
/* IDL: 	PR_GW_MTSIN_ENTRYID=0x66280102, */
/* IDL: 	PR_GW_MTSIN_ENTRYID_ERROR=0x6628000a, */
/* IDL: 	PR_GW_MTSOUT_ENTRYID=0x66290102, */
/* IDL: 	PR_GW_MTSOUT_ENTRYID_ERROR=0x6629000a, */
/* IDL: 	PR_TRANSFER_ENABLED=0x662a000b, */
/* IDL: 	PR_TRANSFER_ENABLED_ERROR=0x662a000a, */
/* IDL: 	PR_TEST_LINE_SPEED=0x662b0102, */
/* IDL: 	PR_TEST_LINE_SPEED_ERROR=0x662b000a, */
/* IDL: 	PR_HIERARCHY_SYNCHRONIZER=0x662c000d, */
/* IDL: 	PR_HIERARCHY_SYNCHRONIZER_ERROR=0x662c000a, */
/* IDL: 	PR_CONTENTS_SYNCHRONIZER=0x662d000d, */
/* IDL: 	PR_CONTENTS_SYNCHRONIZER_ERROR=0x662d000a, */
/* IDL: 	PR_COLLECTOR=0x662e000d, */
/* IDL: 	PR_COLLECTOR_ERROR=0x662e000a, */
/* IDL: 	PR_FAST_TRANSFER=0x662f000d, */
/* IDL: 	PR_FAST_TRANSFER_ERROR=0x662f000a, */
/* IDL: 	PR_IPM_FAVORITES_ENTRYID=0x66300102, */
/* IDL: 	PR_IPM_FAVORITES_ENTRYID_ERROR=0x6630000a, */
/* IDL: 	PR_IPM_PUBLIC_FOLDERS_ENTRYID=0x66310102, */
/* IDL: 	PR_IPM_PUBLIC_FOLDERS_ENTRYID_ERROR=0x6631000a, */
/* IDL: 	PR_STORE_OFFLINE=0x6632000b, */
/* IDL: 	PR_STORE_OFFLINE_ERROR=0x6632000a, */
/* IDL: 	PR_CHANGE_ADVISOR=0x6634000d, */
/* IDL: 	PR_CHANGE_ADVISOR_ERROR=0x6634000a, */
/* IDL: 	PR_FAVORITES_DEFAULT_NAME=0x6635001e, */
/* IDL: 	PR_FAVORITES_DEFAULT_NAME_UNICODE=0x6635001f, */
/* IDL: 	PR_FAVORITES_DEFAULT_NAME_ERROR=0x6635000a, */
/* IDL: 	PR_SYS_CONFIG_FOLDER_ENTRYID=0x66360102, */
/* IDL: 	PR_SYS_CONFIG_FOLDER_ENTRYID_ERROR=0x6636000a, */
/* IDL: 	PR_CHANGE_NOTIFICATION_GUID=0x66370048, */
/* IDL: 	PR_CHANGE_NOTIFICATION_GUID_ERROR=0x6637000a, */
/* IDL: 	PR_FOLDER_CHILD_COUNT=0x66380003, */
/* IDL: 	PR_FOLDER_CHILD_COUNT_ERROR=0x6638000a, */
/* IDL: 	PR_RIGHTS=0x66390003, */
/* IDL: 	PR_RIGHTS_ERROR=0x6639000a, */
/* IDL: 	PR_HAS_RULES=0x663a000b, */
/* IDL: 	PR_HAS_RULES_ERROR=0x663a000a, */
/* IDL: 	PR_ADDRESS_BOOK_ENTRYID=0x663b0102, */
/* IDL: 	PR_ADDRESS_BOOK_ENTRYID_ERROR=0x663b000a, */
/* IDL: 	PR_PUBLIC_FOLDER_ENTRYID=0x663c0102, */
/* IDL: 	PR_PUBLIC_FOLDER_ENTRYID_ERROR=0x663c000a, */
/* IDL: 	PR_OFFLINE_FLAGS=0x663d0003, */
/* IDL: 	PR_OFFLINE_FLAGS_ERROR=0x663d000a, */
/* IDL: 	PR_HIERARCHY_CHANGE_NUM=0x663e0003, */
/* IDL: 	PR_HIERARCHY_CHANGE_NUM_ERROR=0x663e000a, */
/* IDL: 	PR_HAS_MODERATOR_RULES=0x663f000b, */
/* IDL: 	PR_HAS_MODERATOR_RULES_ERROR=0x663f000a, */
/* IDL: 	PR_DELETED_MSG_COUNT=0x66400003, */
/* IDL: 	PR_DELETED_MSG_COUNT_ERROR=0x6640000a, */
/* IDL: 	PR_DELETED_FOLDER_COUNT=0x66410003, */
/* IDL: 	PR_DELETED_FOLDER_COUNT_ERROR=0x6641000a, */
/* IDL: 	PR_OLDEST_DELETED_ON=0x66420040, */
/* IDL: 	PR_OLDEST_DELETED_ON_ERROR=0x6642000a, */
/* IDL: 	PR_DELETED_ASSOC_MSG_COUNT=0x66430003, */
/* IDL: 	PR_DELETED_ASSOC_MSG_COUNT_ERROR=0x6643000a, */
/* IDL: 	PR_REPLICA_SERVER=0x6644001e, */
/* IDL: 	PR_REPLICA_SERVER_UNICODE=0x6644001f, */
/* IDL: 	PR_REPLICA_SERVER_ERROR=0x6644000a, */
/* IDL: 	PR_CLIENT_ACTIONS=0x66450102, */
/* IDL: 	PR_CLIENT_ACTIONS_ERROR=0x6645000a, */
/* IDL: 	PR_DAM_ORIGINAL_ENTRYID=0x66460102, */
/* IDL: 	PR_DAM_ORIGINAL_ENTRYID_ERROR=0x6646000a, */
/* IDL: 	PR_DAM_BACK_PATCHED=0x6647000b, */
/* IDL: 	PR_DAM_BACK_PATCHED_ERROR=0x6647000a, */
/* IDL: 	PR_RULE_ERROR=0x66480003, */
/* IDL: 	PR_RULE_ERROR_ERROR=0x6648000a, */
/* IDL: 	PR_RULE_ACTION_TYPE=0x66490003, */
/* IDL: 	PR_RULE_ACTION_TYPE_ERROR=0x6649000a, */
/* IDL: 	PR_HAS_NAMED_PROPERTIES=0x664a000b, */
/* IDL: 	PR_HAS_NAMED_PROPERTIES_ERROR=0x664a000a, */
/* IDL: 	PR_REPLICA_VERSION=0x664b0014, */
/* IDL: 	PR_REPLICA_VERSION_ERROR=0x664b000a, */
/* IDL: 	PR_RULE_ACTION_NUMBER=0x66500003, */
/* IDL: 	PR_RULE_ACTION_NUMBER_ERROR=0x6650000a, */
/* IDL: 	PR_RULE_FOLDER_ENTRYID=0x66510102, */
/* IDL: 	PR_RULE_FOLDER_ENTRYID_ERROR=0x6651000a, */
/* IDL: 	PR_ACTIVE_USER_ENTRYID=0x66520102, */
/* IDL: 	PR_ACTIVE_USER_ENTRYID_ERROR=0x6652000a, */
/* IDL: 	PR_0X400_ENVELOPE_TYPE=0x66530003, */
/* IDL: 	PR_0X400_ENVELOPE_TYPE_ERROR=0x6653000a, */
/* IDL: 	PR_MSG_FOLD_TIME=0x66540040, */
/* IDL: 	PR_MSG_FOLD_TIME_ERROR=0x6654000a, */
/* IDL: 	PR_ICS_CHANGE_KEY=0x66550102, */
/* IDL: 	PR_ICS_CHANGE_KEY_ERROR=0x6655000a, */
/* IDL: 	PR_GW_ADMIN_OPERATIONS=0x66580003, */
/* IDL: 	PR_GW_ADMIN_OPERATIONS_ERROR=0x6658000a, */
/* IDL: 	PR_INTERNET_CONTENT=0x66590102, */
/* IDL: 	PR_INTERNET_CONTENT_ERROR=0x6659000a, */
/* IDL: 	PR_HAS_ATTACH_FROM_IMAIL=0x665a000b, */
/* IDL: 	PR_HAS_ATTACH_FROM_IMAIL_ERROR=0x665a000a, */
/* IDL: 	PR_ORIGINATOR_NAME=0x665b001e, */
/* IDL: 	PR_ORIGINATOR_NAME_UNICODE=0x665b001f, */
/* IDL: 	PR_ORIGINATOR_NAME_ERROR=0x665b000a, */
/* IDL: 	PR_ORIGINATOR_ADDR=0x665c001e, */
/* IDL: 	PR_ORIGINATOR_ADDR_UNICODE=0x665c001f, */
/* IDL: 	PR_ORIGINATOR_ADDR_ERROR=0x665c000a, */
/* IDL: 	PR_ORIGINATOR_ADDRTYPE=0x665d001e, */
/* IDL: 	PR_ORIGINATOR_ADDRTYPE_UNICODE=0x665d001f, */
/* IDL: 	PR_ORIGINATOR_ADDRTYPE_ERROR=0x665d000a, */
/* IDL: 	PR_ORIGINATOR_ENTRYID=0x665e0102, */
/* IDL: 	PR_ORIGINATOR_ENTRYID_ERROR=0x665e000a, */
/* IDL: 	PR_ARRIVAL_TIME=0x665f0040, */
/* IDL: 	PR_ARRIVAL_TIME_ERROR=0x665f000a, */
/* IDL: 	PR_TRACE_INFO=0x66600102, */
/* IDL: 	PR_TRACE_INFO_ERROR=0x6660000a, */
/* IDL: 	PR_SUBJECT_TRACE_INFO=0x66610102, */
/* IDL: 	PR_SUBJECT_TRACE_INFO_ERROR=0x6661000a, */
/* IDL: 	PR_RECIPIENT_NUMBER=0x66620003, */
/* IDL: 	PR_RECIPIENT_NUMBER_ERROR=0x6662000a, */
/* IDL: 	PR_MTS_SUBJECT_ID=0x66630102, */
/* IDL: 	PR_MTS_SUBJECT_ID_ERROR=0x6663000a, */
/* IDL: 	PR_REPORT_DESTINATION_NAME=0x6664001e, */
/* IDL: 	PR_REPORT_DESTINATION_NAME_UNICODE=0x6664001f, */
/* IDL: 	PR_REPORT_DESTINATION_NAME_ERROR=0x6664000a, */
/* IDL: 	PR_REPORT_DESTINATION_ENTRYID=0x66650102, */
/* IDL: 	PR_REPORT_DESTINATION_ENTRYID_ERROR=0x6665000a, */
/* IDL: 	PR_CONTENT_SEARCH_KEY=0x66660102, */
/* IDL: 	PR_CONTENT_SEARCH_KEY_ERROR=0x6666000a, */
/* IDL: 	PR_FOREIGN_ID=0x66670102, */
/* IDL: 	PR_FOREIGN_ID_ERROR=0x6667000a, */
/* IDL: 	PR_FOREIGN_REPORT_ID=0x66680102, */
/* IDL: 	PR_FOREIGN_REPORT_ID_ERROR=0x6668000a, */
/* IDL: 	PR_FOREIGN_SUBJECT_ID=0x66690102, */
/* IDL: 	PR_FOREIGN_SUBJECT_ID_ERROR=0x6669000a, */
/* IDL: 	PR_INTERNAL_TRACE_INFO=0x666a0102, */
/* IDL: 	PR_INTERNAL_TRACE_INFO_ERROR=0x666a000a, */
/* IDL: 	PR_IN_CONFLICT=0x666c000b, */
/* IDL: 	PR_IN_CONFLICT_ERROR=0x666c000a, */
/* IDL: 	PR_LONGTERM_ENTRYID_FROM_TABLE=0x66700102, */
/* IDL: 	PR_LONGTERM_ENTRYID_FROM_TABLE_ERROR=0x6670000a, */
/* IDL: 	PR_MEMBER_ID=0x66710014, */
/* IDL: 	PR_MEMBER_ID_ERROR=0x6671000a, */
/* IDL: 	PR_MEMBER_NAME=0x6672001e, */
/* IDL: 	PR_MEMBER_NAME_UNICODE=0x6672001f, */
/* IDL: 	PR_MEMBER_NAME_ERROR=0x6672000a, */
/* IDL: 	PR_MEMBER_RIGHTS=0x66730003, */
/* IDL: 	PR_MEMBER_RIGHTS_ERROR=0x6673000a, */
/* IDL: 	PR_RULE_ID=0x66740014, */
/* IDL: 	PR_RULE_ID_ERROR=0x6674000a, */
/* IDL: 	PR_RULE_IDS=0x66750102, */
/* IDL: 	PR_RULE_IDS_ERROR=0x6675000a, */
/* IDL: 	PR_RULE_SEQUENCE=0x66760003, */
/* IDL: 	PR_RULE_SEQUENCE_ERROR=0x6676000a, */
/* IDL: 	PR_RULE_STATE=0x66770003, */
/* IDL: 	PR_RULE_STATE_ERROR=0x6677000a, */
/* IDL: 	PR_RULE_USER_FLAGS=0x66780003, */
/* IDL: 	PR_RULE_USER_FLAGS_ERROR=0x6678000a, */
/* IDL: 	PR_RULE_CONDITION=0x667900fd, */
/* IDL: 	PR_RULE_CONDITION_ERROR=0x6679000a, */
/* IDL: 	PR_PROFILE_MOAB=0x667b001e, */
/* IDL: 	PR_PROFILE_MOAB_UNICODE=0x667b001f, */
/* IDL: 	PR_PROFILE_MOAB_ERROR=0x667b000a, */
/* IDL: 	PR_PROFILE_MOAB_GUID=0x667c001e, */
/* IDL: 	PR_PROFILE_MOAB_GUID_UNICODE=0x667c001f, */
/* IDL: 	PR_PROFILE_MOAB_GUID_ERROR=0x667c000a, */
/* IDL: 	PR_PROFILE_MOAB_SEQ=0x667d0003, */
/* IDL: 	PR_PROFILE_MOAB_SEQ_ERROR=0x667d000a, */
/* IDL: 	PR_IMPLIED_RESTRICTIONS=0x667f1102, */
/* IDL: 	PR_IMPLIED_RESTRICTIONS_ERROR=0x667f000a, */
/* IDL: 	PR_RULE_ACTIONS=0x668000fe, */
/* IDL: 	PR_RULE_ACTIONS_ERROR=0x6680000a, */
/* IDL: 	PR_RULE_PROVIDER=0x6681001e, */
/* IDL: 	PR_RULE_PROVIDER_UNICODE=0x6681001f, */
/* IDL: 	PR_RULE_PROVIDER_ERROR=0x6681000a, */
/* IDL: 	PR_RULE_NAME=0x6682001e, */
/* IDL: 	PR_RULE_NAME_UNICODE=0x6682001f, */
/* IDL: 	PR_RULE_NAME_ERROR=0x6682000a, */
/* IDL: 	PR_RULE_LEVEL=0x66830003, */
/* IDL: 	PR_RULE_LEVEL_ERROR=0x6683000a, */
/* IDL: 	PR_RULE_PROVIDER_DATA=0x66840102, */
/* IDL: 	PR_RULE_PROVIDER_DATA_ERROR=0x6684000a, */
/* IDL: 	PR_LAST_FULL_BACKUP=0x66850040, */
/* IDL: 	PR_LAST_FULL_BACKUP_ERROR=0x6685000a, */
/* IDL: 	PR_PROFILE_ADDR_INFO=0x66870102, */
/* IDL: 	PR_PROFILE_ADDR_INFO_ERROR=0x6687000a, */
/* IDL: 	PR_PROFILE_OPTIONS_DATA=0x66890102, */
/* IDL: 	PR_PROFILE_OPTIONS_DATA_ERROR=0x6689000a, */
/* IDL: 	PR_EVENTS_ROOT_FOLDER_ENTRYID=0x668a0102, */
/* IDL: 	PR_EVENTS_ROOT_FOLDER_ENTRYID_ERROR=0x668a000a, */
/* IDL: 	PR_INBOUND_NEWSFEED_DN=0x668d001e, */
/* IDL: 	PR_INBOUND_NEWSFEED_DN_UNICODE=0x668d001f, */
/* IDL: 	PR_INBOUND_NEWSFEED_DN_ERROR=0x668d000a, */
/* IDL: 	PR_OUTBOUND_NEWSFEED_DN=0x668e001e, */
/* IDL: 	PR_OUTBOUND_NEWSFEED_DN_UNICODE=0x668e001f, */
/* IDL: 	PR_OUTBOUND_NEWSFEED_DN_ERROR=0x668e000a, */
/* IDL: 	PR_DELETED_ON=0x668f0040, */
/* IDL: 	PR_DELETED_ON_ERROR=0x668f000a, */
/* IDL: 	PR_REPLICATION_STYLE=0x66900003, */
/* IDL: 	PR_REPLICATION_STYLE_ERROR=0x6690000a, */
/* IDL: 	PR_REPLICATION_SCHEDULE=0x66910102, */
/* IDL: 	PR_REPLICATION_SCHEDULE_ERROR=0x6691000a, */
/* IDL: 	PR_REPLICATION_MESSAGE_PRIORITY=0x66920003, */
/* IDL: 	PR_REPLICATION_MESSAGE_PRIORITY_ERROR=0x6692000a, */
/* IDL: 	PR_OVERALL_MSG_AGE_LIMIT=0x66930003, */
/* IDL: 	PR_OVERALL_MSG_AGE_LIMIT_ERROR=0x6693000a, */
/* IDL: 	PR_REPLICATION_ALWAYS_INTERVAL=0x66940003, */
/* IDL: 	PR_REPLICATION_ALWAYS_INTERVAL_ERROR=0x6694000a, */
/* IDL: 	PR_REPLICATION_MSG_SIZE=0x66950003, */
/* IDL: 	PR_REPLICATION_MSG_SIZE_ERROR=0x6695000a, */
/* IDL: 	PR_IS_NEWSGROUP_ANCHOR=0x6696000b, */
/* IDL: 	PR_IS_NEWSGROUP_ANCHOR_ERROR=0x6696000a, */
/* IDL: 	PR_IS_NEWSGROUP=0x6697000b, */
/* IDL: 	PR_IS_NEWSGROUP_ERROR=0x6697000a, */
/* IDL: 	PR_REPLICA_LIST=0x66980102, */
/* IDL: 	PR_REPLICA_LIST_ERROR=0x6698000a, */
/* IDL: 	PR_OVERALL_AGE_LIMIT=0x66990003, */
/* IDL: 	PR_OVERALL_AGE_LIMIT_ERROR=0x6699000a, */
/* IDL: 	PR_INTERNET_CHARSET=0x669a001e, */
/* IDL: 	PR_INTERNET_CHARSET_UNICODE=0x669a001f, */
/* IDL: 	PR_INTERNET_CHARSET_ERROR=0x669a000a, */
/* IDL: 	PR_DELETED_MESSAGE_SIZE_EXTENDED=0x669b0014, */
/* IDL: 	PR_DELETED_MESSAGE_SIZE_EXTENDED_ERROR=0x669b000a, */
/* IDL: 	PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED=0x669c0014, */
/* IDL: 	PR_DELETED_NORMAL_MESSAGE_SIZE_EXTENDED_ERROR=0x669c000a, */
/* IDL: 	PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED=0x669d0014, */
/* IDL: 	PR_DELETED_ASSOC_MESSAGE_SIZE_EXTENDED_ERROR=0x669d000a, */
/* IDL: 	PR_SECURE_IN_SITE=0x669e000b, */
/* IDL: 	PR_SECURE_IN_SITE_ERROR=0x669e000a, */
/* IDL: 	PR_NT_USER_NAME=0x66a0001e, */
/* IDL: 	PR_NT_USER_NAME_UNICODE=0x66a0001f, */
/* IDL: 	PR_NT_USER_NAME_ERROR=0x66a0000a, */
/* IDL: 	PR_LOCALE_ID=0x66a10003, */
/* IDL: 	PR_LOCALE_ID_ERROR=0x66a1000a, */
/* IDL: 	PR_LAST_LOGON_TIME=0x66a20040, */
/* IDL: 	PR_LAST_LOGON_TIME_ERROR=0x66a2000a, */
/* IDL: 	PR_LAST_LOGOFF_TIME=0x66a30040, */
/* IDL: 	PR_LAST_LOGOFF_TIME_ERROR=0x66a3000a, */
/* IDL: 	PR_STORAGE_LIMIT_INFORMATION=0x66a40003, */
/* IDL: 	PR_STORAGE_LIMIT_INFORMATION_ERROR=0x66a4000a, */
/* IDL: 	PR_NEWSGROUP_COMPONENT=0x66a5001e, */
/* IDL: 	PR_NEWSGROUP_COMPONENT_UNICODE=0x66a5001f, */
/* IDL: 	PR_NEWSGROUP_COMPONENT_ERROR=0x66a5000a, */
/* IDL: 	PR_NEWSFEED_INFO=0x66a60102, */
/* IDL: 	PR_NEWSFEED_INFO_ERROR=0x66a6000a, */
/* IDL: 	PR_INTERNET_NEWSGROUP_NAME=0x66a7001e, */
/* IDL: 	PR_INTERNET_NEWSGROUP_NAME_UNICODE=0x66a7001f, */
/* IDL: 	PR_INTERNET_NEWSGROUP_NAME_ERROR=0x66a7000a, */
/* IDL: 	PR_FOLDER_FLAGS=0x66a80003, */
/* IDL: 	PR_FOLDER_FLAGS_ERROR=0x66a8000a, */
/* IDL: 	PR_LAST_ACCESS_TIME=0x66a90040, */
/* IDL: 	PR_LAST_ACCESS_TIME_ERROR=0x66a9000a, */
/* IDL: 	PR_RESTRICTION_COUNT=0x66aa0003, */
/* IDL: 	PR_RESTRICTION_COUNT_ERROR=0x66aa000a, */
/* IDL: 	PR_CATEG_COUNT=0x66ab0003, */
/* IDL: 	PR_CATEG_COUNT_ERROR=0x66ab000a, */
/* IDL: 	PR_CACHED_COLUMN_COUNT=0x66ac0003, */
/* IDL: 	PR_CACHED_COLUMN_COUNT_ERROR=0x66ac000a, */
/* IDL: 	PR_NORMAL_MSG_W_ATTACH_COUNT=0x66ad0003, */
/* IDL: 	PR_NORMAL_MSG_W_ATTACH_COUNT_ERROR=0x66ad000a, */
/* IDL: 	PR_ASSOC_MSG_W_ATTACH_COUNT=0x66ae0003, */
/* IDL: 	PR_ASSOC_MSG_W_ATTACH_COUNT_ERROR=0x66ae000a, */
/* IDL: 	PR_RECIPIENT_ON_NORMAL_MSG_COUNT=0x66af0003, */
/* IDL: 	PR_RECIPIENT_ON_NORMAL_MSG_COUNT_ERROR=0x66af000a, */
/* IDL: 	PR_RECIPIENT_ON_ASSOC_MSG_COUNT=0x66b00003, */
/* IDL: 	PR_RECIPIENT_ON_ASSOC_MSG_COUNT_ERROR=0x66b0000a, */
/* IDL: 	PR_ATTACH_ON_NORMAL_MSG_COUNT=0x66b10003, */
/* IDL: 	PR_ATTACH_ON_NORMAL_MSG_COUNT_ERROR=0x66b1000a, */
/* IDL: 	PR_ATTACH_ON_ASSOC_MSG_COUNT=0x66b20003, */
/* IDL: 	PR_ATTACH_ON_ASSOC_MSG_COUNT_ERROR=0x66b2000a, */
/* IDL: 	PR_NORMAL_MESSAGE_SIZE=0x66b30003, */
/* IDL: 	PR_NORMAL_MESSAGE_SIZE_ERROR=0x66b3000a, */
/* IDL: 	PR_NORMAL_MESSAGE_SIZE_EXTENDED=0x66b30014, */
/* IDL: 	PR_ASSOC_MESSAGE_SIZE=0x66b40003, */
/* IDL: 	PR_ASSOC_MESSAGE_SIZE_ERROR=0x66b4000a, */
/* IDL: 	PR_ASSOC_MESSAGE_SIZE_EXTENDED=0x66b40014, */
/* IDL: 	PR_FOLDER_PATHNAME=0x66b5001e, */
/* IDL: 	PR_FOLDER_PATHNAME_UNICODE=0x66b5001f, */
/* IDL: 	PR_FOLDER_PATHNAME_ERROR=0x66b5000a, */
/* IDL: 	PR_OWNER_COUNT=0x66b60003, */
/* IDL: 	PR_OWNER_COUNT_ERROR=0x66b6000a, */
/* IDL: 	PR_CONTACT_COUNT=0x66b70003, */
/* IDL: 	PR_CONTACT_COUNT_ERROR=0x66b7000a, */
/* IDL: 	PR_CODE_PAGE_ID=0x66c30003, */
/* IDL: 	PR_CODE_PAGE_ID_ERROR=0x66c3000a, */
/* IDL: 	PR_RETENTION_AGE_LIMIT=0x66c40003, */
/* IDL: 	PR_RETENTION_AGE_LIMIT_ERROR=0x66c4000a, */
/* IDL: 	PR_DISABLE_PERUSER_READ=0x66c5000b, */
/* IDL: 	PR_DISABLE_PERUSER_READ_ERROR=0x66c5000a, */
/* IDL: 	PR_INTERNET_PARSE_STATE=0x66c60102, */
/* IDL: 	PR_INTERNET_PARSE_STATE_ERROR=0x66c6000a, */
/* IDL: 	PR_INTERNET_MESSAGE_INFO=0x66c70102, */
/* IDL: 	PR_INTERNET_MESSAGE_INFO_ERROR=0x66c7000a, */
/* IDL: 	PR_PST_PATH=0x6700001e, */
/* IDL: 	PR_PST_PATH_UNICODE=0x6700001f, */
/* IDL: 	PR_PST_PATH_ERROR=0x6700000a, */
/* IDL: 	PR_PST_REMEMBER_PW=0x6701000b, */
/* IDL: 	PR_PST_REMEMBER_PW_ERROR=0x6701000a, */
/* IDL: 	PR_OST_ENCRYPTION=0x67020003, */
/* IDL: 	PR_OST_ENCRYPTION_ERROR=0x6702000a, */
/* IDL: 	PR_PST_PW_SZ_OLD=0x6703001e, */
/* IDL: 	PR_PST_PW_SZ_OLD_UNICODE=0x6703001f, */
/* IDL: 	PR_PST_PW_SZ_OLD_ERROR=0x6703000a, */
/* IDL: 	PR_PST_PW_SZ_NEW=0x6704001e, */
/* IDL: 	PR_PST_PW_SZ_NEW_UNICODE=0x6704001f, */
/* IDL: 	PR_PST_PW_SZ_NEW_ERROR=0x6704000a, */
/* IDL: 	PR_SORT_LOCALE_ID=0x67050003, */
/* IDL: 	PR_SORT_LOCALE_ID_ERROR=0x6705000a, */
/* IDL: 	PR_URL_NAME=0x6707001e, */
/* IDL: 	PR_URL_NAME_UNICODE=0x6707001f, */
/* IDL: 	PR_URL_NAME_ERROR=0x6707000a, */
/* IDL: 	PR_LOCAL_COMMIT_TIME=0x67090040, */
/* IDL: 	PR_LOCAL_COMMIT_TIME_ERROR=0x6709000a, */
/* IDL: 	PR_LOCAL_COMMIT_TIME_MAX=0x670a0040, */
/* IDL: 	PR_LOCAL_COMMIT_TIME_MAX_ERROR=0x670a000a, */
/* IDL: 	PR_DELETED_COUNT_TOTAL=0x670b0003, */
/* IDL: 	PR_DELETED_COUNT_TOTAL_ERROR=0x670b000a, */
/* IDL: 	PR_AUTO_RESET=0x670c0048, */
/* IDL: 	PR_AUTO_RESET_ERROR=0x670c000a, */
/* IDL: 	PR_URL_COMP_NAME_HASH=0x67100003, */
/* IDL: 	PR_URL_COMP_NAME_HASH_ERROR=0x6710000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_2=0x67110003, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_2_ERROR=0x6711000a, */
/* IDL: 	PR_RANK=0x67120003, */
/* IDL: 	PR_RANK_ERROR=0x6712000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_4=0x6713000b, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_4_ERROR=0x6713000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_5=0x6714000b, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_5_ERROR=0x6714000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_6=0x6715000b, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_6_ERROR=0x6715000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_7=0x67160102, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_7_ERROR=0x6716000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_8=0x67170102, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_8_ERROR=0x6717000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_9=0x67180102, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_9_ERROR=0x6718000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_10=0x6719001e, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_10_UNICODE=0x6719001f, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_10_ERROR=0x6719000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_11=0x671a001e, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_11_UNICODE=0x671a001f, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_11_ERROR=0x671a000a, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_12=0x671b001e, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_12_UNICODE=0x671b001f, */
/* IDL: 	PR_MSG_FOLDER_TEMPLATE_RES_12_ERROR=0x671b000a, */
/* IDL: 	PR_PF_PLATINUM_HOME_MDB=0x671e000b, */
/* IDL: 	PR_PF_PLATINUM_HOME_MDB_ERROR=0x671e000a, */
/* IDL: 	PR_PF_PROXY_REQUIRED=0x671f000b, */
/* IDL: 	PR_PF_PROXY_REQUIRED_ERROR=0x671f000a, */
/* IDL: 	PR_INTERNET_FREE_DOC_INFO=0x67200102, */
/* IDL: 	PR_INTERNET_FREE_DOC_INFO_ERROR=0x6720000a, */
/* IDL: 	PR_PF_OVER_HARD_QUOTA_LIMIT=0x67210003, */
/* IDL: 	PR_PF_OVER_HARD_QUOTA_LIMIT_ERROR=0x6721000a, */
/* IDL: 	PR_PF_MSG_SIZE_LIMIT=0x67220003, */
/* IDL: 	PR_PF_MSG_SIZE_LIMIT_ERROR=0x6722000a, */
/* IDL: 	PR_CONNECTION_MODULUS=0x67430003, */
/* IDL: 	PR_CONNECTION_MODULUS_ERROR=0x6743000a, */
/* IDL: 	PR_DELIVER_TO_DN=0x6744001e, */
/* IDL: 	PR_DELIVER_TO_DN_UNICODE=0x6744001f, */
/* IDL: 	PR_DELIVER_TO_DN_ERROR=0x6744000a, */
/* IDL: 	PR_MIME_SIZE=0x67460003, */
/* IDL: 	PR_MIME_SIZE_ERROR=0x6746000a, */
/* IDL: 	PR_FILE_SIZE_EXTENDED=0x67470014, */
/* IDL: 	PR_FILE_SIZE_EXTENDED_ERROR=0x6747000a, */
/* IDL: 	PR_FID=0x67480014, */
/* IDL: 	PR_FID_ERROR=0x6748000a, */
/* IDL: 	PR_PARENT_FID=0x67490014, */
/* IDL: 	PR_PARENT_FID_ERROR=0x6749000a, */
/* IDL: 	PR_MID=0x674a0014, */
/* IDL: 	PR_MID_ERROR=0x674a000a, */
/* IDL: 	PR_CATEG_ID=0x674b0014, */
/* IDL: 	PR_CATEG_ID_ERROR=0x674b000a, */
/* IDL: 	PR_PARENT_CATEG_ID=0x674c0014, */
/* IDL: 	PR_PARENT_CATEG_ID_ERROR=0x674c000a, */
/* IDL: 	PR_INST_ID=0x674d0014, */
/* IDL: 	PR_INST_ID_ERROR=0x674d000a, */
/* IDL: 	PR_INSTANCE_NUM=0x674e0003, */
/* IDL: 	PR_INSTANCE_NUM_ERROR=0x674e000a, */
/* IDL: 	PR_ADDRBOOK_MID=0x674f0014, */
/* IDL: 	PR_ADDRBOOK_MID_ERROR=0x674f000a, */
/* IDL: 	PR_ICS_NOTIF=0x67500003, */
/* IDL: 	PR_ICS_NOTIF_ERROR=0x6750000a, */
/* IDL: 	PR_ARTICLE_NUM_NEXT=0x67510003, */
/* IDL: 	PR_ARTICLE_NUM_NEXT_ERROR=0x6751000a, */
/* IDL: 	PR_IMAP_LAST_ARTICLE_ID=0x67520003, */
/* IDL: 	PR_IMAP_LAST_ARTICLE_ID_ERROR=0x6752000a, */
/* IDL: 	PR_NOT_822_RENDERABLE=0x6753000b, */
/* IDL: 	PR_NOT_822_RENDERABLE_ERROR=0x6753000a, */
/* IDL: 	PR_LTID=0x67580102, */
/* IDL: 	PR_LTID_ERROR=0x6758000a, */
/* IDL: 	PR_CN_EXPORT=0x67590102, */
/* IDL: 	PR_CN_EXPORT_ERROR=0x6759000a, */
/* IDL: 	PR_PCL_EXPORT=0x675a0102, */
/* IDL: 	PR_PCL_EXPORT_ERROR=0x675a000a, */
/* IDL: 	PR_CN_MV_EXPORT=0x675b1102, */
/* IDL: 	PR_CN_MV_EXPORT_ERROR=0x675b000a, */
/* IDL: 	PR_PF_QUOTA_STYLE=0x67790003, */
/* IDL: 	PR_PF_QUOTA_STYLE_ERROR=0x6779000a, */
/* IDL: 	PR_PF_STORAGE_QUOTA=0x677b0003, */
/* IDL: 	PR_PF_STORAGE_QUOTA_ERROR=0x677b000a, */
/* IDL: 	PR_SEARCH_FLAGS=0x67830003, */
/* IDL: 	PR_SEARCH_FLAGS_ERROR=0x6783000a, */
/* IDL: 	PR_ASSOCIATED=0x67aa000b, */
/* IDL: 	PR_ASSOCIATED_ERROR=0x67aa000a, */
/* IDL: 	PR_PROFILE_SECURE_MAILBOX=0x67f00102, */
/* IDL: 	PR_PROFILE_SECURE_MAILBOX_ERROR=0x67f0000a, */
/* IDL: 	PR_MAILBEAT_BOUNCE_SERVER=0x6800001e, */
/* IDL: 	PR_MAILBEAT_BOUNCE_SERVER_UNICODE=0x6800001f, */
/* IDL: 	PR_MAILBEAT_BOUNCE_SERVER_ERROR=0x6800000a, */
/* IDL: 	PR_MAILBEAT_REQUEST_SENT=0x68010040, */
/* IDL: 	PR_MAILBEAT_REQUEST_SENT_ERROR=0x6801000a, */
/* IDL: 	PR_USENET_SITE_NAME=0x6802001e, */
/* IDL: 	PR_USENET_SITE_NAME_UNICODE=0x6802001f, */
/* IDL: 	PR_USENET_SITE_NAME_ERROR=0x6802000a, */
/* IDL: 	PR_MAILBEAT_REQUEST_RECEIVED=0x68030040, */
/* IDL: 	PR_MAILBEAT_REQUEST_RECEIVED_ERROR=0x6803000a, */
/* IDL: 	PR_MAILBEAT_REQUEST_PROCESSED=0x68040040, */
/* IDL: 	PR_MAILBEAT_REQUEST_PROCESSED_ERROR=0x6804000a, */
/* IDL: 	PR_MAILBEAT_REPLY_SENT=0x68060040, */
/* IDL: 	PR_MAILBEAT_REPLY_SENT_ERROR=0x6806000a, */
/* IDL: 	PR_MAILBEAT_REPLY_SUBMIT=0x68070040, */
/* IDL: 	PR_MAILBEAT_REPLY_SUBMIT_ERROR=0x6807000a, */
/* IDL: 	PR_MAILBEAT_REPLY_RECEIVED=0x68080040, */
/* IDL: 	PR_MAILBEAT_REPLY_RECEIVED_ERROR=0x6808000a, */
/* IDL: 	PR_MAILBEAT_REPLY_PROCESSED=0x68090040, */
/* IDL: 	PR_MAILBEAT_REPLY_PROCESSED_ERROR=0x6809000a, */
/* IDL: 	PR_DELEGATES_DISPLAY_NAMES=0x6844101e, */
/* IDL: 	PR_DELEGATES_DISPLAY_NAMES_UNICODE=0x6844101f, */
/* IDL: 	PR_DELEGATES_DISPLAY_NAMES_ERROR=0x6844000a, */
/* IDL: 	PR_DELEGATES_ENTRYIDS=0x68451102, */
/* IDL: 	PR_DELEGATES_ENTRYIDS_ERROR=0x6845000a, */
/* IDL: 	PR_FREEBUSY_START_RANGE=0x68470003, */
/* IDL: 	PR_FREEBUSY_START_RANGE_ERROR=0x6847000a, */
/* IDL: 	PR_FREEBUSY_END_RANGE=0x68480003, */
/* IDL: 	PR_FREEBUSY_END_RANGE_ERROR=0x6848000a, */
/* IDL: 	PR_FREEBUSY_EMAIL_ADDRESS=0x6849001e, */
/* IDL: 	PR_FREEBUSY_EMAIL_ADDRESS_UNICODE=0x6849001f, */
/* IDL: 	PR_FREEBUSY_EMAIL_ADDRESS_ERROR=0x6849000a, */
/* IDL: 	PR_FREEBUSY_ALL_MONTHS=0x684f1003, */
/* IDL: 	PR_FREEBUSY_ALL_MONTHS_ERROR=0x684f000a, */
/* IDL: 	PR_FREEBUSY_ALL_EVENTS=0x68501102, */
/* IDL: 	PR_FREEBUSY_ALL_EVENTS_ERROR=0x6850000a, */
/* IDL: 	PR_FREEBUSY_TENTATIVE_MONTHS=0x68511003, */
/* IDL: 	PR_FREEBUSY_TENTATIVE_MONTHS_ERROR=0x6851000a, */
/* IDL: 	PR_FREEBUSY_TENTATIVE_EVENTS=0x68521102, */
/* IDL: 	PR_FREEBUSY_TENTATIVE_EVENTS_ERROR=0x6852000a, */
/* IDL: 	PR_FREEBUSY_BUSY_MONTHS=0x68531003, */
/* IDL: 	PR_FREEBUSY_BUSY_MONTHS_ERROR=0x6853000a, */
/* IDL: 	PR_FREEBUSY_BUSY_EVENTS=0x68541102, */
/* IDL: 	PR_FREEBUSY_BUSY_EVENTS_ERROR=0x6854000a, */
/* IDL: 	PR_FREEBUSY_OOF_MONTHS=0x68551003, */
/* IDL: 	PR_FREEBUSY_OOF_MONTHS_ERROR=0x6855000a, */
/* IDL: 	PR_FREEBUSY_OOF_EVENTS=0x68561102, */
/* IDL: 	PR_FREEBUSY_OOF_EVENTS_ERROR=0x6856000a, */
/* IDL: 	PR_FREEBUSY_LAST_MODIFIED=0x68680040, */
/* IDL: 	PR_FREEBUSY_LAST_MODIFIED_ERROR=0x6868000a, */
/* IDL: 	PR_FREEBUSY_NUM_MONTHS=0x68690003, */
/* IDL: 	PR_FREEBUSY_NUM_MONTHS_ERROR=0x6869000a, */
/* IDL: 	PR_DELEGATES_SEE_PRIVATE=0x686b1003, */
/* IDL: 	PR_DELEGATES_SEE_PRIVATE_ERROR=0x686b000a, */
/* IDL: 	PR_PERSONAL_FREEBUSY=0x686c0102, */
/* IDL: 	PR_PERSONAL_FREEBUSY_ERROR=0x686c000a, */
/* IDL: 	PR_PROCESS_MEETING_REQUESTS=0x686d000b, */
/* IDL: 	PR_PROCESS_MEETING_REQUESTS_ERROR=0x686d000a, */
/* IDL: 	PR_DECLINE_RECURRING_MEETING_REQUESTS=0x686e000b, */
/* IDL: 	PR_DECLINE_RECURRING_MEETING_REQUESTS_ERROR=0x686e000a, */
/* IDL: 	PR_DECLINE_CONFLICTING_MEETING_REQUESTS=0x686f000b, */
/* IDL: 	PR_DECLINE_CONFLICTING_MEETING_REQUESTS_ERROR=0x686f000a, */
/* IDL: 	PR_VD_BINARY=0x70010102, */
/* IDL: 	PR_VD_BINARY_ERROR=0x7001000a, */
/* IDL: 	PR_VD_STRINGS=0x7002001e, */
/* IDL: 	PR_VD_STRINGS_UNICODE=0x7002001f, */
/* IDL: 	PR_VD_STRINGS_ERROR=0x7002000a, */
/* IDL: 	PR_VD_FLAGS=0x70030003, */
/* IDL: 	PR_VD_FLAGS_ERROR=0x7003000a, */
/* IDL: 	PR_VD_LINK_TO=0x70040102, */
/* IDL: 	PR_VD_LINK_TO_ERROR=0x7004000a, */
/* IDL: 	PR_VD_VIEW_FOLDER=0x70050102, */
/* IDL: 	PR_VD_VIEW_FOLDER_ERROR=0x7005000a, */
/* IDL: 	PR_VD_NAME=0x7006001e, */
/* IDL: 	PR_VD_NAME_UNICODE=0x7006001f, */
/* IDL: 	PR_VD_NAME_ERROR=0x7006000a, */
/* IDL: 	PR_VD_VERSION=0x70070003, */
/* IDL: 	PR_VD_VERSION_ERROR=0x7007000a, */
/* IDL: 	PR_FAV_DISPLAY_NAME_A=0x7c00001e, */
/* IDL: 	PR_FAV_DISPLAY_NAME_A_UNICODE=0x7c00001f, */
/* IDL: 	PR_FAV_DISPLAY_NAME_A_ERROR=0x7c00000a, */
/* IDL: 	PR_FAV_PUBLIC_SOURCE_KEY=0x7c020102, */
/* IDL: 	PR_FAV_PUBLIC_SOURCE_KEY_ERROR=0x7c02000a, */
/* IDL: 	PR_OST_OSTID=0x7c040102, */
/* IDL: 	PR_OST_OSTID_ERROR=0x7c04000a, */
/* IDL: 	PR_STORE_SLOWLINK=0x7c0a000b, */
/* IDL: 	PR_STORE_SLOWLINK_ERROR=0x7c0a000a, */
/* IDL: 	PR_FAV_AUTOSUBFOLDERS=0x7d010003, */
/* IDL: 	PR_FAV_AUTOSUBFOLDERS_ERROR=0x7d01000a, */
/* IDL: 	PR_FAV_PARENT_SOURCE_KEY=0x7d020102, */
/* IDL: 	PR_FAV_PARENT_SOURCE_KEY_ERROR=0x7d02000a, */
/* IDL: 	PR_FAV_LEVEL_MASK=0x7d030003, */
/* IDL: 	PR_FAV_LEVEL_MASK_ERROR=0x7d03000a, */
/* IDL: 	PR_FAV_INHERIT_AUTO=0x7d070003, */
/* IDL: 	PR_FAV_INHERIT_AUTO_ERROR=0x7d07000a, */
/* IDL: 	PR_FAV_DEL_SUBS=0x7d080102, */
/* IDL: 	PR_FAV_DEL_SUBS_ERROR=0x7d08000a, */
/* IDL: 	PR_ATTACHMENT_LINKID=0x7ffa0003, */
/* IDL: 	PR_ATTACHMENT_LINKID_ERROR=0x7ffa000a, */
/* IDL: 	PR_EXCEPTION_STARTTIME=0x7ffb0040, */
/* IDL: 	PR_EXCEPTION_STARTTIME_ERROR=0x7ffb000a, */
/* IDL: 	PR_EXCEPTION_ENDTIME=0x7ffc0040, */
/* IDL: 	PR_EXCEPTION_ENDTIME_ERROR=0x7ffc000a, */
/* IDL: 	PR_ATTACHMENT_FLAGS=0x7ffd0003, */
/* IDL: 	PR_ATTACHMENT_FLAGS_ERROR=0x7ffd000a, */
/* IDL: 	PR_ATTACHMENT_HIDDEN=0x7ffe000b, */
/* IDL: 	PR_ATTACHMENT_HIDDEN_ERROR=0x7ffe000a, */
/* IDL: 	PR_EMS_AB_DISPLAY_NAME_OVERRIDE=0x8001000b, */
/* IDL: 	PR_EMS_AB_DISPLAY_NAME_OVERRIDE_ERROR=0x8001000a, */
/* IDL: 	PR_EMS_AB_CA_CERTIFICATE=0x80031102, */
/* IDL: 	PR_EMS_AB_CA_CERTIFICATE_ERROR=0x8003000a, */
/* IDL: 	PR_EMS_AB_FOLDER_PATHNAME=0x8004001e, */
/* IDL: 	PR_EMS_AB_FOLDER_PATHNAME_UNICODE=0x8004001f, */
/* IDL: 	PR_EMS_AB_FOLDER_PATHNAME_ERROR=0x8004000a, */
/* IDL: 	PR_EMS_AB_MANAGER=0x8005000d, */
/* IDL: 	PR_EMS_AB_MANAGER_ERROR=0x8005000a, */
/* IDL: 	PR_EMS_AB_MANAGER_T=0x8005001e, */
/* IDL: 	PR_EMS_AB_MANAGER_T_UNICODE=0x8005001f, */
/* IDL: 	PR_EMS_AB_HOME_MDB_O=0x8006000d, */
/* IDL: 	PR_EMS_AB_HOME_MDB_O_ERROR=0x8006000a, */
/* IDL: 	PR_EMS_AB_HOME_MDB=0x8006001e, */
/* IDL: 	PR_EMS_AB_HOME_MDB_UNICODE=0x8006001f, */
/* IDL: 	PR_EMS_AB_HOME_MTA_O=0x8007000d, */
/* IDL: 	PR_EMS_AB_HOME_MTA_O_ERROR=0x8007000a, */
/* IDL: 	PR_EMS_AB_HOME_MTA=0x8007001e, */
/* IDL: 	PR_EMS_AB_HOME_MTA_UNICODE=0x8007001f, */
/* IDL: 	PR_EMS_AB_IS_MEMBER_OF_DL=0x8008000d, */
/* IDL: 	PR_EMS_AB_IS_MEMBER_OF_DL_ERROR=0x8008000a, */
/* IDL: 	PR_EMS_AB_IS_MEMBER_OF_DL_T=0x8008001e, */
/* IDL: 	PR_EMS_AB_IS_MEMBER_OF_DL_T_UNICODE=0x8008001f, */
/* IDL: 	PR_EMS_AB_MEMBER=0x8009000d, */
/* IDL: 	PR_EMS_AB_MEMBER_ERROR=0x8009000a, */
/* IDL: 	PR_EMS_AB_MEMBER_T=0x8009001e, */
/* IDL: 	PR_EMS_AB_MEMBER_T_UNICODE=0x8009001f, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_MESSAGE=0x800a001e, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_MESSAGE_UNICODE=0x800a001f, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_MESSAGE_ERROR=0x800a000a, */
/* IDL: 	PR_EMS_AB_AUTOREPLY=0x800b000b, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_ERROR=0x800b000a, */
/* IDL: 	PR_EMS_AB_OWNER_O=0x800c000d, */
/* IDL: 	PR_EMS_AB_OWNER_O_ERROR=0x800c000a, */
/* IDL: 	PR_EMS_AB_OWNER=0x800c001e, */
/* IDL: 	PR_EMS_AB_OWNER_UNICODE=0x800c001f, */
/* IDL: 	PR_EMS_AB_KM_SERVER_O=0x800d000d, */
/* IDL: 	PR_EMS_AB_KM_SERVER_O_ERROR=0x800d000a, */
/* IDL: 	PR_EMS_AB_KM_SERVER=0x800d001e, */
/* IDL: 	PR_EMS_AB_KM_SERVER_UNICODE=0x800d001f, */
/* IDL: 	PR_EMS_AB_REPORTS=0x800e000d, */
/* IDL: 	PR_EMS_AB_REPORTS_ERROR=0x800e000a, */
/* IDL: 	PR_EMS_AB_PROXY_ADDRESSES=0x800f101e, */
/* IDL: 	PR_EMS_AB_PROXY_ADDRESSES_UNICODE=0x800f101f, */
/* IDL: 	PR_EMS_AB_PROXY_ADDRESSES_ERROR=0x800f000a, */
/* IDL: 	PR_EMS_AB_HELP_DATA32=0x80100102, */
/* IDL: 	PR_EMS_AB_HELP_DATA32_ERROR=0x8010000a, */
/* IDL: 	PR_EMS_AB_TARGET_ADDRESS=0x8011001e, */
/* IDL: 	PR_EMS_AB_TARGET_ADDRESS_UNICODE=0x8011001f, */
/* IDL: 	PR_EMS_AB_TARGET_ADDRESS_ERROR=0x8011000a, */
/* IDL: 	PR_EMS_AB_TELEPHONE_NUMBER=0x8012101e, */
/* IDL: 	PR_EMS_AB_TELEPHONE_NUMBER_UNICODE=0x8012101f, */
/* IDL: 	PR_EMS_AB_TELEPHONE_NUMBER_ERROR=0x8012000a, */
/* IDL: 	PR_EMS_AB_NT_SECURITY_DESCRIPTOR=0x80130102, */
/* IDL: 	PR_EMS_AB_NT_SECURITY_DESCRIPTOR_ERROR=0x8013000a, */
/* IDL: 	PR_EMS_AB_HOME_MDB_BL_O=0x8014000d, */
/* IDL: 	PR_EMS_AB_HOME_MDB_BL_O_ERROR=0x8014000a, */
/* IDL: 	PR_EMS_AB_HOME_MDB_BL=0x8014101e, */
/* IDL: 	PR_EMS_AB_HOME_MDB_BL_UNICODE=0x8014101f, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES=0x8015000d, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_ERROR=0x8015000a, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_T=0x8015001e, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_T_UNICODE=0x8015001f, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST=0x80160102, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_ERROR=0x8016000a, */
/* IDL: 	PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE=0x80170102, */
/* IDL: 	PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_ERROR=0x8017000a, */
/* IDL: 	PR_EMS_AB_ADDRESS_SYNTAX=0x80180102, */
/* IDL: 	PR_EMS_AB_ADDRESS_SYNTAX_ERROR=0x8018000a, */
/* IDL: 	PR_EMS_AB_BUSINESS_ROLES=0x80230102, */
/* IDL: 	PR_EMS_AB_BUSINESS_ROLES_ERROR=0x8023000a, */
/* IDL: 	PR_EMS_AB_OWNER_BL_O=0x8024000d, */
/* IDL: 	PR_EMS_AB_OWNER_BL_O_ERROR=0x8024000a, */
/* IDL: 	PR_EMS_AB_OWNER_BL=0x8024101e, */
/* IDL: 	PR_EMS_AB_OWNER_BL_UNICODE=0x8024101f, */
/* IDL: 	PR_EMS_AB_CROSS_CERTIFICATE_PAIR=0x80251102, */
/* IDL: 	PR_EMS_AB_CROSS_CERTIFICATE_PAIR_ERROR=0x8025000a, */
/* IDL: 	PR_EMS_AB_AUTHORITY_REVOCATION_LIST=0x80261102, */
/* IDL: 	PR_EMS_AB_AUTHORITY_REVOCATION_LIST_ERROR=0x8026000a, */
/* IDL: 	PR_EMS_AB_ASSOC_NT_ACCOUNT=0x80270102, */
/* IDL: 	PR_EMS_AB_ASSOC_NT_ACCOUNT_ERROR=0x8027000a, */
/* IDL: 	PR_EMS_AB_EXPIRATION_TIME=0x80280040, */
/* IDL: 	PR_EMS_AB_EXPIRATION_TIME_ERROR=0x8028000a, */
/* IDL: 	PR_EMS_AB_USN_CHANGED=0x80290003, */
/* IDL: 	PR_EMS_AB_USN_CHANGED_ERROR=0x8029000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_1=0x802d001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_1_UNICODE=0x802d001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_1_ERROR=0x802d000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_2=0x802e001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_2_UNICODE=0x802e001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_2_ERROR=0x802e000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_3=0x802f001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_3_UNICODE=0x802f001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_3_ERROR=0x802f000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_4=0x8030001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_4_UNICODE=0x8030001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_4_ERROR=0x8030000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_5=0x8031001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_5_UNICODE=0x8031001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_5_ERROR=0x8031000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_6=0x8032001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_6_UNICODE=0x8032001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_6_ERROR=0x8032000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_7=0x8033001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_7_UNICODE=0x8033001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_7_ERROR=0x8033000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_8=0x8034001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_8_UNICODE=0x8034001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_8_ERROR=0x8034000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_9=0x8035001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_9_UNICODE=0x8035001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_9_ERROR=0x8035000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_10=0x8036001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_10_UNICODE=0x8036001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_10_ERROR=0x8036000a, */
/* IDL: 	PR_EMS_AB_SECURITY_PROTOCOL=0x80371102, */
/* IDL: 	PR_EMS_AB_SECURITY_PROTOCOL_ERROR=0x8037000a, */
/* IDL: 	PR_EMS_AB_PF_CONTACTS_O=0x8038000d, */
/* IDL: 	PR_EMS_AB_PF_CONTACTS_O_ERROR=0x8038000a, */
/* IDL: 	PR_EMS_AB_PF_CONTACTS=0x8038101e, */
/* IDL: 	PR_EMS_AB_PF_CONTACTS_UNICODE=0x8038101f, */
/* IDL: 	PR_EMS_AB_HELP_DATA16=0x803a0102, */
/* IDL: 	PR_EMS_AB_HELP_DATA16_ERROR=0x803a000a, */
/* IDL: 	PR_EMS_AB_HELP_FILE_NAME=0x803b001e, */
/* IDL: 	PR_EMS_AB_HELP_FILE_NAME_UNICODE=0x803b001f, */
/* IDL: 	PR_EMS_AB_HELP_FILE_NAME_ERROR=0x803b000a, */
/* IDL: 	PR_EMS_AB_OBJ_DIST_NAME_O=0x803c000d, */
/* IDL: 	PR_EMS_AB_OBJ_DIST_NAME_O_ERROR=0x803c000a, */
/* IDL: 	PR_EMS_AB_OBJ_DIST_NAME=0x803c001e, */
/* IDL: 	PR_EMS_AB_OBJ_DIST_NAME_UNICODE=0x803c001f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER=0x803d001e, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_UNICODE=0x803d001f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_OTHER_ERROR=0x803d000a, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_SUBJECT=0x803e001e, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_SUBJECT_UNICODE=0x803e001f, */
/* IDL: 	PR_EMS_AB_AUTOREPLY_SUBJECT_ERROR=0x803e000a, */
/* IDL: 	PR_EMS_AB_HOME_PUBLIC_SERVER_O=0x803f000d, */
/* IDL: 	PR_EMS_AB_HOME_PUBLIC_SERVER_O_ERROR=0x803f000a, */
/* IDL: 	PR_EMS_AB_HOME_PUBLIC_SERVER=0x803f001e, */
/* IDL: 	PR_EMS_AB_HOME_PUBLIC_SERVER_UNICODE=0x803f001f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_NA=0x8040101e, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_NA_UNICODE=0x8040101f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_NA_ERROR=0x8040000a, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER=0x8041101e, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_UNICODE=0x8041101f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_LIST_OTHER_ERROR=0x8041000a, */
/* IDL: 	PR_EMS_AB_IMPORTED_FROM=0x8042001e, */
/* IDL: 	PR_EMS_AB_IMPORTED_FROM_UNICODE=0x8042001f, */
/* IDL: 	PR_EMS_AB_IMPORTED_FROM_ERROR=0x8042000a, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA=0x8043001e, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_UNICODE=0x8043001f, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ALG_SELECTED_NA_ERROR=0x8043000a, */
/* IDL: 	PR_EMS_AB_ACCESS_CATEGORY=0x80440003, */
/* IDL: 	PR_EMS_AB_ACCESS_CATEGORY_ERROR=0x8044000a, */
/* IDL: 	PR_EMS_AB_ACTIVATION_SCHEDULE=0x80450102, */
/* IDL: 	PR_EMS_AB_ACTIVATION_SCHEDULE_ERROR=0x8045000a, */
/* IDL: 	PR_EMS_AB_ACTIVATION_STYLE=0x80460003, */
/* IDL: 	PR_EMS_AB_ACTIVATION_STYLE_ERROR=0x8046000a, */
/* IDL: 	PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS=0x80470102, */
/* IDL: 	PR_EMS_AB_ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS_ERROR=0x8047000a, */
/* IDL: 	PR_EMS_AB_ADDRESS_TYPE=0x8048001e, */
/* IDL: 	PR_EMS_AB_ADDRESS_TYPE_UNICODE=0x8048001f, */
/* IDL: 	PR_EMS_AB_ADDRESS_TYPE_ERROR=0x8048000a, */
/* IDL: 	PR_EMS_AB_ADMD=0x8049001e, */
/* IDL: 	PR_EMS_AB_ADMD_UNICODE=0x8049001f, */
/* IDL: 	PR_EMS_AB_ADMD_ERROR=0x8049000a, */
/* IDL: 	PR_EMS_AB_ADMIN_DESCRIPTION=0x804a001e, */
/* IDL: 	PR_EMS_AB_ADMIN_DESCRIPTION_UNICODE=0x804a001f, */
/* IDL: 	PR_EMS_AB_ADMIN_DESCRIPTION_ERROR=0x804a000a, */
/* IDL: 	PR_EMS_AB_ADMIN_DISPLAY_NAME=0x804b001e, */
/* IDL: 	PR_EMS_AB_ADMIN_DISPLAY_NAME_UNICODE=0x804b001f, */
/* IDL: 	PR_EMS_AB_ADMIN_DISPLAY_NAME_ERROR=0x804b000a, */
/* IDL: 	PR_EMS_AB_ADMIN_EXTENSION_DLL=0x804c001e, */
/* IDL: 	PR_EMS_AB_ADMIN_EXTENSION_DLL_UNICODE=0x804c001f, */
/* IDL: 	PR_EMS_AB_ADMIN_EXTENSION_DLL_ERROR=0x804c000a, */
/* IDL: 	PR_EMS_AB_ALIASED_OBJECT_NAME_O=0x804d000d, */
/* IDL: 	PR_EMS_AB_ALIASED_OBJECT_NAME_O_ERROR=0x804d000a, */
/* IDL: 	PR_EMS_AB_ALIASED_OBJECT_NAME=0x804d001e, */
/* IDL: 	PR_EMS_AB_ALIASED_OBJECT_NAME_UNICODE=0x804d001f, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_O=0x804e000d, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_O_ERROR=0x804e000a, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT=0x804e001e, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_UNICODE=0x804e001f, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_BL_O=0x804f000d, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_BL_O_ERROR=0x804f000a, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_BL=0x804f101e, */
/* IDL: 	PR_EMS_AB_ALT_RECIPIENT_BL_UNICODE=0x804f101f, */
/* IDL: 	PR_EMS_AB_ANCESTOR_ID=0x80500102, */
/* IDL: 	PR_EMS_AB_ANCESTOR_ID_ERROR=0x8050000a, */
/* IDL: 	PR_EMS_AB_ASSOC_REMOTE_DXA_O=0x8051000d, */
/* IDL: 	PR_EMS_AB_ASSOC_REMOTE_DXA_O_ERROR=0x8051000a, */
/* IDL: 	PR_EMS_AB_ASSOC_REMOTE_DXA=0x8051101e, */
/* IDL: 	PR_EMS_AB_ASSOC_REMOTE_DXA_UNICODE=0x8051101f, */
/* IDL: 	PR_EMS_AB_ASSOCIATION_LIFETIME=0x80520003, */
/* IDL: 	PR_EMS_AB_ASSOCIATION_LIFETIME_ERROR=0x8052000a, */
/* IDL: 	PR_EMS_AB_AUTH_ORIG_BL_O=0x8053000d, */
/* IDL: 	PR_EMS_AB_AUTH_ORIG_BL_O_ERROR=0x8053000a, */
/* IDL: 	PR_EMS_AB_AUTH_ORIG_BL=0x8053101e, */
/* IDL: 	PR_EMS_AB_AUTH_ORIG_BL_UNICODE=0x8053101f, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_DOMAIN=0x8054001e, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_DOMAIN_UNICODE=0x8054001f, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_DOMAIN_ERROR=0x8054000a, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_PASSWORD=0x80550102, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_PASSWORD_ERROR=0x8055000a, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_USER=0x8056001e, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_USER_UNICODE=0x8056001f, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_USER_ERROR=0x8056000a, */
/* IDL: 	PR_EMS_AB_BUSINESS_CATEGORY=0x8057101e, */
/* IDL: 	PR_EMS_AB_BUSINESS_CATEGORY_UNICODE=0x8057101f, */
/* IDL: 	PR_EMS_AB_BUSINESS_CATEGORY_ERROR=0x8057000a, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_O=0x8058000d, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_O_ERROR=0x8058000a, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF=0x8058101e, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_UNICODE=0x8058101f, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_BL_O=0x8059000d, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_BL_O_ERROR=0x8059000a, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_BL=0x8059101e, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_BL_UNICODE=0x8059101f, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_O=0x805a000d, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_O_ERROR=0x805a000a, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL=0x805a101e, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_UNICODE=0x805a101f, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_BL_O=0x805b000d, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_BL_O_ERROR=0x805b000a, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_BL=0x805b101e, */
/* IDL: 	PR_EMS_AB_CAN_CREATE_PF_DL_BL_UNICODE=0x805b101f, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_O=0x805c000d, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_O_ERROR=0x805c000a, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF=0x805c101e, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_UNICODE=0x805c101f, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O=0x805d000d, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_BL_O_ERROR=0x805d000a, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_BL=0x805d101e, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_BL_UNICODE=0x805d101f, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O=0x805e000d, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_O_ERROR=0x805e000a, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL=0x805e101e, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_UNICODE=0x805e101f, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O=0x805f000d, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_O_ERROR=0x805f000a, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL=0x805f101e, */
/* IDL: 	PR_EMS_AB_CAN_NOT_CREATE_PF_DL_BL_UNICODE=0x805f101f, */
/* IDL: 	PR_EMS_AB_CAN_PRESERVE_DNS=0x8060000b, */
/* IDL: 	PR_EMS_AB_CAN_PRESERVE_DNS_ERROR=0x8060000a, */
/* IDL: 	PR_EMS_AB_CLOCK_ALERT_OFFSET=0x80610003, */
/* IDL: 	PR_EMS_AB_CLOCK_ALERT_OFFSET_ERROR=0x8061000a, */
/* IDL: 	PR_EMS_AB_CLOCK_ALERT_REPAIR=0x8062000b, */
/* IDL: 	PR_EMS_AB_CLOCK_ALERT_REPAIR_ERROR=0x8062000a, */
/* IDL: 	PR_EMS_AB_CLOCK_WARNING_OFFSET=0x80630003, */
/* IDL: 	PR_EMS_AB_CLOCK_WARNING_OFFSET_ERROR=0x8063000a, */
/* IDL: 	PR_EMS_AB_CLOCK_WARNING_REPAIR=0x8064000b, */
/* IDL: 	PR_EMS_AB_CLOCK_WARNING_REPAIR_ERROR=0x8064000a, */
/* IDL: 	PR_EMS_AB_COMPUTER_NAME=0x8065001e, */
/* IDL: 	PR_EMS_AB_COMPUTER_NAME_UNICODE=0x8065001f, */
/* IDL: 	PR_EMS_AB_COMPUTER_NAME_ERROR=0x8065000a, */
/* IDL: 	PR_EMS_AB_CONNECTED_DOMAINS=0x8066101e, */
/* IDL: 	PR_EMS_AB_CONNECTED_DOMAINS_UNICODE=0x8066101f, */
/* IDL: 	PR_EMS_AB_CONNECTED_DOMAINS_ERROR=0x8066000a, */
/* IDL: 	PR_EMS_AB_CONTAINER_INFO=0x80670003, */
/* IDL: 	PR_EMS_AB_CONTAINER_INFO_ERROR=0x8067000a, */
/* IDL: 	PR_EMS_AB_COST=0x80680003, */
/* IDL: 	PR_EMS_AB_COST_ERROR=0x8068000a, */
/* IDL: 	PR_EMS_AB_COUNTRY_NAME=0x8069001e, */
/* IDL: 	PR_EMS_AB_COUNTRY_NAME_UNICODE=0x8069001f, */
/* IDL: 	PR_EMS_AB_COUNTRY_NAME_ERROR=0x8069000a, */
/* IDL: 	PR_EMS_AB_DELIV_CONT_LENGTH=0x806a0003, */
/* IDL: 	PR_EMS_AB_DELIV_CONT_LENGTH_ERROR=0x806a000a, */
/* IDL: 	PR_EMS_AB_DELIV_EITS=0x806b1102, */
/* IDL: 	PR_EMS_AB_DELIV_EITS_ERROR=0x806b000a, */
/* IDL: 	PR_EMS_AB_DELIV_EXT_CONT_TYPES=0x806c1102, */
/* IDL: 	PR_EMS_AB_DELIV_EXT_CONT_TYPES_ERROR=0x806c000a, */
/* IDL: 	PR_EMS_AB_DELIVER_AND_REDIRECT=0x806d000b, */
/* IDL: 	PR_EMS_AB_DELIVER_AND_REDIRECT_ERROR=0x806d000a, */
/* IDL: 	PR_EMS_AB_DELIVERY_MECHANISM=0x806e0003, */
/* IDL: 	PR_EMS_AB_DELIVERY_MECHANISM_ERROR=0x806e000a, */
/* IDL: 	PR_EMS_AB_DESCRIPTION=0x806f101e, */
/* IDL: 	PR_EMS_AB_DESCRIPTION_UNICODE=0x806f101f, */
/* IDL: 	PR_EMS_AB_DESCRIPTION_ERROR=0x806f000a, */
/* IDL: 	PR_EMS_AB_DESTINATION_INDICATOR=0x8070101e, */
/* IDL: 	PR_EMS_AB_DESTINATION_INDICATOR_UNICODE=0x8070101f, */
/* IDL: 	PR_EMS_AB_DESTINATION_INDICATOR_ERROR=0x8070000a, */
/* IDL: 	PR_EMS_AB_DIAGNOSTIC_REG_KEY=0x8071001e, */
/* IDL: 	PR_EMS_AB_DIAGNOSTIC_REG_KEY_UNICODE=0x8071001f, */
/* IDL: 	PR_EMS_AB_DIAGNOSTIC_REG_KEY_ERROR=0x8071000a, */
/* IDL: 	PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O=0x8072000d, */
/* IDL: 	PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_O_ERROR=0x8072000a, */
/* IDL: 	PR_EMS_AB_DL_MEM_REJECT_PERMS_BL=0x8072101e, */
/* IDL: 	PR_EMS_AB_DL_MEM_REJECT_PERMS_BL_UNICODE=0x8072101f, */
/* IDL: 	PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O=0x8073000d, */
/* IDL: 	PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_O_ERROR=0x8073000a, */
/* IDL: 	PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL=0x8073101e, */
/* IDL: 	PR_EMS_AB_DL_MEM_SUBMIT_PERMS_BL_UNICODE=0x8073101f, */
/* IDL: 	PR_EMS_AB_DL_MEMBER_RULE=0x80741102, */
/* IDL: 	PR_EMS_AB_DL_MEMBER_RULE_ERROR=0x8074000a, */
/* IDL: 	PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O=0x8075000d, */
/* IDL: 	PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_O_ERROR=0x8075000a, */
/* IDL: 	PR_EMS_AB_DOMAIN_DEF_ALT_RECIP=0x8075001e, */
/* IDL: 	PR_EMS_AB_DOMAIN_DEF_ALT_RECIP_UNICODE=0x8075001f, */
/* IDL: 	PR_EMS_AB_DOMAIN_NAME=0x8076001e, */
/* IDL: 	PR_EMS_AB_DOMAIN_NAME_UNICODE=0x8076001f, */
/* IDL: 	PR_EMS_AB_DOMAIN_NAME_ERROR=0x8076000a, */
/* IDL: 	PR_EMS_AB_DSA_SIGNATURE=0x80770102, */
/* IDL: 	PR_EMS_AB_DSA_SIGNATURE_ERROR=0x8077000a, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_COPY=0x8078000b, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_COPY_ERROR=0x8078000a, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_FORWARD=0x8079000b, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_FORWARD_ERROR=0x8079000a, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_UPDATE=0x807a0003, */
/* IDL: 	PR_EMS_AB_DXA_ADMIN_UPDATE_ERROR=0x807a000a, */
/* IDL: 	PR_EMS_AB_DXA_APPEND_REQCN=0x807b000b, */
/* IDL: 	PR_EMS_AB_DXA_APPEND_REQCN_ERROR=0x807b000a, */
/* IDL: 	PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O=0x807c000d, */
/* IDL: 	PR_EMS_AB_DXA_CONF_CONTAINER_LIST_O_ERROR=0x807c000a, */
/* IDL: 	PR_EMS_AB_DXA_CONF_CONTAINER_LIST=0x807c101e, */
/* IDL: 	PR_EMS_AB_DXA_CONF_CONTAINER_LIST_UNICODE=0x807c101f, */
/* IDL: 	PR_EMS_AB_DXA_CONF_REQ_TIME=0x807d0040, */
/* IDL: 	PR_EMS_AB_DXA_CONF_REQ_TIME_ERROR=0x807d000a, */
/* IDL: 	PR_EMS_AB_DXA_CONF_SEQ=0x807e001e, */
/* IDL: 	PR_EMS_AB_DXA_CONF_SEQ_UNICODE=0x807e001f, */
/* IDL: 	PR_EMS_AB_DXA_CONF_SEQ_ERROR=0x807e000a, */
/* IDL: 	PR_EMS_AB_DXA_CONF_SEQ_USN=0x807f0003, */
/* IDL: 	PR_EMS_AB_DXA_CONF_SEQ_USN_ERROR=0x807f000a, */
/* IDL: 	PR_EMS_AB_DXA_EXCHANGE_OPTIONS=0x80800003, */
/* IDL: 	PR_EMS_AB_DXA_EXCHANGE_OPTIONS_ERROR=0x8080000a, */
/* IDL: 	PR_EMS_AB_DXA_EXPORT_NOW=0x8081000b, */
/* IDL: 	PR_EMS_AB_DXA_EXPORT_NOW_ERROR=0x8081000a, */
/* IDL: 	PR_EMS_AB_DXA_FLAGS=0x80820003, */
/* IDL: 	PR_EMS_AB_DXA_FLAGS_ERROR=0x8082000a, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ=0x8083001e, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_UNICODE=0x8083001f, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_ERROR=0x8083000a, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_TIME=0x80840040, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_TIME_ERROR=0x8084000a, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_USN=0x80850003, */
/* IDL: 	PR_EMS_AB_DXA_IMP_SEQ_USN_ERROR=0x8085000a, */
/* IDL: 	PR_EMS_AB_DXA_IMPORT_NOW=0x8086000b, */
/* IDL: 	PR_EMS_AB_DXA_IMPORT_NOW_ERROR=0x8086000a, */
/* IDL: 	PR_EMS_AB_DXA_IN_TEMPLATE_MAP=0x8087101e, */
/* IDL: 	PR_EMS_AB_DXA_IN_TEMPLATE_MAP_UNICODE=0x8087101f, */
/* IDL: 	PR_EMS_AB_DXA_IN_TEMPLATE_MAP_ERROR=0x8087000a, */
/* IDL: 	PR_EMS_AB_DXA_LOCAL_ADMIN_O=0x8088000d, */
/* IDL: 	PR_EMS_AB_DXA_LOCAL_ADMIN_O_ERROR=0x8088000a, */
/* IDL: 	PR_EMS_AB_DXA_LOCAL_ADMIN=0x8088001e, */
/* IDL: 	PR_EMS_AB_DXA_LOCAL_ADMIN_UNICODE=0x8088001f, */
/* IDL: 	PR_EMS_AB_DXA_LOGGING_LEVEL=0x80890003, */
/* IDL: 	PR_EMS_AB_DXA_LOGGING_LEVEL_ERROR=0x8089000a, */
/* IDL: 	PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE=0x808a001e, */
/* IDL: 	PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_UNICODE=0x808a001f, */
/* IDL: 	PR_EMS_AB_DXA_NATIVE_ADDRESS_TYPE_ERROR=0x808a000a, */
/* IDL: 	PR_EMS_AB_DXA_OUT_TEMPLATE_MAP=0x808b101e, */
/* IDL: 	PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_UNICODE=0x808b101f, */
/* IDL: 	PR_EMS_AB_DXA_OUT_TEMPLATE_MAP_ERROR=0x808b000a, */
/* IDL: 	PR_EMS_AB_DXA_PASSWORD=0x808c001e, */
/* IDL: 	PR_EMS_AB_DXA_PASSWORD_UNICODE=0x808c001f, */
/* IDL: 	PR_EMS_AB_DXA_PASSWORD_ERROR=0x808c000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS=0x808d0003, */
/* IDL: 	PR_EMS_AB_DXA_PREV_EXCHANGE_OPTIONS_ERROR=0x808d000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY=0x808e000b, */
/* IDL: 	PR_EMS_AB_DXA_PREV_EXPORT_NATIVE_ONLY_ERROR=0x808e000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY=0x808f0003, */
/* IDL: 	PR_EMS_AB_DXA_PREV_IN_EXCHANGE_SENSITIVITY_ERROR=0x808f000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O=0x8090000d, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_O_ERROR=0x8090000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES=0x8090001e, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REMOTE_ENTRIES_UNICODE=0x8090001f, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY=0x80910003, */
/* IDL: 	PR_EMS_AB_DXA_PREV_REPLICATION_SENSITIVITY_ERROR=0x8091000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS=0x80920003, */
/* IDL: 	PR_EMS_AB_DXA_PREV_TEMPLATE_OPTIONS_ERROR=0x8092000a, */
/* IDL: 	PR_EMS_AB_DXA_PREV_TYPES=0x80930003, */
/* IDL: 	PR_EMS_AB_DXA_PREV_TYPES_ERROR=0x8093000a, */
/* IDL: 	PR_EMS_AB_DXA_RECIPIENT_CP=0x8094001e, */
/* IDL: 	PR_EMS_AB_DXA_RECIPIENT_CP_UNICODE=0x8094001f, */
/* IDL: 	PR_EMS_AB_DXA_RECIPIENT_CP_ERROR=0x8094000a, */
/* IDL: 	PR_EMS_AB_DXA_REMOTE_CLIENT_O=0x8095000d, */
/* IDL: 	PR_EMS_AB_DXA_REMOTE_CLIENT_O_ERROR=0x8095000a, */
/* IDL: 	PR_EMS_AB_DXA_REMOTE_CLIENT=0x8095001e, */
/* IDL: 	PR_EMS_AB_DXA_REMOTE_CLIENT_UNICODE=0x8095001f, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ=0x8096001e, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_UNICODE=0x8096001f, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_ERROR=0x8096000a, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_TIME=0x80970040, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_TIME_ERROR=0x8097000a, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_USN=0x80980003, */
/* IDL: 	PR_EMS_AB_DXA_REQ_SEQ_USN_ERROR=0x8098000a, */
/* IDL: 	PR_EMS_AB_DXA_REQNAME=0x8099001e, */
/* IDL: 	PR_EMS_AB_DXA_REQNAME_UNICODE=0x8099001f, */
/* IDL: 	PR_EMS_AB_DXA_REQNAME_ERROR=0x8099000a, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ=0x809a001e, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_UNICODE=0x809a001f, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_ERROR=0x809a000a, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_TIME=0x809b0040, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_TIME_ERROR=0x809b000a, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_USN=0x809c0003, */
/* IDL: 	PR_EMS_AB_DXA_SVR_SEQ_USN_ERROR=0x809c000a, */
/* IDL: 	PR_EMS_AB_DXA_TASK=0x809d0003, */
/* IDL: 	PR_EMS_AB_DXA_TASK_ERROR=0x809d000a, */
/* IDL: 	PR_EMS_AB_DXA_TEMPLATE_OPTIONS=0x809e0003, */
/* IDL: 	PR_EMS_AB_DXA_TEMPLATE_OPTIONS_ERROR=0x809e000a, */
/* IDL: 	PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP=0x809f0040, */
/* IDL: 	PR_EMS_AB_DXA_TEMPLATE_TIMESTAMP_ERROR=0x809f000a, */
/* IDL: 	PR_EMS_AB_DXA_TYPES=0x80a00003, */
/* IDL: 	PR_EMS_AB_DXA_TYPES_ERROR=0x80a0000a, */
/* IDL: 	PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O=0x80a1000d, */
/* IDL: 	PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_O_ERROR=0x80a1000a, */
/* IDL: 	PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST=0x80a1101e, */
/* IDL: 	PR_EMS_AB_DXA_UNCONF_CONTAINER_LIST_UNICODE=0x80a1101f, */
/* IDL: 	PR_EMS_AB_ENCAPSULATION_METHOD=0x80a20003, */
/* IDL: 	PR_EMS_AB_ENCAPSULATION_METHOD_ERROR=0x80a2000a, */
/* IDL: 	PR_EMS_AB_ENCRYPT=0x80a3000b, */
/* IDL: 	PR_EMS_AB_ENCRYPT_ERROR=0x80a3000a, */
/* IDL: 	PR_EMS_AB_EXPAND_DLS_LOCALLY=0x80a4000b, */
/* IDL: 	PR_EMS_AB_EXPAND_DLS_LOCALLY_ERROR=0x80a4000a, */
/* IDL: 	PR_EMS_AB_EXPORT_CONTAINERS_O=0x80a5000d, */
/* IDL: 	PR_EMS_AB_EXPORT_CONTAINERS_O_ERROR=0x80a5000a, */
/* IDL: 	PR_EMS_AB_EXPORT_CONTAINERS=0x80a5101e, */
/* IDL: 	PR_EMS_AB_EXPORT_CONTAINERS_UNICODE=0x80a5101f, */
/* IDL: 	PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS=0x80a6000b, */
/* IDL: 	PR_EMS_AB_EXPORT_CUSTOM_RECIPIENTS_ERROR=0x80a6000a, */
/* IDL: 	PR_EMS_AB_EXTENDED_CHARS_ALLOWED=0x80a7000b, */
/* IDL: 	PR_EMS_AB_EXTENDED_CHARS_ALLOWED_ERROR=0x80a7000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_DATA=0x80a81102, */
/* IDL: 	PR_EMS_AB_EXTENSION_DATA_ERROR=0x80a8000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME=0x80a9101e, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME_UNICODE=0x80a9101f, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME_ERROR=0x80a9000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME_INHERITED=0x80aa101e, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME_INHERITED_UNICODE=0x80aa101f, */
/* IDL: 	PR_EMS_AB_EXTENSION_NAME_INHERITED_ERROR=0x80aa000a, */
/* IDL: 	PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER=0x80ab1102, */
/* IDL: 	PR_EMS_AB_FACSIMILE_TELEPHONE_NUMBER_ERROR=0x80ab000a, */
/* IDL: 	PR_EMS_AB_FILE_VERSION=0x80ac0102, */
/* IDL: 	PR_EMS_AB_FILE_VERSION_ERROR=0x80ac000a, */
/* IDL: 	PR_EMS_AB_FILTER_LOCAL_ADDRESSES=0x80ad000b, */
/* IDL: 	PR_EMS_AB_FILTER_LOCAL_ADDRESSES_ERROR=0x80ad000a, */
/* IDL: 	PR_EMS_AB_FOLDERS_CONTAINER_O=0x80ae000d, */
/* IDL: 	PR_EMS_AB_FOLDERS_CONTAINER_O_ERROR=0x80ae000a, */
/* IDL: 	PR_EMS_AB_FOLDERS_CONTAINER=0x80ae001e, */
/* IDL: 	PR_EMS_AB_FOLDERS_CONTAINER_UNICODE=0x80ae001f, */
/* IDL: 	PR_EMS_AB_GARBAGE_COLL_PERIOD=0x80af0003, */
/* IDL: 	PR_EMS_AB_GARBAGE_COLL_PERIOD_ERROR=0x80af000a, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_CRED=0x80b0001e, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_CRED_UNICODE=0x80b0001f, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_CRED_ERROR=0x80b0000a, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_DESIG=0x80b1001e, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_DESIG_UNICODE=0x80b1001f, */
/* IDL: 	PR_EMS_AB_GATEWAY_LOCAL_DESIG_ERROR=0x80b1000a, */
/* IDL: 	PR_EMS_AB_GATEWAY_PROXY=0x80b2101e, */
/* IDL: 	PR_EMS_AB_GATEWAY_PROXY_UNICODE=0x80b2101f, */
/* IDL: 	PR_EMS_AB_GATEWAY_PROXY_ERROR=0x80b2000a, */
/* IDL: 	PR_EMS_AB_GATEWAY_ROUTING_TREE=0x80b30102, */
/* IDL: 	PR_EMS_AB_GATEWAY_ROUTING_TREE_ERROR=0x80b3000a, */
/* IDL: 	PR_EMS_AB_GWART_LAST_MODIFIED=0x80b40040, */
/* IDL: 	PR_EMS_AB_GWART_LAST_MODIFIED_ERROR=0x80b4000a, */
/* IDL: 	PR_EMS_AB_HAS_FULL_REPLICA_NCS_O=0x80b5000d, */
/* IDL: 	PR_EMS_AB_HAS_FULL_REPLICA_NCS_O_ERROR=0x80b5000a, */
/* IDL: 	PR_EMS_AB_HAS_FULL_REPLICA_NCS=0x80b5101e, */
/* IDL: 	PR_EMS_AB_HAS_FULL_REPLICA_NCS_UNICODE=0x80b5101f, */
/* IDL: 	PR_EMS_AB_HAS_MASTER_NCS_O=0x80b6000d, */
/* IDL: 	PR_EMS_AB_HAS_MASTER_NCS_O_ERROR=0x80b6000a, */
/* IDL: 	PR_EMS_AB_HAS_MASTER_NCS=0x80b6101e, */
/* IDL: 	PR_EMS_AB_HAS_MASTER_NCS_UNICODE=0x80b6101f, */
/* IDL: 	PR_EMS_AB_HEURISTICS=0x80b70003, */
/* IDL: 	PR_EMS_AB_HEURISTICS_ERROR=0x80b7000a, */
/* IDL: 	PR_EMS_AB_HIDE_DL_MEMBERSHIP=0x80b8000b, */
/* IDL: 	PR_EMS_AB_HIDE_DL_MEMBERSHIP_ERROR=0x80b8000a, */
/* IDL: 	PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK=0x80b9000b, */
/* IDL: 	PR_EMS_AB_HIDE_FROM_ADDRESS_BOOK_ERROR=0x80b9000a, */
/* IDL: 	PR_EMS_AB_IMPORT_CONTAINER_O=0x80ba000d, */
/* IDL: 	PR_EMS_AB_IMPORT_CONTAINER_O_ERROR=0x80ba000a, */
/* IDL: 	PR_EMS_AB_IMPORT_CONTAINER=0x80ba001e, */
/* IDL: 	PR_EMS_AB_IMPORT_CONTAINER_UNICODE=0x80ba001f, */
/* IDL: 	PR_EMS_AB_IMPORT_SENSITIVITY=0x80bb0003, */
/* IDL: 	PR_EMS_AB_IMPORT_SENSITIVITY_ERROR=0x80bb000a, */
/* IDL: 	PR_EMS_AB_INBOUND_SITES_O=0x80bc000d, */
/* IDL: 	PR_EMS_AB_INBOUND_SITES_O_ERROR=0x80bc000a, */
/* IDL: 	PR_EMS_AB_INBOUND_SITES=0x80bc101e, */
/* IDL: 	PR_EMS_AB_INBOUND_SITES_UNICODE=0x80bc101f, */
/* IDL: 	PR_EMS_AB_INSTANCE_TYPE=0x80bd0003, */
/* IDL: 	PR_EMS_AB_INSTANCE_TYPE_ERROR=0x80bd000a, */
/* IDL: 	PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER=0x80be101e, */
/* IDL: 	PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_UNICODE=0x80be101f, */
/* IDL: 	PR_EMS_AB_INTERNATIONAL_ISDN_NUMBER_ERROR=0x80be000a, */
/* IDL: 	PR_EMS_AB_INVOCATION_ID=0x80bf0102, */
/* IDL: 	PR_EMS_AB_INVOCATION_ID_ERROR=0x80bf000a, */
/* IDL: 	PR_EMS_AB_IS_DELETED=0x80c0000b, */
/* IDL: 	PR_EMS_AB_IS_DELETED_ERROR=0x80c0000a, */
/* IDL: 	PR_EMS_AB_IS_SINGLE_VALUED=0x80c1000b, */
/* IDL: 	PR_EMS_AB_IS_SINGLE_VALUED_ERROR=0x80c1000a, */
/* IDL: 	PR_EMS_AB_KCC_STATUS=0x80c21102, */
/* IDL: 	PR_EMS_AB_KCC_STATUS_ERROR=0x80c2000a, */
/* IDL: 	PR_EMS_AB_KNOWLEDGE_INFORMATION=0x80c3101e, */
/* IDL: 	PR_EMS_AB_KNOWLEDGE_INFORMATION_UNICODE=0x80c3101f, */
/* IDL: 	PR_EMS_AB_KNOWLEDGE_INFORMATION_ERROR=0x80c3000a, */
/* IDL: 	PR_EMS_AB_LINE_WRAP=0x80c40003, */
/* IDL: 	PR_EMS_AB_LINE_WRAP_ERROR=0x80c4000a, */
/* IDL: 	PR_EMS_AB_LINK_ID=0x80c50003, */
/* IDL: 	PR_EMS_AB_LINK_ID_ERROR=0x80c5000a, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD=0x80c6001e, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD_UNICODE=0x80c6001f, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD_ERROR=0x80c6000a, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS=0x80c7001e, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_UNICODE=0x80c7001f, */
/* IDL: 	PR_EMS_AB_LOCAL_BRIDGE_HEAD_ADDRESS_ERROR=0x80c7000a, */
/* IDL: 	PR_EMS_AB_LOCAL_INITIAL_TURN=0x80c8000b, */
/* IDL: 	PR_EMS_AB_LOCAL_INITIAL_TURN_ERROR=0x80c8000a, */
/* IDL: 	PR_EMS_AB_LOCAL_SCOPE_O=0x80c9000d, */
/* IDL: 	PR_EMS_AB_LOCAL_SCOPE_O_ERROR=0x80c9000a, */
/* IDL: 	PR_EMS_AB_LOCAL_SCOPE=0x80c9101e, */
/* IDL: 	PR_EMS_AB_LOCAL_SCOPE_UNICODE=0x80c9101f, */
/* IDL: 	PR_EMS_AB_LOG_FILENAME=0x80ca001e, */
/* IDL: 	PR_EMS_AB_LOG_FILENAME_UNICODE=0x80ca001f, */
/* IDL: 	PR_EMS_AB_LOG_FILENAME_ERROR=0x80ca000a, */
/* IDL: 	PR_EMS_AB_LOG_ROLLOVER_INTERVAL=0x80cb0003, */
/* IDL: 	PR_EMS_AB_LOG_ROLLOVER_INTERVAL_ERROR=0x80cb000a, */
/* IDL: 	PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY=0x80cc000b, */
/* IDL: 	PR_EMS_AB_MAINTAIN_AUTOREPLY_HISTORY_ERROR=0x80cc000a, */
/* IDL: 	PR_EMS_AB_MAPI_DISPLAY_TYPE=0x80cd0003, */
/* IDL: 	PR_EMS_AB_MAPI_DISPLAY_TYPE_ERROR=0x80cd000a, */
/* IDL: 	PR_EMS_AB_MAPI_ID=0x80ce0003, */
/* IDL: 	PR_EMS_AB_MAPI_ID_ERROR=0x80ce000a, */
/* IDL: 	PR_EMS_AB_MDB_BACKOFF_INTERVAL=0x80cf0003, */
/* IDL: 	PR_EMS_AB_MDB_BACKOFF_INTERVAL_ERROR=0x80cf000a, */
/* IDL: 	PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD=0x80d00003, */
/* IDL: 	PR_EMS_AB_MDB_MSG_TIME_OUT_PERIOD_ERROR=0x80d0000a, */
/* IDL: 	PR_EMS_AB_MDB_OVER_QUOTA_LIMIT=0x80d10003, */
/* IDL: 	PR_EMS_AB_MDB_OVER_QUOTA_LIMIT_ERROR=0x80d1000a, */
/* IDL: 	PR_EMS_AB_MDB_STORAGE_QUOTA=0x80d20003, */
/* IDL: 	PR_EMS_AB_MDB_STORAGE_QUOTA_ERROR=0x80d2000a, */
/* IDL: 	PR_EMS_AB_MDB_UNREAD_LIMIT=0x80d30003, */
/* IDL: 	PR_EMS_AB_MDB_UNREAD_LIMIT_ERROR=0x80d3000a, */
/* IDL: 	PR_EMS_AB_MDB_USE_DEFAULTS=0x80d4000b, */
/* IDL: 	PR_EMS_AB_MDB_USE_DEFAULTS_ERROR=0x80d4000a, */
/* IDL: 	PR_EMS_AB_MESSAGE_TRACKING_ENABLED=0x80d5000b, */
/* IDL: 	PR_EMS_AB_MESSAGE_TRACKING_ENABLED_ERROR=0x80d5000a, */
/* IDL: 	PR_EMS_AB_MONITOR_CLOCK=0x80d6000b, */
/* IDL: 	PR_EMS_AB_MONITOR_CLOCK_ERROR=0x80d6000a, */
/* IDL: 	PR_EMS_AB_MONITOR_SERVERS=0x80d7000b, */
/* IDL: 	PR_EMS_AB_MONITOR_SERVERS_ERROR=0x80d7000a, */
/* IDL: 	PR_EMS_AB_MONITOR_SERVICES=0x80d8000b, */
/* IDL: 	PR_EMS_AB_MONITOR_SERVICES_ERROR=0x80d8000a, */
/* IDL: 	PR_EMS_AB_MONITORED_CONFIGURATIONS_O=0x80d9000d, */
/* IDL: 	PR_EMS_AB_MONITORED_CONFIGURATIONS_O_ERROR=0x80d9000a, */
/* IDL: 	PR_EMS_AB_MONITORED_CONFIGURATIONS=0x80d9101e, */
/* IDL: 	PR_EMS_AB_MONITORED_CONFIGURATIONS_UNICODE=0x80d9101f, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVERS_O=0x80da000d, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVERS_O_ERROR=0x80da000a, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVERS=0x80da101e, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVERS_UNICODE=0x80da101f, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVICES=0x80db101e, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVICES_UNICODE=0x80db101f, */
/* IDL: 	PR_EMS_AB_MONITORED_SERVICES_ERROR=0x80db000a, */
/* IDL: 	PR_EMS_AB_MONITORING_ALERT_DELAY=0x80dc0003, */
/* IDL: 	PR_EMS_AB_MONITORING_ALERT_DELAY_ERROR=0x80dc000a, */
/* IDL: 	PR_EMS_AB_MONITORING_ALERT_UNITS=0x80dd0003, */
/* IDL: 	PR_EMS_AB_MONITORING_ALERT_UNITS_ERROR=0x80dd000a, */
/* IDL: 	PR_EMS_AB_MONITORING_AVAILABILITY_STYLE=0x80de0003, */
/* IDL: 	PR_EMS_AB_MONITORING_AVAILABILITY_STYLE_ERROR=0x80de000a, */
/* IDL: 	PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW=0x80df0102, */
/* IDL: 	PR_EMS_AB_MONITORING_AVAILABILITY_WINDOW_ERROR=0x80df000a, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O=0x80e0000d, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_O_ERROR=0x80e0000a, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_MAIL=0x80e0101e, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_MAIL_UNICODE=0x80e0101f, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O=0x80e1000d, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_RPC_O_ERROR=0x80e1000a, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_RPC=0x80e1101e, */
/* IDL: 	PR_EMS_AB_MONITORING_CACHED_VIA_RPC_UNICODE=0x80e1101f, */
/* IDL: 	PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE=0x80e21102, */
/* IDL: 	PR_EMS_AB_MONITORING_ESCALATION_PROCEDURE_ERROR=0x80e2000a, */
/* IDL: 	PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL=0x80e30003, */
/* IDL: 	PR_EMS_AB_MONITORING_HOTSITE_POLL_INTERVAL_ERROR=0x80e3000a, */
/* IDL: 	PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS=0x80e40003, */
/* IDL: 	PR_EMS_AB_MONITORING_HOTSITE_POLL_UNITS_ERROR=0x80e4000a, */
/* IDL: 	PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL=0x80e50003, */
/* IDL: 	PR_EMS_AB_MONITORING_MAIL_UPDATE_INTERVAL_ERROR=0x80e5000a, */
/* IDL: 	PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS=0x80e60003, */
/* IDL: 	PR_EMS_AB_MONITORING_MAIL_UPDATE_UNITS_ERROR=0x80e6000a, */
/* IDL: 	PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL=0x80e70003, */
/* IDL: 	PR_EMS_AB_MONITORING_NORMAL_POLL_INTERVAL_ERROR=0x80e7000a, */
/* IDL: 	PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS=0x80e80003, */
/* IDL: 	PR_EMS_AB_MONITORING_NORMAL_POLL_UNITS_ERROR=0x80e8000a, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_O=0x80e9000d, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_O_ERROR=0x80e9000a, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS=0x80e9101e, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_UNICODE=0x80e9101f, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O=0x80ea000d, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_NDR_O_ERROR=0x80ea000a, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_NDR=0x80ea101e, */
/* IDL: 	PR_EMS_AB_MONITORING_RECIPIENTS_NDR_UNICODE=0x80ea101f, */
/* IDL: 	PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL=0x80eb0003, */
/* IDL: 	PR_EMS_AB_MONITORING_RPC_UPDATE_INTERVAL_ERROR=0x80eb000a, */
/* IDL: 	PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS=0x80ec0003, */
/* IDL: 	PR_EMS_AB_MONITORING_RPC_UPDATE_UNITS_ERROR=0x80ec000a, */
/* IDL: 	PR_EMS_AB_MONITORING_WARNING_DELAY=0x80ed0003, */
/* IDL: 	PR_EMS_AB_MONITORING_WARNING_DELAY_ERROR=0x80ed000a, */
/* IDL: 	PR_EMS_AB_MONITORING_WARNING_UNITS=0x80ee0003, */
/* IDL: 	PR_EMS_AB_MONITORING_WARNING_UNITS_ERROR=0x80ee000a, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_CRED=0x80ef001e, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_CRED_UNICODE=0x80ef001f, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_CRED_ERROR=0x80ef000a, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_DESIG=0x80f0001e, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_DESIG_UNICODE=0x80f0001f, */
/* IDL: 	PR_EMS_AB_MTA_LOCAL_DESIG_ERROR=0x80f0000a, */
/* IDL: 	PR_EMS_AB_N_ADDRESS=0x80f10102, */
/* IDL: 	PR_EMS_AB_N_ADDRESS_ERROR=0x80f1000a, */
/* IDL: 	PR_EMS_AB_N_ADDRESS_TYPE=0x80f20003, */
/* IDL: 	PR_EMS_AB_N_ADDRESS_TYPE_ERROR=0x80f2000a, */
/* IDL: 	PR_EMS_AB_NT_MACHINE_NAME=0x80f3001e, */
/* IDL: 	PR_EMS_AB_NT_MACHINE_NAME_UNICODE=0x80f3001f, */
/* IDL: 	PR_EMS_AB_NT_MACHINE_NAME_ERROR=0x80f3000a, */
/* IDL: 	PR_EMS_AB_NUM_OF_OPEN_RETRIES=0x80f40003, */
/* IDL: 	PR_EMS_AB_NUM_OF_OPEN_RETRIES_ERROR=0x80f4000a, */
/* IDL: 	PR_EMS_AB_NUM_OF_TRANSFER_RETRIES=0x80f50003, */
/* IDL: 	PR_EMS_AB_NUM_OF_TRANSFER_RETRIES_ERROR=0x80f5000a, */
/* IDL: 	PR_EMS_AB_OBJECT_CLASS_CATEGORY=0x80f60003, */
/* IDL: 	PR_EMS_AB_OBJECT_CLASS_CATEGORY_ERROR=0x80f6000a, */
/* IDL: 	PR_EMS_AB_OBJECT_VERSION=0x80f70003, */
/* IDL: 	PR_EMS_AB_OBJECT_VERSION_ERROR=0x80f7000a, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O=0x80f8000d, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_CONTAINERS_O_ERROR=0x80f8000a, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_CONTAINERS=0x80f8101e, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_CONTAINERS_UNICODE=0x80f8101f, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SCHEDULE=0x80f90102, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SCHEDULE_ERROR=0x80f9000a, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SERVER_O=0x80fa000d, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SERVER_O_ERROR=0x80fa000a, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SERVER=0x80fa001e, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_SERVER_UNICODE=0x80fa001f, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_STYLE=0x80fb0003, */
/* IDL: 	PR_EMS_AB_OFF_LINE_AB_STYLE_ERROR=0x80fb000a, */
/* IDL: 	PR_EMS_AB_OID_TYPE=0x80fc0003, */
/* IDL: 	PR_EMS_AB_OID_TYPE_ERROR=0x80fc000a, */
/* IDL: 	PR_EMS_AB_OM_OBJECT_CLASS=0x80fd0102, */
/* IDL: 	PR_EMS_AB_OM_OBJECT_CLASS_ERROR=0x80fd000a, */
/* IDL: 	PR_EMS_AB_OM_SYNTAX=0x80fe0003, */
/* IDL: 	PR_EMS_AB_OM_SYNTAX_ERROR=0x80fe000a, */
/* IDL: 	PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR=0x80ff000b, */
/* IDL: 	PR_EMS_AB_OOF_REPLY_TO_ORIGINATOR_ERROR=0x80ff000a, */
/* IDL: 	PR_EMS_AB_OPEN_RETRY_INTERVAL=0x81000003, */
/* IDL: 	PR_EMS_AB_OPEN_RETRY_INTERVAL_ERROR=0x8100000a, */
/* IDL: 	PR_EMS_AB_ORGANIZATION_NAME=0x8101101e, */
/* IDL: 	PR_EMS_AB_ORGANIZATION_NAME_UNICODE=0x8101101f, */
/* IDL: 	PR_EMS_AB_ORGANIZATION_NAME_ERROR=0x8101000a, */
/* IDL: 	PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME=0x8102101e, */
/* IDL: 	PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_UNICODE=0x8102101f, */
/* IDL: 	PR_EMS_AB_ORGANIZATIONAL_UNIT_NAME_ERROR=0x8102000a, */
/* IDL: 	PR_EMS_AB_ORIGINAL_DISPLAY_TABLE=0x81030102, */
/* IDL: 	PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_ERROR=0x8103000a, */
/* IDL: 	PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS=0x81040102, */
/* IDL: 	PR_EMS_AB_ORIGINAL_DISPLAY_TABLE_MSDOS_ERROR=0x8104000a, */
/* IDL: 	PR_EMS_AB_OUTBOUND_SITES_O=0x8105000d, */
/* IDL: 	PR_EMS_AB_OUTBOUND_SITES_O_ERROR=0x8105000a, */
/* IDL: 	PR_EMS_AB_OUTBOUND_SITES=0x8105101e, */
/* IDL: 	PR_EMS_AB_OUTBOUND_SITES_UNICODE=0x8105101f, */
/* IDL: 	PR_EMS_AB_P_SELECTOR=0x81060102, */
/* IDL: 	PR_EMS_AB_P_SELECTOR_ERROR=0x8106000a, */
/* IDL: 	PR_EMS_AB_P_SELECTOR_INBOUND=0x81070102, */
/* IDL: 	PR_EMS_AB_P_SELECTOR_INBOUND_ERROR=0x8107000a, */
/* IDL: 	PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE=0x81080102, */
/* IDL: 	PR_EMS_AB_PER_MSG_DIALOG_DISPLAY_TABLE_ERROR=0x8108000a, */
/* IDL: 	PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE=0x81090102, */
/* IDL: 	PR_EMS_AB_PER_RECIP_DIALOG_DISPLAY_TABLE_ERROR=0x8109000a, */
/* IDL: 	PR_EMS_AB_PERIOD_REP_SYNC_TIMES=0x810a0102, */
/* IDL: 	PR_EMS_AB_PERIOD_REP_SYNC_TIMES_ERROR=0x810a000a, */
/* IDL: 	PR_EMS_AB_PERIOD_REPL_STAGGER=0x810b0003, */
/* IDL: 	PR_EMS_AB_PERIOD_REPL_STAGGER_ERROR=0x810b000a, */
/* IDL: 	PR_EMS_AB_POSTAL_ADDRESS=0x810c1102, */
/* IDL: 	PR_EMS_AB_POSTAL_ADDRESS_ERROR=0x810c000a, */
/* IDL: 	PR_EMS_AB_PREFERRED_DELIVERY_METHOD=0x810d1003, */
/* IDL: 	PR_EMS_AB_PREFERRED_DELIVERY_METHOD_ERROR=0x810d000a, */
/* IDL: 	PR_EMS_AB_PRMD=0x810e001e, */
/* IDL: 	PR_EMS_AB_PRMD_UNICODE=0x810e001f, */
/* IDL: 	PR_EMS_AB_PRMD_ERROR=0x810e000a, */
/* IDL: 	PR_EMS_AB_PROXY_GENERATOR_DLL=0x810f001e, */
/* IDL: 	PR_EMS_AB_PROXY_GENERATOR_DLL_UNICODE=0x810f001f, */
/* IDL: 	PR_EMS_AB_PROXY_GENERATOR_DLL_ERROR=0x810f000a, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_BL_O=0x8110000d, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_BL_O_ERROR=0x8110000a, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_BL=0x8110101e, */
/* IDL: 	PR_EMS_AB_PUBLIC_DELEGATES_BL_UNICODE=0x8110101f, */
/* IDL: 	PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE=0x81110102, */
/* IDL: 	PR_EMS_AB_QUOTA_NOTIFICATION_SCHEDULE_ERROR=0x8111000a, */
/* IDL: 	PR_EMS_AB_QUOTA_NOTIFICATION_STYLE=0x81120003, */
/* IDL: 	PR_EMS_AB_QUOTA_NOTIFICATION_STYLE_ERROR=0x8112000a, */
/* IDL: 	PR_EMS_AB_RANGE_LOWER=0x81130003, */
/* IDL: 	PR_EMS_AB_RANGE_LOWER_ERROR=0x8113000a, */
/* IDL: 	PR_EMS_AB_RANGE_UPPER=0x81140003, */
/* IDL: 	PR_EMS_AB_RANGE_UPPER_ERROR=0x8114000a, */
/* IDL: 	PR_EMS_AB_RAS_CALLBACK_NUMBER=0x8115001e, */
/* IDL: 	PR_EMS_AB_RAS_CALLBACK_NUMBER_UNICODE=0x8115001f, */
/* IDL: 	PR_EMS_AB_RAS_CALLBACK_NUMBER_ERROR=0x8115000a, */
/* IDL: 	PR_EMS_AB_RAS_PHONE_NUMBER=0x8116001e, */
/* IDL: 	PR_EMS_AB_RAS_PHONE_NUMBER_UNICODE=0x8116001f, */
/* IDL: 	PR_EMS_AB_RAS_PHONE_NUMBER_ERROR=0x8116000a, */
/* IDL: 	PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME=0x8117001e, */
/* IDL: 	PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_UNICODE=0x8117001f, */
/* IDL: 	PR_EMS_AB_RAS_PHONEBOOK_ENTRY_NAME_ERROR=0x8117000a, */
/* IDL: 	PR_EMS_AB_RAS_REMOTE_SRVR_NAME=0x8118001e, */
/* IDL: 	PR_EMS_AB_RAS_REMOTE_SRVR_NAME_UNICODE=0x8118001f, */
/* IDL: 	PR_EMS_AB_RAS_REMOTE_SRVR_NAME_ERROR=0x8118000a, */
/* IDL: 	PR_EMS_AB_REGISTERED_ADDRESS=0x81191102, */
/* IDL: 	PR_EMS_AB_REGISTERED_ADDRESS_ERROR=0x8119000a, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD=0x811a001e, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD_UNICODE=0x811a001f, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD_ERROR=0x811a000a, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS=0x811b001e, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_UNICODE=0x811b001f, */
/* IDL: 	PR_EMS_AB_REMOTE_BRIDGE_HEAD_ADDRESS_ERROR=0x811b000a, */
/* IDL: 	PR_EMS_AB_REMOTE_OUT_BH_SERVER_O=0x811c000d, */
/* IDL: 	PR_EMS_AB_REMOTE_OUT_BH_SERVER_O_ERROR=0x811c000a, */
/* IDL: 	PR_EMS_AB_REMOTE_OUT_BH_SERVER=0x811c001e, */
/* IDL: 	PR_EMS_AB_REMOTE_OUT_BH_SERVER_UNICODE=0x811c001f, */
/* IDL: 	PR_EMS_AB_REMOTE_SITE_O=0x811d000d, */
/* IDL: 	PR_EMS_AB_REMOTE_SITE_O_ERROR=0x811d000a, */
/* IDL: 	PR_EMS_AB_REMOTE_SITE=0x811d001e, */
/* IDL: 	PR_EMS_AB_REMOTE_SITE_UNICODE=0x811d001f, */
/* IDL: 	PR_EMS_AB_REPLICATION_SENSITIVITY=0x811e0003, */
/* IDL: 	PR_EMS_AB_REPLICATION_SENSITIVITY_ERROR=0x811e000a, */
/* IDL: 	PR_EMS_AB_REPLICATION_STAGGER=0x811f0003, */
/* IDL: 	PR_EMS_AB_REPLICATION_STAGGER_ERROR=0x811f000a, */
/* IDL: 	PR_EMS_AB_REPORT_TO_ORIGINATOR=0x8120000b, */
/* IDL: 	PR_EMS_AB_REPORT_TO_ORIGINATOR_ERROR=0x8120000a, */
/* IDL: 	PR_EMS_AB_REPORT_TO_OWNER=0x8121000b, */
/* IDL: 	PR_EMS_AB_REPORT_TO_OWNER_ERROR=0x8121000a, */
/* IDL: 	PR_EMS_AB_REQ_SEQ=0x81220003, */
/* IDL: 	PR_EMS_AB_REQ_SEQ_ERROR=0x8122000a, */
/* IDL: 	PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O=0x8123000d, */
/* IDL: 	PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_O_ERROR=0x8123000a, */
/* IDL: 	PR_EMS_AB_RESPONSIBLE_LOCAL_DXA=0x8123001e, */
/* IDL: 	PR_EMS_AB_RESPONSIBLE_LOCAL_DXA_UNICODE=0x8123001f, */
/* IDL: 	PR_EMS_AB_RID_SERVER_O=0x8124000d, */
/* IDL: 	PR_EMS_AB_RID_SERVER_O_ERROR=0x8124000a, */
/* IDL: 	PR_EMS_AB_RID_SERVER=0x8124001e, */
/* IDL: 	PR_EMS_AB_RID_SERVER_UNICODE=0x8124001f, */
/* IDL: 	PR_EMS_AB_ROLE_OCCUPANT_O=0x8125000d, */
/* IDL: 	PR_EMS_AB_ROLE_OCCUPANT_O_ERROR=0x8125000a, */
/* IDL: 	PR_EMS_AB_ROLE_OCCUPANT=0x8125101e, */
/* IDL: 	PR_EMS_AB_ROLE_OCCUPANT_UNICODE=0x8125101f, */
/* IDL: 	PR_EMS_AB_ROUTING_LIST=0x8126101e, */
/* IDL: 	PR_EMS_AB_ROUTING_LIST_UNICODE=0x8126101f, */
/* IDL: 	PR_EMS_AB_ROUTING_LIST_ERROR=0x8126000a, */
/* IDL: 	PR_EMS_AB_RTS_CHECKPOINT_SIZE=0x81270003, */
/* IDL: 	PR_EMS_AB_RTS_CHECKPOINT_SIZE_ERROR=0x8127000a, */
/* IDL: 	PR_EMS_AB_RTS_RECOVERY_TIMEOUT=0x81280003, */
/* IDL: 	PR_EMS_AB_RTS_RECOVERY_TIMEOUT_ERROR=0x8128000a, */
/* IDL: 	PR_EMS_AB_RTS_WINDOW_SIZE=0x81290003, */
/* IDL: 	PR_EMS_AB_RTS_WINDOW_SIZE_ERROR=0x8129000a, */
/* IDL: 	PR_EMS_AB_RUNS_ON_O=0x812a000d, */
/* IDL: 	PR_EMS_AB_RUNS_ON_O_ERROR=0x812a000a, */
/* IDL: 	PR_EMS_AB_RUNS_ON=0x812a101e, */
/* IDL: 	PR_EMS_AB_RUNS_ON_UNICODE=0x812a101f, */
/* IDL: 	PR_EMS_AB_S_SELECTOR=0x812b0102, */
/* IDL: 	PR_EMS_AB_S_SELECTOR_ERROR=0x812b000a, */
/* IDL: 	PR_EMS_AB_S_SELECTOR_INBOUND=0x812c0102, */
/* IDL: 	PR_EMS_AB_S_SELECTOR_INBOUND_ERROR=0x812c000a, */
/* IDL: 	PR_EMS_AB_SEARCH_FLAGS=0x812d0003, */
/* IDL: 	PR_EMS_AB_SEARCH_FLAGS_ERROR=0x812d000a, */
/* IDL: 	PR_EMS_AB_SEARCH_GUIDE=0x812e1102, */
/* IDL: 	PR_EMS_AB_SEARCH_GUIDE_ERROR=0x812e000a, */
/* IDL: 	PR_EMS_AB_SEE_ALSO_O=0x812f000d, */
/* IDL: 	PR_EMS_AB_SEE_ALSO_O_ERROR=0x812f000a, */
/* IDL: 	PR_EMS_AB_SEE_ALSO=0x812f101e, */
/* IDL: 	PR_EMS_AB_SEE_ALSO_UNICODE=0x812f101f, */
/* IDL: 	PR_EMS_AB_SERIAL_NUMBER=0x8130101e, */
/* IDL: 	PR_EMS_AB_SERIAL_NUMBER_UNICODE=0x8130101f, */
/* IDL: 	PR_EMS_AB_SERIAL_NUMBER_ERROR=0x8130000a, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_FIRST=0x81310003, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_FIRST_ERROR=0x8131000a, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_OTHER=0x81320003, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_OTHER_ERROR=0x8132000a, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_SECOND=0x81330003, */
/* IDL: 	PR_EMS_AB_SERVICE_ACTION_SECOND_ERROR=0x8133000a, */
/* IDL: 	PR_EMS_AB_SERVICE_RESTART_DELAY=0x81340003, */
/* IDL: 	PR_EMS_AB_SERVICE_RESTART_DELAY_ERROR=0x8134000a, */
/* IDL: 	PR_EMS_AB_SERVICE_RESTART_MESSAGE=0x8135001e, */
/* IDL: 	PR_EMS_AB_SERVICE_RESTART_MESSAGE_UNICODE=0x8135001f, */
/* IDL: 	PR_EMS_AB_SERVICE_RESTART_MESSAGE_ERROR=0x8135000a, */
/* IDL: 	PR_EMS_AB_SESSION_DISCONNECT_TIMER=0x81360003, */
/* IDL: 	PR_EMS_AB_SESSION_DISCONNECT_TIMER_ERROR=0x8136000a, */
/* IDL: 	PR_EMS_AB_SITE_AFFINITY=0x8137101e, */
/* IDL: 	PR_EMS_AB_SITE_AFFINITY_UNICODE=0x8137101f, */
/* IDL: 	PR_EMS_AB_SITE_AFFINITY_ERROR=0x8137000a, */
/* IDL: 	PR_EMS_AB_SITE_PROXY_SPACE=0x8138101e, */
/* IDL: 	PR_EMS_AB_SITE_PROXY_SPACE_UNICODE=0x8138101f, */
/* IDL: 	PR_EMS_AB_SITE_PROXY_SPACE_ERROR=0x8138000a, */
/* IDL: 	PR_EMS_AB_SPACE_LAST_COMPUTED=0x81390040, */
/* IDL: 	PR_EMS_AB_SPACE_LAST_COMPUTED_ERROR=0x8139000a, */
/* IDL: 	PR_EMS_AB_STREET_ADDRESS=0x813a001e, */
/* IDL: 	PR_EMS_AB_STREET_ADDRESS_UNICODE=0x813a001f, */
/* IDL: 	PR_EMS_AB_STREET_ADDRESS_ERROR=0x813a000a, */
/* IDL: 	PR_EMS_AB_SUB_REFS_O=0x813b000d, */
/* IDL: 	PR_EMS_AB_SUB_REFS_O_ERROR=0x813b000a, */
/* IDL: 	PR_EMS_AB_SUB_REFS=0x813b101e, */
/* IDL: 	PR_EMS_AB_SUB_REFS_UNICODE=0x813b101f, */
/* IDL: 	PR_EMS_AB_SUBMISSION_CONT_LENGTH=0x813c0003, */
/* IDL: 	PR_EMS_AB_SUBMISSION_CONT_LENGTH_ERROR=0x813c000a, */
/* IDL: 	PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT=0x813d1102, */
/* IDL: 	PR_EMS_AB_SUPPORTED_APPLICATION_CONTEXT_ERROR=0x813d000a, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_O=0x813e000d, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_O_ERROR=0x813e000a, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK=0x813e101e, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_UNICODE=0x813e101f, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_BL_O=0x813f000d, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_BL_O_ERROR=0x813f000a, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_BL=0x813f101e, */
/* IDL: 	PR_EMS_AB_SUPPORTING_STACK_BL_UNICODE=0x813f101f, */
/* IDL: 	PR_EMS_AB_T_SELECTOR=0x81400102, */
/* IDL: 	PR_EMS_AB_T_SELECTOR_ERROR=0x8140000a, */
/* IDL: 	PR_EMS_AB_T_SELECTOR_INBOUND=0x81410102, */
/* IDL: 	PR_EMS_AB_T_SELECTOR_INBOUND_ERROR=0x8141000a, */
/* IDL: 	PR_EMS_AB_TARGET_MTAS=0x8142101e, */
/* IDL: 	PR_EMS_AB_TARGET_MTAS_UNICODE=0x8142101f, */
/* IDL: 	PR_EMS_AB_TARGET_MTAS_ERROR=0x8142000a, */
/* IDL: 	PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER=0x81431102, */
/* IDL: 	PR_EMS_AB_TELETEX_TERMINAL_IDENTIFIER_ERROR=0x8143000a, */
/* IDL: 	PR_EMS_AB_TEMP_ASSOC_THRESHOLD=0x81440003, */
/* IDL: 	PR_EMS_AB_TEMP_ASSOC_THRESHOLD_ERROR=0x8144000a, */
/* IDL: 	PR_EMS_AB_TOMBSTONE_LIFETIME=0x81450003, */
/* IDL: 	PR_EMS_AB_TOMBSTONE_LIFETIME_ERROR=0x8145000a, */
/* IDL: 	PR_EMS_AB_TRACKING_LOG_PATH_NAME=0x8146001e, */
/* IDL: 	PR_EMS_AB_TRACKING_LOG_PATH_NAME_UNICODE=0x8146001f, */
/* IDL: 	PR_EMS_AB_TRACKING_LOG_PATH_NAME_ERROR=0x8146000a, */
/* IDL: 	PR_EMS_AB_TRANS_RETRY_MINS=0x81470003, */
/* IDL: 	PR_EMS_AB_TRANS_RETRY_MINS_ERROR=0x8147000a, */
/* IDL: 	PR_EMS_AB_TRANS_TIMEOUT_MINS=0x81480003, */
/* IDL: 	PR_EMS_AB_TRANS_TIMEOUT_MINS_ERROR=0x8148000a, */
/* IDL: 	PR_EMS_AB_TRANSFER_RETRY_INTERVAL=0x81490003, */
/* IDL: 	PR_EMS_AB_TRANSFER_RETRY_INTERVAL_ERROR=0x8149000a, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT=0x814a0003, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_NON_URGENT_ERROR=0x814a000a, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL=0x814b0003, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_NORMAL_ERROR=0x814b000a, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_URGENT=0x814c0003, */
/* IDL: 	PR_EMS_AB_TRANSFER_TIMEOUT_URGENT_ERROR=0x814c000a, */
/* IDL: 	PR_EMS_AB_TRANSLATION_TABLE_USED=0x814d0003, */
/* IDL: 	PR_EMS_AB_TRANSLATION_TABLE_USED_ERROR=0x814d000a, */
/* IDL: 	PR_EMS_AB_TRANSPORT_EXPEDITED_DATA=0x814e000b, */
/* IDL: 	PR_EMS_AB_TRANSPORT_EXPEDITED_DATA_ERROR=0x814e000a, */
/* IDL: 	PR_EMS_AB_TRUST_LEVEL=0x814f0003, */
/* IDL: 	PR_EMS_AB_TRUST_LEVEL_ERROR=0x814f000a, */
/* IDL: 	PR_EMS_AB_TURN_REQUEST_THRESHOLD=0x81500003, */
/* IDL: 	PR_EMS_AB_TURN_REQUEST_THRESHOLD_ERROR=0x8150000a, */
/* IDL: 	PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY=0x8151000b, */
/* IDL: 	PR_EMS_AB_TWO_WAY_ALTERNATE_FACILITY_ERROR=0x8151000a, */
/* IDL: 	PR_EMS_AB_UNAUTH_ORIG_BL_O=0x8152000d, */
/* IDL: 	PR_EMS_AB_UNAUTH_ORIG_BL_O_ERROR=0x8152000a, */
/* IDL: 	PR_EMS_AB_UNAUTH_ORIG_BL=0x8152101e, */
/* IDL: 	PR_EMS_AB_UNAUTH_ORIG_BL_UNICODE=0x8152101f, */
/* IDL: 	PR_EMS_AB_USER_PASSWORD=0x81531102, */
/* IDL: 	PR_EMS_AB_USER_PASSWORD_ERROR=0x8153000a, */
/* IDL: 	PR_EMS_AB_USN_CREATED=0x81540003, */
/* IDL: 	PR_EMS_AB_USN_CREATED_ERROR=0x8154000a, */
/* IDL: 	PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED=0x81550003, */
/* IDL: 	PR_EMS_AB_USN_DSA_LAST_OBJ_REMOVED_ERROR=0x8155000a, */
/* IDL: 	PR_EMS_AB_USN_LAST_OBJ_REM=0x81560003, */
/* IDL: 	PR_EMS_AB_USN_LAST_OBJ_REM_ERROR=0x8156000a, */
/* IDL: 	PR_EMS_AB_USN_SOURCE=0x81570003, */
/* IDL: 	PR_EMS_AB_USN_SOURCE_ERROR=0x8157000a, */
/* IDL: 	PR_EMS_AB_X121_ADDRESS=0x8158101e, */
/* IDL: 	PR_EMS_AB_X121_ADDRESS_UNICODE=0x8158101f, */
/* IDL: 	PR_EMS_AB_X121_ADDRESS_ERROR=0x8158000a, */
/* IDL: 	PR_EMS_AB_X25_CALL_USER_DATA_INCOMING=0x81590102, */
/* IDL: 	PR_EMS_AB_X25_CALL_USER_DATA_INCOMING_ERROR=0x8159000a, */
/* IDL: 	PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING=0x815a0102, */
/* IDL: 	PR_EMS_AB_X25_CALL_USER_DATA_OUTGOING_ERROR=0x815a000a, */
/* IDL: 	PR_EMS_AB_X25_FACILITIES_DATA_INCOMING=0x815b0102, */
/* IDL: 	PR_EMS_AB_X25_FACILITIES_DATA_INCOMING_ERROR=0x815b000a, */
/* IDL: 	PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING=0x815c0102, */
/* IDL: 	PR_EMS_AB_X25_FACILITIES_DATA_OUTGOING_ERROR=0x815c000a, */
/* IDL: 	PR_EMS_AB_X25_LEASED_LINE_PORT=0x815d0102, */
/* IDL: 	PR_EMS_AB_X25_LEASED_LINE_PORT_ERROR=0x815d000a, */
/* IDL: 	PR_EMS_AB_X25_LEASED_OR_SWITCHED=0x815e000b, */
/* IDL: 	PR_EMS_AB_X25_LEASED_OR_SWITCHED_ERROR=0x815e000a, */
/* IDL: 	PR_EMS_AB_X25_REMOTE_MTA_PHONE=0x815f001e, */
/* IDL: 	PR_EMS_AB_X25_REMOTE_MTA_PHONE_UNICODE=0x815f001f, */
/* IDL: 	PR_EMS_AB_X25_REMOTE_MTA_PHONE_ERROR=0x815f000a, */
/* IDL: 	PR_EMS_AB_X400_ATTACHMENT_TYPE=0x81600102, */
/* IDL: 	PR_EMS_AB_X400_ATTACHMENT_TYPE_ERROR=0x8160000a, */
/* IDL: 	PR_EMS_AB_X400_SELECTOR_SYNTAX=0x81610003, */
/* IDL: 	PR_EMS_AB_X400_SELECTOR_SYNTAX_ERROR=0x8161000a, */
/* IDL: 	PR_EMS_AB_X500_ACCESS_CONTROL_LIST=0x81620102, */
/* IDL: 	PR_EMS_AB_X500_ACCESS_CONTROL_LIST_ERROR=0x8162000a, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT=0x81630003, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_NON_URGENT_ERROR=0x8163000a, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_NORMAL=0x81640003, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_NORMAL_ERROR=0x8164000a, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_URGENT=0x81650003, */
/* IDL: 	PR_EMS_AB_XMIT_TIMEOUT_URGENT_ERROR=0x8165000a, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_GUID=0x81660102, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_GUID_ERROR=0x8166000a, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_SERVER_O=0x8167000d, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_SERVER_O_ERROR=0x8167000a, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_SERVER=0x8167001e, */
/* IDL: 	PR_EMS_AB_SITE_FOLDER_SERVER_UNICODE=0x8167001f, */
/* IDL: 	PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE=0x81680003, */
/* IDL: 	PR_EMS_AB_REPLICATION_MAIL_MSG_SIZE_ERROR=0x8168000a, */
/* IDL: 	PR_EMS_AB_MAXIMUM_OBJECT_ID=0x81690102, */
/* IDL: 	PR_EMS_AB_MAXIMUM_OBJECT_ID_ERROR=0x8169000a, */
/* IDL: 	PR_EMS_AB_NETWORK_ADDRESS=0x8170101e, */
/* IDL: 	PR_EMS_AB_NETWORK_ADDRESS_UNICODE=0x8170101f, */
/* IDL: 	PR_EMS_AB_NETWORK_ADDRESS_ERROR=0x8170000a, */
/* IDL: 	PR_EMS_AB_LDAP_DISPLAY_NAME=0x8171101e, */
/* IDL: 	PR_EMS_AB_LDAP_DISPLAY_NAME_UNICODE=0x8171101f, */
/* IDL: 	PR_EMS_AB_LDAP_DISPLAY_NAME_ERROR=0x8171000a, */
/* IDL: 	PR_EMS_AB_SCHEMA_FLAGS=0x81730003, */
/* IDL: 	PR_EMS_AB_SCHEMA_FLAGS_ERROR=0x8173000a, */
/* IDL: 	PR_EMS_AB_BRIDGEHEAD_SERVERS_O=0x8174000d, */
/* IDL: 	PR_EMS_AB_BRIDGEHEAD_SERVERS_O_ERROR=0x8174000a, */
/* IDL: 	PR_EMS_AB_BRIDGEHEAD_SERVERS=0x8174101e, */
/* IDL: 	PR_EMS_AB_BRIDGEHEAD_SERVERS_UNICODE=0x8174101f, */
/* IDL: 	PR_EMS_AB_WWW_HOME_PAGE=0x8175001e, */
/* IDL: 	PR_EMS_AB_WWW_HOME_PAGE_UNICODE=0x8175001f, */
/* IDL: 	PR_EMS_AB_WWW_HOME_PAGE_ERROR=0x8175000a, */
/* IDL: 	PR_EMS_AB_NNTP_CONTENT_FORMAT=0x8176001e, */
/* IDL: 	PR_EMS_AB_NNTP_CONTENT_FORMAT_UNICODE=0x8176001f, */
/* IDL: 	PR_EMS_AB_NNTP_CONTENT_FORMAT_ERROR=0x8176000a, */
/* IDL: 	PR_EMS_AB_POP_CONTENT_FORMAT=0x8177001e, */
/* IDL: 	PR_EMS_AB_POP_CONTENT_FORMAT_UNICODE=0x8177001f, */
/* IDL: 	PR_EMS_AB_POP_CONTENT_FORMAT_ERROR=0x8177000a, */
/* IDL: 	PR_EMS_AB_LANGUAGE=0x81780003, */
/* IDL: 	PR_EMS_AB_LANGUAGE_ERROR=0x8178000a, */
/* IDL: 	PR_EMS_AB_POP_CHARACTER_SET=0x8179001e, */
/* IDL: 	PR_EMS_AB_POP_CHARACTER_SET_UNICODE=0x8179001f, */
/* IDL: 	PR_EMS_AB_POP_CHARACTER_SET_ERROR=0x8179000a, */
/* IDL: 	PR_EMS_AB_USN_INTERSITE=0x817a0003, */
/* IDL: 	PR_EMS_AB_USN_INTERSITE_ERROR=0x817a000a, */
/* IDL: 	PR_EMS_AB_SUB_SITE=0x817b001e, */
/* IDL: 	PR_EMS_AB_SUB_SITE_UNICODE=0x817b001f, */
/* IDL: 	PR_EMS_AB_SUB_SITE_ERROR=0x817b000a, */
/* IDL: 	PR_EMS_AB_SCHEMA_VERSION=0x817c1003, */
/* IDL: 	PR_EMS_AB_SCHEMA_VERSION_ERROR=0x817c000a, */
/* IDL: 	PR_EMS_AB_NNTP_CHARACTER_SET=0x817d001e, */
/* IDL: 	PR_EMS_AB_NNTP_CHARACTER_SET_UNICODE=0x817d001f, */
/* IDL: 	PR_EMS_AB_NNTP_CHARACTER_SET_ERROR=0x817d000a, */
/* IDL: 	PR_EMS_AB_USE_SERVER_VALUES=0x817e000b, */
/* IDL: 	PR_EMS_AB_USE_SERVER_VALUES_ERROR=0x817e000a, */
/* IDL: 	PR_EMS_AB_ENABLED_PROTOCOLS=0x817f0003, */
/* IDL: 	PR_EMS_AB_ENABLED_PROTOCOLS_ERROR=0x817f000a, */
/* IDL: 	PR_EMS_AB_CONNECTION_LIST_FILTER=0x81800102, */
/* IDL: 	PR_EMS_AB_CONNECTION_LIST_FILTER_ERROR=0x8180000a, */
/* IDL: 	PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES=0x8181101e, */
/* IDL: 	PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_UNICODE=0x8181101f, */
/* IDL: 	PR_EMS_AB_AVAILABLE_AUTHORIZATION_PACKAGES_ERROR=0x8181000a, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET_LIST=0x8182101e, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET_LIST_UNICODE=0x8182101f, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET_LIST_ERROR=0x8182000a, */
/* IDL: 	PR_EMS_AB_USE_SITE_VALUES=0x8183000b, */
/* IDL: 	PR_EMS_AB_USE_SITE_VALUES_ERROR=0x8183000a, */
/* IDL: 	PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES=0x8184101e, */
/* IDL: 	PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_UNICODE=0x8184101f, */
/* IDL: 	PR_EMS_AB_ENABLED_AUTHORIZATION_PACKAGES_ERROR=0x8184000a, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET=0x8185001e, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET_UNICODE=0x8185001f, */
/* IDL: 	PR_EMS_AB_CHARACTER_SET_ERROR=0x8185000a, */
/* IDL: 	PR_EMS_AB_CONTENT_TYPE=0x81860003, */
/* IDL: 	PR_EMS_AB_CONTENT_TYPE_ERROR=0x8186000a, */
/* IDL: 	PR_EMS_AB_ANONYMOUS_ACCESS=0x8187000b, */
/* IDL: 	PR_EMS_AB_ANONYMOUS_ACCESS_ERROR=0x8187000a, */
/* IDL: 	PR_EMS_AB_CONTROL_MSG_FOLDER_ID=0x81880102, */
/* IDL: 	PR_EMS_AB_CONTROL_MSG_FOLDER_ID_ERROR=0x8188000a, */
/* IDL: 	PR_EMS_AB_USENET_SITE_NAME=0x8189001e, */
/* IDL: 	PR_EMS_AB_USENET_SITE_NAME_UNICODE=0x8189001f, */
/* IDL: 	PR_EMS_AB_USENET_SITE_NAME_ERROR=0x8189000a, */
/* IDL: 	PR_EMS_AB_CONTROL_MSG_RULES=0x818a0102, */
/* IDL: 	PR_EMS_AB_CONTROL_MSG_RULES_ERROR=0x818a000a, */
/* IDL: 	PR_EMS_AB_AVAILABLE_DISTRIBUTIONS=0x818b001e, */
/* IDL: 	PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_UNICODE=0x818b001f, */
/* IDL: 	PR_EMS_AB_AVAILABLE_DISTRIBUTIONS_ERROR=0x818b000a, */
/* IDL: 	PR_EMS_AB_OUTBOUND_HOST=0x818d0102, */
/* IDL: 	PR_EMS_AB_OUTBOUND_HOST_ERROR=0x818d000a, */
/* IDL: 	PR_EMS_AB_INBOUND_HOST=0x818e101e, */
/* IDL: 	PR_EMS_AB_INBOUND_HOST_UNICODE=0x818e101f, */
/* IDL: 	PR_EMS_AB_INBOUND_HOST_ERROR=0x818e000a, */
/* IDL: 	PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT=0x818f0003, */
/* IDL: 	PR_EMS_AB_OUTGOING_MSG_SIZE_LIMIT_ERROR=0x818f000a, */
/* IDL: 	PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT=0x81900003, */
/* IDL: 	PR_EMS_AB_INCOMING_MSG_SIZE_LIMIT_ERROR=0x8190000a, */
/* IDL: 	PR_EMS_AB_SEND_TNEF=0x8191000b, */
/* IDL: 	PR_EMS_AB_SEND_TNEF_ERROR=0x8191000a, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM=0x81920102, */
/* IDL: 	PR_EMS_AB_AUTHORIZED_PASSWORD_CONFIRM_ERROR=0x8192000a, */
/* IDL: 	PR_EMS_AB_INBOUND_NEWSFEED=0x8193001e, */
/* IDL: 	PR_EMS_AB_INBOUND_NEWSFEED_UNICODE=0x8193001f, */
/* IDL: 	PR_EMS_AB_INBOUND_NEWSFEED_ERROR=0x8193000a, */
/* IDL: 	PR_EMS_AB_NEWSFEED_TYPE=0x81940003, */
/* IDL: 	PR_EMS_AB_NEWSFEED_TYPE_ERROR=0x8194000a, */
/* IDL: 	PR_EMS_AB_OUTBOUND_NEWSFEED=0x8195001e, */
/* IDL: 	PR_EMS_AB_OUTBOUND_NEWSFEED_UNICODE=0x8195001f, */
/* IDL: 	PR_EMS_AB_OUTBOUND_NEWSFEED_ERROR=0x8195000a, */
/* IDL: 	PR_EMS_AB_NEWSGROUP_LIST=0x81960102, */
/* IDL: 	PR_EMS_AB_NEWSGROUP_LIST_ERROR=0x8196000a, */
/* IDL: 	PR_EMS_AB_NNTP_DISTRIBUTIONS=0x8197101e, */
/* IDL: 	PR_EMS_AB_NNTP_DISTRIBUTIONS_UNICODE=0x8197101f, */
/* IDL: 	PR_EMS_AB_NNTP_DISTRIBUTIONS_ERROR=0x8197000a, */
/* IDL: 	PR_EMS_AB_NEWSGROUP=0x8198001e, */
/* IDL: 	PR_EMS_AB_NEWSGROUP_UNICODE=0x8198001f, */
/* IDL: 	PR_EMS_AB_NEWSGROUP_ERROR=0x8198000a, */
/* IDL: 	PR_EMS_AB_MODERATOR=0x8199001e, */
/* IDL: 	PR_EMS_AB_MODERATOR_UNICODE=0x8199001f, */
/* IDL: 	PR_EMS_AB_MODERATOR_ERROR=0x8199000a, */
/* IDL: 	PR_EMS_AB_AUTHENTICATION_TO_USE=0x819a001e, */
/* IDL: 	PR_EMS_AB_AUTHENTICATION_TO_USE_UNICODE=0x819a001f, */
/* IDL: 	PR_EMS_AB_AUTHENTICATION_TO_USE_ERROR=0x819a000a, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_GAL=0x819b000b, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_GAL_ERROR=0x819b000a, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_GAL_LIMIT=0x819c0003, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_GAL_LIMIT_ERROR=0x819c000a, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_PF=0x819e1102, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_PF_ERROR=0x819e000a, */
/* IDL: 	PR_EMS_AB_X500_RDN=0x81a1001e, */
/* IDL: 	PR_EMS_AB_X500_RDN_UNICODE=0x81a1001f, */
/* IDL: 	PR_EMS_AB_X500_RDN_ERROR=0x81a1000a, */
/* IDL: 	PR_EMS_AB_X500_NC=0x81a2001e, */
/* IDL: 	PR_EMS_AB_X500_NC_UNICODE=0x81a2001f, */
/* IDL: 	PR_EMS_AB_X500_NC_ERROR=0x81a2000a, */
/* IDL: 	PR_EMS_AB_REFERRAL_LIST=0x81a3101e, */
/* IDL: 	PR_EMS_AB_REFERRAL_LIST_UNICODE=0x81a3101f, */
/* IDL: 	PR_EMS_AB_REFERRAL_LIST_ERROR=0x81a3000a, */
/* IDL: 	PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG=0x81a4000b, */
/* IDL: 	PR_EMS_AB_NNTP_DISTRIBUTIONS_FLAG_ERROR=0x81a4000a, */
/* IDL: 	PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O=0x81a5000d, */
/* IDL: 	PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_O_ERROR=0x81a5000a, */
/* IDL: 	PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP=0x81a5001e, */
/* IDL: 	PR_EMS_AB_ASSOC_PROTOCOL_CFG_NNTP_UNICODE=0x81a5001f, */
/* IDL: 	PR_EMS_AB_NNTP_NEWSFEEDS_O=0x81a6000d, */
/* IDL: 	PR_EMS_AB_NNTP_NEWSFEEDS_O_ERROR=0x81a6000a, */
/* IDL: 	PR_EMS_AB_NNTP_NEWSFEEDS=0x81a6101e, */
/* IDL: 	PR_EMS_AB_NNTP_NEWSFEEDS_UNICODE=0x81a6101f, */
/* IDL: 	PR_EMS_AB_ENABLED_PROTOCOL_CFG=0x81a8000b, */
/* IDL: 	PR_EMS_AB_ENABLED_PROTOCOL_CFG_ERROR=0x81a8000a, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES=0x81a9101e, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_UNICODE=0x81a9101f, */
/* IDL: 	PR_EMS_AB_HTTP_PUB_AB_ATTRIBUTES_ERROR=0x81a9000a, */
/* IDL: 	PR_EMS_AB_HTTP_SERVERS=0x81ab101e, */
/* IDL: 	PR_EMS_AB_HTTP_SERVERS_UNICODE=0x81ab101f, */
/* IDL: 	PR_EMS_AB_HTTP_SERVERS_ERROR=0x81ab000a, */
/* IDL: 	PR_EMS_AB_MODERATED=0x81ac000b, */
/* IDL: 	PR_EMS_AB_MODERATED_ERROR=0x81ac000a, */
/* IDL: 	PR_EMS_AB_RAS_ACCOUNT=0x81ad001e, */
/* IDL: 	PR_EMS_AB_RAS_ACCOUNT_UNICODE=0x81ad001f, */
/* IDL: 	PR_EMS_AB_RAS_ACCOUNT_ERROR=0x81ad000a, */
/* IDL: 	PR_EMS_AB_RAS_PASSWORD=0x81ae0102, */
/* IDL: 	PR_EMS_AB_RAS_PASSWORD_ERROR=0x81ae000a, */
/* IDL: 	PR_EMS_AB_INCOMING_PASSWORD=0x81af0102, */
/* IDL: 	PR_EMS_AB_INCOMING_PASSWORD_ERROR=0x81af000a, */
/* IDL: 	PR_EMS_AB_OUTBOUND_HOST_TYPE=0x81b0000b, */
/* IDL: 	PR_EMS_AB_OUTBOUND_HOST_TYPE_ERROR=0x81b0000a, */
/* IDL: 	PR_EMS_AB_PROXY_GENERATION_ENABLED=0x81b1000b, */
/* IDL: 	PR_EMS_AB_PROXY_GENERATION_ENABLED_ERROR=0x81b1000a, */
/* IDL: 	PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID=0x81b20102, */
/* IDL: 	PR_EMS_AB_ROOT_NEWSGROUPS_FOLDER_ID_ERROR=0x81b2000a, */
/* IDL: 	PR_EMS_AB_CONNECTION_TYPE=0x81b3000b, */
/* IDL: 	PR_EMS_AB_CONNECTION_TYPE_ERROR=0x81b3000a, */
/* IDL: 	PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE=0x81b40003, */
/* IDL: 	PR_EMS_AB_CONNECTION_LIST_FILTER_TYPE_ERROR=0x81b4000a, */
/* IDL: 	PR_EMS_AB_PORT_NUMBER=0x81b50003, */
/* IDL: 	PR_EMS_AB_PORT_NUMBER_ERROR=0x81b5000a, */
/* IDL: 	PR_EMS_AB_PROTOCOL_SETTINGS=0x81b6101e, */
/* IDL: 	PR_EMS_AB_PROTOCOL_SETTINGS_UNICODE=0x81b6101f, */
/* IDL: 	PR_EMS_AB_PROTOCOL_SETTINGS_ERROR=0x81b6000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_1=0x81b7001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_1_UNICODE=0x81b7001f, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_1_ERROR=0x81b7000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_2=0x81b8001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_2_UNICODE=0x81b8001f, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_2_ERROR=0x81b8000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_3=0x81b9001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_3_UNICODE=0x81b9001f, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_3_ERROR=0x81b9000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_4=0x81ba001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_4_UNICODE=0x81ba001f, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_4_ERROR=0x81ba000a, */
/* IDL: 	PR_EMS_AB_VIEW_SITE=0x81be001e, */
/* IDL: 	PR_EMS_AB_VIEW_SITE_UNICODE=0x81be001f, */
/* IDL: 	PR_EMS_AB_VIEW_SITE_ERROR=0x81be000a, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_1=0x81bf001e, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_1_UNICODE=0x81bf001f, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_1_ERROR=0x81bf000a, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_2=0x81c0001e, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_2_UNICODE=0x81c0001f, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_2_ERROR=0x81c0000a, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_3=0x81c1001e, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_3_UNICODE=0x81c1001f, */
/* IDL: 	PR_EMS_AB_VIEW_CONTAINER_3_ERROR=0x81c1000a, */
/* IDL: 	PR_EMS_AB_PROMO_EXPIRATION=0x81c20040, */
/* IDL: 	PR_EMS_AB_PROMO_EXPIRATION_ERROR=0x81c2000a, */
/* IDL: 	PR_EMS_AB_DISABLED_GATEWAY_PROXY=0x81c3101e, */
/* IDL: 	PR_EMS_AB_DISABLED_GATEWAY_PROXY_UNICODE=0x81c3101f, */
/* IDL: 	PR_EMS_AB_DISABLED_GATEWAY_PROXY_ERROR=0x81c3000a, */
/* IDL: 	PR_EMS_AB_COMPROMISED_KEY_LIST=0x81c40102, */
/* IDL: 	PR_EMS_AB_COMPROMISED_KEY_LIST_ERROR=0x81c4000a, */
/* IDL: 	PR_EMS_AB_INSADMIN_O=0x81c5000d, */
/* IDL: 	PR_EMS_AB_INSADMIN_O_ERROR=0x81c5000a, */
/* IDL: 	PR_EMS_AB_INSADMIN=0x81c5001e, */
/* IDL: 	PR_EMS_AB_INSADMIN_UNICODE=0x81c5001f, */
/* IDL: 	PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT=0x81c6000b, */
/* IDL: 	PR_EMS_AB_OVERRIDE_NNTP_CONTENT_FORMAT_ERROR=0x81c6000a, */
/* IDL: 	PR_EMS_AB_OBJ_VIEW_CONTAINERS_O=0x81c7000d, */
/* IDL: 	PR_EMS_AB_OBJ_VIEW_CONTAINERS_O_ERROR=0x81c7000a, */
/* IDL: 	PR_EMS_AB_OBJ_VIEW_CONTAINERS=0x81c7101e, */
/* IDL: 	PR_EMS_AB_OBJ_VIEW_CONTAINERS_UNICODE=0x81c7101f, */
/* IDL: 	PR_EMS_AB_VIEW_FLAGS=0x8c180003, */
/* IDL: 	PR_EMS_AB_VIEW_FLAGS_ERROR=0x8c18000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR=0x8c19001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_UNICODE=0x8c19001f, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_STR_ERROR=0x8c19000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O=0x8c1a000d, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_O_ERROR=0x8c1a000a, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN=0x8c1a001e, */
/* IDL: 	PR_EMS_AB_GROUP_BY_ATTR_VALUE_DN_UNICODE=0x8c1a001f, */
/* IDL: 	PR_EMS_AB_VIEW_DEFINITION=0x8c1b1102, */
/* IDL: 	PR_EMS_AB_VIEW_DEFINITION_ERROR=0x8c1b000a, */
/* IDL: 	PR_EMS_AB_MIME_TYPES=0x8c1c0102, */
/* IDL: 	PR_EMS_AB_MIME_TYPES_ERROR=0x8c1c000a, */
/* IDL: 	PR_EMS_AB_LDAP_SEARCH_CFG=0x8c1d0003, */
/* IDL: 	PR_EMS_AB_LDAP_SEARCH_CFG_ERROR=0x8c1d000a, */
/* IDL: 	PR_EMS_AB_INBOUND_DN_O=0x8c1e000d, */
/* IDL: 	PR_EMS_AB_INBOUND_DN_O_ERROR=0x8c1e000a, */
/* IDL: 	PR_EMS_AB_INBOUND_DN=0x8c1e001e, */
/* IDL: 	PR_EMS_AB_INBOUND_DN_UNICODE=0x8c1e001f, */
/* IDL: 	PR_EMS_AB_INBOUND_NEWSFEED_TYPE=0x8c1f000b, */
/* IDL: 	PR_EMS_AB_INBOUND_NEWSFEED_TYPE_ERROR=0x8c1f000a, */
/* IDL: 	PR_EMS_AB_INBOUND_ACCEPT_ALL=0x8c20000b, */
/* IDL: 	PR_EMS_AB_INBOUND_ACCEPT_ALL_ERROR=0x8c20000a, */
/* IDL: 	PR_EMS_AB_ENABLED=0x8c21000b, */
/* IDL: 	PR_EMS_AB_ENABLED_ERROR=0x8c21000a, */
/* IDL: 	PR_EMS_AB_PRESERVE_INTERNET_CONTENT=0x8c22000b, */
/* IDL: 	PR_EMS_AB_PRESERVE_INTERNET_CONTENT_ERROR=0x8c22000a, */
/* IDL: 	PR_EMS_AB_DISABLE_DEFERRED_COMMIT=0x8c23000b, */
/* IDL: 	PR_EMS_AB_DISABLE_DEFERRED_COMMIT_ERROR=0x8c23000a, */
/* IDL: 	PR_EMS_AB_CLIENT_ACCESS_ENABLED=0x8c24000b, */
/* IDL: 	PR_EMS_AB_CLIENT_ACCESS_ENABLED_ERROR=0x8c24000a, */
/* IDL: 	PR_EMS_AB_REQUIRE_SSL=0x8c25000b, */
/* IDL: 	PR_EMS_AB_REQUIRE_SSL_ERROR=0x8c25000a, */
/* IDL: 	PR_EMS_AB_ANONYMOUS_ACCOUNT=0x8c26001e, */
/* IDL: 	PR_EMS_AB_ANONYMOUS_ACCOUNT_UNICODE=0x8c26001f, */
/* IDL: 	PR_EMS_AB_ANONYMOUS_ACCOUNT_ERROR=0x8c26000a, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_CHAIN_V3=0x8c270102, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_CHAIN_V3_ERROR=0x8c27000a, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3=0x8c280102, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V3_ERROR=0x8c28000a, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1=0x8c290102, */
/* IDL: 	PR_EMS_AB_CERTIFICATE_REVOCATION_LIST_V1_ERROR=0x8c29000a, */
/* IDL: 	PR_EMS_AB_CROSS_CERTIFICATE_CRL=0x8c301102, */
/* IDL: 	PR_EMS_AB_CROSS_CERTIFICATE_CRL_ERROR=0x8c30000a, */
/* IDL: 	PR_EMS_AB_SEND_EMAIL_MESSAGE=0x8c31000b, */
/* IDL: 	PR_EMS_AB_SEND_EMAIL_MESSAGE_ERROR=0x8c31000a, */
/* IDL: 	PR_EMS_AB_ENABLE_COMPATIBILITY=0x8c32000b, */
/* IDL: 	PR_EMS_AB_ENABLE_COMPATIBILITY_ERROR=0x8c32000a, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_NA=0x8c33101e, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_NA_UNICODE=0x8c33101f, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_NA_ERROR=0x8c33000a, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_OTHER=0x8c34101e, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_OTHER_UNICODE=0x8c34101f, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_LIST_OTHER_ERROR=0x8c34000a, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_NA=0x8c35001e, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_NA_UNICODE=0x8c35001f, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_NA_ERROR=0x8c35000a, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_OTHER=0x8c36001e, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_UNICODE=0x8c36001f, */
/* IDL: 	PR_EMS_AB_SMIME_ALG_SELECTED_OTHER_ERROR=0x8c36000a, */
/* IDL: 	PR_EMS_AB_DEFAULT_MESSAGE_FORMAT=0x8c37000b, */
/* IDL: 	PR_EMS_AB_DEFAULT_MESSAGE_FORMAT_ERROR=0x8c37000a, */
/* IDL: 	PR_EMS_AB_TYPE=0x8c38001e, */
/* IDL: 	PR_EMS_AB_TYPE_UNICODE=0x8c38001f, */
/* IDL: 	PR_EMS_AB_TYPE_ERROR=0x8c38000a, */
/* IDL: 	PR_EMS_AB_DO_OAB_VERSION=0x8c3a0003, */
/* IDL: 	PR_EMS_AB_DO_OAB_VERSION_ERROR=0x8c3a000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID=0x8c3b0102, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_SYSTEM_GUID_ERROR=0x8c3b000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_USER_ID=0x8c3c001e, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_USER_ID_UNICODE=0x8c3c001f, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_USER_ID_ERROR=0x8c3c000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_PASSWORD=0x8c3d001e, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_PASSWORD_UNICODE=0x8c3d001f, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_PASSWORD_ERROR=0x8c3d000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_RECORDED_NAME=0x8c3e0102, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_RECORDED_NAME_ERROR=0x8c3e000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_GREETINGS=0x8c3f101e, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_GREETINGS_UNICODE=0x8c3f101f, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_GREETINGS_ERROR=0x8c3f000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_FLAGS=0x8c401102, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_FLAGS_ERROR=0x8c40000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_VOLUME=0x8c410003, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_VOLUME_ERROR=0x8c41000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_SPEED=0x8c420003, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_SPEED_ERROR=0x8c42000a, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH=0x8c431003, */
/* IDL: 	PR_EMS_AB_VOICE_MAIL_RECORDING_LENGTH_ERROR=0x8c43000a, */
/* IDL: 	PR_EMS_AB_DISPLAY_NAME_SUFFIX=0x8c44001e, */
/* IDL: 	PR_EMS_AB_DISPLAY_NAME_SUFFIX_UNICODE=0x8c44001f, */
/* IDL: 	PR_EMS_AB_DISPLAY_NAME_SUFFIX_ERROR=0x8c44000a, */
/* IDL: 	PR_EMS_AB_ATTRIBUTE_CERTIFICATE=0x8c451102, */
/* IDL: 	PR_EMS_AB_ATTRIBUTE_CERTIFICATE_ERROR=0x8c45000a, */
/* IDL: 	PR_EMS_AB_DELTA_REVOCATION_LIST=0x8c461102, */
/* IDL: 	PR_EMS_AB_DELTA_REVOCATION_LIST_ERROR=0x8c46000a, */
/* IDL: 	PR_EMS_AB_SECURITY_POLICY=0x8c471102, */
/* IDL: 	PR_EMS_AB_SECURITY_POLICY_ERROR=0x8c47000a, */
/* IDL: 	PR_EMS_AB_SUPPORT_SMIME_SIGNATURES=0x8c48000b, */
/* IDL: 	PR_EMS_AB_SUPPORT_SMIME_SIGNATURES_ERROR=0x8c48000a, */
/* IDL: 	PR_EMS_AB_DELEGATE_USER=0x8c49000b, */
/* IDL: 	PR_EMS_AB_DELEGATE_USER_ERROR=0x8c49000a, */
/* IDL: 	PR_EMS_AB_LIST_PUBLIC_FOLDERS=0x8c50000b, */
/* IDL: 	PR_EMS_AB_LIST_PUBLIC_FOLDERS_ERROR=0x8c50000a, */
/* IDL: 	PR_EMS_AB_LABELEDURI=0x8c51001e, */
/* IDL: 	PR_EMS_AB_LABELEDURI_UNICODE=0x8c51001f, */
/* IDL: 	PR_EMS_AB_LABELEDURI_ERROR=0x8c51000a, */
/* IDL: 	PR_EMS_AB_RETURN_EXACT_MSG_SIZE=0x8c52000b, */
/* IDL: 	PR_EMS_AB_RETURN_EXACT_MSG_SIZE_ERROR=0x8c52000a, */
/* IDL: 	PR_EMS_AB_GENERATION_QUALIFIER=0x8c53001e, */
/* IDL: 	PR_EMS_AB_GENERATION_QUALIFIER_UNICODE=0x8c53001f, */
/* IDL: 	PR_EMS_AB_GENERATION_QUALIFIER_ERROR=0x8c53000a, */
/* IDL: 	PR_EMS_AB_HOUSE_IDENTIFIER=0x8c54001e, */
/* IDL: 	PR_EMS_AB_HOUSE_IDENTIFIER_UNICODE=0x8c54001f, */
/* IDL: 	PR_EMS_AB_HOUSE_IDENTIFIER_ERROR=0x8c54000a, */
/* IDL: 	PR_EMS_AB_SUPPORTED_ALGORITHMS=0x8c550102, */
/* IDL: 	PR_EMS_AB_SUPPORTED_ALGORITHMS_ERROR=0x8c55000a, */
/* IDL: 	PR_EMS_AB_DMD_NAME=0x8c56001e, */
/* IDL: 	PR_EMS_AB_DMD_NAME_UNICODE=0x8c56001f, */
/* IDL: 	PR_EMS_AB_DMD_NAME_ERROR=0x8c56000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_11=0x8c57001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_11_UNICODE=0x8c57001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_11_ERROR=0x8c57000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_12=0x8c58001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_12_UNICODE=0x8c58001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_12_ERROR=0x8c58000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_13=0x8c59001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_13_UNICODE=0x8c59001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_13_ERROR=0x8c59000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_14=0x8c60001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_14_UNICODE=0x8c60001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_14_ERROR=0x8c60000a, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_15=0x8c61001e, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_15_UNICODE=0x8c61001f, */
/* IDL: 	PR_EMS_AB_EXTENSION_ATTRIBUTE_15_ERROR=0x8c61000a, */
/* IDL: 	PR_EMS_AB_REPLICATED_OBJECT_VERSION=0x8c620003, */
/* IDL: 	PR_EMS_AB_REPLICATED_OBJECT_VERSION_ERROR=0x8c62000a, */
/* IDL: 	PR_EMS_AB_MAIL_DROP=0x8c63001e, */
/* IDL: 	PR_EMS_AB_MAIL_DROP_UNICODE=0x8c63001f, */
/* IDL: 	PR_EMS_AB_MAIL_DROP_ERROR=0x8c63000a, */
/* IDL: 	PR_EMS_AB_FORWARDING_ADDRESS=0x8c64001e, */
/* IDL: 	PR_EMS_AB_FORWARDING_ADDRESS_UNICODE=0x8c64001f, */
/* IDL: 	PR_EMS_AB_FORWARDING_ADDRESS_ERROR=0x8c64000a, */
/* IDL: 	PR_EMS_AB_FORM_DATA=0x8c650102, */
/* IDL: 	PR_EMS_AB_FORM_DATA_ERROR=0x8c65000a, */
/* IDL: 	PR_EMS_AB_OWA_SERVER=0x8c66001e, */
/* IDL: 	PR_EMS_AB_OWA_SERVER_UNICODE=0x8c66001f, */
/* IDL: 	PR_EMS_AB_OWA_SERVER_ERROR=0x8c66000a, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_NUMBER=0x8c67001e, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_NUMBER_UNICODE=0x8c67001f, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_NUMBER_ERROR=0x8c67000a, */
/* IDL: 	PR_EMS_AB_TELEPHONE_PERSONAL_PAGER=0x8c68001e, */
/* IDL: 	PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_UNICODE=0x8c68001f, */
/* IDL: 	PR_EMS_AB_TELEPHONE_PERSONAL_PAGER_ERROR=0x8c68000a, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_TYPE=0x8c69001e, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_TYPE_UNICODE=0x8c69001f, */
/* IDL: 	PR_EMS_AB_EMPLOYEE_TYPE_ERROR=0x8c69000a, */
/* IDL: 	PR_EMS_AB_TAGGED_X509_CERT=0x8c6a1102, */
/* IDL: 	PR_EMS_AB_TAGGED_X509_CERT_ERROR=0x8c6a000a, */
/* IDL: 	PR_EMS_AB_PERSONAL_TITLE=0x8c6b001e, */
/* IDL: 	PR_EMS_AB_PERSONAL_TITLE_UNICODE=0x8c6b001f, */
/* IDL: 	PR_EMS_AB_PERSONAL_TITLE_ERROR=0x8c6b000a, */
/* IDL: 	PR_EMS_AB_LANGUAGE_ISO639=0x8c6c001e, */
/* IDL: 	PR_EMS_AB_LANGUAGE_ISO639_UNICODE=0x8c6c001f, */
/* IDL: 	PR_EMS_AB_LANGUAGE_ISO639_ERROR=0x8c6c000a, */
/* IDL: 	PR_EMS_AB_OTHER_RECIPS=0xf000000d, */
/* IDL: 	PR_EMS_AB_OTHER_RECIPS_ERROR=0xf000000a, */
/* IDL: 	PR_EMS_AB_CHILD_RDNS=0xfff8101e, */
/* IDL: 	PR_EMS_AB_CHILD_RDNS_UNICODE=0xfff8101f, */
/* IDL: 	PR_EMS_AB_CHILD_RDNS_ERROR=0xfff8000a, */
/* IDL: 	PR_EMS_AB_HIERARCHY_PATH=0xfff9001e, */
/* IDL: 	PR_EMS_AB_HIERARCHY_PATH_UNICODE=0xfff9001f, */
/* IDL: 	PR_EMS_AB_HIERARCHY_PATH_ERROR=0xfff9000a, */
/* IDL: 	PR_EMS_AB_OBJECT_OID=0xfffa0102, */
/* IDL: 	PR_EMS_AB_OBJECT_OID_ERROR=0xfffa000a, */
/* IDL: 	PR_EMS_AB_IS_MASTER=0xfffb000b, */
/* IDL: 	PR_EMS_AB_IS_MASTER_ERROR=0xfffb000a, */
/* IDL: 	PR_EMS_AB_PARENT_ENTRYID=0xfffc0102, */
/* IDL: 	PR_EMS_AB_PARENT_ENTRYID_ERROR=0xfffc000a, */
/* IDL: 	PR_EMS_AB_CONTAINERID=0xfffd0003, */
/* IDL: 	PR_EMS_AB_CONTAINERID_ERROR=0xfffd000a, */
/* IDL: 	PR_EMS_AB_SERVER=0xfffe001e, */
/* IDL: 	PR_EMS_AB_SERVER_UNICODE=0xfffe001f, */
/* IDL: 	PR_EMS_AB_SERVER_ERROR=0xfffe000a, */
/* IDL: 	MAPI_PROP_RESERVED=0xFFFFFFFF, */
/* IDL: } */

int
nspi_dissect_enum_MAPITAGS(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep, int hf_index, guint32 *param)
{
	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, param);
	return offset;
}


/* IDL: enum { */
/* IDL: 	MAPI_E_SUCCESS=0x00000000, */
/* IDL: 	MAPI_E_NO_SUPPORT=0x80040102, */
/* IDL: 	MAPI_E_BAD_CHARWIDTH=0x80040103, */
/* IDL: 	MAPI_E_STRING_TOO_LONG=0x80040105, */
/* IDL: 	MAPI_E_UNKNOWN_FLAGS=0x80040106, */
/* IDL: 	MAPI_E_INVALID_ENTRYID=0x80040107, */
/* IDL: 	MAPI_E_INVALID_OBJECT=0x80040108, */
/* IDL: 	MAPI_E_OBJECT_CHANGED=0x80040109, */
/* IDL: 	MAPI_E_OBJECT_DELETED=0x8004010A, */
/* IDL: 	MAPI_E_BUSY=0x8004010B, */
/* IDL: 	MAPI_E_NOT_ENOUGH_DISK=0x8004010D, */
/* IDL: 	MAPI_E_NOT_ENOUGH_RESOURCES=0x8004010E, */
/* IDL: 	MAPI_E_NOT_FOUND=0x8004010F, */
/* IDL: 	MAPI_E_VERSION=0x80040110, */
/* IDL: 	MAPI_E_LOGON_FAILED=0x80040111, */
/* IDL: 	MAPI_E_SESSION_LIMIT=0x80040112, */
/* IDL: 	MAPI_E_USER_CANCEL=0x80040113, */
/* IDL: 	MAPI_E_UNABLE_TO_ABORT=0x80040114, */
/* IDL: 	MAPI_E_NETWORK_ERROR=0x80040115, */
/* IDL: 	MAPI_E_DISK_ERROR=0x80040116, */
/* IDL: 	MAPI_E_TOO_COMPLEX=0x80040117, */
/* IDL: 	MAPI_E_BAD_COLUMN=0x80040118, */
/* IDL: 	MAPI_E_EXTENDED_ERROR=0x80040119, */
/* IDL: 	MAPI_E_COMPUTED=0x8004011A, */
/* IDL: 	MAPI_E_CORRUPT_DATA=0x8004011B, */
/* IDL: 	MAPI_E_UNCONFIGURED=0x8004011C, */
/* IDL: 	MAPI_E_FAILONEPROVIDER=0x8004011D, */
/* IDL: 	MAPI_E_UNKNOWN_CPID=0x8004011E, */
/* IDL: 	MAPI_E_UNKNOWN_LCID=0x8004011F, */
/* IDL: 	MAPI_E_PASSWORD_CHANGE_REQUIRED=0x80040120, */
/* IDL: 	MAPI_E_PASSWORD_EXPIRED=0x80040121, */
/* IDL: 	MAPI_E_INVALID_WORKSTATION_ACCOUNT=0x80040122, */
/* IDL: 	MAPI_E_INVALID_ACCESS_TIME=0x80040123, */
/* IDL: 	MAPI_E_ACCOUNT_DISABLED=0x80040124, */
/* IDL: 	MAPI_E_END_OF_SESSION=0x80040200, */
/* IDL: 	MAPI_E_UNKNOWN_ENTRYID=0x80040201, */
/* IDL: 	MAPI_E_MISSING_REQUIRED_COLUMN=0x80040202, */
/* IDL: 	MAPI_W_NO_SERVICE=0x80040203, */
/* IDL: 	MAPI_E_BAD_VALUE=0x80040301, */
/* IDL: 	MAPI_E_INVALID_TYPE=0x80040302, */
/* IDL: 	MAPI_E_TYPE_NO_SUPPORT=0x80040303, */
/* IDL: 	MAPI_E_UNEXPECTED_TYPE=0x80040304, */
/* IDL: 	MAPI_E_TOO_BIG=0x80040305, */
/* IDL: 	MAPI_E_DECLINE_COPY=0x80040306, */
/* IDL: 	MAPI_E_UNEXPECTED_ID=0x80040307, */
/* IDL: 	MAPI_W_ERRORS_RETURNED=0x80040380, */
/* IDL: 	MAPI_E_UNABLE_TO_COMPLETE=0x80040400, */
/* IDL: 	MAPI_E_TIMEOUT=0x80040401, */
/* IDL: 	MAPI_E_TABLE_EMPTY=0x80040402, */
/* IDL: 	MAPI_E_TABLE_TOO_BIG=0x80040403, */
/* IDL: 	MAPI_E_INVALID_BOOKMARK=0x80040405, */
/* IDL: 	MAPI_W_POSITION_CHANGED=0x80040481, */
/* IDL: 	MAPI_W_APPROX_COUNT=0x80040482, */
/* IDL: 	MAPI_E_WAIT=0x80040500, */
/* IDL: 	MAPI_E_CANCEL=0x80040501, */
/* IDL: 	MAPI_E_NOT_ME=0x80040502, */
/* IDL: 	MAPI_W_CANCEL_MESSAGE=0x80040580, */
/* IDL: 	MAPI_E_CORRUPT_STORE=0x80040600, */
/* IDL: 	MAPI_E_NOT_IN_QUEUE=0x80040601, */
/* IDL: 	MAPI_E_NO_SUPPRESS=0x80040602, */
/* IDL: 	MAPI_E_COLLISION=0x80040604, */
/* IDL: 	MAPI_E_NOT_INITIALIZED=0x80040605, */
/* IDL: 	MAPI_E_NON_STANDARD=0x80040606, */
/* IDL: 	MAPI_E_NO_RECIPIENTS=0x80040607, */
/* IDL: 	MAPI_E_SUBMITTED=0x80040608, */
/* IDL: 	MAPI_E_HAS_FOLDERS=0x80040609, */
/* IDL: 	MAPI_E_HAS_MESAGES=0x8004060A, */
/* IDL: 	MAPI_E_FOLDER_CYCLE=0x8004060B, */
/* IDL: 	MAPI_W_PARTIAL_COMPLETION=0x80040680, */
/* IDL: 	MAPI_E_AMBIGUOUS_RECIP=0x80040700, */
/* IDL: 	MAPI_E_RESERVED=0xFFFFFFFF, */
/* IDL: } */

int
nspi_dissect_enum_MAPISTATUS(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep, int hf_index, guint32 *param)
{
	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, param);
	return offset;
}


/* IDL: struct _MAPIUID { */
/* IDL: 	uint8 ab[16]; */
/* IDL: } */

static int
nspi_dissect_element_MAPIUID_ab(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	int i;
	for (i = 0; i < 16; i++)
		offset = nspi_dissect_element_MAPIUID_ab_(tvb, offset, pinfo, tree, drep);

	return offset;
}

static int
nspi_dissect_element_MAPIUID_ab_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint8(tvb, offset, pinfo, tree, drep, hf_nspi_MAPIUID_ab, 0);

	return offset;
}

int
nspi_dissect_struct_MAPIUID(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_MAPIUID);
	}
	
	offset = nspi_dissect_element_MAPIUID_ab(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 language; */
/* IDL: 	uint32 method; */
/* IDL: } */

static int
nspi_dissect_element_input_locale_language(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_input_locale_language, 0);

	return offset;
}

static int
nspi_dissect_element_input_locale_method(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_input_locale_method, 0);

	return offset;
}

int
nspi_dissect_struct_input_locale(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_input_locale);
	}
	
	offset = nspi_dissect_element_input_locale_language(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_input_locale_method(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 handle; */
/* IDL: 	uint32 flag; */
/* IDL: 	MAPIUID service_provider; */
/* IDL: 	uint32 codepage; */
/* IDL: 	input_locale input_locale; */
/* IDL: } */

static int
nspi_dissect_element_MAPI_SETTINGS_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPI_SETTINGS_handle, 0);

	return offset;
}

static int
nspi_dissect_element_MAPI_SETTINGS_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPI_SETTINGS_flag, 0);

	return offset;
}

static int
nspi_dissect_element_MAPI_SETTINGS_service_provider(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPIUID(tvb,offset,pinfo,tree,drep,hf_nspi_MAPI_SETTINGS_service_provider,0);

	return offset;
}

static int
nspi_dissect_element_MAPI_SETTINGS_codepage(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPI_SETTINGS_codepage, 0);

	return offset;
}

static int
nspi_dissect_element_MAPI_SETTINGS_input_locale(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_input_locale(tvb,offset,pinfo,tree,drep,hf_nspi_MAPI_SETTINGS_input_locale,0);

	return offset;
}

int
nspi_dissect_struct_MAPI_SETTINGS(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_MAPI_SETTINGS);
	}
	
	offset = nspi_dissect_element_MAPI_SETTINGS_handle(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPI_SETTINGS_flag(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPI_SETTINGS_service_provider(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPI_SETTINGS_codepage(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPI_SETTINGS_input_locale(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	[unique(1)] [length_is(cValues-1)] [size_is(cValues-1)] MAPITAGS *aulPropTag; */
/* IDL: 	uint32 cValues; */
/* IDL: } */

static int
nspi_dissect_element_SPropTagArray_aulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropTagArray_aulPropTag_, NDR_POINTER_UNIQUE, "Pointer to Aulproptag (MAPITAGS)",hf_nspi_SPropTagArray_aulPropTag);

	return offset;
}

static int
nspi_dissect_element_SPropTagArray_aulPropTag_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucvarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropTagArray_aulPropTag__);

	return offset;
}

static int
nspi_dissect_element_SPropTagArray_aulPropTag__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_enum_MAPITAGS(tvb, offset, pinfo, tree, drep, hf_nspi_SPropTagArray_aulPropTag, 0);

	return offset;
}

static int
nspi_dissect_element_SPropTagArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropTagArray_cValues, 0);

	return offset;
}

int
nspi_dissect_struct_SPropTagArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SPropTagArray);
	}
	
	offset = nspi_dissect_element_SPropTagArray_aulPropTag(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SPropTagArray_cValues(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	[unique(1)] [length_is(cValues-1)] [size_is(cValues-1)] uint32 *value; */
/* IDL: 	uint32 cValues; */
/* IDL: } */

static int
nspi_dissect_element_instance_key_value(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_instance_key_value_, NDR_POINTER_UNIQUE, "Pointer to Value (uint32)",hf_nspi_instance_key_value);

	return offset;
}

static int
nspi_dissect_element_instance_key_value_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucvarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_instance_key_value__);

	return offset;
}

static int
nspi_dissect_element_instance_key_value__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_instance_key_value, 0);

	return offset;
}

static int
nspi_dissect_element_instance_key_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_instance_key_cValues, 0);

	return offset;
}

int
nspi_dissect_struct_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_instance_key);
	}
	
	offset = nspi_dissect_element_instance_key_value(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_instance_key_cValues(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	[unique(1)] MAPIUID *lpguid; */
/* IDL: 	uint32 ulKind; */
/* IDL: 	uint32 lID; */
/* IDL: } */

static int
nspi_dissect_element_MAPINAMEID_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_MAPINAMEID_lpguid_, NDR_POINTER_UNIQUE, "Pointer to Lpguid (MAPIUID)",hf_nspi_MAPINAMEID_lpguid);

	return offset;
}

static int
nspi_dissect_element_MAPINAMEID_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPIUID(tvb,offset,pinfo,tree,drep,hf_nspi_MAPINAMEID_lpguid,0);

	return offset;
}

static int
nspi_dissect_element_MAPINAMEID_ulKind(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPINAMEID_ulKind, 0);

	return offset;
}

static int
nspi_dissect_element_MAPINAMEID_lID(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPINAMEID_lID, 0);

	return offset;
}

int
nspi_dissect_struct_MAPINAMEID(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_MAPINAMEID);
	}
	
	offset = nspi_dissect_element_MAPINAMEID_lpguid(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPINAMEID_ulKind(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MAPINAMEID_lID(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: enum { */
/* IDL: 	RES_AND=0, */
/* IDL: 	RES_OR=1, */
/* IDL: 	RES_NOT=2, */
/* IDL: 	RES_CONTENT=3, */
/* IDL: 	RES_PROPERTY=4, */
/* IDL: 	RES_COMPAREPROPS=5, */
/* IDL: 	RES_BITMASK=6, */
/* IDL: 	RES_SIZE=7, */
/* IDL: 	RES_EXIST=8, */
/* IDL: 	RES_SUBRESTRICTION=9, */
/* IDL: 	RES_COMMENT=10, */
/* IDL: } */

int
nspi_dissect_enum_RestrictionType(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep, int hf_index, guint32 *param)
{
	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, param);
	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 relop; */
/* IDL: 	MAPITAGS ulPropTag; */
/* IDL: 	[unique(1)] SPropValue *lpProp; */
/* IDL: } */

static int
nspi_dissect_element_SPropertyRestriction_relop(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropertyRestriction_relop, 0);

	return offset;
}

static int
nspi_dissect_element_SPropertyRestriction_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_enum_MAPITAGS(tvb, offset, pinfo, tree, drep, hf_nspi_SPropertyRestriction_ulPropTag, 0);

	return offset;
}

static int
nspi_dissect_element_SPropertyRestriction_lpProp(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropertyRestriction_lpProp_, NDR_POINTER_UNIQUE, "Pointer to Lpprop (SPropValue)",hf_nspi_SPropertyRestriction_lpProp);

	return offset;
}

static int
nspi_dissect_element_SPropertyRestriction_lpProp_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropValue(tvb,offset,pinfo,tree,drep,hf_nspi_SPropertyRestriction_lpProp,0);

	return offset;
}

int
nspi_dissect_struct_SPropertyRestriction(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SPropertyRestriction);
	}
	
	offset = nspi_dissect_element_SPropertyRestriction_relop(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SPropertyRestriction_ulPropTag(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SPropertyRestriction_lpProp(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cRes; */
/* IDL: 	[unique(1)] [size_is(cRes)] SRestriction *lpRes; */
/* IDL: } */

static int
nspi_dissect_element_SAndRestriction_cRes(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SAndRestriction_cRes, 0);

	return offset;
}

static int
nspi_dissect_element_SAndRestriction_lpRes(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SAndRestriction_lpRes_, NDR_POINTER_UNIQUE, "Pointer to Lpres (SRestriction)",hf_nspi_SAndRestriction_lpRes);

	return offset;
}

static int
nspi_dissect_element_SAndRestriction_lpRes_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SAndRestriction_lpRes__);

	return offset;
}

static int
nspi_dissect_element_SAndRestriction_lpRes__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRestriction(tvb,offset,pinfo,tree,drep,hf_nspi_SAndRestriction_lpRes,0);

	return offset;
}

int
nspi_dissect_struct_SAndRestriction(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SAndRestriction);
	}
	
	offset = nspi_dissect_element_SAndRestriction_cRes(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SAndRestriction_lpRes(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: [switch_type(nspi_RestrictionType)] union { */
/* IDL: [case(RES_AND)] [case(RES_AND)] SAndRestriction resAnd; */
/* IDL: [case(RES_PROPERTY)] [case(RES_PROPERTY)] SPropertyRestriction resProperty; */
/* IDL: } */

static int
nspi_dissect_element_SRestriction_CTR_resAnd(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SAndRestriction(tvb,offset,pinfo,tree,drep,hf_nspi_SRestriction_CTR_resAnd,0);

	return offset;
}

static int
nspi_dissect_element_SRestriction_CTR_resProperty(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropertyRestriction(tvb,offset,pinfo,tree,drep,hf_nspi_SRestriction_CTR_resProperty,0);

	return offset;
}

static int
nspi_dissect_SRestriction_CTR(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;
	guint32 level;

	ALIGN_TO_4_BYTES;

	old_offset = offset;
	if (parent_tree) {
		item = proto_tree_add_text(parent_tree, tvb, offset, -1, "SRestriction_CTR");
		tree = proto_item_add_subtree(item, ett_nspi_SRestriction_CTR);
	}

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, &level);
	switch(level) {
		case RES_AND:
			offset = nspi_dissect_element_SRestriction_CTR_resAnd(tvb, offset, pinfo, tree, drep);
		break;

		case RES_PROPERTY:
			offset = nspi_dissect_element_SRestriction_CTR_resProperty(tvb, offset, pinfo, tree, drep);
		break;
	}
	proto_item_set_len(item, offset-old_offset);

	return offset;
}

/* IDL: struct _SRestriction { */
/* IDL: 	nspi_RestrictionType rt; */
/* IDL: 	[switch_is(rt)] SRestriction_CTR res; */
/* IDL: } */

static int
nspi_dissect_element_SRestriction_rt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_enum_RestrictionType(tvb, offset, pinfo, tree, drep, hf_nspi_SRestriction_PTTYPE, 0);

	return offset;
}

static int
nspi_dissect_element_SRestriction_res(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_SRestriction_CTR(tvb, offset, pinfo, tree, drep, hf_nspi_SRestriction_PTTYPE, 0);

	return offset;
}

int
nspi_dissect_struct_SRestriction(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SRestriction);
	}
	
	offset = nspi_dissect_element_SRestriction_rt(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SRestriction_res(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct _SSortOrder { */
/* IDL: 	uint32 ulPropTag; */
/* IDL: 	uint32 ulOrder; */
/* IDL: } */

static int
nspi_dissect_element_SSortOrder_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SSortOrder_ulPropTag, 0);

	return offset;
}

static int
nspi_dissect_element_SSortOrder_ulOrder(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SSortOrder_ulOrder, 0);

	return offset;
}

int
nspi_dissect_struct_SSortOrder(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SSortOrder);
	}
	
	offset = nspi_dissect_element_SSortOrder_ulPropTag(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SSortOrder_ulOrder(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct _SSortOrderSet { */
/* IDL: 	uint32 cSorts; */
/* IDL: 	uint32 cCategories; */
/* IDL: 	uint32 cExpanded; */
/* IDL: 	[unique(1)] [size_is(cSorts)] SSortOrder *aSort; */
/* IDL: } */

static int
nspi_dissect_element_SSortOrderSet_cSorts(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SSortOrderSet_cSorts, 0);

	return offset;
}

static int
nspi_dissect_element_SSortOrderSet_cCategories(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SSortOrderSet_cCategories, 0);

	return offset;
}

static int
nspi_dissect_element_SSortOrderSet_cExpanded(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SSortOrderSet_cExpanded, 0);

	return offset;
}

static int
nspi_dissect_element_SSortOrderSet_aSort(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SSortOrderSet_aSort_, NDR_POINTER_UNIQUE, "Pointer to Asort (SSortOrder)",hf_nspi_SSortOrderSet_aSort);

	return offset;
}

static int
nspi_dissect_element_SSortOrderSet_aSort_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SSortOrderSet_aSort__);

	return offset;
}

static int
nspi_dissect_element_SSortOrderSet_aSort__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SSortOrder(tvb,offset,pinfo,tree,drep,hf_nspi_SSortOrderSet_aSort,0);

	return offset;
}

int
nspi_dissect_struct_SSortOrderSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SSortOrderSet);
	}
	
	offset = nspi_dissect_element_SSortOrderSet_cSorts(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SSortOrderSet_cCategories(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SSortOrderSet_cExpanded(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SSortOrderSet_aSort(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	[unique(1)] [charset(DOS)] uint8 *str; */
/* IDL: } */

static int
nspi_dissect_element_NAME_STRING_str(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NAME_STRING_str_, NDR_POINTER_UNIQUE, "Pointer to Str (uint8)",hf_nspi_NAME_STRING_str);

	return offset;
}

static int
nspi_dissect_element_NAME_STRING_str_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	char *data;

	offset = dissect_ndr_cvstring(tvb, offset, pinfo, tree, drep, sizeof(guint8), hf_nspi_NAME_STRING_str, FALSE, &data);
	proto_item_append_text(tree, ": %s", data);

	return offset;
}

int
nspi_dissect_struct_NAME_STRING(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_NAME_STRING);
	}
	
	offset = nspi_dissect_element_NAME_STRING_str(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: enum { */
/* IDL: 	PT_UNSPECIFIED=0x0000, */
/* IDL: 	PT_NULL=0x0001, */
/* IDL: 	PT_I2=0x0002, */
/* IDL: 	PT_LONG=0x0003, */
/* IDL: 	PT_R4=0x0004, */
/* IDL: 	PT_DOUBLE=0x0005, */
/* IDL: 	PT_CURRENCY=0x0006, */
/* IDL: 	PT_APPTIME=0x0007, */
/* IDL: 	PT_ERROR=0x000a, */
/* IDL: 	PT_BOOLEAN=0x000b, */
/* IDL: 	PT_OBJECT=0x000d, */
/* IDL: 	PT_I8=0x0014, */
/* IDL: 	PT_STRING8=0x001e, */
/* IDL: 	PT_UNICODE=0x001f, */
/* IDL: 	PT_SYSTIME=0x0040, */
/* IDL: 	PT_CLSID=0x0048, */
/* IDL: 	PT_BINARY=0x0102, */
/* IDL: 	PT_MV_I2=0x1002, */
/* IDL: 	PT_MV_LONG=0x1003, */
/* IDL: 	PT_MV_R4=0x1004, */
/* IDL: 	PT_MV_DOUBLE=0x1005, */
/* IDL: 	PT_MV_CURRENCY=0x1006, */
/* IDL: 	PT_MV_APPTIME=0x1007, */
/* IDL: 	PT_MV_I8=0x1014, */
/* IDL: 	PT_MV_STRING8=0x101e, */
/* IDL: 	PT_MV_TSTRING=0x101e, */
/* IDL: 	PT_MV_UNICODE=0x101f, */
/* IDL: 	PT_MV_SYSTIME=0x1040, */
/* IDL: 	PT_MV_CLSID=0x1048, */
/* IDL: 	PT_MV_BINARY=0x1102, */
/* IDL: } */

int
nspi_dissect_enum_property_types(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep, int hf_index, guint32 *param)
{
	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, param);
	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cb; */
/* IDL: 	[unique(1)] [size_is(cb)] uint8 *lpb; */
/* IDL: } */

static int
nspi_dissect_element_SBinary_cb(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SBinary_cb, 0);

	return offset;
}

static int
nspi_dissect_element_SBinary_lpb(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SBinary_lpb_, NDR_POINTER_UNIQUE, "Pointer to Lpb (uint8)",hf_nspi_SBinary_lpb);

	return offset;
}

static int
nspi_dissect_element_SBinary_lpb_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SBinary_lpb__);

	return offset;
}

static int
nspi_dissect_element_SBinary_lpb__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint8(tvb, offset, pinfo, tree, drep, hf_nspi_SBinary_lpb, 0);

	return offset;
}

int
nspi_dissect_struct_SBinary(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SBinary);
	}
	
	offset = nspi_dissect_element_SBinary_cb(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SBinary_lpb(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 dwLowDateTime; */
/* IDL: 	uint32 dwHighDateTime; */
/* IDL: } */

static int
nspi_dissect_element_FILETIME_dwLowDateTime(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_FILETIME_dwLowDateTime, 0);

	return offset;
}

static int
nspi_dissect_element_FILETIME_dwHighDateTime(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_FILETIME_dwHighDateTime, 0);

	return offset;
}

int
nspi_dissect_struct_FILETIME(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_FILETIME);
	}
	
	offset = nspi_dissect_element_FILETIME_dwLowDateTime(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_FILETIME_dwHighDateTime(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] uint16 *lpi; */
/* IDL: } */

static int
nspi_dissect_element_SShortArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SShortArray_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SShortArray_lpi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SShortArray_lpi_, NDR_POINTER_UNIQUE, "Pointer to Lpi (uint16)",hf_nspi_SShortArray_lpi);

	return offset;
}

static int
nspi_dissect_element_SShortArray_lpi_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SShortArray_lpi__);

	return offset;
}

static int
nspi_dissect_element_SShortArray_lpi__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint16(tvb, offset, pinfo, tree, drep, hf_nspi_SShortArray_lpi, 0);

	return offset;
}

int
nspi_dissect_struct_SShortArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SShortArray);
	}
	
	offset = nspi_dissect_element_SShortArray_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SShortArray_lpi(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] uint32 *lpl; */
/* IDL: } */

static int
nspi_dissect_element_MV_LONG_STRUCT_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MV_LONG_STRUCT_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_MV_LONG_STRUCT_lpl(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_MV_LONG_STRUCT_lpl_, NDR_POINTER_UNIQUE, "Pointer to Lpl (uint32)",hf_nspi_MV_LONG_STRUCT_lpl);

	return offset;
}

static int
nspi_dissect_element_MV_LONG_STRUCT_lpl_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_MV_LONG_STRUCT_lpl__);

	return offset;
}

static int
nspi_dissect_element_MV_LONG_STRUCT_lpl__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MV_LONG_STRUCT_lpl, 0);

	return offset;
}

int
nspi_dissect_struct_MV_LONG_STRUCT(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_MV_LONG_STRUCT);
	}
	
	offset = nspi_dissect_element_MV_LONG_STRUCT_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MV_LONG_STRUCT_lpl(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	[flag(LIBNDR_FLAG_STR_NOTERM|LIBNDR_FLAG_STR_ASCII|LIBNDR_FLAG_STR_SIZE4|LIBNDR_FLAG_STR_LEN4)] string lppszA; */
/* IDL: } */

static int
nspi_dissect_element_LPSTR_lppszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	char *data;

	offset = dissect_ndr_cvstring(tvb, offset, pinfo, tree, drep, 1, hf_nspi_LPSTR_lppszA, FALSE, &data);
	proto_item_append_text(tree, ": %s", data);

	return offset;
}

int
nspi_dissect_struct_LPSTR(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_LPSTR);
	}
	
	offset = nspi_dissect_element_LPSTR_lppszA(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] LPSTR **strings; */
/* IDL: } */

static int
nspi_dissect_element_SLPSTRArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SLPSTRArray_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SLPSTRArray_strings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SLPSTRArray_strings_, NDR_POINTER_UNIQUE, "Pointer to Strings (LPSTR)",hf_nspi_SLPSTRArray_strings);

	return offset;
}

static int
nspi_dissect_element_SLPSTRArray_strings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SLPSTRArray_strings__);

	return offset;
}

static int
nspi_dissect_element_SLPSTRArray_strings__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SLPSTRArray_strings___, NDR_POINTER_UNIQUE, "Pointer to Strings (LPSTR)",hf_nspi_SLPSTRArray_strings);

	return offset;
}

static int
nspi_dissect_element_SLPSTRArray_strings___(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_LPSTR(tvb,offset,pinfo,tree,drep,hf_nspi_SLPSTRArray_strings,0);

	return offset;
}

int
nspi_dissect_struct_SLPSTRArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SLPSTRArray);
	}
	
	offset = nspi_dissect_element_SLPSTRArray_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SLPSTRArray_strings(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] SBinary *lpbin; */
/* IDL: } */

static int
nspi_dissect_element_SBinaryArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SBinaryArray_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SBinaryArray_lpbin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SBinaryArray_lpbin_, NDR_POINTER_UNIQUE, "Pointer to Lpbin (SBinary)",hf_nspi_SBinaryArray_lpbin);

	return offset;
}

static int
nspi_dissect_element_SBinaryArray_lpbin_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SBinaryArray_lpbin__);

	return offset;
}

static int
nspi_dissect_element_SBinaryArray_lpbin__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SBinary(tvb,offset,pinfo,tree,drep,hf_nspi_SBinaryArray_lpbin,0);

	return offset;
}

int
nspi_dissect_struct_SBinaryArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SBinaryArray);
	}
	
	offset = nspi_dissect_element_SBinaryArray_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SBinaryArray_lpbin(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] uint32 *lpguid; */
/* IDL: } */

static int
nspi_dissect_element_SGuidArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SGuidArray_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SGuidArray_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SGuidArray_lpguid_, NDR_POINTER_UNIQUE, "Pointer to Lpguid (uint32)",hf_nspi_SGuidArray_lpguid);

	return offset;
}

static int
nspi_dissect_element_SGuidArray_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SGuidArray_lpguid__);

	return offset;
}

static int
nspi_dissect_element_SGuidArray_lpguid__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SGuidArray_lpguid, 0);

	return offset;
}

int
nspi_dissect_struct_SGuidArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SGuidArray);
	}
	
	offset = nspi_dissect_element_SGuidArray_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SGuidArray_lpguid(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] uint32 *lpi; */
/* IDL: } */

static int
nspi_dissect_element_MV_UNICODE_STRUCT_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MV_UNICODE_STRUCT_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_MV_UNICODE_STRUCT_lpi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_MV_UNICODE_STRUCT_lpi_, NDR_POINTER_UNIQUE, "Pointer to Lpi (uint32)",hf_nspi_MV_UNICODE_STRUCT_lpi);

	return offset;
}

static int
nspi_dissect_element_MV_UNICODE_STRUCT_lpi_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_MV_UNICODE_STRUCT_lpi__);

	return offset;
}

static int
nspi_dissect_element_MV_UNICODE_STRUCT_lpi__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MV_UNICODE_STRUCT_lpi, 0);

	return offset;
}

int
nspi_dissect_struct_MV_UNICODE_STRUCT(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_MV_UNICODE_STRUCT);
	}
	
	offset = nspi_dissect_element_MV_UNICODE_STRUCT_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_MV_UNICODE_STRUCT_lpi(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] FILETIME *lpft; */
/* IDL: } */

static int
nspi_dissect_element_SDateTimeArray_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SDateTimeArray_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SDateTimeArray_lpft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SDateTimeArray_lpft_, NDR_POINTER_UNIQUE, "Pointer to Lpft (FILETIME)",hf_nspi_SDateTimeArray_lpft);

	return offset;
}

static int
nspi_dissect_element_SDateTimeArray_lpft_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SDateTimeArray_lpft__);

	return offset;
}

static int
nspi_dissect_element_SDateTimeArray_lpft__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_FILETIME(tvb,offset,pinfo,tree,drep,hf_nspi_SDateTimeArray_lpft,0);

	return offset;
}

int
nspi_dissect_struct_SDateTimeArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SDateTimeArray);
	}
	
	offset = nspi_dissect_element_SDateTimeArray_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SDateTimeArray_lpft(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: [switch_type(property_types)] union { */
/* IDL: [case(PT_I2)] [case(PT_I2)] uint16 i; */
/* IDL: [case(PT_LONG)] [case(PT_LONG)] uint32 l; */
/* IDL: [case(PT_BOOLEAN)] [case(PT_BOOLEAN)] uint16 b; */
/* IDL: [case(PT_STRING8)] [unique(1)] [charset(DOS)] [case(PT_STRING8)] uint8 *lpszA; */
/* IDL: [case(PT_BINARY)] [case(PT_BINARY)] SBinary bin; */
/* IDL: [case(PT_UNICODE)] [unique(1)] [charset(UTF16)] [case(PT_UNICODE)] uint16 *lpszW; */
/* IDL: [case(PT_CLSID)] [unique(1)] [case(PT_CLSID)] MAPIUID *lpguid; */
/* IDL: [case(PT_SYSTIME)] [case(PT_SYSTIME)] FILETIME ft; */
/* IDL: [case(PT_ERROR)] [case(PT_ERROR)] MAPISTATUS err; */
/* IDL: [case(PT_MV_I2)] [case(PT_MV_I2)] SShortArray MVi; */
/* IDL: [case(PT_MV_LONG)] [case(PT_MV_LONG)] MV_LONG_STRUCT MVl; */
/* IDL: [case(PT_MV_STRING8)] [case(PT_MV_STRING8)] SLPSTRArray MVszA; */
/* IDL: [case(PT_MV_BINARY)] [case(PT_MV_BINARY)] SBinaryArray MVbin; */
/* IDL: [case(PT_MV_CLSID)] [case(PT_MV_CLSID)] SGuidArray MVguid; */
/* IDL: [case(PT_MV_UNICODE)] [case(PT_MV_UNICODE)] MV_UNICODE_STRUCT MVszW; */
/* IDL: [case(PT_MV_SYSTIME)] [case(PT_MV_SYSTIME)] SDateTimeArray MVft; */
/* IDL: [case(PT_NULL)] [case(PT_NULL)] uint32 null; */
/* IDL: [case(PT_OBJECT)] [case(PT_OBJECT)] uint32 object; */
/* IDL: } */

static int
nspi_dissect_element_SPropValue_CTR_i(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint16(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_i, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_l(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_l, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_b(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint16(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_b, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropValue_CTR_lpszA_, NDR_POINTER_UNIQUE, "Pointer to Lpsza (uint8)",hf_nspi_SPropValue_CTR_lpszA);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpszA_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	char *data;

	offset = dissect_ndr_cvstring(tvb, offset, pinfo, tree, drep, sizeof(guint8), hf_nspi_SPropValue_CTR_lpszA, FALSE, &data);
	proto_item_append_text(tree, ": %s", data);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_bin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SBinary(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_bin,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpszW(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropValue_CTR_lpszW_, NDR_POINTER_UNIQUE, "Pointer to Lpszw (uint16)",hf_nspi_SPropValue_CTR_lpszW);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpszW_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	char *data;

	offset = dissect_ndr_cvstring(tvb, offset, pinfo, tree, drep, sizeof(guint16), hf_nspi_SPropValue_CTR_lpszW, FALSE, &data);
	proto_item_append_text(tree, ": %s", data);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SPropValue_CTR_lpguid_, NDR_POINTER_UNIQUE, "Pointer to Lpguid (MAPIUID)",hf_nspi_SPropValue_CTR_lpguid);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_lpguid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPIUID(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_lpguid,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_ft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_FILETIME(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_ft,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_err(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_enum_MAPISTATUS(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_err, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVi(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SShortArray(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVi,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVl(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MV_LONG_STRUCT(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVl,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVszA(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SLPSTRArray(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVszA,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVbin(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SBinaryArray(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVbin,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVguid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SGuidArray(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVguid,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVszW(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MV_UNICODE_STRUCT(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVszW,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_MVft(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SDateTimeArray(tvb,offset,pinfo,tree,drep,hf_nspi_SPropValue_CTR_MVft,0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_null(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_null, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_CTR_object(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_CTR_object, 0);

	return offset;
}

static int
nspi_dissect_SPropValue_CTR(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;
	guint32 level;

	ALIGN_TO_4_BYTES;

	old_offset = offset;
	if (parent_tree) {
		item = proto_tree_add_text(parent_tree, tvb, offset, -1, "SPropValue_CTR");
		tree = proto_item_add_subtree(item, ett_nspi_SPropValue_CTR);
	}

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_index, &level);
	switch(level) {
		case PT_I2:
			offset = nspi_dissect_element_SPropValue_CTR_i(tvb, offset, pinfo, tree, drep);
		break;

		case PT_LONG:
			offset = nspi_dissect_element_SPropValue_CTR_l(tvb, offset, pinfo, tree, drep);
		break;

		case PT_BOOLEAN:
			offset = nspi_dissect_element_SPropValue_CTR_b(tvb, offset, pinfo, tree, drep);
		break;

		case PT_STRING8:
			offset = nspi_dissect_element_SPropValue_CTR_lpszA(tvb, offset, pinfo, tree, drep);
		break;

		case PT_BINARY:
			offset = nspi_dissect_element_SPropValue_CTR_bin(tvb, offset, pinfo, tree, drep);
		break;

		case PT_UNICODE:
			offset = nspi_dissect_element_SPropValue_CTR_lpszW(tvb, offset, pinfo, tree, drep);
		break;

		case PT_CLSID:
			offset = nspi_dissect_element_SPropValue_CTR_lpguid(tvb, offset, pinfo, tree, drep);
		break;

		case PT_SYSTIME:
			offset = nspi_dissect_element_SPropValue_CTR_ft(tvb, offset, pinfo, tree, drep);
		break;

		case PT_ERROR:
			offset = nspi_dissect_element_SPropValue_CTR_err(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_I2:
			offset = nspi_dissect_element_SPropValue_CTR_MVi(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_LONG:
			offset = nspi_dissect_element_SPropValue_CTR_MVl(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_STRING8:
			offset = nspi_dissect_element_SPropValue_CTR_MVszA(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_BINARY:
			offset = nspi_dissect_element_SPropValue_CTR_MVbin(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_CLSID:
			offset = nspi_dissect_element_SPropValue_CTR_MVguid(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_UNICODE:
			offset = nspi_dissect_element_SPropValue_CTR_MVszW(tvb, offset, pinfo, tree, drep);
		break;

		case PT_MV_SYSTIME:
			offset = nspi_dissect_element_SPropValue_CTR_MVft(tvb, offset, pinfo, tree, drep);
		break;

		case PT_NULL:
			offset = nspi_dissect_element_SPropValue_CTR_null(tvb, offset, pinfo, tree, drep);
		break;

		case PT_OBJECT:
			offset = nspi_dissect_element_SPropValue_CTR_object(tvb, offset, pinfo, tree, drep);
		break;
	}
	proto_item_set_len(item, offset-old_offset);

	return offset;
}

/* IDL: struct { */
/* IDL: 	MAPITAGS ulPropTag; */
/* IDL: 	uint32 dwAlignPad; */
/* IDL: 	[switch_is(ulPropTag&0xFFFF)] SPropValue_CTR value; */
/* IDL: } */

static int
nspi_dissect_element_SPropValue_ulPropTag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_enum_MAPITAGS(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_ulPropTag, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_dwAlignPad(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SPropValue_dwAlignPad, 0);

	return offset;
}

static int
nspi_dissect_element_SPropValue_value(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_SPropValue_CTR(tvb, offset, pinfo, tree, drep, hf_nspi_property_type, 0);

	return offset;
}

int
nspi_dissect_struct_SPropValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SPropValue);
	}
	
	offset = nspi_dissect_element_SPropValue_ulPropTag(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SPropValue_dwAlignPad(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SPropValue_value(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 ulAdrEntryPad; */
/* IDL: 	uint32 cValues; */
/* IDL: 	[unique(1)] [size_is(cValues)] SPropValue *lpProps; */
/* IDL: } */

static int
nspi_dissect_element_SRow_ulAdrEntryPad(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SRow_ulAdrEntryPad, 0);

	return offset;
}

static int
nspi_dissect_element_SRow_cValues(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SRow_cValues, 0);

	return offset;
}

static int
nspi_dissect_element_SRow_lpProps(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SRow_lpProps_, NDR_POINTER_UNIQUE, "Pointer to Lpprops (SPropValue)",hf_nspi_SRow_lpProps);

	return offset;
}

static int
nspi_dissect_element_SRow_lpProps_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SRow_lpProps__);

	return offset;
}

static int
nspi_dissect_element_SRow_lpProps__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropValue(tvb,offset,pinfo,tree,drep,hf_nspi_SRow_lpProps,0);

	return offset;
}

int
nspi_dissect_struct_SRow(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SRow);
	}
	
	offset = nspi_dissect_element_SRow_ulAdrEntryPad(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SRow_cValues(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SRow_lpProps(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}


/* IDL: struct { */
/* IDL: 	uint32 cRows; */
/* IDL: 	[size_is(cRows)] SRow aRow[*]; */
/* IDL: } */

static int
nspi_dissect_element_SRowSet_cRows(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_SRowSet_cRows, 0);

	return offset;
}

static int
nspi_dissect_element_SRowSet_aRow(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_SRowSet_aRow_);

	return offset;
}

static int
nspi_dissect_element_SRowSet_aRow_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRow(tvb,offset,pinfo,tree,drep,hf_nspi_SRowSet_aRow,0);

	return offset;
}

int
nspi_dissect_struct_SRowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *parent_tree, guint8 *drep, int hf_index, guint32 param _U_)
{
	proto_item *item = NULL;
	proto_tree *tree = NULL;
	int old_offset;

	ALIGN_TO_4_BYTES;

	old_offset = offset;

	if (parent_tree) {
		item = proto_tree_add_item(parent_tree, hf_index, tvb, offset, -1, TRUE);
		tree = proto_item_add_subtree(item, ett_nspi_SRowSet);
	}
	
	offset = nspi_dissect_element_SRowSet_cRows(tvb, offset, pinfo, tree, drep);

	offset = nspi_dissect_element_SRowSet_aRow(tvb, offset, pinfo, tree, drep);


	proto_item_set_len(item, offset-old_offset);

	return offset;
}

static int
nspi_dissect_element_NspiBind_unknown(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiBind_unknown, 0);

	return offset;
}

static int
nspi_dissect_element_NspiBind_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiBind_settings_, NDR_POINTER_REF, "Pointer to Settings (MAPI_SETTINGS)",hf_nspi_NspiBind_settings);

	return offset;
}

static int
nspi_dissect_element_NspiBind_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPI_SETTINGS(tvb,offset,pinfo,tree,drep,hf_nspi_NspiBind_settings,0);

	return offset;
}

static int
nspi_dissect_element_NspiBind_mapiuid(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiBind_mapiuid_, NDR_POINTER_UNIQUE, "Pointer to Mapiuid (GUID)",hf_nspi_NspiBind_mapiuid);

	return offset;
}

static int
nspi_dissect_element_NspiBind_mapiuid_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_uuid_t(tvb, offset, pinfo, tree, drep, hf_nspi_NspiBind_mapiuid, NULL);

	return offset;
}

static int
nspi_dissect_element_NspiBind_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiBind_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiBind_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0x0001);

	return offset;
}

/* IDL: MAPISTATUS NspiBind( */
/* IDL: [in] uint32 unknown, */
/* IDL: [ref] [in] MAPI_SETTINGS *settings, */
/* IDL: [out] [unique(1)] [in] GUID *mapiuid, */
/* IDL: [out] [ref] policy_handle *handle */
/* IDL: ); */

static int
nspi_dissect_NspiBind_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiBind";
	offset = nspi_dissect_element_NspiBind_mapiuid(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = nspi_dissect_element_NspiBind_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiBind_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiBind";
	offset = nspi_dissect_element_NspiBind_unknown(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiBind_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiBind_mapiuid(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

static int
nspi_dissect_element_NspiUnbind_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiUnbind_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiUnbind_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0x0002);

	return offset;
}

static int
nspi_dissect_element_NspiUnbind_status(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiUnbind_status, 0);

	return offset;
}

/* IDL: MAPISTATUS NspiUnbind( */
/* IDL: [out] [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 status */
/* IDL: ); */

static int
nspi_dissect_NspiUnbind_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiUnbind";
	offset = nspi_dissect_element_NspiUnbind_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiUnbind_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiUnbind";
	offset = nspi_dissect_element_NspiUnbind_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiUnbind_status(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiUpdateStat( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiUpdateStat_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiUpdateStat";
	return offset;
}

static int
nspi_dissect_NspiUpdateStat_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiUpdateStat";
	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiQueryRows_flag, 0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_settings_, NDR_POINTER_REF, "Pointer to Settings (MAPI_SETTINGS)",hf_nspi_NspiQueryRows_settings);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPI_SETTINGS(tvb,offset,pinfo,tree,drep,hf_nspi_NspiQueryRows_settings,0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_lRows(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiQueryRows_lRows, 0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_instance_key_, NDR_POINTER_UNIQUE, "Pointer to Instance Key (uint32)",hf_nspi_NspiQueryRows_instance_key);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_instance_key__);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_instance_key__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiQueryRows_instance_key, 0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_unknown(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiQueryRows_unknown, 0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_REQ_properties_, NDR_POINTER_REF, "Pointer to Req Properties (SPropTagArray)",hf_nspi_NspiQueryRows_REQ_properties);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropTagArray(tvb,offset,pinfo,tree,drep,hf_nspi_NspiQueryRows_REQ_properties,0);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_RowSet_, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiQueryRows_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiQueryRows_RowSet__, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiQueryRows_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiQueryRows_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRowSet(tvb,offset,pinfo,tree,drep,hf_nspi_NspiQueryRows_RowSet,0);

	return offset;
}

/* IDL: MAPISTATUS NspiQueryRows( */
/* IDL: [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 flag, */
/* IDL: [out] [ref] [in] MAPI_SETTINGS *settings, */
/* IDL: [in] uint32 lRows, */
/* IDL: [unique(1)] [in] [size_is(lRows)] uint32 *instance_key, */
/* IDL: [in] uint32 unknown, */
/* IDL: [ref] [in] SPropTagArray *REQ_properties, */
/* IDL: [out] [ref] SRowSet **RowSet */
/* IDL: ); */

static int
nspi_dissect_NspiQueryRows_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiQueryRows";
	offset = nspi_dissect_element_NspiQueryRows_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = nspi_dissect_element_NspiQueryRows_RowSet(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiQueryRows_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiQueryRows";
	offset = nspi_dissect_element_NspiQueryRows_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_flag(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_lRows(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_instance_key(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_unknown(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiQueryRows_REQ_properties(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiSeekEntries( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiSeekEntries_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiSeekEntries";
	return offset;
}

static int
nspi_dissect_NspiSeekEntries_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiSeekEntries";
	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_unknown1(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetMatches_unknown1, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_settings_, NDR_POINTER_REF, "Pointer to Settings (MAPI_SETTINGS)",hf_nspi_NspiGetMatches_settings);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPI_SETTINGS(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_settings,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_PropTagArray(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_PropTagArray_, NDR_POINTER_UNIQUE, "Pointer to Proptagarray (SPropTagArray)",hf_nspi_NspiGetMatches_PropTagArray);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_PropTagArray_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropTagArray(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_PropTagArray,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_unknown2(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetMatches_unknown2, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_restrictions(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_restrictions_, NDR_POINTER_UNIQUE, "Pointer to Restrictions (SRestriction)",hf_nspi_NspiGetMatches_restrictions);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_restrictions_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRestriction(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_restrictions,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_unknown3(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetMatches_unknown3, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_instance_key_, NDR_POINTER_REF, "Pointer to Instance Key (instance_key)",hf_nspi_NspiGetMatches_instance_key);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_instance_key(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_instance_key,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_REQ_properties_, NDR_POINTER_UNIQUE, "Pointer to Req Properties (SPropTagArray)",hf_nspi_NspiGetMatches_REQ_properties);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropTagArray(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_REQ_properties,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_RowSet_, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiGetMatches_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetMatches_RowSet__, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiGetMatches_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiGetMatches_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRowSet(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetMatches_RowSet,0);

	return offset;
}

/* IDL: MAPISTATUS NspiGetMatches( */
/* IDL: [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 unknown1, */
/* IDL: [out] [ref] [in] MAPI_SETTINGS *settings, */
/* IDL: [unique(1)] [in] SPropTagArray *PropTagArray, */
/* IDL: [in] uint32 unknown2, */
/* IDL: [unique(1)] [in] SRestriction *restrictions, */
/* IDL: [in] uint32 unknown3, */
/* IDL: [out] [ref] instance_key *instance_key, */
/* IDL: [unique(1)] [in] SPropTagArray *REQ_properties, */
/* IDL: [out] [ref] SRowSet **RowSet */
/* IDL: ); */

static int
nspi_dissect_NspiGetMatches_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiGetMatches";
	offset = nspi_dissect_element_NspiGetMatches_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = nspi_dissect_element_NspiGetMatches_instance_key(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = nspi_dissect_element_NspiGetMatches_RowSet(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiGetMatches_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetMatches";
	offset = nspi_dissect_element_NspiGetMatches_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_unknown1(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_PropTagArray(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_unknown2(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_restrictions(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_unknown3(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetMatches_REQ_properties(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiResortRestriction( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiResortRestriction_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResortRestriction";
	return offset;
}

static int
nspi_dissect_NspiResortRestriction_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResortRestriction";
	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiDNToEph_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiDNToEph_flag, 0);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_size(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiDNToEph_size, 0);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_server_dn(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiDNToEph_server_dn_, NDR_POINTER_REF, "Pointer to Server Dn (NAME_STRING)",hf_nspi_NspiDNToEph_server_dn);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_server_dn_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_ucarray(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiDNToEph_server_dn__);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_server_dn__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_NAME_STRING(tvb,offset,pinfo,tree,drep,hf_nspi_NspiDNToEph_server_dn,0);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_instance_key(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiDNToEph_instance_key_, NDR_POINTER_REF, "Pointer to Instance Key (instance_key)",hf_nspi_NspiDNToEph_instance_key);

	return offset;
}

static int
nspi_dissect_element_NspiDNToEph_instance_key_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_instance_key(tvb,offset,pinfo,tree,drep,hf_nspi_NspiDNToEph_instance_key,0);

	return offset;
}

/* IDL: MAPISTATUS NspiDNToEph( */
/* IDL: [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 flag, */
/* IDL: [in] uint32 size, */
/* IDL: [ref] [in] [size_is(size)] NAME_STRING *server_dn, */
/* IDL: [out] [ref] instance_key *instance_key */
/* IDL: ); */

static int
nspi_dissect_NspiDNToEph_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiDNToEph";
	offset = nspi_dissect_element_NspiDNToEph_instance_key(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiDNToEph_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiDNToEph";
	offset = nspi_dissect_element_NspiDNToEph_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiDNToEph_flag(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiDNToEph_size(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiDNToEph_server_dn(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiGetPropList( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiGetPropList_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetPropList";
	return offset;
}

static int
nspi_dissect_NspiGetPropList_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetPropList";
	return offset;
}

static int
nspi_dissect_element_NspiGetProps_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetProps_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_flag(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetProps_flag, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetProps_settings_, NDR_POINTER_REF, "Pointer to Settings (MAPI_SETTINGS)",hf_nspi_NspiGetProps_settings);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPI_SETTINGS(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetProps_settings,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_REQ_properties(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetProps_REQ_properties_, NDR_POINTER_REF, "Pointer to Req Properties (SPropTagArray)",hf_nspi_NspiGetProps_REQ_properties);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_REQ_properties_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SPropTagArray(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetProps_REQ_properties,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_REPL_values(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetProps_REPL_values_, NDR_POINTER_REF, "Pointer to Repl Values (SRow)",hf_nspi_NspiGetProps_REPL_values);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_REPL_values_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetProps_REPL_values__, NDR_POINTER_REF, "Pointer to Repl Values (SRow)",hf_nspi_NspiGetProps_REPL_values);

	return offset;
}

static int
nspi_dissect_element_NspiGetProps_REPL_values__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRow(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetProps_REPL_values,0);

	return offset;
}

/* IDL: MAPISTATUS NspiGetProps( */
/* IDL: [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 flag, */
/* IDL: [ref] [in] MAPI_SETTINGS *settings, */
/* IDL: [ref] [in] SPropTagArray *REQ_properties, */
/* IDL: [out] [ref] SRow **REPL_values */
/* IDL: ); */

static int
nspi_dissect_NspiGetProps_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiGetProps";
	offset = nspi_dissect_element_NspiGetProps_REPL_values(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiGetProps_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetProps";
	offset = nspi_dissect_element_NspiGetProps_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetProps_flag(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetProps_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetProps_REQ_properties(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiCompareDNTs( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiCompareDNTs_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiCompareDNTs";
	return offset;
}

static int
nspi_dissect_NspiCompareDNTs_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiCompareDNTs";
	return offset;
}

/* IDL: void NspiModProps( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiModProps_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiModProps";
	return offset;
}

static int
nspi_dissect_NspiModProps_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiModProps";
	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_handle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetHierarchyInfo_handle_, NDR_POINTER_REF, "Pointer to Handle (policy_handle)",hf_nspi_handle);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_handle_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_policy_hnd(tvb, offset, pinfo, tree, drep, hf_nspi_handle, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_unknown1(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetHierarchyInfo_unknown1, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_settings(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetHierarchyInfo_settings_, NDR_POINTER_REF, "Pointer to Settings (MAPI_SETTINGS)",hf_nspi_NspiGetHierarchyInfo_settings);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_settings_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_MAPI_SETTINGS(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetHierarchyInfo_settings,0);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_unknown2(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetHierarchyInfo_unknown2_, NDR_POINTER_REF, "Pointer to Unknown2 (uint32)",hf_nspi_NspiGetHierarchyInfo_unknown2);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_unknown2_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = PIDL_dissect_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_NspiGetHierarchyInfo_unknown2, 0);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_RowSet(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_toplevel_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetHierarchyInfo_RowSet_, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiGetHierarchyInfo_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_RowSet_(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = dissect_ndr_embedded_pointer(tvb, offset, pinfo, tree, drep, nspi_dissect_element_NspiGetHierarchyInfo_RowSet__, NDR_POINTER_REF, "Pointer to Rowset (SRowSet)",hf_nspi_NspiGetHierarchyInfo_RowSet);

	return offset;
}

static int
nspi_dissect_element_NspiGetHierarchyInfo_RowSet__(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep)
{
	offset = nspi_dissect_struct_SRowSet(tvb,offset,pinfo,tree,drep,hf_nspi_NspiGetHierarchyInfo_RowSet,0);

	return offset;
}

/* IDL: MAPISTATUS NspiGetHierarchyInfo( */
/* IDL: [ref] [in] policy_handle *handle, */
/* IDL: [in] uint32 unknown1, */
/* IDL: [ref] [in] MAPI_SETTINGS *settings, */
/* IDL: [out] [ref] [in] uint32 *unknown2, */
/* IDL: [out] [ref] SRowSet **RowSet */
/* IDL: ); */

static int
nspi_dissect_NspiGetHierarchyInfo_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	guint32 status;

	pinfo->dcerpc_procedure_name="NspiGetHierarchyInfo";
	offset = nspi_dissect_element_NspiGetHierarchyInfo_unknown2(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = nspi_dissect_element_NspiGetHierarchyInfo_RowSet(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);

	offset = dissect_ndr_uint32(tvb, offset, pinfo, tree, drep, hf_nspi_MAPISTATUS_status, &status);
	if (status != 0 && check_col(pinfo->cinfo, COL_INFO))
		col_append_fstr(pinfo->cinfo, COL_INFO, ", Status: %s", val_to_str(status, nspi_MAPISTATUS_vals, "Unknown MAPISTATUS error 0x%08x"));

	return offset;
}

static int
nspi_dissect_NspiGetHierarchyInfo_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetHierarchyInfo";
	offset = nspi_dissect_element_NspiGetHierarchyInfo_handle(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetHierarchyInfo_unknown1(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetHierarchyInfo_settings(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	offset = nspi_dissect_element_NspiGetHierarchyInfo_unknown2(tvb, offset, pinfo, tree, drep);
	offset = dissect_deferred_pointers(pinfo, tvb, offset, drep);
	return offset;
}

/* IDL: void NspiGetTemplateInfo( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiGetTemplateInfo_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetTemplateInfo";
	return offset;
}

static int
nspi_dissect_NspiGetTemplateInfo_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetTemplateInfo";
	return offset;
}

/* IDL: void NspiModLInkAtt( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiModLInkAtt_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiModLInkAtt";
	return offset;
}

static int
nspi_dissect_NspiModLInkAtt_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiModLInkAtt";
	return offset;
}

/* IDL: void NspiDeleteEntries( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiDeleteEntries_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiDeleteEntries";
	return offset;
}

static int
nspi_dissect_NspiDeleteEntries_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiDeleteEntries";
	return offset;
}

/* IDL: void NspiQueryColumns( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiQueryColumns_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiQueryColumns";
	return offset;
}

static int
nspi_dissect_NspiQueryColumns_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiQueryColumns";
	return offset;
}

/* IDL: void NspiGetNamesFromIDs( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiGetNamesFromIDs_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetNamesFromIDs";
	return offset;
}

static int
nspi_dissect_NspiGetNamesFromIDs_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetNamesFromIDs";
	return offset;
}

/* IDL: void NspiGetIDsFromNames( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiGetIDsFromNames_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetIDsFromNames";
	return offset;
}

static int
nspi_dissect_NspiGetIDsFromNames_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiGetIDsFromNames";
	return offset;
}

/* IDL: void NspiResolveNames( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiResolveNames_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResolveNames";
	return offset;
}

static int
nspi_dissect_NspiResolveNames_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResolveNames";
	return offset;
}

/* IDL: void NspiResolveNamesW( */
/* IDL:  */
/* IDL: ); */

static int
nspi_dissect_NspiResolveNamesW_response(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResolveNamesW";
	return offset;
}

static int
nspi_dissect_NspiResolveNamesW_request(tvbuff_t *tvb _U_, int offset _U_, packet_info *pinfo, proto_tree *tree _U_, guint8 *drep _U_)
{
	pinfo->dcerpc_procedure_name="NspiResolveNamesW";
	return offset;
}


static dcerpc_sub_dissector nspi_dissectors[] = {
	{ 0, "NspiBind",
	   nspi_dissect_NspiBind_request, nspi_dissect_NspiBind_response},
	{ 1, "NspiUnbind",
	   nspi_dissect_NspiUnbind_request, nspi_dissect_NspiUnbind_response},
	{ 2, "NspiUpdateStat",
	   nspi_dissect_NspiUpdateStat_request, nspi_dissect_NspiUpdateStat_response},
	{ 3, "NspiQueryRows",
	   nspi_dissect_NspiQueryRows_request, nspi_dissect_NspiQueryRows_response},
	{ 4, "NspiSeekEntries",
	   nspi_dissect_NspiSeekEntries_request, nspi_dissect_NspiSeekEntries_response},
	{ 5, "NspiGetMatches",
	   nspi_dissect_NspiGetMatches_request, nspi_dissect_NspiGetMatches_response},
	{ 6, "NspiResortRestriction",
	   nspi_dissect_NspiResortRestriction_request, nspi_dissect_NspiResortRestriction_response},
	{ 7, "NspiDNToEph",
	   nspi_dissect_NspiDNToEph_request, nspi_dissect_NspiDNToEph_response},
	{ 8, "NspiGetPropList",
	   nspi_dissect_NspiGetPropList_request, nspi_dissect_NspiGetPropList_response},
	{ 9, "NspiGetProps",
	   nspi_dissect_NspiGetProps_request, nspi_dissect_NspiGetProps_response},
	{ 10, "NspiCompareDNTs",
	   nspi_dissect_NspiCompareDNTs_request, nspi_dissect_NspiCompareDNTs_response},
	{ 11, "NspiModProps",
	   nspi_dissect_NspiModProps_request, nspi_dissect_NspiModProps_response},
	{ 12, "NspiGetHierarchyInfo",
	   nspi_dissect_NspiGetHierarchyInfo_request, nspi_dissect_NspiGetHierarchyInfo_response},
	{ 13, "NspiGetTemplateInfo",
	   nspi_dissect_NspiGetTemplateInfo_request, nspi_dissect_NspiGetTemplateInfo_response},
	{ 14, "NspiModLInkAtt",
	   nspi_dissect_NspiModLInkAtt_request, nspi_dissect_NspiModLInkAtt_response},
	{ 15, "NspiDeleteEntries",
	   nspi_dissect_NspiDeleteEntries_request, nspi_dissect_NspiDeleteEntries_response},
	{ 16, "NspiQueryColumns",
	   nspi_dissect_NspiQueryColumns_request, nspi_dissect_NspiQueryColumns_response},
	{ 17, "NspiGetNamesFromIDs",
	   nspi_dissect_NspiGetNamesFromIDs_request, nspi_dissect_NspiGetNamesFromIDs_response},
	{ 18, "NspiGetIDsFromNames",
	   nspi_dissect_NspiGetIDsFromNames_request, nspi_dissect_NspiGetIDsFromNames_response},
	{ 19, "NspiResolveNames",
	   nspi_dissect_NspiResolveNames_request, nspi_dissect_NspiResolveNames_response},
	{ 20, "NspiResolveNamesW",
	   nspi_dissect_NspiResolveNamesW_request, nspi_dissect_NspiResolveNamesW_response},
	{ 0, NULL, NULL, NULL }
};

void proto_register_dcerpc_nspi(void)
{
	static hf_register_info hf[] = {
	{ &hf_nspi_NspiGetHierarchyInfo_unknown1, 
	  { "Unknown1", "nspi.NspiGetHierarchyInfo.unknown1", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_ft, 
	  { "Ft", "nspi.SPropValue_CTR.ft", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_object, 
	  { "Object", "nspi.SPropValue_CTR.object", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_input_locale_language, 
	  { "Language", "nspi.input_locale.language", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVft, 
	  { "Mvft", "nspi.SPropValue_CTR.MVft", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SDateTimeArray_lpft, 
	  { "Lpft", "nspi.SDateTimeArray.lpft", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrderSet_aSort, 
	  { "Asort", "nspi.SSortOrderSet.aSort", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SGuidArray_lpguid, 
	  { "Lpguid", "nspi.SGuidArray.lpguid", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_dwAlignPad, 
	  { "Dwalignpad", "nspi.SPropValue.dwAlignPad", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_instance_key, 
	  { "Instance Key", "nspi.NspiQueryRows.instance_key", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrderSet_cCategories, 
	  { "Ccategories", "nspi.SSortOrderSet.cCategories", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPINAMEID_ulKind, 
	  { "Ulkind", "nspi.MAPINAMEID.ulKind", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiDNToEph_server_dn, 
	  { "Server Dn", "nspi.NspiDNToEph.server_dn", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPIUID_ab, 
	  { "Ab", "nspi.MAPIUID.ab", FT_UINT8, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_b, 
	  { "B", "nspi.SPropValue_CTR.b", FT_UINT16, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRow_ulAdrEntryPad, 
	  { "Uladrentrypad", "nspi.SRow.ulAdrEntryPad", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_null, 
	  { "Null", "nspi.SPropValue_CTR.null", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SBinary_cb, 
	  { "Cb", "nspi.SBinary.cb", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SBinaryArray_lpbin, 
	  { "Lpbin", "nspi.SBinaryArray.lpbin", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SShortArray_cValues, 
	  { "Cvalues", "nspi.SShortArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRowSet_aRow, 
	  { "Arow", "nspi.SRowSet.aRow", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiBind_mapiuid, 
	  { "Mapiuid", "nspi.NspiBind.mapiuid", FT_GUID, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SGuidArray_cValues, 
	  { "Cvalues", "nspi.SGuidArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_instance_key_cValues, 
	  { "Cvalues", "nspi.instance_key.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRestriction_CTR_resProperty, 
	  { "Resproperty", "nspi.SRestriction_CTR.resProperty", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_lRows, 
	  { "Lrows", "nspi.NspiQueryRows.lRows", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MV_UNICODE_STRUCT_cValues, 
	  { "Cvalues", "nspi.MV_UNICODE_STRUCT.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrder_ulOrder, 
	  { "Ulorder", "nspi.SSortOrder.ulOrder", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetHierarchyInfo_RowSet, 
	  { "Rowset", "nspi.NspiGetHierarchyInfo.RowSet", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPINAMEID_lID, 
	  { "Lid", "nspi.MAPINAMEID.lID", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_settings, 
	  { "Settings", "nspi.NspiQueryRows.settings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SBinaryArray_cValues, 
	  { "Cvalues", "nspi.SBinaryArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_unknown2, 
	  { "Unknown2", "nspi.NspiGetMatches.unknown2", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SLPSTRArray_strings, 
	  { "Strings", "nspi.SLPSTRArray.strings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_FILETIME_dwHighDateTime, 
	  { "Dwhighdatetime", "nspi.FILETIME.dwHighDateTime", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetProps_REPL_values, 
	  { "Repl Values", "nspi.NspiGetProps.REPL_values", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRow_cValues, 
	  { "Cvalues", "nspi.SRow.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_ulPropTag, 
	  { "Ulproptag", "nspi.SPropValue.ulPropTag", FT_UINT32, BASE_DEC, VALS(nspi_MAPITAGS_vals), 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetProps_settings, 
	  { "Settings", "nspi.NspiGetProps.settings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiDNToEph_instance_key, 
	  { "Instance Key", "nspi.NspiDNToEph.instance_key", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRestriction_CTR_resAnd, 
	  { "Resand", "nspi.SRestriction_CTR.resAnd", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_RowSet, 
	  { "Rowset", "nspi.NspiGetMatches.RowSet", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MV_UNICODE_STRUCT_lpi, 
	  { "Lpi", "nspi.MV_UNICODE_STRUCT.lpi", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrderSet_cSorts, 
	  { "Csorts", "nspi.SSortOrderSet.cSorts", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrder_ulPropTag, 
	  { "Ulproptag", "nspi.SSortOrder.ulPropTag", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVbin, 
	  { "Mvbin", "nspi.SPropValue_CTR.MVbin", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_opnum, 
	  { "Operation", "nspi.opnum", FT_UINT16, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiUnbind_status, 
	  { "Status", "nspi.NspiUnbind.status", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SAndRestriction_lpRes, 
	  { "Lpres", "nspi.SAndRestriction.lpRes", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiDNToEph_size, 
	  { "Size", "nspi.NspiDNToEph.size", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVi, 
	  { "Mvi", "nspi.SPropValue_CTR.MVi", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropertyRestriction_lpProp, 
	  { "Lpprop", "nspi.SPropertyRestriction.lpProp", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_unknown3, 
	  { "Unknown3", "nspi.NspiGetMatches.unknown3", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPI_SETTINGS_flag, 
	  { "Flag", "nspi.MAPI_SETTINGS.flag", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_instance_key_value, 
	  { "Value", "nspi.instance_key.value", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_settings, 
	  { "Settings", "nspi.NspiGetMatches.settings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MV_LONG_STRUCT_cValues, 
	  { "Cvalues", "nspi.MV_LONG_STRUCT.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRowSet_cRows, 
	  { "Crows", "nspi.SRowSet.cRows", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPI_SETTINGS_service_provider, 
	  { "Service Provider", "nspi.MAPI_SETTINGS.service_provider", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_l, 
	  { "L", "nspi.SPropValue_CTR.l", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SAndRestriction_cRes, 
	  { "Cres", "nspi.SAndRestriction.cRes", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_flag, 
	  { "Flag", "nspi.NspiQueryRows.flag", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_REQ_properties, 
	  { "Req Properties", "nspi.NspiQueryRows.REQ_properties", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVl, 
	  { "Mvl", "nspi.SPropValue_CTR.MVl", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetProps_flag, 
	  { "Flag", "nspi.NspiGetProps.flag", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPISTATUS_status, 
	  { "MAPISTATUS", "nspi.MAPISTATUS_status", FT_UINT32, BASE_HEX, VALS(nspi_MAPISTATUS_vals), 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVszW, 
	  { "Mvszw", "nspi.SPropValue_CTR.MVszW", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_LPSTR_lppszA, 
	  { "Lppsza", "nspi.LPSTR.lppszA", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVszA, 
	  { "Mvsza", "nspi.SPropValue_CTR.MVszA", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_property_type, 
	  { "Value", "nspi.SPropValue.value", FT_UINT32, BASE_HEX, VALS(nspi_property_types_vals), 0, NULL, HFILL }},
	{ &hf_nspi_handle, 
	  { "Handle", "nspi.handle", FT_BYTES, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_restrictions, 
	  { "Restrictions", "nspi.NspiGetMatches.restrictions", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_RowSet, 
	  { "Rowset", "nspi.NspiQueryRows.RowSet", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SRow_lpProps, 
	  { "Lpprops", "nspi.SRow.lpProps", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPI_SETTINGS_input_locale, 
	  { "Input Locale", "nspi.MAPI_SETTINGS.input_locale", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropertyRestriction_ulPropTag, 
	  { "Ulproptag", "nspi.SPropertyRestriction.ulPropTag", FT_UINT32, BASE_DEC, VALS(nspi_MAPITAGS_vals), 0, NULL, HFILL }},
	{ &hf_nspi_SPropTagArray_aulPropTag, 
	  { "Aulproptag", "nspi.SPropTagArray.aulPropTag", FT_UINT32, BASE_DEC, VALS(nspi_MAPITAGS_vals), 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetHierarchyInfo_unknown2, 
	  { "Unknown2", "nspi.NspiGetHierarchyInfo.unknown2", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SBinary_lpb, 
	  { "Lpb", "nspi.SBinary.lpb", FT_UINT8, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetHierarchyInfo_settings, 
	  { "Settings", "nspi.NspiGetHierarchyInfo.settings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SSortOrderSet_cExpanded, 
	  { "Cexpanded", "nspi.SSortOrderSet.cExpanded", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiQueryRows_unknown, 
	  { "Unknown", "nspi.NspiQueryRows.unknown", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_input_locale_method, 
	  { "Method", "nspi.input_locale.method", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_instance_key, 
	  { "Instance Key", "nspi.NspiGetMatches.instance_key", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_lpszA, 
	  { "Lpsza", "nspi.SPropValue_CTR.lpszA", FT_STRING, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_bin, 
	  { "Bin", "nspi.SPropValue_CTR.bin", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SShortArray_lpi, 
	  { "Lpi", "nspi.SShortArray.lpi", FT_UINT16, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_lpszW, 
	  { "Lpszw", "nspi.SPropValue_CTR.lpszW", FT_STRING, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_PropTagArray, 
	  { "Proptagarray", "nspi.NspiGetMatches.PropTagArray", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SLPSTRArray_cValues, 
	  { "Cvalues", "nspi.SLPSTRArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_err, 
	  { "Err", "nspi.SPropValue_CTR.err", FT_UINT32, BASE_DEC, VALS(nspi_MAPISTATUS_vals), 0, NULL, HFILL }},
	{ &hf_nspi_SRestriction_PTTYPE, 
	  { "Restriction Type", "nspi.property_type", FT_UINT32, BASE_HEX, VALS(nspi_nspi_RestrictionType_vals), 0, NULL, HFILL }},
	{ &hf_nspi_NAME_STRING_str, 
	  { "Str", "nspi.NAME_STRING.str", FT_STRING, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropertyRestriction_relop, 
	  { "Relop", "nspi.SPropertyRestriction.relop", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_MVguid, 
	  { "Mvguid", "nspi.SPropValue_CTR.MVguid", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPINAMEID_lpguid, 
	  { "Lpguid", "nspi.MAPINAMEID.lpguid", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_FILETIME_dwLowDateTime, 
	  { "Dwlowdatetime", "nspi.FILETIME.dwLowDateTime", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_REQ_properties, 
	  { "Req Properties", "nspi.NspiGetMatches.REQ_properties", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropTagArray_cValues, 
	  { "Cvalues", "nspi.SPropTagArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_lpguid, 
	  { "Lpguid", "nspi.SPropValue_CTR.lpguid", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SPropValue_CTR_i, 
	  { "I", "nspi.SPropValue_CTR.i", FT_UINT16, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPI_SETTINGS_codepage, 
	  { "Codepage", "nspi.MAPI_SETTINGS.codepage", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiDNToEph_flag, 
	  { "Flag", "nspi.NspiDNToEph.flag", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiBind_unknown, 
	  { "Unknown", "nspi.NspiBind.unknown", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MAPI_SETTINGS_handle, 
	  { "Handle", "nspi.MAPI_SETTINGS.handle", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetProps_REQ_properties, 
	  { "Req Properties", "nspi.NspiGetProps.REQ_properties", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_MV_LONG_STRUCT_lpl, 
	  { "Lpl", "nspi.MV_LONG_STRUCT.lpl", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiGetMatches_unknown1, 
	  { "Unknown1", "nspi.NspiGetMatches.unknown1", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_SDateTimeArray_cValues, 
	  { "Cvalues", "nspi.SDateTimeArray.cValues", FT_UINT32, BASE_DEC, NULL, 0, NULL, HFILL }},
	{ &hf_nspi_NspiBind_settings, 
	  { "Settings", "nspi.NspiBind.settings", FT_NONE, BASE_NONE, NULL, 0, NULL, HFILL }},
	};


	static gint *ett[] = {
		&ett_dcerpc_nspi,
		&ett_nspi_MAPIUID,
		&ett_nspi_input_locale,
		&ett_nspi_MAPI_SETTINGS,
		&ett_nspi_SPropTagArray,
		&ett_nspi_instance_key,
		&ett_nspi_MAPINAMEID,
		&ett_nspi_SPropertyRestriction,
		&ett_nspi_SAndRestriction,
		&ett_nspi_SRestriction_CTR,
		&ett_nspi_SRestriction,
		&ett_nspi_SSortOrder,
		&ett_nspi_SSortOrderSet,
		&ett_nspi_NAME_STRING,
		&ett_nspi_SBinary,
		&ett_nspi_FILETIME,
		&ett_nspi_SShortArray,
		&ett_nspi_MV_LONG_STRUCT,
		&ett_nspi_LPSTR,
		&ett_nspi_SLPSTRArray,
		&ett_nspi_SBinaryArray,
		&ett_nspi_SGuidArray,
		&ett_nspi_MV_UNICODE_STRUCT,
		&ett_nspi_SDateTimeArray,
		&ett_nspi_SPropValue_CTR,
		&ett_nspi_SPropValue,
		&ett_nspi_SRow,
		&ett_nspi_SRowSet,
	};

	proto_dcerpc_nspi = proto_register_protocol("Exchange 5.5 Name Service Provider", "NSPI", "nspi");
	proto_register_field_array(proto_dcerpc_nspi, hf, array_length (hf));
	proto_register_subtree_array(ett, array_length(ett));
}

void proto_reg_handoff_dcerpc_nspi(void)
{
	dcerpc_init_uuid(proto_dcerpc_nspi, ett_dcerpc_nspi,
		&uuid_dcerpc_nspi, ver_dcerpc_nspi,
		nspi_dissectors, hf_nspi_opnum);
}
